package com.vdv.stuntcar;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.Region;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.widget.Toast;

import com.vdv.sc.R;

import java.util.Random;

public final class GameView extends SurfaceView implements SurfaceHolder.Callback, Runnable {

	private static final long REFRESH_PERIOD = 20;// ms, 50 Hz

	private Game mGame = null;

	private SurfaceHolder mSurfaceHolder;
	private Bitmap mOffScreenBuffer;
	private Canvas mOffScreenCanvas = new Canvas();
    private Rect mScaleRect = null;
	private Thread mThread = null;
	private final Object mPauseLock = new Object();
	private volatile boolean mIsPaused = false;

	private String getString(final int textId) {
		return getResources().getString( textId );
	}


	public GameView(final Context context) {
		super( context );
		init();
	}

	public GameView(final Context context, final AttributeSet attribute) {
		super( context, attribute );
		init();
	}

	public GameView(final Context context, final AttributeSet attrs, final int defStyleAttr) {
		super( context, attrs, defStyleAttr );
		init();
	}

	private void init() {
		// subscription for SurfaceHolder.Callback events
		mSurfaceHolder = getHolder();
		mSurfaceHolder.addCallback( this );
		mGame = new Game( getResources() );
		// to get key events
		setFocusable( true );
		setFocusableInTouchMode( true );
		requestFocus();
	}

	// start SurfaceHolder.Callback
	@Override
	public final void surfaceCreated(final SurfaceHolder surfaceHolder) {
		// mPF = 0.5f * ((float)getWidth() / Game.SCREEN_WIDTH + (float)getHeight() / Game.SCREEN_HEIGHT);
		mScaleRect = new Rect( 0, 0, getWidth(), getHeight() );
		SoundManager.init( getResources() );
		//  mIsRunning = true;
		new Thread( this, getString( R.string.AppName ) ).start();
		new Thread( mGame, getString( R.string.AppName ) ).start();
	}

	@Override
	public final void surfaceChanged(final SurfaceHolder surfaceHolder, final int format, final int width, final int height) {
	}

	@Override
	public final void surfaceDestroyed(final SurfaceHolder surfaceHolder) {
		synchronized( mPauseLock ) {
			mIsPaused = false;
			// mIsRunning = false;
			mPauseLock.notifyAll();
		}
		// SoundManager.release();
		mOffScreenBuffer.recycle();
	}
	// end SurfaceHolder.Callback

	final void setKeyboardAccessInterface(final MainActivity activity) {
		mGame.setKeyboardAccessInterface( activity );
	}

	final void keyPressed(final int key) {
		mGame.setKey( key );
	}

	final void keyReleased(final int key) {
		mGame.resetKey( key );
	}

	final void keyTyped(final char key) {
		if( key != 0 ) {// 0 is set after reading
			mGame.setKeyTyped(key );
		}
	}

	private void paintAndWait(long time) {
		Canvas c = null;
		try {
			synchronized( Game.class ) {
				c = mSurfaceHolder.lockCanvas();
				mOffScreenCanvas.drawBitmap( mGame.getScene(), 0, 0, null );
				Game.vertbInterrupt( mOffScreenCanvas );
			}
			c.drawBitmap( mOffScreenBuffer, null, mScaleRect, null );
			// c.drawBitmap( mOffScreenBuffer, 0f, 0f, null );
			// mSurfaceHolder.unlockCanvasAndPost( c );
			// set specified refresh rate
			time += REFRESH_PERIOD;
			time -= System.currentTimeMillis();
			if( time > 0 ) {
				Thread.sleep( time );
			}
		} catch(final Exception e) {// if c == null || m_sh = null || interrupt
		} finally {
			if( c != null ) {
				mSurfaceHolder.unlockCanvasAndPost( c );
			}
		}
		synchronized( mPauseLock ) {
			if( mIsPaused ) {
				while( mIsPaused ) {
					try {
						mPauseLock.wait();
					} catch (final InterruptedException e) {// nothing to do
					}
				}
			}
		}
	}

	// Runnable
	@Override
	public final void run() {
		// Creating bitmap with attaching it to the buffer-canvas,
		// it means that all the changes
		// done with the canvas are captured into the attached bitmap
		mOffScreenBuffer = Bitmap.createBitmap( Game.SCREEN_WIDTH, Game.SCREEN_HEIGHT, Bitmap.Config.ARGB_8888 );
		mOffScreenCanvas = new Canvas( mOffScreenBuffer );
		mOffScreenCanvas.drawColor( 0xFF000000 );

		while( true ) {
			final long curr_time = System.currentTimeMillis();
			paintAndWait( curr_time );
		} // while running
	}

	final void pause() {
		mGame.pause();
	}

	final void resume() {
		mGame.resume();
	}
}
