package com.vdv.stuntcar;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.inputmethod.InputMethodManager;

import com.vdv.sc.BuildConfig;
import com.vdv.sc.R;
import com.vdv.sc.databinding.ActivityMainBinding;

import java.io.File;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

public final class MainActivity extends Activity implements View.OnTouchListener {

	public static final String PREF_NAME = "StuntCar";

	/**
	 * Returns the app specific directory for files.
	 * If the directory does not exist, it creates.
	 *
	 * @return a File object that represents a path to save app data.
	 */
	public File getAppFileDirectory() {
		if( Environment.MEDIA_MOUNTED.equals( Environment.getExternalStorageState() ) ) {
			final File dir = getExternalFilesDir( null );// sContext.getExternalCacheDir();
			// null if API < 18, Android < 4.4 due to uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE"
			// but if the permission is not requested, the tool is not able to send a file using Intent SEND_TO
			if( dir != null ) {
				return dir;
			}
		}
		return getFilesDir();// sContext.getCacheDir();
	}

	private ActivityMainBinding mBinding;

	@Override
	protected final void onCreate(Bundle savedInstanceState) {
		super.onCreate( savedInstanceState );
 if( BuildConfig.DEBUG ) {
			Thread.setDefaultUncaughtExceptionHandler( new Thread.UncaughtExceptionHandler() {
				@Override
				public final void uncaughtException(final Thread thread, final Throwable ex) {
					//Log.e("Error", "Unhandled exception: " + ex.getMessage());
					// Extract to the file.
					// BufferedReader reader = null;
					PrintStream writer = null;
					try {
						// Make file name - file must be saved to external storage or it wont be readable by
						// the email app.
						final File file = new File( getAppFileDirectory(),
								new SimpleDateFormat("'" + PREF_NAME + "_CrashReport_'yyyy-dd-MM_HH-mm-ss'.txt'", Locale.US )
										.format( new Date( System.currentTimeMillis() ) ) );
						//if( ! file.getParentFile().exists() ) {
						//	file.getParentFile().mkdirs();
						//}
						// write output stream
						writer = new PrintStream( file );
						writer.println( "Android version: " + Build.VERSION.SDK_INT );
						String model = Build.MODEL;
						if( ! model.startsWith( Build.MANUFACTURER ) ) {
							model = Build.MANUFACTURER + " " + model;
						}
						writer.println( "Device: " + model );
						PackageInfo info = null;
						try {
							info = getPackageManager().getPackageInfo( getPackageName(), 0 );
						} catch(final PackageManager.NameNotFoundException e2) {
						}
						writer.println( "App version: " + (info == null ? "(null)" : info.versionCode) );
						writer.println( "Thread: " + thread.getName() );
						writer.println( "Message: " + ex.getMessage() );
						ex.printStackTrace( writer );
					} catch (final Exception e) {
						// You might want to write a failure message to the log here.
					} finally {
						if( writer != null ) {
							writer.close();
						}
					}
					System.exit( 1 );
				}
			} );
}// if debug
		if( ! isTaskRoot()
				&& getIntent().hasCategory( Intent.CATEGORY_LAUNCHER )
				&& Intent.ACTION_MAIN.equals( getIntent().getAction() ) )
		{
			finish();
			return;
		}

		mBinding = ActivityMainBinding.inflate( getLayoutInflater() );
		setContentView( mBinding.getRoot() );

		mBinding.leftBoostAcceleration.setOnTouchListener( this );
		mBinding.leftAcceleration.setOnTouchListener( this );
		mBinding.leftBoostBreak.setOnTouchListener( this );
		mBinding.leftBreak.setOnTouchListener( this );
		mBinding.leftSteeringLeft.setOnTouchListener( this );

		mBinding.rightBoostAcceleration.setOnTouchListener( this );
		mBinding.rightAcceleration.setOnTouchListener( this );
		mBinding.rightBoostBreak.setOnTouchListener( this );
		mBinding.rightBreak.setOnTouchListener( this );
		mBinding.rightSteeringRight.setOnTouchListener( this );
		// setSplashScreen();
		mBinding.gameview.setKeyboardAccessInterface( this );
		hideKeyboard();
	}

	@Override
	protected final void onPause() {
		mBinding.gameview.pause();
		super.onPause();
	}

	@Override
	protected final void onResume() {
		mBinding.gameview.resume();
		super.onResume();
	}

	// OnTouchListener
	@Override
	public final boolean onTouch(final View v, final MotionEvent event) {
		switch( event.getAction() ) {
			case MotionEvent.ACTION_DOWN:// PRESSED
				switch( v.getId() ) {
					case R.id.leftBoostAcceleration:
					case R.id.rightBoostAcceleration:
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_DPAD_UP );
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_ENTER );
						break;
					case R.id.leftAcceleration:
					case R.id.rightAcceleration:
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_DPAD_UP );
						break;
					case R.id.leftBoostBreak:
					case R.id.rightBoostBreak:
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_DPAD_DOWN );
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_ENTER );
						break;
					case R.id.leftBreak:
					case R.id.rightBreak:
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_DPAD_DOWN );
						break;
					case R.id.leftSteeringLeft:
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_DPAD_LEFT );
						break;
					case R.id.rightSteeringRight:
						mBinding.gameview.keyPressed( KeyEvent.KEYCODE_DPAD_RIGHT );
						mBinding.gameview.keyTyped( '\n' );//  (char) KeyEvent.KEYCODE_ENTER );
						break;
				}
				return false;
			case MotionEvent.ACTION_UP:// RELEASED
				switch( v.getId() ) {
					case R.id.leftBoostAcceleration:
					case R.id.rightBoostAcceleration:
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_DPAD_UP );
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_ENTER );
						break;
					case R.id.leftAcceleration:
					case R.id.rightAcceleration:
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_DPAD_UP );
						break;
					case R.id.leftBoostBreak:
					case R.id.rightBoostBreak:
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_DPAD_DOWN );
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_ENTER );
						break;
					case R.id.leftBreak:
					case R.id.rightBreak:
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_DPAD_DOWN );
						break;
					case R.id.leftSteeringLeft:
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_DPAD_LEFT );
						break;
					case R.id.rightSteeringRight:
						mBinding.gameview.keyReleased( KeyEvent.KEYCODE_DPAD_RIGHT );
						break;
				}
				return false;
		}
		return false;
	}
	/*
	There are two separate bugs (issues 42904 and 62306) affecting KEYCODE_DEL
	 generation in the default Google Keyboard (LatinIME).
	  Some versions of the keyboard have one bug, some have the other,
	   and some don't have either. So, that is why some of your devices exhibit
		a problem while others do not; it depends upon the version of
		 Google Keyboard that is installed on the device.

	 https://stackoverflow.com/questions/18581636/android-cannot-capture-backspace-delete-press-in-soft-keyboard/19980975#19980975
	*/

   @Override
	public final boolean dispatchKeyEvent(final KeyEvent event) {

		if( event.getAction() == KeyEvent.ACTION_DOWN ) {
			mBinding.gameview.keyPressed( event.getKeyCode() );
			mBinding.gameview.keyTyped( (char)event.getUnicodeChar( event.getMetaState() ) );
		}

		return super.dispatchKeyEvent( event );
	}

/*
	@Override
	public final boolean onKeyDown(final int keyCode, final KeyEvent event) {
		mBinding.gameview.keyTyped( (char)event.getUnicodeChar( event.getMetaState() ) );
		return super.onKeyDown( keyCode, event );
	}
*/
/*
	final void setSplashScreen() {
		getFragmentManager()
				.beginTransaction()
				.replace( R.id.placeholder, FirstFragment.newFragment() )
				.commit();
	}

	final void setUserNameScreen() {

	}
*/

	final void showKeyboard() {
		final InputMethodManager im = (InputMethodManager) getSystemService( Context.INPUT_METHOD_SERVICE );
		// im.hideSoftInputFromWindow( mBinding.gameview.getWindowToken(), 0 );
		mBinding.gameview.requestFocus();
		im.toggleSoftInput( InputMethodManager.SHOW_FORCED, InputMethodManager.HIDE_IMPLICIT_ONLY );
		//mBinding.gameview.setIn
		// im.showSoftInput( mBinding.gameview, InputMethodManager.SHOW_IMPLICIT );
	}

	final void hideKeyboard() {
		final InputMethodManager im = (InputMethodManager) getSystemService( Context.INPUT_METHOD_SERVICE );
		im.hideSoftInputFromWindow( mBinding.gameview.getWindowToken(), 0 );
	}
}