package com.vdv.stuntcar;

import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.view.KeyEvent;

import com.vdv.sc.R;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;

public final class Game implements Runnable {

	/** Picture scale factor, greater or equal to 1. */
	private static final int PF = 2;

	private static final Path path = new Path();

	private static final Paint paint = new Paint();
	static {
		paint.setStyle( Paint.Style.FILL );
		paint.setStrokeWidth( PF );
		paint.setXfermode( new PorterDuffXfermode( PorterDuff.Mode.SRC_OVER ) );
	}

	/**
	 * Screen width in px
	 */
	public static final int SCREEN_WIDTH = 320 * PF;

	/**
	 * Screen height in px
	 */
	public static final int SCREEN_HEIGHT = 200 * PF;

	private static final int SCREEN_AREA_WIDTH = 256 * PF;
	private static final int SCREEN_AREA_HEIGHT = 128 * PF;
	private static final int SCREEN_AREA_LEFT_X = 32 * PF;// (SCREEN_WIDTH - SCREEN_AREA_WIDTH) / 2;
	private static final int SCREEN_AREA_TOP_Y = 16 * PF;
	private static final int SCREEN_AREA_RIGHT_X = (SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH);
	private static final int SCREEN_AREA_BOTTOM_Y = (SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT);

	private static final int MAX_DAMAGE_LINE_START_X = 8 * PF;
	private static final int MAX_DAMAGE_LINE_X = 240 * PF;

	private static final byte MIN_FRAMES = 6;// originally 6

	private static final int FADE_FRAMES = 30;// 30 * 20ms = 600 ms

	private static final byte SLAVE = 0x40;
	private static final byte MASTER = (byte) 0x80;

	private static final int NUMBER_OF_RACERS = 12;

	/**
	 * (238 / 256)
	 */
	private static final int REDUCTION = 238;

	private static final int CAR_WEIGHT = 317;

	// graphic_info
	/**
	 * Word 1  --  not used
	 * Word 2  --  not used
	 * Word 3  --  number of words wide - 1
	 * Word 4  --  number of lines high - 1
	 * Word 5  --  x position in words
	 * Word 6  --  y position
	 * Word 7  --  not used
	 * Word 8  --  not used
	 */
	private static final class GraphicsInfo {

		private Bitmap mImage = null;

		/**
		 * X from left
		 */
		private int mX;

		/**
		 * Y from top
		 */
		private int mY;
		// private int mLines;

		private GraphicsInfo(final int x, final int y) {
			mX = x;
			mY = y;
		}
	}

	private static final int SPRITE_WHEEL_SPRITES = 3;
	private static final int SPRITE_RIGHT_WHEELS = 0;
	private static final int SPRITE_LEFT_WHEELS = 3;

	private static final int SPRITE_LEFT_FLAMES = 6;
	private static final int SPRITE_LEFT_FLAME_3 = 49;
	private static final int SPRITE_RIGHT_FLAMES = 8;
	private static final int SPRITE_RIGHT_FLAME_3 = 50;
	private static final int SPRITE_FLAME_SPRITES = 3;
	private static final int SPRITE_ENGINE_BLOCK = 10;
	private static final int SPRITE_LEFT_EXHAUST_PANEL = 11;
	private static final int SPRITE_RIGHT_EXHAUST_PANEL = 12;
	private static final int SPRITE_LEFT_TOP_CORNER = 13;
	private static final int SPRITE_RIGHT_TOP_CORNER = 14;
	private static final int SPRITE_CHEQUERED_FLAG_BRIGHT = 15;
	private static final int SPRITE_CHEQUERED_FLAG_DULL = 16;
	private static final int SPRITE_STOP_WATCH_BRIGHT = 17;
	private static final int SPRITE_STOP_WATCH_DULL = 18;
	private static final int SPRITE_LEFT_CHAIN_TOP = 19;
	private static final int SPRITE_LEFT_CHAIN_BOTTOM = 20;
	private static final int SPRITE_RIGHT_CHAIN_TOP = 21;
	private static final int SPRITE_RIGHT_CHAIN_BOTTOM = 22;

	private static final int SPRITE_HOLE_1 = 23;
	private static final int SPRITE_HOLE_2 = 24;

	private static final int SPRITE_SMASH_1 = 25;
	private static final int SPRITE_SMASH_2 = 26;

	private static final int SPRITE_DAMAGE_BAR_CLEAR_1 = 27;
	private static final int SPRITE_DAMAGE_BAR_CLEAR_2 = 28;

	private static final int SPRITE_MESSAGE_PANEL = 51;

	private static final int J_SPRITE_CURRENT_LEFT_FLAME = 46;
	private static final int J_SPRITE_CURRENT_RIGHT_FLAME = 43;

	/**
	 * 16 bytes per index
	 */
	private static final GraphicsInfo[] graphic_info = {
			// dc.w 0, 0, 1, 57, 16, 119, 0, 0 right wheels
			// dc.w 2, 0, 1, 57, 16, 119, 0, 0
			// dc.w 4, 0, 1, 57, 16, 119, 0, 0
			// 0 right wheels
			new GraphicsInfo( (16 * 16 + 8) * PF, 119 * PF ),// java +8
			new GraphicsInfo( (16 * 16 + 8) * PF, 119 * PF ),
			new GraphicsInfo( (16 * 16 + 8) * PF, 119 * PF ),

			// dc.w 8, 0, 1, 57, 2, 119, 0, 0 left wheels
			// dc.w 10, 0, 1, 57, 2, 119, 0, 0
			// dc.w 12, 0, 1, 57, 2, 119, 0, 0
			// 3 left wheels
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),

			// dc.w 0, 68, 3, 27, 2, 123, 0, 0 left flames
			// dc.w 4, 68, 3, 27, 2, 123, 0, 0
			// 6 left flames
			new GraphicsInfo( (2 * 16 + 15) * PF, (123 + 1) * PF ),// java +15, +1
			new GraphicsInfo( (2 * 16 + 15) * PF, (123 + 1) * PF ),

			// dc.w 8, 68, 3, 27, 14, 123, 0, 0 right flames
			// dc.w 12, 68, 3, 27, 14, 123, 0, 0
			// 8 right flames
			new GraphicsInfo( (14 * 16 + 1) * PF, (123 + 1) * PF ),// java +1, +1
			new GraphicsInfo( (14 * 16 + 1) * PF, (123 + 1) * PF ),

			// dc.w 2, 123, 15, 20, 2, 123, 0, 0 engine block
			// 10 engine block
			new GraphicsInfo( (2 * 16 + 73) * PF, 123 * PF ),// java +73

			// dc.w 2, 144, 1, 14, 2, 144, 0, 0 left / right exhaust covering wheel
			// dc.w 16, 144, 1, 14, 16, 144, 0, 0
			// 11 left / right exhaust covering wheel
			new GraphicsInfo( (2 * 16 + 7) * PF, (144 - 19) * PF ),// java +7,-19
			new GraphicsInfo( (16 * 16 - 38) * PF, (144 - 19) * PF ),// java -37,-19

			// dc.w 2, 16, 0, 5, 2, 16, 0, 0 left / right top corner
			// dc.w 17, 16, 0, 5, 17, 16, 0, 0
			// 13 left / right top corner
			new GraphicsInfo( (2 * 16) * PF, 16 * PF ),
			new GraphicsInfo( (17 * 16 + 8) * PF, 16 * PF ),// java +8

			// dc.w 14, 0, 0, 7, 6, 190, 0, 0 chequered flag bright / dull
			// dc.w 14, 8, 0, 7, 6, 190, 0, 0
			// 15 chequered flag bright / dull
			new GraphicsInfo( (6 * 16 + 2) * PF, 190 * PF ),// java +2
			new GraphicsInfo( (6 * 16 + 2) * PF, 190 * PF ),

			// dc.w 14, 16, 0, 7, 13, 190, 0, 0 stop watch bright / dull
			// dc.w 14, 24, 0, 7, 13, 190, 0, 0
			// 17 stop watch bright / dull
			new GraphicsInfo( (13 * 16 + 2) * PF, (190 + 1) * PF ),// java +2,+1
			new GraphicsInfo( (13 * 16 + 2) * PF, (190 + 1) * PF ),

			// dc.w 15, 0, 0, 7, 4, 16, 0, 0 left chain top / bottom
			// dc.w 15, 8, 0, 7, 4, 24, 0, 0
			// 19 left chain top / bottom
			new GraphicsInfo( (4 * 16 + 6) * PF, 16 * PF ),// java +6
			new GraphicsInfo( (4 * 16 + 6) * PF, 24 * PF ),

			// dc.w 15, 16, 0, 7, 15, 16, 0, 0 right chain top / bottom
			// dc.w 15, 24, 0, 7, 15, 24, 0, 0
			// 21 right chain top / bottom
			new GraphicsInfo( 15 * 16 * PF, 16 * PF ),
			new GraphicsInfo( 15 * 16 * PF, 24 * PF ),

			// dc.w 16, 0, 1, 7, 4, 0, 0, 0 hole position 1 / 2
			// dc.w 16, 8, 1, 7, 4, 0, 0, 0
			// 23 hole position 1 / 2
			new GraphicsInfo( 4 * 16 * PF, 0 * PF ),
			new GraphicsInfo( 4 * 16 * PF, 0 * PF ),

			// dc.w 16, 16, 1, 7, 4, 0, 0, 0 smash position 1 / 2
			// dc.w 16, 24, 1, 7, 4, 0, 0, 0
			// 25 smash position 1 / 2
			new GraphicsInfo( 4 * 16 * PF, 0 * PF ),
			new GraphicsInfo( 4 * 16 * PF, 0 * PF ),

			// dc.w 16, 32, 1, 7, 4, 0, 0, 0 damage bar clear position 1 / 2
			// dc.w 16, 40, 1, 7, 4, 0, 0, 0
			// 27 damage bar clear position 1 / 2
			new GraphicsInfo( 4 * 16 * PF, 0 * PF ),
			new GraphicsInfo( 4 * 16 * PF, 0 * PF ),

			// dc.w 0, 96, 3, 33, 0, 0, 0, 0 dust clouds
			// dc.w 5, 96, 3, 30, 0, 0, 0, 0
			// dc.w 9, 96, 3, 37, 0, 0, 0, 0
			// dc.w 13, 96, 4, 35, 0, 0, 0, 0
			// dc.w 0, 134, 2, 27, 0, 0, 0, 0
			// dc.w 3, 134, 3, 33, 0, 0, 0, 0
			// dc.w 7, 134, 3, 33, 0, 0, 0, 0
			// dc.w 11, 134, 3, 35, 0, 0, 0, 0
			// 29 dust clouds
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),
			new GraphicsInfo( 0, 0 ),

			//--- java: not used
			// dc.w 1, 176, 0, 15, 17, 119, 0, 0 sprites
			// 37 3 right wheel parts
			new GraphicsInfo( 17 * 16 * PF, 119 * PF ),
			// dc.w 3, 176, 0, 15, 17, 119, 0, 0
			new GraphicsInfo( 17 * 16 * PF, 119 * PF ),
			// dc.w 5, 176, 0, 15, 17, 119, 0, 0
			new GraphicsInfo( 17 * 16 * PF, 119 * PF ),

			// dc.w 8, 176, 0, 15, 2, 119, 0, 0
			// 40 3 left wheel parts
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			// dc.w 10, 176, 0, 15, 2, 119, 0, 0
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			// dc.w 12, 176, 0, 15, 2, 119, 0, 0
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),

			// dc.w 0, 176, 0, 15, 17, 119, 0, 0
			// 43 3 right wheel parts
			// new GraphicsInfo( 17 * 16 * PF, 119 * PF ),
			// java changed: current right flame
			new GraphicsInfo( (14 * 16 + 1) * PF, (123 + 1) * PF ),// java +1, +1

			// dc.w 2, 176, 0, 15, 17, 119, 0, 0
			new GraphicsInfo( 17 * 16 * PF, 119 * PF ),
			// dc.w 4, 176, 0, 15, 17, 119, 0, 0
			new GraphicsInfo( 17 * 16 * PF, 119 * PF ),

			// dc.w 7, 176, 0, 15, 2, 119, 0, 0
			// 46 3 left wheel part
			// new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			// java changed: current left flame
			new GraphicsInfo( (2 * 16 + 15) * PF, (123 + 1) * PF ),// java +15, +1

			// dc.w 9, 176, 0, 15, 2, 119, 0, 0
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			// dc.w 11, 176, 0, 15, 2, 119, 0, 0
			new GraphicsInfo( 2 * 16 * PF, 119 * PF ),
			//--- end java: not used

			// dc.w 16, 68, 3, 27, 2, 123, 0, 0 left flame
			// 49 left flame 3
			new GraphicsInfo( (2 * 16 + 15) * PF, (123 + 1) * PF ),// java +15, +1

			// dc.w 16, 172, 3, 27, 14, 123, 0, 0 right flame
			// 50 right flame 3
			new GraphicsInfo( (14 * 16 + 1) * PF, (123 + 1) * PF ),// java +1, +1

			// dc.w 16, 134, 3, 27, 8, 27, 0, 0 message panel
			// 51 message panel
			new GraphicsInfo( 8 * 16 * PF, 27 * PF )
	};

	/**
	 * Java: a current screen to draw
	 */
	// private static Bitmap screen_mem = null;
	private static Canvas screen_mem_canvas = null;

	/**
	 * Java: the 1st off-screen buffer
	 */
	private static Bitmap screen1 = null;
	private static Canvas screen1_canvas = null;

	/**
	 * Java: the 2nd off-screen buffer
	 */
	private static Bitmap screen2 = null;
	private static Canvas screen2_canvas = null;

	// x and y position to print next character
	private static byte b_print_column = 0;
	private static byte b_print_row = 0;

	private static boolean adjust_sprites = false;

	/**
	 * 0...15
	 */
	private static byte b_57c61;
	private static byte b_57c62;
	private static byte b_57c63;
	/**
	 * unsigned byte
	 */
	private static byte b_57c64;
	private static byte b_57c66;
	private static byte b_57c67;

	/**
	 * 0, 0x41, 0x42, 0x80
	 */
	private static byte b_5d724;
	/**
	 * 0, 0x40, 0x80
	 */
	private static byte b_text_array_selection = 0;
	/**
	 * 1 ... 7
	 */
	private static byte b_multi_no_of_players = 0;
	/**
	 * 0...4
	 */
	private static byte b_5eb77 = 0;
	private static byte b_max_races_in_division = 6;
	/**
	 * Player id for races.
	 * Computer ids are: {@link #b_racer_id_1} and {@link #b_racer_id_2}
	 */
	private static byte b_racer_id_3 = (byte) (NUMBER_OF_RACERS - 1);
	private static boolean b_5eb7a = false;
	private static byte b_current_race_number = 0;
	private static byte b_5eb7c = 0;
	private static byte b_5eb7d = 0;
	private static byte b_5eb7e = 0;
	private static boolean b_game_started = false;

	private static short w_57c72 = 0;

	private static volatile byte b_frame_count = 0;//	dc.b	0,0
	private static volatile byte fade_frame_count = 0;//	dc.b	0,

	private static boolean b_57c3a = false;// dc.b	0,0
	/**
	 * 0, 0x80, 0x40
	 */
	private static byte b_machine;
	private static short transmit_byte_total = 0;
	private static short w_com_receive_byte_total = 0;
	private static short w_copy_receive_byte_total = 0;
	private static short w_com_57c44 = 0;
	private static short w_com_opp_rear_left_actual_height = 0;
	private static short w_com_57c48 = 0;
	private static short w_com_opp_front_actual_height = 0;
	private static short w_com_opponents_z_speed = 0;
	private static short w_com_opponents_road_x_position = 0;
	private static byte b_com_machine = 0;
	private static byte b_com_57c51 = 0;
	private static byte b_com_57c52 = 0;
	private static byte b_com_57c53 = 0;
	private static byte b_com_car_on_chains_countdown = 0;
	private static byte b_com_57c55 = 0;
	private static byte b_com_time_results_min = 0;
	private static byte b_com_time_results_sec = 0;
	private static byte b_com_time_results_ms = 0;
	private static byte b_com_57c59 = 0;
	private static byte b_com_loop_counter = 0;
	private static boolean b_57c5b = false;
	private static byte b_57c5c = 0;// received info bytes
	private static byte b_57c5d = 0;
	private static byte b_57c5e = 0;
	private static byte b_57c5f = 0;
	private static byte b_57c60 = 0;
	private static byte b_com_opponents_road_piece = 0;
	private static byte b_57c6b = 0;
	private static byte b_57c6c = 0;
	private static byte b_57c6d = 0;
	private static boolean b_57c6e = false;

	private static short w_com_opponents_distance_into_piece = 0;

	private static byte b_prompt_groups = 0;
	// private static byte prompt_offset = 0;
	private static String b_prompt_offset = null;

	private static byte b_grounded_delay = 0;//	dc.b	0,0

	/**
	 * offset x4
	 */
	private static short w_road_section_offset = 0;

	/**
	 * x or z position
	 */
	private static short w_surface_position = 0;

	private static byte fp_level = 0;//	dc.b	0,0
	private static short bottom_ptrs_offset = 0;
	private static short fp_y_flag = 0;
	private static int fp_ptr1 = 0;
	private static int fp_ptr2 = 0;
	private static byte b_yellow_road_lines = 0;//	dc.b	0,0
	/**
	 * 0x80, 0x40
	 */
	private static byte b_pit_and_start_byte = 0;// dc.b 0,0
	private static boolean b_closest_section_flag = false;//	dc.b	0,0
	private static byte b_left_side_flag = 0;//	dc.b	0,0
	// dc.w	0
	private static byte b_right_side_flag = 0;//	dc.b	0,0
	// dc.w	0
	private static short w_next_road_section_offset = 0;
	private static short w_copy_road_section_offset = 0;
	private static byte b_start_line_flag = 0;//	dc.b	0,0
	/**
	 * Color index for fill polygon
	 */
	private static byte b_fill_polygon_color = 0;//	dc.b	0,0
	// dc.w	0
	private static int fp_start = 0;
	private static int fp_end = 0;
	private static int fp_diff = 0;

	private static final int SECTION_SIDE_PTRS_OFF = 32;
	/**
	 * it is an array of 64 ints, and initially it point to the middle.
	 * Amiga to java:
	 * section_side_ptrs is section_side_ptrs[ SECTION_SIDE_PTRS_OFF ]
	 */
	private static final int[] lw_section_side_ptrs = new int[ 64 ];

	private static final class Bottom {

		/**
		 * y coordinate
		 */
		private short mD0_i16;

		/**
		 * offset to {@link #lw_section_side_ptrs}
		 */
		private int mD2_i32;

		/**
		 * offset to {@link #lw_section_side_ptrs}
		 */
		private int mD6_i32;
	}

	/*
	 * Holds the structures:
	 * {int16, int32, int22}
	 */
	// private static final int[] bottom_ptrs = new int[ 64 ];
	private static final Bottom[] bottom_ptrs = {
			new Bottom(), new Bottom(), new Bottom(), new Bottom(), new Bottom(),
			new Bottom(), new Bottom(), new Bottom(), new Bottom(), new Bottom()
	};

	private static short w_far_road_limit = 0;

	private static byte b_max_far_coord = 0;//	dc.b	0,0
	private static short w_old_left_x = 0;
	private static short w_old_right_x = 0;

	/**
	 * 0, 0x4000, 0x8000
	 */
	private static short w_current_far_section_flag = 0;

	/**
	 * offset for {@link #w_far_section_coordinates}
	 * Amiga: 2 aligned offset, Java: just an offset for short[] in bytes, use /2
	 */
	private static int lw_end_far_sections_ptr = 0;

	private static final char ASCII_DELETE = 127;
	private static volatile int key_code = 0;// java: key code
	// private static final short[] key_array = new short[ 64 ];
	private static final HashSet< Integer > key_array = new HashSet<>();// java: key code

	/**
	 * offset for short[] {@link #w_screen_coords} as bytes.
	 * use edge_space[ edge_space_ptr / 2 ]
	 */
	private static int lw_screen_coords_ptr = 0;

	/**
	 * Offset to {@link #w_screen_coords}. For offset in bytes, use x2
	 */
	private static final int END_SCREEN_COORDS = 4500;
	/**
	 * struct{ y1, y2, x1, x2 }, x,..., straight edge count, 0x8000
	 * y is changed by 1 automatically for each x value
	 */
	private static final short[] w_screen_coords = new short[ 4500 + 500 ];
	// private static final short[] end_edge_space = new short[ 500 ];

	/**
	 * offset for {@link #w_far_section_coordinates}
	 * Amiga: 2 aligned offset, Java: just an offset to short[], in bytes, use /2
	 */
	private static final int[] lw_far_section_ptrs = new int[ 80 ];
	/**
	 * (pit / start line flag << 8) | (color flag)
	 */
	private static final short[] w_far_section_flags = new short[ 64 ];

	/* private static final class FarSectionCoordinates {
		short mFlag;

		// 0 - left, 1 - right
		final Coord3D[] mCoords = { new Coord3D(), new Coord3D() };
	} */

	/**
	 * (((flag << 8) | section), x1, z1, y1_fp5, x2, z2, y2_fp5)
	 * flag: 0x4000 or 0x8000 if curve
	 */
	private static final short[] w_far_section_coordinates = new short[ 600 ];
	/* private static final FarSectionCoordinates[] w_far_section_coordinates = new FarSectionCoordinates[ 42 ];
	static {
		for( int i = 0; i < w_far_section_coordinates.length; i++ ) {
			w_far_section_coordinates[ i ] = new FarSectionCoordinates();
		}
	} */

	// java: offsets are used instead of addresses, so 0 is valid data.
	/**
	 * Amiga: 0x80000000
	 * Java: 0x80FFFFFF
	 */
	private static final int SECTION_DATA_END_MARKER = 0x80FFFFFF;

	/**
	 * Amiga: 0
	 * Java: 0xFFFFFF
	 */
	private static final int SECTION_DATA_NO_DATA = 0xFFFFFF;

	/**
	 * Flags and offsets for short[] {@link #w_screen_coords} as bytes.
	 * 32 bytes per section, 8 ints
	 * <p>
	 * int 0 (+0): low 3 bytes is a pointer to {@link #w_screen_coords} for left road line
	 * 3 byte:
	 * bit 6: left/right side flag, line: y1 < y2
	 * <p>
	 * int 1 (+4): low 3 bytes is a pointer to {@link #w_screen_coords} for right road line
	 * <p>
	 * int 2 (+8): ?
	 * 3 byte:
	 * bit 6: left/right side flag
	 * <p>
	 * int 3 (+12): ?
	 * <p>
	 * int 4 (+16): low 3 bytes is a pointer to {@link #w_screen_coords} for left side line
	 * <p>
	 * int 5 (+20): low 3 bytes is a pointer to {@link #w_screen_coords} for right side line
	 * 3 byte:
	 * bit 6: left/right side flag
	 * <p>
	 * int 6 (+24): ?
	 * <p>
	 * int 7 (+28):
	 * 3 byte: flags
	 * 0 bit: 0 - use {@link #SIDES_COLOR_A}, 1 - use {@link #b_side_lines_color}
	 * <p>
	 * 2 byte: bit 0...3 is color, bit 7 is to draw or not
	 */
	private static final int[] lw_screen_data = new int[ 412 ];

	private static final byte DEFAULT_VISIBLE_RANGE = 120;
	/**
	 * default value is 120, {@link #DEFAULT_VISIBLE_RANGE},
	 * for some sections it can be lower
	 */
	private static final byte[] b_road_visible_range_table = new byte[ 80 ];
	/**
	 * map_x << 8 |
	 */

	private static short w_road_piece_x_offset = 0;// TODO verify using
	/**
	 * map_z << 8 |
	 */
	private static short w_road_piece_z_offset = 0;// TODO verify using

	//---
	private static byte b_57c65 = 0;
	/**
	 * 0, 1, 0x80
	 */
	private static byte b_opponent_draw_flag = 0;
	private static byte b_57c6f = 0;
	//---
	/**
	 * unsigned byte
	 */
	private static byte b_opponent_max_speed = 0;
	private static byte b_opponent_base_speed = 0;
	private static byte b_damaged_limit = 0;//	dc.b	0,0

	private static short w_opp_smallest_difference = 0;
	/**
	 * signed
	 */
	private static byte b_cars_collided_delay = 0;// dc.b	0,0

	private static byte b_offset_for_last_two_coords = 0;//	dc.b	0,0

	// See: http://amigadev.elowar.com/read/ADCD_2.1/Devices_Manual_guide/node019D.html
	// http://amigadev.elowar.com/read/ADCD_2.1/Libraries_Manual_guide/node0479.html
	private static final int AMIGA_KEY_PRESSED = 0xB3;
	private static final int AMIGA_KEY_CODE_P = 0x19;
	private static final int AMIGA_KEY_CODE_RET = 0x44;
	private static final int AMIGA_KEY_CODE_ESC = 0x45;
	private static final int AMIGA_KEY_CODE_UP = 0x4C;
	private static final int AMIGA_KEY_CODE_DOWN = 0x4D;
	private static final int AMIGA_KEY_CODE_RIGHT = 0x4E;
	private static final int AMIGA_KEY_CODE_LEFT = 0x4F;
	private static final int AMIGA_KEY_CODE_HELP = 0x5F;
	private static final int AMIGA_KEY_CODE_RIGHT_AMIGA = 0x67;
	private static final int AMIGA_KEY_CODE_F1 = 0xD1;

	private static final int NUM_PALETTE_ENTRIES = 42;

	private static final int[] SCPalette = new int[/* NUM_PALETTE_ENTRIES */ ] {
			// menu
			0xFF000000,// menu text in lines
			0xFFffffff,// menu text "select"
			0xFFbbbbbb,// temp background gray line
			0xFF555555,// menu background

			0xFF000000,
			0xFF000000,
			0xFF000000,
			0xFFffff00,// hall of fame, names
			0xFF000000,
			0xFF000000,

			0xFF000000,
			0xFF0099DD,// 11, empty file name in load/save
			0xFF000000,
			0xFF000000,
			0xFF000000,// 15 file name in load/save
			0xFFff9900,// menu text "division"
			0xFF000000,
			0xFF000000,

			// car colors 2
			0xFF000000,
			0xFF220088,
			0xFF3300aa,
			0xFF4400cc,
			0xFF5500ee,
			0xFF222233,
			0xFF444444,
			0xFF333333,

			// track colors (i.e. Stunt Car Racer car colors)
			0xFF000000,
			0xFF999977,
			0xFFbbbb99,
			0xFFffff00,
			0xFF99bb33,
			0xFF557777,
			0xFF55bbff,
			0xFF5599ff,
			0xFF335577,
			0xFF550000,
			0xFF773333,
			0xFF995555,
			0xFFdd9999,
			0xFF777755,
			0xFFbbbbbb,
			0xFFffffff
	};

	// java
	private static final int COLOR_BASE_MENU = 0;
	private static final int COLOR_BASE_PREVIEW = 26;
	private static final int COLOR_BASE_CARS = 26;
	private static final int COLOR_BASE_GAME = 26;
	private static int j_color_pallete_offset = COLOR_BASE_MENU;
	// end java

	private static final byte ROAD_COLOR_A = 1;
	private static final byte ROAD_COLOR_B = 2;
	private static final byte ROAD_PIT_COLOR = 0;

	private static final byte START_LINE_COLOR = 15;

	private static final byte SIDES_COLOR_A = 15;
	private static final byte SIDES_COLOR_B = 10;
	private static final byte SUPER_SIDES_COLOR_B = 8;
	/**
	 * Darkest red
	 */
	private static final byte SIDE_LINES_COLOR = 9;
	/**
	 * Brighter red
	 */
	private static final byte SIDE_LINES_COLOR_BRIGHTER = 10;
	/**
	 * Brightest red
	 */
	private static final byte SIDE_LINES_COLOR_BRIGHTEST = 11;
	/**
	 * Yellow
	 */
	private static final byte SIDE_LINES_COLOR_YELLOW = 3;
	private static final byte SUPER_SIDE_LINES_COLOR = 0;

	private static final byte SKY_COLOR = 7;
	private static final byte GROUND_COLOR = 13;

	// private static final byte CAR_WHEEL_COLOR = 0;
	// private static final byte CAR_SIDE_COLOR = 12;
	// private static final byte CAR_BACK_COLOR = 10;
	// private static final byte CAR_TOP_COLOR = 15;
	// private static final byte CAR_BOTTOM_COLOR = 9;

	private static final int CAR_SHADOW_COLOR = 5;
	private static final int CAR_WHEEL_COLOR = 0;
	private static final int CAR_BODY_PANEL_COLOR = 12;
	private static final int CAR_REAR_BODY_PANEL_COLOR = 10;
	private static final int CAR_TOP_BODY_PANEL_COLOR = 15;
	private static final int CAR_BOTTOM_BODY_PANEL_COLOR = 9;

	private static final int SPEED_BAR_COLOR = 3;
	private static final int SPEED_BAR_BACKGROUND_COLOR = 0;

	private static final int DAMAGE_POINT_COLOR_1 = 11;
	private static final int DAMAGE_POINT_COLOR_2 = 12;

	private static final int SPARK_COLOR = 3;
	private static final int SPARK_LIGHT_COLOR = 15;

	private static byte b_sides_color = 10;
	private static byte b_side_lines_color = 9;

	/*
09 0d 09 0d 47 54 52 41 43 4b 0d 09 0d 09 6d 6f // 192
76 65 2e 62 09 64 31 2c 64 30 0d 09 61 73 6c 2e // 208

22 20 62 20 3e 04 30 14 4a 10 08 00   65 2e 62 09 // 224 attribute sources

64 30 2c 64 32 0d 69 6d 6f 76 65 61 2e 6c 69 23 // 240
	 */
	private static final int OBSTRUCTS_PLAYER = 0x2;
	private static final int WHEELIE = 0x4;
	private static final int DRIVES_NEAR_EDGE = 0x8;
	private static final int UNUSED_10 = 0x10;
	private static final int PUSH_PLAYER = 0x20;
	private static final int UNUSED_40 = 0x40;

	private static final byte[] opponent_attributes_source = {
			PUSH_PLAYER | OBSTRUCTS_PLAYER,
			PUSH_PLAYER,
			UNUSED_40 | PUSH_PLAYER | OBSTRUCTS_PLAYER,
			PUSH_PLAYER,
			UNUSED_10 | PUSH_PLAYER | DRIVES_NEAR_EDGE | WHEELIE | OBSTRUCTS_PLAYER,
			WHEELIE,
			UNUSED_10 | PUSH_PLAYER,
			UNUSED_10 | WHEELIE,
			UNUSED_40 | DRIVES_NEAR_EDGE | OBSTRUCTS_PLAYER,
			UNUSED_10,
			DRIVES_NEAR_EDGE,
			0// unused
	};

	// opponent.attributes
	private static final byte[] b_opponent_attributes = {
			PUSH_PLAYER | OBSTRUCTS_PLAYER,
			PUSH_PLAYER,
			UNUSED_40 | PUSH_PLAYER | OBSTRUCTS_PLAYER,
			PUSH_PLAYER,
			UNUSED_10 | PUSH_PLAYER | DRIVES_NEAR_EDGE | WHEELIE | OBSTRUCTS_PLAYER,
			WHEELIE,
			UNUSED_10 | PUSH_PLAYER,
			UNUSED_10 | WHEELIE,
			UNUSED_40 | DRIVES_NEAR_EDGE | OBSTRUCTS_PLAYER,
			UNUSED_10,
			DRIVES_NEAR_EDGE,
			0// unused
	};

	// b_opponents_names
	private static final char[][] opponents_names =
			new char[ 1 + Text.opponents_names_sources.length ][ Text.OPPONENTS_NAME_FULL_LENGTH ];

	// private static final byte[] b_opponents_names_192 = new byte[ 16 ];// TODO verify size

	private static final int TRACK_NAME_STORE_LENGTH = 16;
	private static final int TRACK_NAME_LENGTH = 15;
	// track_names
	private static final char[][] track_names = {
			"LITTLE RAMP     ".toCharArray(),
			"STEPPING STONES ".toCharArray(),
			"HUMP BACK       ".toCharArray(),
			"BIG RAMP        ".toCharArray(),
			"SKI JUMP        ".toCharArray(),
			"DRAW BRIDGE     ".toCharArray(),
			"HIGH JUMP       ".toCharArray(),
			"ROLLER COASTER  ".toCharArray(),
	};

	private static final char[] b_tracks_short_names = {
			'L', 'R',
			'H', 'B',
			'S', 'S',
			'B', 'R',
			'H', 'J',
			'R', 'C',
			'S', 'J',
			'D', 'B',
	};

	/*
    private static byte[] league_values = {
            // Standard league
            0x48, 0x00,
            (byte)0xf0, 0x00,// engine_power
            (byte)0xec, 0x00,// opponents_engine_power
            16,// boost.unit.value
            0x60, 0x5b,
            0,// road.cushion.value
            0,
            // Super league
            0x54, 0x0c,
            0x40, 0x01,
            0x3a, 0x01,
            12,
            0x6e, 0x69,
            1,
            0
        };
    */

	private static final short ENGINE_POWER_TO_BREAK = (short) 0xFF10;
	// Standard league
	private static final short STANDARD_LEAGUE_w_1baf8 = 0x0048;
	private static final short STANDARD_LEAGUE_ENGINE_POWER = 0x00F0;
	private static final short STANDARD_LEAGUE_OPPONENT_ENGINE_POWER = 0x00EC;
	private static final byte STANDARD_LEAGUE_BOOST_UNIT_VALUE = 16;
	// 0x60, 0x5b,
	private static final byte STANDARD_LEAGUE_ROAD_CUSHION_VALUE = 0;
	// 0

	// Super league
	private static final short SUPER_LEAGUE_w_1baf8 = 0x0C54;
	private static final short SUPER_LEAGUE_ENGINE_POWER = 0x0140;
	private static final short SUPER_LEAGUE_OPPONENT_ENGINE_POWER = 0x013A;
	private static final byte SUPER_LEAGUE_BOOST_UNIT_VALUE = 12;
	// 0x6e, 0x69,
	private static final byte SUPER_LEAGUE_ROAD_CUSHION_VALUE = 1;
	// 0

	// indexes for wheel parameter arrays
	private static final int LEFT_FRONT_WHEEL = 0;
	private static final int RIGHT_FRONT_WHEEL = 1;
	private static final int REAR_WHEEL = 2;
	private static final int MAX_WHEELS = 3;

	private static final int OPP_REAR_LEFT_WHEEL = 0;
	private static final int OPP_REAR_RIGHT_WHEEL = 1;
	private static final int OPP_FRONT_WHEEL = 2;
	private static final int OPP_MAX_WHEELS = 3;

	// --- cleared in loop in initialization
	// --- cleared in loop in initialise.data
	private static short w_1baf8 = 0;
	private static short engine_power = 0;
	private static short w_opponents_engine_power = 0;
	private static byte boost_unit_value = 0;// dc.b 0,0
	// dc.b 0
	/**
	 * {@link #STANDARD_LEAGUE_ROAD_CUSHION_VALUE}
	 * {@link #SUPER_LEAGUE_ROAD_CUSHION_VALUE}
	 */
	private static byte b_road_cushion_value = 0;// dc.b 0
	// dc.b 0,0

	private static byte b_players_map_x = 0;// dc.b 0,0
	private static byte b_players_map_z = 0;// dc.b 0
	private static byte b_players_fine_map_x = 0;// dc.b 0,0
	private static byte b_players_fine_map_z = 0;// dc.b 0

	/**
	 * player's center (after adjustment for opposite direction)
	 * high byte is section
	 */
	private static short w_players_distance_into_section = 0;

	/**
	 * opponent's center
	 */
	private static short w_opponents_distance_into_piece = 0;

	/**
	 * player's front, can be negative when player's front is after the last piece
	 */
	private static short w_players_distance_into_section_plus64 = 0;

	/**
	 * player's center (before adjustment for opposite direction)
	 */
	private static short w_normal_distance_into_section = 0;

	/**
	 * opponent's rear
	 */
	private static short w_opponents_distance_into_section_minus64 = 0;
	// dc.w 0
	/**
	 * Menu bar index 0...17
	 */
	private static byte b_menu_bar_index = 0;
	private static byte b_thousands = 0;

	// acceleration.adjust
	// road.height	dc.w	0; actually a longword, but bytes have more than 1 use
	// factor1	dc.b	0
	// value
	// dc.b	0
	private static int lw_road_height_fp8 = 0;// actually a longword, but bytes have
	private static byte b_factor_high_value_low = 0;
	private static byte b_value_low_factor_high = 0;

	static byte b_players_road_piece = 0;
	static byte b_opponents_road_piece = 0;

	private static byte b_1bb1e = 0;
	private static byte b_1bb1f = 0;

	private static byte b_players_lap = 0;
	private static byte b_opponents_lap = 0;

	/**
	 * road_near_x_offset = ((road_section_x_offset << 3) + road_finer_x_offset);
	 * <p>
	 * FIXME verify: low byte of road_near_x_offset is road_finest_x_offset
	 */
	private static short w_road_near_x_offset = 0;
	// private static byte road_finest_x_offset = 0;
	// dc.w 0

	/**
	 * road_near_z_offset = ((road_section_z_offset << 3) + road_finer_z_offset);
	 * <p>
	 * FIXME verify: low byte of road_near_z_offset is road_finest_z_offset
	 */
	private static short w_road_near_z_offset = 0;
	// private static byte road_finest_z_offset = 0;
	// dc.w 0

	private static short w_opp_zdiff = 0;
	/**
	 * unsigned 0...255
	 */
	private static byte b_inclination_sin_fp8 = 0;

	// opp.xdiff
	private static short w_opp_xdiff = 0;
	private static byte b_surface_cosx_fp8 = 0;// locations have more than 1 use
	/**
	 * unsigned byte 0...255
	 */
	private static byte b_inclination_cos_fp8 = 0;

	private static byte b_road_finer_x_offset = 0;
	// dc.w 0
	private static byte b_road_finer_z_offset = 0;// dc.b 0,0
	private static short w_previous_left_top_y = 0;
	private static short w_previous_right_top_y = 0;
	private static boolean b_daft_flag = false;// dc.b 0,0
	// dc.w 0
	private static byte byte_count = 0;
	private static byte boost_unit = 0;
	// dc.b 0
	/**
	 * isOnBridge
	 */
	static byte b_1bb3f = 0;
	private static byte b_opponents_road_piece_m64_high = 0;
	private static byte b_opponents_road_piece_m64_low = 0;
	private static byte opponents_x_span = 0;
	private static boolean b_opp_touching_road = false;
	private static boolean b_opponents_required_z_speed_reached = false;
	private static boolean b_cars_collided = false;

	private static final int CTRL_PLAYER_ACCELERATE = 0x01;
	private static final int CTRL_PLAYER_BREAK = 0x02;
	private static final int CTRL_PLAYER_LEFT = 0x04;
	private static final int CTRL_PLAYER_RIGHT = 0x08;
	private static final int CTRL_PLAYER_BOOST = 0x10;
	private static byte b_players_input = 0;
	private static byte b_prompt_chars = 0;// dc.b 0,0
	private static byte b_opponents_required_z_speed = 0;// dc.b 0,0
	// dc.b 0
	/**
	 * current piece type. 0x80 - curve
	 */
	private static byte b_curved_section = 0;
	// dc.b 0

	/*
	 * unsigned bytes
	 *
	 * front_left_damage,
	 * front_right_damage,
	 * rear_damage
	 */
	// private static byte[] wheel_damages = {
	/**
	 * unsigned byte 0...255
	 */
	private static byte b_front_left_damage = 0;
	/**
	 * unsigned byte 0...255
	 */
	private static byte b_front_right_damage = 0;
	/**
	 * unsigned byte 0...255
	 */
	private static byte b_rear_damage = 0;
	// dc.w 0
	//};
	private static boolean b_damaged = false;

	/** unsigned byte, up to screen width, {@link #MAX_DAMAGE_LINE_X} */
	private static byte b_damage_line_x = 0;
	private static byte b_damaged_count = 0;
	private static byte camera_preview_position_id = 0;
	private static byte b_near_sections_done = 0;
	private static byte b_number_of_coords = 0;
	private static byte b_coords_of_last_section = 0;

	private static boolean b_1bb5b = false;
	private static byte b_1bb5c = 0;
	private static byte b_left_hand_bend = 0;
	private static byte b_current_print_row = 0;

	private static final byte MAX_DIVISIONS = 4;
	private static byte b_division = 0;
	/**
	 * unsigned byte. can be >= 128
	 */
	private static byte b_offset_for_coord_pair_to_start_at = 0;
	private static byte b_section_flags2 = 0;
	private static boolean b_boost_activated = false;
	private static boolean b_turn_engine_off = false;
	private static byte b_1bb64 = 0;
	private static boolean b_wheel_off_road = false;
	private static boolean b_use_lines_color = false;
	private static boolean b_opponent_reached = false;
	private static boolean b_unused_flag = false;
	/**
	 * 0, 1, 2
	 */
	private static byte b_flame_counter = 0;
	/**
	 * 0 ... 13
	 */
	private static byte b_number_of_segments = 0;
	/**
	 * 0: read {@link #b_players_division}, {@link #b_damage_hole_position}, {@link #b_league_offset}
	 * 0x40: read 512 bytes
	 * others: read 256 bytes
	 */
	private static byte b_1bb6b = 0;
	/**
	 * 0x2c, 0x3c, 0x80
	 */
	private static byte b_1bb6c = 0;
	private static byte b_print_fine_x = 0;
	private static byte b_print_fine_y = 0;
	private static boolean b_1bb6f = false;

	private static boolean b_boost_flag = false;
	private static boolean b_draw_bridge_make_far_coords = false;
	private static boolean b_1bb72 = false;

	private static final int SMASHED_COUNTDOWN_MAX = 69;// 3.36 s
	private static byte b_smashed_countdown = 0;
	private static byte b_1bb74 = 0;
	private static boolean b_1bb75 = false;
	private static byte b_1bb76 = 0;
	static boolean b_standard_clip_flag = false;
	// dc.b 0
	private static byte b_y_coords_stored_as_words = 0;
	private static byte b_current_y_coord_offset = 0;
	/**
	 * unsigned byte
	 */
	private static byte b_road_width_reduction = 0;
	/**
	 * a positive value or 255
	 */
	private static byte b_opponents_coord = 0;
	private static byte b_grounded_count = 0;
	private static boolean b_touching_road = false;
	private static boolean b_pits_are_black = false;
	// dc.w 0
	// dc.w 0
	// dc.b 0
	private static byte b_current_road_piece = 0;
	/**
	 * 0...15
	 */
	private static byte b_near_section_piece = 0;// dc.b 0,0
	private static boolean b_white_prompts = false;
	// private static byte b_1bb89 = 0;
	// dc.b 0
	private static byte b_map_x = 0;
	private static byte b_map_z = 0;
	private static boolean b_prompt_required = false;
	private static boolean b_track_preview = false;
	private static byte b_copy_prompt_groups = 0;
	private static byte b_piece_coords_offset = 0;
	private static boolean b_finish_file_name_editing = false;
	private static boolean b_1bb93 = false;
	private static byte b_1bb94 = 0;
	private static byte b_boost_max_units = 0;// Amiga: in BCD
	// dc.b 0
	/**
	 * <= 28
	 */
	private static byte b_number_of_sections_x2 = 0;
	private static byte b_number_of_sections_x2_minus2 = 0;
	private static byte b_lap_that_finishes_race = 0;
	/**
	 * 0, 0x40, 0x80, 0xC0
	 */
	private static byte b_at_side_byte = 0;
	private static byte b_1bb9b = 0;
	/**
	 * 0, 0x40, 0x80
	 */
	private static byte b_off_map_status = 0;
	private static byte b_1bb9d = 0;
	private static byte b_1bb9e = 0;
	private static byte b_1bb9f = 0;
	// dc.b 0
	/**
	 * unsigned byte?
	 */
	private static byte b_rear_wheel_surface_x_position = 0;
	// dc.b 0
	/**
	 * >= 128 means that section change is required
	 */
	private static byte b_surface_start_coord_x2 = 0;
	private static boolean b_1bba4 = false;
	/**
	 * {@link #b_machine} or 0 for data link
	 */
	private static byte b_second_machine = 0;
	private static byte b_opponents_road_section_m255 = 0;
	/**
	 * unsigned 0...255
	 */
	private static byte b_opponents_suggested_road_x_position = 0;
	// private static byte temp = 0;// location has more than 1 use

	private static boolean b_accelerating = false;// dc.b 0,0
	private static boolean b_1bbaa = false;
	private static byte b_main_loop_count = 0;
	private static boolean b_dont_copy_coords = false;
	private static byte b_laps_when_race_finished = 0;
	static byte b_draw_bridge_frame_count = 0;
	/**
	 * bit 0 : new track record
	 * bit 7 : best race time
	 * bit 6 : best lap time
	 */
	private static byte b_1bbb1 = 0;
	private static byte b_1bbb2 = 0;// dc.b 0,0
	/**
	 * Flags.
	 * bit 7 - race lost
	 */
	private static byte b_1bbb4 = 0;
	private static byte b_1bbb5 = 0;
	private static byte b_new_damage = 0;// dc.b 0,0
	private static boolean b_opponent_behind_player = false;
	private static boolean b_no_wheel_update = false;
	/**
	 * flag to not draw opponent's shadow
	 */
	private static boolean b_do_not_draw_opponents_shadow = false;
	private static boolean b_copy_swing_from_left = false;
	private static boolean b_print_text_on_screen_2 = false;
	/**
	 * signed byte
	 */
	private static byte b_1bbbd = 0;
	private static byte b_1bbbe = 0;
	private static byte b_1bbbf = 0;
	private static byte b_opponents_random_steering_count = 0;
	private static boolean b_collision_in_air = false;
	private static byte b_1bbc2 = 0;
	private static byte b_collision_counter = 0;
	private static boolean b_drop_start_done = false;
	private static byte b_coord_offset_zero_or_four = 0;
	private static byte b_left_right_value = 0;
	private static boolean b_player_close_to_opponent = false;
	/**
	 * Java:
	 * {@code true} - draw on {@link #screen1}, show {@link #screen2}.
	 * {@code false} - draw on {@link #screen2}, show {@link #screen1}
	 */
	private static volatile boolean b_which_screen = false;
	private static boolean show_new_screen = false;
	private static byte b_loop_counter = 0;
	private static byte b_1bbca = 0;
	private static boolean b_1bbcb = false;
	/**
	 * 0, 0x1b (27)
	 */
	private static byte b_1bbcc = 0;
	static byte b_fourteen_frames_elapsed = 0;// TODO verify using
	// dc.b 0
	private static byte b_1bbcf = 0;
	private static byte b_1bbd0 = 0;
	/**
	 * on road: 0, off-road right: 0x02, off-road left: 0x80
	 */
	private static byte b_players_position_flags = 0;
	private static boolean b_draw_road_surface_flag = false;// FIXME is not required
	private static boolean b_or_char_with_screen = false;
	/**
	 * 0 ... 0x3e
	 */
	private static byte b_section_steering_amount = 0;
	/**
	 * 0...15
	 */
	private static byte b_map_x_shift = 0;
	/**
	 * 0...15
	 */
	private static byte b_map_z_shift = 0;

	/**
	 * 50 max
	 */
	private static byte b_ferocity_of_sparks_or_clouds = 0;
	// dc.b 0
	/**
	 * unsigned byte
	 */
	private static byte b_road_length_reduction = 0;
	/**
	 * 0x40, 0x80
	 */
	private static byte b_which_side_byte = 0;

	/**
	 * 0...2
	 */
	private static byte b_wheel_frame_number = 0;
	/**
	 * 0 or 2
	 */
	private static byte b_other_road_line_color = 0;
	/**
	 * 0xBA
	 */
	private static byte b_1bbdd = 0;

	private static final int ON_CHAIN_STAGE_START_OFF_GROUND = 240;
	private static final int ON_CHAIN_STAGE_SET_FIXED_HEIGHT = 230;
	private static final int ON_CHAIN_STAGE_1 = 229;
	private static final int ON_CHAIN_STAGE_2 = 228;
	private static final int ON_CHAIN_PRACTICE_WAIT_KEY = 140;
	private static final int ON_CHAIN_RACE_MIN_WAIT_KEY = 160;
	/**
	 * unsigned byte 230, 240
	 */
	private static byte b_car_on_chains_countdown = 0;
	private static byte b_1bbe0 = 0;
	private static boolean b_swing_from_left = false;
	private static byte b_1bbe2 = 0;
	private static byte b_wheel_frame_count = 0;
	/** unsigned byte */
	private static byte b_edge_x1_offset = 0;
	private static byte b_coord_pair_to_start_at = 0;// FIXME not used?
	private static boolean b_next_segment = false;// dc.b 0,0
	private static byte b_start_offset_for_rating_in_divisions = 0;
	private static byte b_1bbe9 = 0;
	/**
	 * init value is 176
	 */
	private static byte b_1bbea = 0;
	private static boolean b_1bbeb = false;

	private static short w_opponents_road_x_position = 0;
	private static short w_opponents_z_speed = 0;
	private static short w_opponents_engine_z_acceleration = 0;

	/**
	 * signed
	 */
	private static short w_rough_difference_angle = 0;
	private static short w_engine_revs_change = 0;

	private static short w_max_difference = 0;// FIXME param for limitOpponentWheels
	private static boolean b_player_to_right = false;
	// difference_angle
	static short w_near_x_coord = 0;
	// location has more than 1 use
	// private static byte near_x_coord_1 = 0;

	private static byte b_abs_x_difference = 0;
	// location has more than 1 use
	// private static byte x_difference_1 = 0;

	private static short w_height_adjust = 0;// just a local?
	private static short w_near_z_coord = 0;// location has more than 1 use

	private static short w_y_perspective_shift = 0;
	// dc.w 0
	// dc.w 0

	/**
	 * signed short
	 */
	private static short w_swing_magnitude = 0;

	private static short w_surface_y_coord1 = 0;
	private static short w_surface_y_coord2 = 0;
	private static short w_surface_y_coord3 = 0;
	private static short w_surface_y_coord4 = 0;
	private static short w_surface_y_coord5 = 0;
	private static short w_surface_y_coord6 = 0;
	/**
	 * 0 ... 32768
	 */
	static short w_overall_left_y_shift = 0;
	private static short w_overall_right_y_shift = 0;
	private static short w_opponents_offset = 0;
	private static byte b_edge_offset_left_x2 = 0;// dc.b 0,0
	private static byte b_edge_x2_offset_1 = 0;// TODO remove
	private static byte b_edge_offset_right_x2 = 0;// dc.b 0,0
	private static short w_new_speed_bar = 0;
	// dc.w 0
	private static short w_1bc1c = 0;
	private static short w_dnr_value = 0;
	/**
	 * initial value is 127
	 */
	private static short w_old_speed_bar = 0;

	private static short w_wheel_road_x_position = 0;
	private static short w_opponents_distance_into_section_minus255 = 0;
	private static short w_near_sections_done2 = 0;
	private static short w_1bc28 = 0;
	static short w_pos_difference_angle = 0;
	// dc.w 0
	private static short w_x_shift = 0;
	private static short w_rough_player_angle = 0;
	/**
	 * 0 or 0x80
	 */
	private static byte b_plus_180_degrees = 0;
	private static short w_engine_revs = 0;
	private static short w_perspective_z = 0;
	private static short w_smallest_distance_between_players = 0;
	private static short w_damage_value = 0;// dc.b 0,0
	private static short w_pos_difference_angle_m16 = 0;
	private static short w_difference_between_players = 0;
	/**
	 * high byte is subsection number
	 */
	private static short w_surface_z_position = 0;
	static short w_y_shift = 0;

	/**
	 * 0 or 0x8000 for left
	 */
	private static short w_curve_to_left = 0;
	// dc.w 0
	// dc.w 0
	private static short w_rough_piece_angle = 0;
	private static short w_surface_x_position = 0;
	// dc.w 0
	// dc.w 0
	// dc.w 0
	private static byte b_1bc55 = 0;
	private static short w_1bc56 = 0;
	static short w_fp_y2 = 0;
	static short w_fp_y = 0;

	private static short w_players_x_offset_from_road_centre = 0;
	private static short w_players_road_x_position = 0;

	private static short w_required_raise_height = 0;
	/**
	 * unsigned int16, up to 0xFF00
	 */
	private static char w_wheel_rotation_speed = 0;
	private static short w_1bc64 = 0;
	private static short w_1bc66 = 0;
	private static short w_1bc68 = 0;// abs half y diff, max 255
	private static short w_1bc6a = 0;// abs half x diff, max 255
	private static short w_1bc6c = 0;
	private static short w_1bc6e = 0;
	private static short w_1bc70 = 0;
	private static short w_1bc72 = 0;
	private static short w_1bc74 = 0;
	private static short w_1bc76 = 0;
	private static short w_1bc78 = 0;
	private static short w_1bc7a = 0;
	private static short w_1bc7c = 0;
	private static short w_1bc7e = 0;
	private static short w_1bc80 = 0;
	private static short w_1bc82 = 0;
	private static short w_car_x_shift = 0;
	private static short w_car_y_shift = 0;
	private static short w_1bc88 = 0;
	private static short w_1bc8a = 0;
	/**
	 * Amiga: short w_left_y_coord_offset
	 */
	private static byte[] w_left_y_coord_offset = null;
	// dc.w 0
	/**
	 * Amiga: short w_right_y_coord_offset
	 */
	private static byte[] w_right_y_coord_offset = null;
	// dc.w 0

	// array of 3
	private static int lw_front_left_actual_height_fp8 = 0;
	private static int lw_front_right_actual_height_fp8 = 0;
	private static int lw_rear_actual_height_fp8 = 0;

	private static final int SCRATCH_HEIGHT_REDUCTION = 0x200;
	/**
	 * 0 or 0x200 when scratching a road
	 */
	private static int lw_wreck_wheel_height_reduction_fp8 = 0;

	// array of 3
	private static int lw_front_left_road_height_fp8 = 0;
	private static int lw_front_right_road_height_fp8 = 0;
	private static int lw_rear_road_height_fp8 = 0;

	private static int lw_front_left_height_difference = 0;
	private static int lw_front_right_height_difference = 0;
	private static int lw_rear_height_difference = 0;

	/**
	 * Amiga: short w_piece_data_offset
	 */
	private static byte[] w_piece_data_offset = null;
	// dc.w 0
	/**
	 * Amiga: short w_road_data_offset = 0;
	 * The pointer to a road data. They is different byte arrays.
	 */
	private static byte[] w_road_data_offset = null;
	// dc.w 0
	// dc.w 0
	// dc.w 0
	private static volatile boolean b_which_screen2 = false;// dc.b 0,0
	// dc.w 0
	private static int lw_current_players_x = 0;
	private static short w_players_smaller_y = 0;
	// dc.w 0
	private static int lw_current_players_z = 0;

	private static int lw_players_world_x = 0;
	private static int lw_players_world_y_fp16 = 0;
	private static int lw_players_world_z = 0;

	public static short w_players_x_angle = 0;
	public static short w_players_y_angle = 0;
	public static short w_players_z_angle = 0;

	private static short w_players_world_x_speed = 0;
	private static short w_players_world_y_speed = 0;
	private static short w_players_world_z_speed = 0;

	private static short w_players_x_rotation_speed = 0;
	private static short w_players_y_rotation_speed = 0;
	private static short w_players_z_rotation_speed = 0;

	private static short w_total_world_x_acceleration = 0;
	private static short w_total_world_y_acceleration = 0;
	private static short w_total_world_z_acceleration = 0;

	private static short w_players_x_rotation_acceleration = 0;
	private static short w_players_y_rotation_acceleration = 0;
	private static short w_players_z_rotation_acceleration = 0;

	/**
	 * front_left_wheel_x_offset,
	 * front_right_wheel_x_offset,
	 * rear_wheel_x_offset
	 */
	private static short[] w_wheel_x_offsets = {
			0,// private static short front_left_wheel_x_offset = 0;
			0,// private static short front_right_wheel_x_offset = 0;
			0// private static short rear_wheel_x_offset = 0;
	};

	/**
	 * front_left_wheel_z_offset,
	 * front_right_wheel_z_offset,
	 * rear_wheel_z_offset
	 */
	private static final short[] w_wheel_z_offsets = {
			0,// private static short front_left_wheel_z_offset = 0;
			0,// private static short front_right_wheel_z_offset = 0;
			0// private static short rear_wheel_z_offset = 0;
	};

	private static short w_gravity_x_acceleration = 0;
	private static short w_gravity_y_acceleration = 0;
	private static short w_gravity_z_acceleration = 0;

	/**
	 * new_front_left_difference,
	 * new_front_right_difference,
	 * new_rear_difference
	 */
	private static final short[] w_new_wheel_differences = {
			0, // private static short new_front_left_difference = 0;
			0, // private static short new_front_right_difference = 0;
			0// private static short new_rear_difference = 0;
	};

	private static short w_old_front_left_difference = 0;
	private static short w_old_front_right_difference = 0;
	private static short w_old_rear_difference = 0;

	private static short w_front_left_amount_below_road = 0;
	private static short w_front_right_amount_below_road = 0;
	private static short w_rear_amount_below_road = 0;

	private static short w_overall_difference_below_road = 0;
	private static short w_front_difference_below_road = 0;

	private static short w_engine_z_acceleration = 0;

	private static short w_players_x_speed = 0;
	private static short w_players_y_speed = 0;
	private static short w_players_z_speed = 0;

	private static short w_players_x_acceleration = 0;
	private static short w_players_y_acceleration = 0;
	private static short w_players_z_acceleration = 0;

	private static short w_average_amount_below_road = 0;

	private static short w_players_final_x_rotation_speed = 0;
	private static short w_players_final_y_rotation_speed = 0;
	private static short w_players_final_z_rotation_speed = 0;

	private static short w_car_collision_x_acceleration = 0;
	private static short w_car_collision_y_acceleration = 0;
	private static short w_car_collision_z_acceleration = 0;

	private static short w_car_to_road_collision_z_acceleration = 0;

	private static short w_z_inclination_to_road = 0;
	private static short w_y_inclination_to_road = 0;
	private static short w_x_inclination_to_road = 0;

	private static byte b_surface_cosx_sinz_fp8 = 0;// dc.b 0,0
	private static byte b_surface_cosx_cosz_fp8 = 0;// dc.b 0,0
	private static byte b_surface_sinx_fp8 = 0;// dc.b 0,0

	private static short w_car_to_car_x_acceleration = 0;
	private static short w_car_to_car_y_acceleration = 0;
	private static short w_car_to_car_z_acceleration = 0;

	private static short w_section_y_angle = 0;
	private static short w_pos_players_z_speed = 0;
	/**
	 * opp_new_rear_left_difference,
	 * opp_new_rear_right_difference,
	 * opp_new_front_difference
	 */
	private static final short[] w_opp_new_wheel_differences = {
			0,// private static short opp_new_rear_left_difference = 0;
			0,// private static short opp_new_rear_right_difference = 0;
			0// private static short opp_new_front_difference = 0;
	};
	// dc.w 0

	// array of 3 shorts
	/**
	 * opp_rear_left_actual_height,
	 * opp_rear_right_actual_height,
	 * opp_front_actual_height
	 */
	private static final short[] w_opp_wheel_actual_heights = {
			0,// private static short opp_rear_left_actual_height = 0;
			0,// private static short opp_rear_right_actual_height = 0;
			0,// private static short opp_front_actual_height = 0;
			0,// dc.w 0
	};

	private static short w_opp_old_rear_left_difference = 0;
	private static short w_opp_old_rear_right_difference = 0;
	private static short w_opp_old_front_difference = 0;
	// dc.w 0

	/**
	 * opp_rear_left_y_speed,
	 * opp_rear_right_y_speed,
	 * opp_front_y_speed
	 */
	private static final short[] w_opp_wheel_y_speeds = {
			0,// private static short opp_rear_left_y_speed = 0;
			0,// private static short opp_rear_right_y_speed = 0;
			0// private static short opp_front_y_speed = 0;
			// dc.w 0
	};

	private static short w_opp_rear_left_y_acceleration = 0;
	private static short w_opp_rear_right_y_acceleration = 0;
	private static short w_opp_front_y_acceleration = 0;
	// dc.w 0

	// array of 3 shorts
	/**
	 * opp_rear_left_road_height,
	 * opp_rear_right_road_height,
	 * opp_front_road_height
	 */
	private static final short[] w_opp_wheel_road_heights = {
			0,// private static short opp_rear_left_road_height = 0;
			0,// private static short opp_rear_right_road_height = 0;
			0,// private static short opp_front_road_height = 0;
			0// dc.w 0
	};

	private static final short[] w_segment_x_coords = new short[ 16 ];// ds.w 16
	private static final short[] w_segment_z_coords = new short[ 16 ];// ds.w 16
	// dc.w 36

	private static final byte PIT = 0x20;

	/**
	 * 0,
	 * 0x01 - near start,
	 * 0x20 - pit flag,
	 * 0x80 - do not use coordinates for far sections
	 */
	private static final byte[] b_near_section_flags = new byte[ 160 ];// ds.w 80
	// --- end cleared in loop in initialise.data

	private static final int OFFSET_FOR_BOTTOM_X2 = 120;

	/**
	 * Even numbers: left y. Odd numbers: right y
	 */
	private static final short[] w_coord_y_values_fp5 = new short[ 160 ];// ds.w 160

	private static final int OFFSET_COPY_LAST_NEAR_SECTION_COORD = 240 / 2;
	private static final short[] w_screen_x_coords = new short[ 160 ];// ds.w 160
	private static final short[] w_screen_y_coords = new short[ 160 ];// ds.w 160

	/**
	 * Java: points to draw polygons. 10 screen points maximum, x coordinates
	 */
	private static final int[] jXPoints = new int[ 30 ];

	/**
	 * Java: points to draw polygons. 10 screen points maximum, y coordinates
	 */
	private static final int[] jYPoints = new int[ 30 ];

	private static final short[] w_sin_cos_values = new short[ 36 ];// ds.w 36. matrix. 3x3
	// ds.w 4

	/**
	 * [0,0] is left top. 16 x 16
	 * Data stored by columns.
	 * 0-15 is column 0, 16-31 is column 1 and so on.
	 */
	private static final byte[] b_road_under_map = new byte[ 256 ];

	private static final short[] w_sparks_x = new short[ 32 ];
	private static final short SPARK_Y_DEFAULT = 212 * PF;
	private static final short SPARK_Y_OFF_SCREEN = 210 * PF;
	private static final short[] w_sparks_y = new short[ 32 ];
	private static final short[] w_sparks_clouds_dx = new short[ 32 ];
	private static final short[] w_sparks_clouds_dy = new short[ 32 ];
	// ds.w 32

	// TODO Amiga: always 0
	private static byte b_engine_fluctuation = 0;//	dc.b	0,0

	/**
	 * Bit 7 indicates that the y coords for that section are stored as words
	 * e.g. for steeper sections on the roller coaster or the high jump
	 */
	private static final byte[] b_left_y_coordinate_ids = new byte[ 100 ];

	/**
	 * Bit 7 goes to {@link #b_other_road_line_color}
	 */
	private static final byte[] b_right_y_coordinate_ids = new byte[ 100 ];

	/**
	 * Unsigned byte.
	 * Top nibble = Z position, Bottom nibble = X position
	 */
	private static final byte[] b_road_piece_xz_positions = new byte[ 100 ];

	/**
	 * Top two bits are the rough angle for the piece (0, 90, 180 or 270
	 * degrees). Bit 4 indicates that piece is rotated through a further 180
	 * degrees. Bottom nibble is the near section piece to use.
	 */
	private static final byte[] b_road_section_angle_and_piece = new byte[ 100 ];

	/**
	 * A value for each road section, used to shift all the left side y
	 * coordinates up by the same amount.
	 */
	static final short[] w_overall_left_y_shifts = new short[ 100 ];

	/**
	 * A value for each road section, used to shift all the right side y
	 * coordinates up by the same amount.
	 */
	private static final short[] w_overall_right_y_shifts = new short[ 100 ];

	/**
	 * unsigned
	 */
	private static final short[] w_distances_around_road = new short[ 100 ];

	// private static short[] DAT_1c8a8 = new short[ 16 ];
	// private static short[] DAT_1c8c8 = new short[ 16 ];
	// private static short[] DAT_1c8e8 = new short[ 16 ];
	/**
	 * unsigned byte
	 */
	private static final byte[] b_DAT_1c8a8 = new byte[ 32 ];
	/**
	 * unsigned byte
	 */
	private static final byte[] b_DAT_1c8c8 = new byte[ 32 ];
	/**
	 * unsigned byte
	 */
	private static final byte[] b_DAT_1c8e8 = new byte[ 16 ];

	// records, time
	/**
	 * time, s, high byte. Amiga: bcd. 0-59
	 */
	private static final byte[] b_time_results_min = new byte[ 24 ];
	/**
	 * ime, s, low byte. Amiga: bcd. 0-59
	 */
	private static final byte[] b_time_results_sec = new byte[ 24 ];
	/**
	 * time, ms. Amiga: bcd. 0-99
	 */
	private static final byte[] b_time_results_ms = new byte[ 24 ];

	// dc.w 0
	/**
	 * signed bytes
	 */
	static byte[] b_opponents_speed_values = new byte[ 100 ];

	/**
	 * Player's name ^ 0x3B
	 */
	private static final byte[] b_encoded_players_name = new byte[ 12 ];

	private static final byte[] b_DAT_1c9c2 = new byte[ 12 ];

	private static byte b_players_division = 0;
	private static byte b_damage_hole_position = 0;
	private static byte b_league_offset = 0;// dc.b 0,0

	/** unsigned byte up to 255 */
	private static final byte[] b_DAT_1c9d2 = new byte[ 12 ];

	private static final byte[] b_won_races = new byte[ 12 ];

	private static final byte[] b_best_lap_races = new byte[ 12 ];

	private static final byte[] b_raced = new byte[ 12 ];

	private static final byte[] b_total_points = new byte[ 12 ];
	/** players in divisions. 0...2 - division 1, 3...5 division 2, 6...8 division 3, 9...11 division 4 */
	private static final byte[] b_rating_racer_ids = new byte[ 12 ];

	private static final byte[] b_DAT_1ca36 = new byte[ 12 ];

	/**
	 * 44 ... 78
	 */
	private static byte b_number_of_road_pieces = 0;
	/**
	 * 44 ... 78
	 */
	private static byte b_players_start_piece = 0;
	/**
	 * 44 ... 78
	 */
	private static byte b_near_start_line_section = 0;
	/**
	 * 44 ... 78
	 */
	private static byte b_half_a_lap_section = 0;
	private static short w_total_road_distance = 0;
	private static byte b_boost_reserve = 0;// Amiga: stored in BCD
	private static byte b_1ca21 = 0;
	/** menu practice select road: 0 or 1, 0x80: race mode */
	private static byte b_race_mode = 0;
	private static byte b_best_lap_racer_id = 0;
	private static byte b_race_winner_id = 0;
	/**
	 * index for {@link #opponents_names}
	 */
	private static byte b_1ca25 = 0;
	/**
	 * index for {@link #opponents_names}
	 */
	private static byte b_1ca26 = 0;
	private static byte b_racer_id_1 = 0;
	private static byte b_racer_id_2 = 0;
	private static byte b_opponents_id = 0;
	private static byte b_standard_league_damage_limit = 0;
	private static byte b_super_league_damage_limit = 0;
	private static byte b_standard_league_boost = 0;
	private static byte b_super_league_boost = 0;
	private static byte b_1ca2e = 0;
	private static byte b_1ca2f = 0;
	// d.b 0
	/** 0 load, 1 save */
	private static boolean b_save = false;
	private static byte b_start_finish_section = 0;
	static byte b_road_id = 0;// 5 - Draw Bridge
	private static byte b_1ca34 = 0;
	private static byte b_1ca35 = 0;
	// --- end cleared in loop in initialization

	// TODO remove Java: debug
	static void jLogErrLn(final String msg) {
		System.err.println( msg );
	}

	static void jLogErr(final String msg, final Object... args) {
		System.err.printf( msg, args );
	}

	static void jLogErrLn(final String msg, final Object... args) {
		System.err.printf( msg, args );
		System.err.println();
	}

	/**
	 * @param delay in ms
	 */
	private static void jDelay(final int delay) {
		try { Thread.sleep( delay ); } catch( final InterruptedException ignored ) {}
	}

	private static void jKeyPollDelay() {
		jDelay( 25 );
	}

	/**
	 * Load an image with given name from image folder
	 *
	 * @param name an image name without extension. ".png" is used.
	 * @return the image or {@code null} if there is no image with the given
	 * name.
	 */
	private static Bitmap jGetImage(final int name) {
		try {
			// final BitmapFactory.Options options = new BitmapFactory.Options();
			// options.inScaled = false;
			// options.inPremultiplied = false;
			// resources must be in drawable-nodpi to avoid scaling
			final Bitmap b = BitmapFactory.decodeResource( sResources, name );
			final int width = b.getWidth() * PF;
			final int height = b.getHeight() * PF;
			return Bitmap.createScaledBitmap( b, width, height, false );
		} catch( final Exception ie ) {
			return null;
		}
	}

	// vertb_interrupt
	/**
	 * Should be called at screen update or every 20 ms
	 */
	public static void vertbInterrupt(final Canvas g) {
		framesWheelsEngine( g );
	}

	// FIXME set_sprite_copper
	private static void setSpriteCopper(int d0, int d1) {
		/*
		final short[] sa0 = sprite0;// little endian pointers
		int sa0i = 2;
		d1 <<= 3;
		sa0[ (sa0i + d1 + 4) / 2 ] = (short) d0;
		sa0[ (sa0i + d1) / 2 ] = (short) (d0 >> 8);
		*/
	}

	// set_wheel_sprite_pointers
	private static void setWheelSpritePointers(final Canvas g, int d0, int d1) {
		/*
		gp_a1 = lw_graphic_pointers;
		d0 = gp_a1[ d0 & 0xFF ];
		setSpriteCopper( d0, d1 );
		*/
		// java version
		// right: 43, 44, 45
		// left: 48, 47, 46
		// java: draw on visible screen
		/* final Canvas g;
		synchronized( Game.class ) {
			g = b_which_screen ? screen2.getGraphics() : screen1.getGraphics();
		}
		*/
		g.save();
		g.clipRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
				SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH,
				SCREEN_AREA_TOP_Y + (158 + 1) * PF - SCREEN_AREA_TOP_Y );

		switch( d1 ) {
			case 0: {// right
				if( d0 >= 43 ) {
					d0 -= 43;
					// g.clipRect( SCREEN_WIDTH - SCREEN_AREA_LEFT_X - 24 * PF, (151 - 50) * PF,
					//		24 * PF, ((158 + 1) - (151 - 50)) * PF );
					final GraphicsInfo gi = graphic_info[ d0 + SPRITE_RIGHT_WHEELS ];
					g.drawBitmap( gi.mImage, gi.mX, gi.mY, paint );
				}
				GraphicsInfo gi = graphic_info[ SPRITE_RIGHT_EXHAUST_PANEL ];
				g.drawBitmap( gi.mImage, gi.mX, gi.mY, paint );
				gi = graphic_info[ J_SPRITE_CURRENT_RIGHT_FLAME ];
				if( gi.mImage != null ) {// TODO remove after testing
					g.drawBitmap( gi.mImage, gi.mX, gi.mY, paint );
				}
				break;
			}
			case 1: {// left
				if( d0 >= 43 ) {
					d0 -= 46;
					//g.clipRect( SCREEN_AREA_LEFT_X, (151 - 50) * PF,
					//		24 * PF, ((158 + 1) - (151 - 50)) * PF );
					final GraphicsInfo gi = graphic_info[ d0 + SPRITE_LEFT_WHEELS ];
					g.drawBitmap( gi.mImage, gi.mX, gi.mY, paint );
				}
				GraphicsInfo gi = graphic_info[ SPRITE_LEFT_EXHAUST_PANEL ];
				g.drawBitmap( gi.mImage, gi.mX, gi.mY, paint );
				gi = graphic_info[ J_SPRITE_CURRENT_LEFT_FLAME ];
				if( gi.mImage != null ) {// TODO remove after testing
					g.drawBitmap( gi.mImage, gi.mX, gi.mY, paint );
				}
				break;
			}
		}
		g.restore();
	}

	private static int getRGB(int colorIndex) {
		return SCPalette[ j_color_pallete_offset + colorIndex ];
	}
	// end java specific

	private static void jClearLoopPlayersVariables() {// synthetic for java
		lw_current_players_x = 0;
		w_players_smaller_y = 0;
		// dc.w 0
		lw_current_players_z = 0;

		lw_players_world_x = 0;
		lw_players_world_y_fp16 = 0;
		lw_players_world_z = 0;

		w_players_x_angle = 0;
		w_players_y_angle = 0;
		w_players_z_angle = 0;

		w_players_world_x_speed = 0;
		w_players_world_y_speed = 0;
		w_players_world_z_speed = 0;

		w_players_x_rotation_speed = 0;
		w_players_y_rotation_speed = 0;
		w_players_z_rotation_speed = 0;

		w_total_world_x_acceleration = 0;
		w_total_world_y_acceleration = 0;
		w_total_world_z_acceleration = 0;

		w_players_x_rotation_acceleration = 0;
		w_players_y_rotation_acceleration = 0;
		w_players_z_rotation_acceleration = 0;

		Arrays.fill( w_wheel_x_offsets, (short) 0 );
		// front_left_wheel_x_offset = 0;
		// front_right_wheel_x_offset = 0;
		// rear_wheel_x_offset = 0;

		Arrays.fill( w_wheel_z_offsets, (short) 0 );
		// front_left_wheel_z_offset = 0;
		// front_right_wheel_z_offset = 0;
		// rear_wheel_z_offset = 0;

		w_gravity_x_acceleration = 0;
		w_gravity_y_acceleration = 0;
		w_gravity_z_acceleration = 0;

		Arrays.fill( w_new_wheel_differences, (short) 0 );
		// new_front_left_difference = 0;
		// new_front_right_difference = 0;
		// new_rear_difference = 0;

		w_old_front_left_difference = 0;
		w_old_front_right_difference = 0;
		w_old_rear_difference = 0;

		w_front_left_amount_below_road = 0;
		w_front_right_amount_below_road = 0;
		w_rear_amount_below_road = 0;

		w_overall_difference_below_road = 0;
		w_front_difference_below_road = 0;

		w_engine_z_acceleration = 0;

		w_players_x_speed = 0;
		w_players_y_speed = 0;
		w_players_z_speed = 0;

		w_players_x_acceleration = 0;
		w_players_y_acceleration = 0;
		w_players_z_acceleration = 0;

		w_average_amount_below_road = 0;

		w_players_final_x_rotation_speed = 0;
		w_players_final_y_rotation_speed = 0;
		w_players_final_z_rotation_speed = 0;

		w_car_collision_x_acceleration = 0;
		w_car_collision_y_acceleration = 0;
		w_car_collision_z_acceleration = 0;

		w_car_to_road_collision_z_acceleration = 0;

		w_z_inclination_to_road = 0;
		w_y_inclination_to_road = 0;
		w_x_inclination_to_road = 0;

		b_surface_cosx_sinz_fp8 = 0;// dc.b 0,0
		b_surface_cosx_cosz_fp8 = 0;// dc.b 0,0
		b_surface_sinx_fp8 = 0;// dc.b 0,0

		w_car_to_car_x_acceleration = 0;
		w_car_to_car_y_acceleration = 0;
		w_car_to_car_z_acceleration = 0;

		w_section_y_angle = 0;
		w_pos_players_z_speed = 0;
	}

	private static void jClearLoop() {// synthetic for java
		w_1baf8 = 0;
		engine_power = 0;
		w_opponents_engine_power = 0;
		boost_unit_value = 0;// dc.b 0,0
		// dc.b 0
		b_road_cushion_value = 0;// dc.b 0
		// dc.b 0,0

		b_players_map_x = 0;// dc.b 0,0
		b_players_map_z = 0;// dc.b 0
		b_players_fine_map_x = 0;// dc.b 0,0
		b_players_fine_map_z = 0;// dc.b 0

		w_players_distance_into_section = 0;

		w_opponents_distance_into_piece = 0;

		w_players_distance_into_section_plus64 = 0;

		w_normal_distance_into_section = 0;

		w_opponents_distance_into_section_minus64 = 0;
		// dc.w 0
		b_menu_bar_index = 0;
		b_thousands = 0;

		// acceleration.adjust
		lw_road_height_fp8 = 0;// actually a longword, but bytes have more than 1 use
		// road_height_1 = 0;
		b_factor_high_value_low = 0;
		b_value_low_factor_high = 0;

		b_players_road_piece = 0;
		b_opponents_road_piece = 0;
		b_1bb1e = 0;
		b_1bb1f = 0;
		b_players_lap = 0;
		b_opponents_lap = 0;

		w_road_near_x_offset = 0;
		// road_finest_x_offset = 0;// low byte of road_near_x_offset
		// dc.w 0

		w_road_near_z_offset = 0;
		// road_finest_z_offset = 0;// low byte of road_near_x_offset
		// dc.w 0

		w_opp_zdiff = 0;
		b_inclination_sin_fp8 = 0;

		w_opp_xdiff = 0;
		b_surface_cosx_fp8 = 0;// locations have more than 1 use
		b_inclination_cos_fp8 = 0;

		b_road_finer_x_offset = 0;
		// dc.w 0
		b_road_finer_z_offset = 0;// dc.b 0,0
		w_previous_left_top_y = 0;
		w_previous_right_top_y = 0;
		b_daft_flag = false;// dc.b 0,0
		// dc.w 0
		byte_count = 0;
		boost_unit = 0;
		// dc.b 0
		b_1bb3f = 0;
		b_opponents_road_piece_m64_high = 0;
		b_opponents_road_piece_m64_low = 0;
		// dc.b 0
		opponents_x_span = 0;
		b_opp_touching_road = false;
		b_opponents_required_z_speed_reached = false;
		b_cars_collided = false;
		b_players_input = 0;
		b_prompt_chars = 0;// dc.b 0,0
		b_opponents_required_z_speed = 0;// dc.b 0,0
		// dc.b 0
		b_curved_section = 0;
		// dc.b 0

		b_front_left_damage = 0;
		b_front_right_damage = 0;
		b_rear_damage = 0;

		// dc.w 0
		b_damaged = false;
		b_damage_line_x = 0;
		b_damaged_count = 0;
		camera_preview_position_id = 0;
		b_near_sections_done = 0;
		b_number_of_coords = 0;
		b_coords_of_last_section = 0;
		b_1bb5b = false;
		b_1bb5c = 0;
		b_left_hand_bend = 0;
		b_current_print_row = 0;
		b_division = 0;
		b_offset_for_coord_pair_to_start_at = 0;
		b_section_flags2 = 0;
		b_boost_activated = false;
		b_turn_engine_off = false;
		b_1bb64 = 0;
		b_wheel_off_road = false;
		b_use_lines_color = false;
		b_opponent_reached = false;
		b_unused_flag = false;
		b_flame_counter = 0;
		b_number_of_segments = 0;
		b_1bb6b = 0;
		b_1bb6c = 0;
		b_print_fine_x = 0;
		b_print_fine_y = 0;
		b_1bb6f = false;
		b_boost_flag = false;
		b_draw_bridge_make_far_coords = false;
		b_1bb72 = false;
		b_smashed_countdown = 0;
		b_1bb74 = 0;
		b_1bb75 = false;
		b_1bb76 = 0;
		b_standard_clip_flag = false;
		// dc.b 0
		b_y_coords_stored_as_words = 0;
		b_current_y_coord_offset = 0;
		b_road_width_reduction = 0;
		b_opponents_coord = 0;
		b_grounded_count = 0;
		b_touching_road = false;
		b_pits_are_black = false;
		// dc.w 0
		// dc.w 0
		// dc.b 0
		b_current_road_piece = 0;
		b_near_section_piece = 0;// dc.b 0,0
		b_white_prompts = false;
		// b_1bb89 = 0;
		// dc.b 0
		b_map_x = 0;
		b_map_z = 0;
		b_prompt_required = false;
		b_track_preview = false;
		b_copy_prompt_groups = 0;
		b_piece_coords_offset = 0;
		b_finish_file_name_editing = false;
		b_1bb93 = false;
		b_1bb94 = 0;
		b_boost_max_units = 0;
		// dc.b 0
		b_number_of_sections_x2 = 0;
		b_number_of_sections_x2_minus2 = 0;
		b_lap_that_finishes_race = 0;
		b_at_side_byte = 0;
		b_1bb9b = 0;
		b_off_map_status = 0;
		b_1bb9d = 0;
		b_1bb9e = 0;
		b_1bb9f = 0;
		// dc.b 0
		b_rear_wheel_surface_x_position = 0;
		// dc.b 0
		b_surface_start_coord_x2 = 0;
		b_1bba4 = false;
		b_second_machine = 0;
		b_opponents_road_section_m255 = 0;

		b_opponents_suggested_road_x_position = 0;
		// temp = 0;// location has more than 1 use

		b_accelerating = false;// dc.b 0,0
		b_1bbaa = false;
		b_main_loop_count = 0;
		b_dont_copy_coords = false;
		b_laps_when_race_finished = 0;
		b_draw_bridge_frame_count = 0;
		b_1bbb1 = 0;
		b_1bbb2 = 0;// dc.b 0,0
		b_1bbb4 = 0;
		b_1bbb5 = 0;
		b_new_damage = 0;// dc.b 0,0
		b_opponent_behind_player = false;
		b_no_wheel_update = false;
		b_do_not_draw_opponents_shadow = false;
		b_copy_swing_from_left = false;
		b_print_text_on_screen_2 = false;
		b_1bbbd = 0;
		b_1bbbe = 0;
		b_1bbbf = 0;
		b_opponents_random_steering_count = 0;
		b_collision_in_air = false;
		b_1bbc2 = 0;
		b_collision_counter = 0;
		b_drop_start_done = false;
		b_coord_offset_zero_or_four = 0;
		b_left_right_value = 0;
		b_player_close_to_opponent = false;
		b_which_screen = false;
		b_loop_counter = 0;
		b_1bbca = 0;
		b_1bbcb = false;
		b_1bbcc = 0;
		b_fourteen_frames_elapsed = 0;
		// dc.b 0
		b_1bbcf = 0;
		b_1bbd0 = 0;
		b_players_position_flags = 0;
		b_draw_road_surface_flag = false;
		b_or_char_with_screen = false;
		b_section_steering_amount = 0;
		b_map_x_shift = 0;
		b_map_z_shift = 0;
		b_ferocity_of_sparks_or_clouds = 0;
		// dc.b 0
		b_road_length_reduction = 0;
		b_which_side_byte = 0;
		b_wheel_frame_number = 0;
		b_other_road_line_color = 0;
		b_1bbdd = 0;
		b_car_on_chains_countdown = 0;
		b_1bbe0 = 0;
		b_swing_from_left = false;
		b_1bbe2 = 0;
		b_wheel_frame_count = 0;
		b_edge_x1_offset = 0;
		b_coord_pair_to_start_at = 0;
		b_next_segment = false;// dc.b 0,0
		b_start_offset_for_rating_in_divisions = 0;
		b_1bbe9 = 0;
		b_1bbea = 0;
		b_1bbeb = false;

		w_opponents_road_x_position = 0;
		w_opponents_z_speed = 0;
		w_opponents_engine_z_acceleration = 0;

		w_rough_difference_angle = 0;
		w_engine_revs_change = 0;

		w_max_difference = 0;
		b_player_to_right = false;
		// difference_angle
		w_near_x_coord = 0;
		// dc.b 0 ; location has more than 1 use
		b_abs_x_difference = 0;
		// dc.b 0 ; location has more than 1 use

		w_height_adjust = 0;
		w_near_z_coord = 0;// location has more than 1 use

		w_y_perspective_shift = 0;
		// dc.w 0
		// dc.w 0

		w_swing_magnitude = 0;

		w_surface_y_coord1 = 0;
		w_surface_y_coord2 = 0;
		w_surface_y_coord3 = 0;
		w_surface_y_coord4 = 0;
		w_surface_y_coord5 = 0;
		w_surface_y_coord6 = 0;

		w_overall_left_y_shift = 0;
		w_overall_right_y_shift = 0;
		w_opponents_offset = 0;
		b_edge_offset_left_x2 = 0;// dc.b 0,0
		b_edge_x2_offset_1 = 0;
		b_edge_offset_right_x2 = 0;// dc.b 0,0
		w_new_speed_bar = 0;
		// dc.w 0
		w_1bc1c = 0;
		w_dnr_value = 0;
		w_old_speed_bar = 0;

		w_wheel_road_x_position = 0;
		w_opponents_distance_into_section_minus255 = 0;
		w_near_sections_done2 = 0;
		w_1bc28 = 0;
		w_pos_difference_angle = 0;
		// dc.w 0
		w_x_shift = 0;
		w_rough_player_angle = 0;
		b_plus_180_degrees = 0;
		w_engine_revs = 0;
		w_perspective_z = 0;
		w_smallest_distance_between_players = 0;
		w_damage_value = 0;// dc.b 0,0
		w_pos_difference_angle_m16 = 0;
		w_difference_between_players = 0;
		w_surface_z_position = 0;
		w_y_shift = 0;
		w_curve_to_left = 0;
		// dc.w 0
		// dc.w 0
		w_rough_piece_angle = 0;
		w_surface_x_position = 0;
		// dc.w 0
		// dc.w 0
		// dc.w 0
		b_1bc55 = 0;
		w_1bc56 = 0;
		w_fp_y2 = 0;
		w_fp_y = 0;

		w_players_x_offset_from_road_centre = 0;
		w_players_road_x_position = 0;

		w_required_raise_height = 0;
		w_wheel_rotation_speed = 0;
		w_1bc64 = 0;
		w_1bc66 = 0;
		w_1bc68 = 0;
		w_1bc6a = 0;
		w_1bc6c = 0;
		w_1bc6e = 0;
		w_1bc70 = 0;
		w_1bc72 = 0;
		w_1bc74 = 0;
		w_1bc76 = 0;
		w_1bc78 = 0;
		w_1bc7a = 0;
		w_1bc7c = 0;
		w_1bc7e = 0;
		w_1bc80 = 0;
		w_1bc82 = 0;
		w_car_x_shift = 0;
		w_car_y_shift = 0;
		w_1bc88 = 0;
		w_1bc8a = 0;
		w_left_y_coord_offset = null;
		// dc.w 0
		w_right_y_coord_offset = null;
		// dc.w 0

		lw_front_left_actual_height_fp8 = 0;
		lw_front_right_actual_height_fp8 = 0;
		lw_rear_actual_height_fp8 = 0;

		lw_wreck_wheel_height_reduction_fp8 = 0;

		lw_front_left_road_height_fp8 = 0;
		lw_front_right_road_height_fp8 = 0;
		lw_rear_road_height_fp8 = 0;

		lw_front_left_height_difference = 0;
		lw_front_right_height_difference = 0;
		lw_rear_height_difference = 0;

		w_piece_data_offset = null;
		// dc.w 0
		w_road_data_offset = null;
		// dc.w 0
		// dc.w 0
		// dc.w 0
		b_which_screen2 = false;// dc.b 0,0
		// dc.w 0
		jClearLoopPlayersVariables();

		Arrays.fill( w_opp_new_wheel_differences, (short) 0 );
		// opp_new_rear_left_difference = 0;
		// opp_new_rear_right_difference = 0;
		// opp_new_front_difference = 0;
		// dc.w 0

		Arrays.fill( w_opp_wheel_actual_heights, (short) 0 );
		// opp_rear_left_actual_height = 0;
		// opp_rear_right_actual_height = 0;
		// opp_front_actual_height = 0;
		// dc.w 0

		w_opp_old_rear_left_difference = 0;
		w_opp_old_rear_right_difference = 0;
		w_opp_old_front_difference = 0;
		// dc.w 0

		Arrays.fill( w_opp_wheel_y_speeds, (short) 0 );
		// opp_rear_left_y_speed = 0;
		// opp_rear_right_y_speed = 0;
		// opp_front_y_speed = 0;
		// dc.w 0

		w_opp_rear_left_y_acceleration = 0;
		w_opp_rear_right_y_acceleration = 0;
		w_opp_front_y_acceleration = 0;
		// dc.w 0

		Arrays.fill( w_opp_wheel_road_heights, (short) 0 );
		// opp_rear_left_road_height = 0;
		// opp_rear_right_road_height = 0;
		// opp_front_road_height = 0;
		// dc.w 0

		Arrays.fill( w_segment_x_coords, (short) 0 );// ds.w 16
		Arrays.fill( w_segment_z_coords, (short) 0 );// ds.w 16
		// dc.w 36

		Arrays.fill( b_near_section_flags, (byte) 0 );// ds.w 80
	}

	private static void resetTimeResult(final int id) {
		b_time_results_sec[ id ] = 0;
		b_time_results_ms[ id ] = 0;
		b_time_results_min[ id ] = 0;
	}

	private static void delay_1ms() {
		jDelay( 1 );
	}

	private static void delay_2ms() {
		jDelay( 2 );
	}

	private static void delay_5ms() {
		jDelay( 5 );
	}
	
	private static void delay_10ms() {
		jDelay( 10 );
	}
	
	private static void delay_30ms() {
		jDelay( 30 );
	}

	private static void delay_60ms() {
		jDelay( 60 );
	}

	private static void delay_100ms() {
		jDelay( 100 );
	}

	// signal_prompt_required
	/**
	 * @param d0_group group, as byte
	 * @param d2       Java: string for prompting.
	 *                 Amiga: offset to b_race_prompt_text
	 */
	private static void signalPromptRequired(final int d0_group, final String d2) {
		b_prompt_required = true;
		b_prompt_offset = d2;
		b_prompt_groups = (byte) d0_group;
	}

	// fetch_near_section_stuff
	/**
	 * @param d1_piece road section as byte in d1
	 */
	private static void fetchNearSectionStuff(final int d1_piece) {
		b_y_coords_stored_as_words = b_left_y_coordinate_ids[ d1_piece ];
		w_left_y_coord_offset = Road.w_y_coordinate_offsets[ b_y_coords_stored_as_words & 0x7F ];

		final int right_y_id = b_right_y_coordinate_ids[ d1_piece ];// a1
		b_other_road_line_color = (byte) ((right_y_id & 0x80) == 0 ? 0 : 2);
		w_right_y_coord_offset = Road.w_y_coordinate_offsets[ right_y_id & 0x7F ];

		w_overall_left_y_shift = w_overall_left_y_shifts[ d1_piece ];
		w_overall_right_y_shift = w_overall_right_y_shifts[ d1_piece ];

		w_rough_piece_angle = (short) ((b_road_section_angle_and_piece[ d1_piece ] & 0xC0) << 8);

		b_plus_180_degrees = (byte) ((b_road_section_angle_and_piece[ d1_piece ] & 0x10) << 3);

		b_near_section_piece = (byte) (b_road_section_angle_and_piece[ d1_piece ] & 0x0F);

		w_piece_data_offset = Road.w_piece_data_offsets[ b_near_section_piece ];

		// d0 = w_piece_data_offset;
		// d0 = ((((d0 << 8) & 0xFF00) | ((d0 >> 8) & 0xFF)) - 0xb100) & 0xffff;
		final byte[] ba0 = w_piece_data_offset;// Road.offsetToPiece( d0 );

		b_curved_section = ba0[ 1 ];

		int offset = ba0[ 0 ];// offset for number_of_coords
		b_number_of_sections_x2 = ba0[ offset++ ];
		b_number_of_coords = (byte) (b_number_of_sections_x2 << 1);
		b_number_of_sections_x2_minus2 = (byte) (b_number_of_sections_x2 - 2);
		b_coords_of_last_section = (byte) (b_number_of_sections_x2_minus2 << 1);
		b_number_of_segments = (byte) ((b_number_of_sections_x2 >>> 1) - 1);

		w_curve_to_left = (short) ((ba0[ offset++ ] & 0x01) == 0 ? 0 : 0x8000);

		b_road_width_reduction = ba0[ offset++ ];
		b_road_length_reduction = ba0[ offset ];
		offset += 3;

		b_section_steering_amount = ba0[ offset ];
		// d2++;
	}

	// get_road_data_left_y
	/**
	 * @param bd2_index index
	 * @return Java: d0 word
	 * Amiga: high byte in d0 byte, low byte in some "value".
	 * ((factor1 << 8) | value) is the result
	 */
	private static int getRoadDataLeftY(int bd2_index) {
		// final int ba0i = (getROL16( w_left_y_coord_offset, 8 ) - 0xb100) & 0xffff;

		if( b_y_coords_stored_as_words < 0 ) {
			// final char[] ba0 = (char[]) w_left_y_coord_offset;// Road.offsetToNearPieceYdataWords( ba0i );
			bd2_index <<= 1;
			return Road.getCoordinateAsWord( w_left_y_coord_offset, bd2_index );
			// return ((ba0[ bd2 ] & 0x7F) << 8) | (ba0[ bd2 + 1 ] & 0xFF);
			// return ba0[ bd2 ];
		}
		// final byte[] ba0 = w_left_y_coord_offset;// Road.offsetToNearPieceYdataBytes( ba0i );
		// return ((ba0[ bd2 ] & 0x0F) << 8) | ((ba0[ bd2 ] << 1) & 0xE0);
		return Road.getCoordinateAsByte( w_left_y_coord_offset, bd2_index );
	}

	// get_road_data_right_y
	/**
	 * @param bd2_index index
	 * @return Java: d0 word
	 * Amiga: high byte in d0 byte, low byte in some "value".
	 * ((factor1 << 8) | value) is the result
	 */
	private static int getRoadDataRightY(final int bd2_index) {
		w_left_y_coord_offset = w_right_y_coord_offset;
		return getRoadDataLeftY( bd2_index );
	}

	// set_speed_base_data
	private static void setSpeedBaseData() {
		if( (0 == b_multi_no_of_players) || (0 == b_1ca35) ) {
			int d4 = 0xFF;
			do {
				randomizeLong();
			} while( --d4 >= 0 );

			int d1 = b_road_id;
			int d2 = b_standard_league_damage_limit;
			if( 0 != b_league_offset ) {
				d1 += 32;
				d2 = b_super_league_damage_limit;
			}
			b_damaged_limit = (byte) d2;

			int d0 = randomizeLong();
			d0 &= Road.opponents_speed_data[ d1 ];
			d0 += Road.opponents_speed_data[ d1 + 8 ];
			b_opponent_max_speed = (byte) d0;

			d0 = randomizeLong();
			d0 &= Road.opponents_speed_data[ d1 + 16 ];
			d0 += Road.opponents_speed_data[ d1 + 24 ];
			b_opponent_base_speed = (byte) d0;
		}
	}

	// set_road_data1
	/**
	 * @param trackID road id
	 */
	private static void setRoadData1(final int trackID) {
		w_road_data_offset = Road.road_data_offsets[ trackID ];// a2
		// int d0 = (getROL16( w_road_data_offset, 8 ) - 0xB100) & 0xFFFF;
		final byte[] ba5 = w_road_data_offset;// Road.offsetToRoad( d0 );

		// get.road.data.byte
		//	move.b	(a5,d5.w),d0
		//	addq.w	#1,d5
		//	andi.b	#$ff,d0
		//	rts
		int d5_offset = 0;

		b_number_of_road_pieces = ba5[ d5_offset++ ];
		b_players_start_piece = ba5[ d5_offset++ ];
		b_near_start_line_section = ba5[ d5_offset++ ];
		b_half_a_lap_section = ba5[ d5_offset++ ];

		w_near_z_coord = w_near_x_coord =
				(short) ((ba5[ d5_offset + 1 ] << 8)
						| (ba5[ d5_offset ] & 0xFF));
		d5_offset += 2;

		b_other_road_line_color = 0;
		int total_road_distance = 0;// w_pos_difference_angle = 0;
		b_prompt_chars = 0;

		int d1_piece = 0;

		// srd11a:
		// int factor1;
		do {
			// factor1 = 0;// to fix "may not be initialized"
			// srd11a:
			while( 0 == b_prompt_chars ) {
				final byte d0 = ba5[ d5_offset++ ];
				b_factor_high_value_low = d0;
				b_road_section_angle_and_piece[ d1_piece ] = d0;
				if( 0x0F != (d0 & 0x0F) ) {
					break;
				}
				b_prompt_chars = (byte) ((b_factor_high_value_low & 0xFF) >>> 4);
			}
			byte d0_map_position_x;
			if( 0 == b_prompt_chars ) {
				// srd14:
				b_1bc55 = b_road_section_angle_and_piece[ d1_piece ];
				d0_map_position_x = ba5[ d5_offset++ ];
			} else {
				--b_prompt_chars;
				b_factor_high_value_low = b_1bc55;
				b_road_section_angle_and_piece[ d1_piece ] = b_1bc55;

				if( 0 != (b_1bc55 & 0x10) ) {
					b_factor_high_value_low ^= 0xC0;
				}
				d0_map_position_x = b_map_x;
				if( b_factor_high_value_low >= 0 ) {// jsr srd1.sub3
					d0_map_position_x += (0 != (b_factor_high_value_low & 0x40)) ? 1 : 16;
				} else {
					d0_map_position_x -= (0 != (b_factor_high_value_low & 0x40)) ? 1 : 16;
				}
			}
			// srd14a:
			b_road_piece_xz_positions[ d1_piece ] = d0_map_position_x;
			b_map_x = d0_map_position_x;

			// lsr.b	#2,d0
			// roxr.b	#1,d0
			// d0 = (other_road_line_colour >>> 3) & 0x1F;
			// d0 |= (other_road_line_colour << 6) & 0x80;
			// value = (byte) d0;// bit 1 to bit 7 ?
			b_value_low_factor_high = (byte) (b_other_road_line_color << 6);

			int d0 = (b_factor_high_value_low & 0x0F);
			if( d0 >= 12 ) {
				b_road_section_angle_and_piece[ d1_piece ] &= 0xF0;
				b_left_y_coordinate_ids[ d1_piece ] = Road.b_TAB_5b2c4[ d0 - 12 ];
				d0 = Road.b_TAB_5b2c4[ d0 - 12 + 2 ];
			} else {
				// srd15:
				b_left_y_coordinate_ids[ d1_piece ] = ba5[ d5_offset++ ];
				d0 = (0 != (b_factor_high_value_low & 0x20))
						? b_left_y_coordinate_ids[ d1_piece ]
						: ba5[ d5_offset++ ];
			}
			// srd16a:
			b_right_y_coordinate_ids[ d1_piece ] = (byte) ((d0 & 0x7F) | b_value_low_factor_high);
			w_distances_around_road[ d1_piece ] = (short) (total_road_distance << 5);// (w_pos_difference_angle << 5);
			fetchNearSectionStuff( d1_piece );
			b_other_road_line_color = (byte) ((b_other_road_line_color + b_number_of_sections_x2_minus2) & 2);
			total_road_distance += b_number_of_segments;// w_pos_difference_angle += b_number_of_segments & 0xFF;

			int d4 = w_near_x_coord - getRoadDataLeftY( 0 );
			w_overall_left_y_shifts[ d1_piece ] = (short) d4;

			d4 += getRoadDataLeftY( b_number_of_segments );
			w_near_x_coord = (short) d4;

			d4 = w_near_z_coord - getRoadDataRightY( 0 );
			w_overall_right_y_shifts[ d1_piece ] = (short) d4;

			d4 += getRoadDataRightY( b_number_of_segments );
			w_near_z_coord = (short) d4;

		} while( ++d1_piece != b_number_of_road_pieces );
		// srd17:
		b_start_finish_section = b_near_start_line_section;
		if( ++b_start_finish_section >= b_number_of_road_pieces ) {
			b_start_finish_section = 0;
		}
		w_total_road_distance = (short) (total_road_distance << 5);// (w_pos_difference_angle << 5);

		// srd19:
		// read 6 bytes in the loop
		b_standard_league_damage_limit = ba5[ d5_offset++ ];
		b_super_league_damage_limit = ba5[ d5_offset++ ];
		b_standard_league_boost = ba5[ d5_offset++ ];
		b_super_league_boost = ba5[ d5_offset++ ];
		b_1ca2e = ba5[ d5_offset++ ];// byte count to read
		b_1ca2f = ba5[ d5_offset++ ];// byte count to read

		int d0 = b_1ca2e;
		if( 0 != d0 ) {
			d1_piece = 0;
			do {// srd1a:
				b_DAT_1c8a8[ d1_piece ] = ba5[ d5_offset++ ];
				b_DAT_1c8c8[ d1_piece ] = ba5[ d5_offset++ ];
			} while( ++d1_piece < b_1ca2e );
		}
		// srd1b:
		d0 = b_1ca2f;
		if( 0 != d0 ) {
			d1_piece = 0;
			// srd1c:
			do {
				b_DAT_1c8e8[ d1_piece ] = ba5[ d5_offset++ ];
			} while( ++d1_piece != b_1ca2f );
		}
		// srd1d:
		setSpeedBaseData();
		b_prompt_chars = 0;
		b_value_low_factor_high = 0x7C;// base speed
		b_copy_prompt_groups = 1;
		// srd116:
		d1_piece = b_number_of_road_pieces - 1;
		// srd1e:
		int d2 = b_1ca2e;
		for( ; ; ) {
			// srd1fa
			if( --d2 >= 0 ) {
				d0 = d1_piece;
				if( d0 != b_DAT_1c8a8[ d2 ] ) {
					continue;
				}
				// srd110:
				d0 = b_DAT_1c8c8[ d2 ];
				b_opponents_speed_values[ d1_piece ] = (byte) d0;
				if( 0 != (d0 & 0x80) ) {
					b_prompt_chars = 3;
				}
				// srd111:
				b_value_low_factor_high = (byte) (d0 & 0x7F);// base speed
			} else {
				// srd111a:
				d0 = b_road_section_angle_and_piece[ d1_piece ] & 0x0F;
				d0 = Road.sections_car_can_be_put_on[ d0 ];
				if( d0 < 0 ) {
					b_value_low_factor_high = (byte) (b_opponent_base_speed - 10);
					d0 = b_opponent_base_speed;
				} else {
					// srd112:
					d0 = b_value_low_factor_high + 10;
					if( 0 == (d0 & 0x80) ) {
						b_value_low_factor_high = (byte) d0;
					}
					d0 = b_value_low_factor_high;
				}
				// srd113a:
				if( 0 != b_prompt_chars ) {
					--b_prompt_chars;
					d0 |= 0x80;
				}
				// srd114:
				b_opponents_speed_values[ d1_piece ] = (byte) d0;
			}
			// srd115:
			if( --d1_piece >= 0 ) {
				// bpl srd1e
				d2 = b_1ca2e;
				continue;
			}
			// srd116:
			d1_piece = b_number_of_road_pieces;
			--d1_piece;
			if( --b_copy_prompt_groups != 0 ) {
				// bne srd1e
				d2 = b_1ca2e;
				continue;
			}
			return;
		}// for ;;
	}

	// make_near_top_y_coords
	private static void makeNearTopYcoords() {
		// int d0 = w_left_y_coord_offset;
		// d0 = ((((d0 << 8) & 0xFF00) | ((d0 >>> 8) & 0xFF)) - 0xB100) & 0xFFFF;
		final byte[] ba4 = w_left_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

		// d0 = w_right_y_coord_offset;
		// d0 = ((((d0 << 8) & 0xFF00) | ((d0 >>> 8) & 0xFF)) - 0xB100) & 0xFFFF;
		final byte[] ba5 = w_right_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

		int d7 = 46 - w_near_sections_done2;
		if( d7 < 0 ) {
			d7 = 0;
		}
		w_near_sections_done2 += b_number_of_segments;

		b_near_section_flags[ b_number_of_sections_x2_minus2 << 1 ] =
				(byte) ((b_current_road_piece == b_near_start_line_section)
						? 1 : 0);// set start line flag

		int d2_y_coord_offset = 0;
		int d1_section_x2 = b_coord_offset_zero_or_four;
		// java does not allow goto
		boolean j_is_nearest = false;
		if( (b_y_coords_stored_as_words >= 0) ) {
			boolean j_use_for_far_road_coords = false;
			if( d1_section_x2 == 0 ) {
				j_is_nearest = true;
				// For the nearest section
				b_current_y_coord_offset = (byte) d2_y_coord_offset;// start with first coordinates
				// y_coords_are_bytes
				final int y = ba4[ d2_y_coord_offset ];
				if( y < 0 ) {
					j_use_for_far_road_coords = true;
					// bmi use_for_far_road_coords;
				}
				// bra make_top_y_coords;
			} else {
				// For other near sections
				// mntyc3:
				d2_y_coord_offset = 1;
				b_current_y_coord_offset = (byte) d2_y_coord_offset;// start with the second coordinate
			}
			// save_next_two_y_coords:
			do {// bytes
				do {
					if( !j_is_nearest ) {
						final int d0_y = ba4[ d2_y_coord_offset ];
						if( d0_y < 0 ) {
							j_use_for_far_road_coords = true;
						}
					}
					if( !j_is_nearest && !j_use_for_far_road_coords && (d2_y_coord_offset >= d7) ) {
						w_coord_y_values_fp5[ (d1_section_x2) / 2 ] = (short) 0x8000;// if top coordinates are not visible
						w_coord_y_values_fp5[ (d1_section_x2 + 2) / 2 ] = (short) 0x8000;
					} else {
						if( j_use_for_far_road_coords ) {
							j_use_for_far_road_coords = false;
							// use_for_far_road_coords:
							b_near_section_flags[ d1_section_x2 ] = 0;
						}
						j_is_nearest = false;
						// make_top_y_coords:
						// y = ba4[ y_coord_offset ];// get current left y coordinate
						// y = w_overall_left_y_shift + (((y << 1) & 0xE0) | ((y & 0x0F) << 8));
						int y = w_overall_left_y_shift + Road.getCoordinateAsByte( ba4, d2_y_coord_offset );
						w_coord_y_values_fp5[ (d1_section_x2) / 2 ] = (short) y;// save left y coordinate

						if( 0 == (b_near_section_flags[ d1_section_x2 ] & 0x80) ) {
							// TODO java, verify, is this a bug in Amiga ?
							int prev_y;
							if( d1_section_x2 < 4 ) {
								prev_y = b_near_section_flags[ b_near_section_flags.length + (d1_section_x2 - 4) / 2 ];
							} else {
								prev_y = w_coord_y_values_fp5[ (d1_section_x2 - 4) / 2 ];
							}
							if( prev_y < 0 ) {
								if( b_pits_are_black && (d1_section_x2 >= 4) ) {
									// prev_y = ba4[ y_coord_offset - 1 ];// get previous left y coordinate
									// prev_y = w_overall_left_y_shift + (((prev_y << 1) & 0xE0) | ((prev_y & 0x0F) << 8));
									prev_y = w_overall_left_y_shift + Road.getCoordinateAsByte( ba4, d2_y_coord_offset - 1 );
									// previous_left_y_positive:
									y -= prev_y;// current y - previous y
									if( y < 0 ) {
										y = -y;
									}
									// y_change_positive:
									// if y change is not large enough
									if( (y >= Road.STEEP) && (d1_section_x2 >= 4) ) {
										b_near_section_flags[ d1_section_x2 ] |= PIT;// store pit flag
									}
								}
							} else {
								// previous_left_y_positive:
								y -= prev_y;// current y - previous y
								if( y < 0 ) {
									y = -y;
								}
								// y_change_positive:
								// if y change is not large enough
								if( (y >= Road.STEEP) && (d1_section_x2 >= 4) ) {
									b_near_section_flags[ d1_section_x2 ] |= PIT;// store pit flag
								}
							}
						}
						// save_right_y_coord:
						// y = ba5[ y_coord_offset ];// get current right y coordinate
						// y = w_overall_right_y_shift + (((y << 1) & 0xE0) | ((y & 0x0F) << 8));
						y = w_overall_right_y_shift + Road.getCoordinateAsByte( ba5, d2_y_coord_offset );
						w_coord_y_values_fp5[ (d1_section_x2 + 2) / 2 ] = (short) y;// save right y coordinate
					}
					// next_two_y_coords:
					++d2_y_coord_offset;
					d1_section_x2 += 4;
				} while( d1_section_x2 < b_coords_of_last_section );

				// beq make_top_y_coords;
				j_is_nearest = true;
				// j_use_for_far_road_coords = false;
			} while( d1_section_x2 == b_coords_of_last_section );// save_next_two_y_coords
			// bra mntyc4;
			// mntyc4:
			--d2_y_coord_offset;

			// mntyc5:
			final int y = ba4[ d2_y_coord_offset ];
			if( y < 0 ) {
				b_near_section_flags[ d1_section_x2 - 4 ] = (byte) 0x80;
				d2_y_coord_offset = b_number_of_segments;
				if( d2_y_coord_offset >= d7 ) {
					w_coord_y_values_fp5[ (d1_section_x2 - 4) / 2 ] = (short) 0x8000;
					w_coord_y_values_fp5[ (d1_section_x2 - 2) / 2 ] = (short) 0x8000;
				}
			}
			return;
		}
		// words
		boolean j_use_for_far_road_coords2 = false;
		if( d1_section_x2 == 0 ) {
			j_is_nearest = true;
			// For the nearest section
			b_current_y_coord_offset = (byte) d2_y_coord_offset;// start with first coordinates
			// words
			d2_y_coord_offset <<= 1;
			final int y = ba4[ d2_y_coord_offset ];
			if( y < 0 ) {
				// bra use_for_far_road_coords2;
				j_use_for_far_road_coords2 = true;
			}
			// bpl make_top_y_coords2;
		} else {
			// For other near sections
			// mntyc3:
			d2_y_coord_offset = 1;
			b_current_y_coord_offset = (byte) d2_y_coord_offset;// start with the second coordinate
		}
		// save_next_two_y_coords2:
		do {// words
			do {
				if( !j_is_nearest && !j_use_for_far_road_coords2 ) {
					d2_y_coord_offset = b_current_y_coord_offset << 1;
					if( ba4[ d2_y_coord_offset ] < 0 ) {
						j_use_for_far_road_coords2 = true;
					}
				}
				if( !j_is_nearest && !j_use_for_far_road_coords2 && (b_current_y_coord_offset >= d7) ) {
					w_coord_y_values_fp5[ (d1_section_x2) / 2 ] = (short) 0x8000;
					w_coord_y_values_fp5[ (d1_section_x2 + 2) / 2 ] = (short) 0x8000;
				} else {
					if( j_use_for_far_road_coords2 ) {
						j_use_for_far_road_coords2 = false;
						// use_for_far_road_coords2:
						b_near_section_flags[ d1_section_x2 ] = 0;
					}
					j_is_nearest = false;
					// make_top_y_coords2:
					// get current left y coordinate
					// y = w_overall_left_y_shift + (((ba4[ y_coord_offset ] & 0x7F) << 8) | (ba4[ y_coord_offset + 1 ] & 0xFF));
					int y = w_overall_left_y_shift + Road.getCoordinateAsWord( ba4, d2_y_coord_offset );
					w_coord_y_values_fp5[ (d1_section_x2) / 2 ] = (short) y;// save left y coordinate

					if( b_near_section_flags[ d1_section_x2 ] >= 0 ) {
						// TODO java, verify, is this a bug in Amiga ?
						int prev_y;
						if( d1_section_x2 < 4 ) {
							prev_y = b_near_section_flags[ b_near_section_flags.length + (d1_section_x2 - 4) / 2 ];
						} else {
							prev_y = w_coord_y_values_fp5[ (d1_section_x2 - 4) / 2 ];
						}
						// store pit flag
						// current y - previous y
						if( prev_y < 0 ) {
							if( b_pits_are_black && (d1_section_x2 >= 4) ) {
								// get previous left y coordinate
								// prev_y = w_overall_left_y_shift + (((ba4[ y_coord_offset - 2 ] & 0x7f) << 8) | (ba4[ y_coord_offset - 2 + 1 ] & 0xFF));
								prev_y = w_overall_left_y_shift + Road.getCoordinateAsWord( ba4, d2_y_coord_offset - 2 );
								// previous_left_y_positive2:
								// if y change is not large enough
								y -= prev_y;// current y - previous y
								if( y < 0 ) {
									y = -y;
								}
								if( (y >= Road.STEEP) && (d1_section_x2 >= 4) ) {
									b_near_section_flags[ d1_section_x2 ] |= PIT;// store pit flag
								}
							}
						} else {
							// previous_left_y_positive2:
							// if y change is not large enough
							y -= prev_y;// current y - previous y
							if( y < 0 ) {
								y = -y;
							}
							if( (y >= Road.STEEP) && (d1_section_x2 >= 4) ) {
								b_near_section_flags[ d1_section_x2 ] |= PIT;// store pit flag
							}
						}
					}
					// save_right_y_coord2
					// get current right y coordinate
					// y = w_overall_right_y_shift + (((ba5[ y_coord_offset ] & 0x7F) << 8) | (ba5[ y_coord_offset + 1 ] & 0xFF));
					y = w_overall_right_y_shift + Road.getCoordinateAsWord( ba5, d2_y_coord_offset );
					// save right y coordinate
					w_coord_y_values_fp5[ (d1_section_x2 + 2) / 2 ] = (short) y;
				}
				// next_two_y_coords2:
				b_current_y_coord_offset++;
				d1_section_x2 += 4;
			} while( d1_section_x2 < b_coords_of_last_section );
			if( d1_section_x2 != b_coords_of_last_section ) {
				// mntyc5:
				if( ba4[ d2_y_coord_offset ] < 0 ) {
					b_near_section_flags[ d1_section_x2 - 4 ] = (byte) 0x80;
					if( b_number_of_segments >= d7 ) {
						w_coord_y_values_fp5[ (d1_section_x2 - 4) / 2 ] = (short) 0x8000;
						w_coord_y_values_fp5[ (d1_section_x2 - 2) / 2 ] = (short) 0x8000;
					}
				}
				return;
			}
			d2_y_coord_offset = b_current_y_coord_offset << 1;
			j_is_nearest = true;
			// j_use_for_far_road_coords2 = false;
		} while( true );
		// goto make_top_y_coords2;
	}

	// fetch_near_xz_coords
	/**
	 * Get {@link #w_near_x_coord} and {@link #w_near_z_coord}
	 *
	 * @param d2 offset for coordinate pair (x 4) as word
	 * @return d2 as word
	 * {@link #w_near_x_coord} and {@link #w_near_z_coord}
	 */
	private static int fetchNearXZcoords(int d2) {
		// int d0 = w_piece_data_offset;
		// d0 = ((((d0 << 8) & 0xFF00) | ((d0 >> 8) & 0xFF)) - 0xb100) & 0xffff;
		final byte[] ba5 = w_piece_data_offset;// Road.offsetToPiece( d0 );
		if( d2 >= ba5.length ) {// java: TODO verify
			// Amiga reads 4 coordinates for a section,
			// but sometimes uses only 2 the closest coordinates in
			// {@link ##makeOptionalSideEdges}
			w_near_x_coord = 0;
			w_near_z_coord = 0;
			return d2 + 4;
		}
		final int x = Road.getSignedWordDataFromPiece( ba5, d2 );
		d2 += 2;
		final int z = Road.getSignedWordDataFromPiece( ba5, d2 );
		d2 += 2;
		if( 0 == (w_rough_difference_angle & Trigonometry.ANGLE_180_DEGREES) ) {
			if( 0 == (w_rough_difference_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				// Save X and Z
				w_near_x_coord = (short) (w_road_near_x_offset + x);
				w_near_z_coord = (short) (w_road_near_z_offset + z);
				return d2;
			}
			// Save Z and $800 -X
			w_near_z_coord = (short) (w_road_near_z_offset + x);
			w_near_x_coord = (short) (w_road_near_x_offset - z + 0x800);
			return d2;
		}
		if( 0 == (w_rough_difference_angle & Trigonometry.ANGLE_90_DEGREES) ) {
			// Save $800 -X and $800 -Z
			w_near_x_coord = (short) (w_road_near_x_offset - x + 0x800);
			w_near_z_coord = (short) (w_road_near_z_offset - z + 0x800);
			return d2;
		}
		// Save $800 - Z and X
		w_near_z_coord = (short) (w_road_near_z_offset - x + 0x800);
		w_near_x_coord = (short) (w_road_near_x_offset + z);
		return d2;
	}

	// set_visible_range_data
	private static void setVisibleRangeData() {
		for( int i = b_number_of_road_pieces; --i >= 0; ) {
			b_road_visible_range_table[ i ] = DEFAULT_VISIBLE_RANGE;
		}
		int offset = b_road_id << 3;
		int i = 3;
		do {
			final int j = Road.b_default_visible_range_for_tracks[ offset ];
			if( j < 0 ) {
				return;
			}
			b_road_visible_range_table[ j ] = Road.b_default_visible_range_for_tracks[ offset + 1 ];
			offset += 2;
		} while( --i >= 0 );
	}

	// set_road_data2
	/**
	 * sa6 = {@link #w_coord_y_values_fp5}
	 */
	private static void setRoadData2() {
		setVisibleRangeData();

		lw_end_far_sections_ptr = 0;// offset to far_section_coordinates;
		b_coord_offset_zero_or_four = 0;
		int d1_piece = 0;

		// srd21:
		do {
			b_current_road_piece = (byte) d1_piece;
			lw_far_section_ptrs[ d1_piece ] = lw_end_far_sections_ptr;

			// FIXME is this required?
			// toNextRoadSection, toPreviousRoadSection, fetchNearSectionStuff
			// does not use this flag
			w_current_far_section_flag = 0;
			d1_piece = toNextRoadPiece();
			fetchNearSectionStuff( d1_piece );

			if( b_curved_section < 0 ) {
				w_current_far_section_flag = (short) (0x4000 | (w_current_far_section_flag & 0xFF));
				if( 0 != (((w_curve_to_left >> 8) ^ b_plus_180_degrees) & 0x80) ) {
					w_current_far_section_flag = (short) (0x8000 | (w_current_far_section_flag & 0xFF));
				}
			}
			// srd22:
			d1_piece = toPreviousRoadPiece();
			fetchNearSectionStuff( d1_piece );
			w_near_sections_done2 = 0;
			// TODO verify low byte
			w_rough_difference_angle = (short) (((-((w_rough_piece_angle >> 8) & 0xFF)) << 8)
					| (w_rough_difference_angle & 0xFF));
			makeNearTopYcoords();

			int coord_offset_x2 = b_coord_offset_zero_or_four;
			int sa4i = lw_end_far_sections_ptr;// far_section_coordinates[ sa4i ]
			// int d0 = w_piece_data_offset;
			// d0 = ((((d0 << 8) & 0xFF00) | ((d0 >> 8) & 0x00FF)) - 0xb100) & 0xffff;
			// final byte[] ba0 = w_piece_data_offset;// Road.offsetToPiece( d0 );

			int offset_for_coords = w_piece_data_offset[ 0 ] + 7;
			if( b_plus_180_degrees < 0 ) {
				offset_for_coords += (b_number_of_sections_x2 - 1) << 2;
			}
			b_piece_coords_offset = (byte) offset_for_coords;

			// srd24:
			do {
				int flags = b_near_section_flags[ coord_offset_x2 ];
				if( 0 != (flags & 0x80) ) {
					coord_offset_x2 += 4;
				} else {
					b_near_section_flags[ coord_offset_x2 ] = (byte) 0x80;
					if( coord_offset_x2 < 4 ) {
						coord_offset_x2 += 4;
					} else {
						if( coord_offset_x2 < b_coords_of_last_section ) {
							if( (0 == (b_near_section_flags[ b_coords_of_last_section ] & 0x80))
									|| (coord_offset_x2 != (b_coords_of_last_section - 4)) ) {
								// srd25:
								if( 0 == (b_curved_section & 0x80) ) {// straight
									if( (b_left_y_coordinate_ids[ b_current_road_piece ] != 37)
											|| ((12 * 2) != coord_offset_x2) ) {
										int d3_y = w_coord_y_values_fp5[ coord_offset_x2 / 2 ] >>> 1;
										d3_y -= (w_coord_y_values_fp5[ (coord_offset_x2 - 4) / 2 ]
												+ w_coord_y_values_fp5[ (coord_offset_x2 + 4) / 2 ]) >>> 2;
										if( d3_y < 0 ) {
											d3_y = -d3_y;
										}
										if( d3_y < 80 ) {
											coord_offset_x2 += 4;
											continue;// java bra srd2d;
										}
									}
								}
							}
						}
						// srd28:
						flags = (flags & 0x3F) << 8;
						if( 0 == (b_curved_section & 0x80) ) {// straight
							flags |= w_current_far_section_flag;
						}
						// srd29:
						flags = (flags & 0xFF00) | (b_current_road_piece & 0xFF);
						w_far_section_coordinates[ sa4i / 2 ] = (short) flags;
						sa4i += 2;
						// srd2a:
						do {// left/right x,z,y
							int offset_for_coord_pair;
							if( 0 != (b_plus_180_degrees & 0x80) ) {
								offset_for_coord_pair = b_piece_coords_offset - (coord_offset_x2 << 1);
							} else {
								offset_for_coord_pair = b_piece_coords_offset + (coord_offset_x2 << 1);
							}
							fetchNearXZcoords( offset_for_coord_pair );
							w_far_section_coordinates[ sa4i / 2 ] = w_near_x_coord;
							sa4i += 2;
							w_far_section_coordinates[ sa4i / 2 ] = w_near_z_coord;
							sa4i += 2;
							w_far_section_coordinates[ sa4i / 2 ] = w_coord_y_values_fp5[ coord_offset_x2 / 2 ];
							sa4i += 2;
							coord_offset_x2 += 2;
						} while( 0 != (coord_offset_x2 & 0x02) );
					}// else if( coord_offset_x2 < 4 ) {
				} // else if( 0 != (flags & 0x80) )
				// srd2d:
			} while( coord_offset_x2 != b_number_of_coords );
			lw_end_far_sections_ptr = sa4i;
			d1_piece = b_current_road_piece;

		} while( ++d1_piece < b_number_of_road_pieces );
	}

	// make_road_under_map
	private static void makeRoadUnderMap() {
		Arrays.fill( b_road_under_map, (byte) 0xFF );
		byte d1 = 0;
		do {
			b_road_under_map[ b_road_piece_xz_positions[ d1 ] & 0xFF ] = d1;
		} while( ++d1 < b_number_of_road_pieces );
	}

	// move_draw_bridge
	private static void moveDrawBridge() {
		if( Road.TRACK_DRAW_BRIDGE != b_road_id ) {
			return;
		}
		int d0;
		if( ((b_players_road_piece < 56) && (b_players_road_piece >= 51))
				||
				((b_opponents_road_piece < 56) && (b_opponents_road_piece >= 51))
				||
				((0 != b_1bb3f) && (b_opponents_road_piece >= 48)) ) {
			// do not move mdb3:
			d0 = 12;
			b_1bb3f = (byte) d0;
			d0 += b_draw_bridge_frame_count;
		} else {
			// move mdb4:
			if( b_fourteen_frames_elapsed >= 0 ) {
				++b_draw_bridge_frame_count;
			}
			w_pos_difference_angle = 0;
			b_1bb3f = 0;
			d0 = (b_draw_bridge_frame_count & 0x1F) - 16;
			if( d0 < 0 ) {
				d0 = ~d0;// -d0 - 1
			}
			int d2 = d0;
			d0 += 4;
			b_opponents_speed_values[ 52 ]
					= b_opponents_speed_values[ 51 ]
					= Road.b_draw_bridge_speed_values[ d2 ];
			d0 <<= 5;
			w_near_x_coord = (short) d0;
			d2 = 2;
			w_road_data_offset = Road.w_y_coordinate_offsets[ 190 / 2 ];
			// move.l	#y_coordinate_offsets,a1
			// d0 = w_road_data_offset;
			// d0 = ((((d0 & 0xFF) << 8) | ((d0 & 0xFF00) >>> 8)) - 0xb100) & 0xffff;
			final byte[] ba0 = w_road_data_offset;// Road.offsetToNearPieceYdataBytes( d0 );
			int d1 = 15;
			// mdb7:
			do {
				d0 = w_pos_difference_angle + w_near_x_coord;
				w_pos_difference_angle = (short) d0;
				do {
					d0 = (w_pos_difference_angle >> 8);
					if( 32 == d2 ) {
						d0 |= 0x80;// do not use for far coordinates
					}
					/* Amiga code
					final int sp1 = d0;
					ba0[ d2 ] = (byte) d0;
					++d2;
					ba0[ d2 ] = (byte) w_pos_difference_angle;
					++d2;

					int value = d2;
					d2 = 72 - value;// 18 bytes in the piece, 72 - 4
					ba0[ d2 ] = (byte) sp1;
					++d2;
					ba0[ d2 ] = (byte) w_pos_difference_angle;

					d2 = value;
					*/
					// java code, TODO clean the code
					final int sp1 = d0;
					int offset = d2;
					byte[] p = ba0;
					if( offset >= ba0.length ) {
						offset -= ba0.length;
						p = Road.w_y_coordinate_offsets[ 96 ];
					}
					p[ offset ] = (byte) d0;
					++d2;

					p = ba0;
					offset = d2;
					if( offset >= ba0.length ) {
						offset -= ba0.length;
						p = Road.w_y_coordinate_offsets[ 96 ];
					}
					p[ offset ] = (byte) w_pos_difference_angle;
					++d2;
					// find the opposite piece
					offset = 72 - d2;
					int piece = 95;
					do {
						p = Road.w_y_coordinate_offsets[ piece ];
						if( p.length > offset ) {
							p[ offset ] = (byte) sp1;
							if( ++offset >= p.length ) {
								offset -= p.length;
								p = Road.w_y_coordinate_offsets[ ++piece ];
							}
							p[ offset ] = (byte) w_pos_difference_angle;
							break;
						}
						offset -= p.length;
						++piece;
					} while( true );
					/* */
				} while( 18 == d2 );
			} while( --d1 >= 0 );
			if( b_draw_bridge_make_far_coords ) {
				final int sa1i = lw_far_section_ptrs[ 51 ];
				// d0 = ((ba0[ 16 ] << 8) | (ba0[ 19 ] & 0xFF)) & 0x7FFF;
				d0 = ((ba0[ 16 ] << 8) | (Road.w_y_coordinate_offsets[ 96 ][ 19 - ba0.length ] & 0xFF)) & 0x7FFF;
				d0 += w_overall_left_y_shifts[ 51 ];// left
				w_far_section_coordinates[ (sa1i + 2 + 4) / 2 ] = (short) d0;// left
				w_far_section_coordinates[ (sa1i + 2 + 10) / 2 ] = (short) d0;// right
				w_far_section_coordinates[ (sa1i + 5 * 7 * 2 + 2 + 4) / 2 ] = (short) d0;
				w_far_section_coordinates[ (sa1i + 5 * 7 * 2 + 2 + 10) / 2 ] = (short) d0;
				// d0 = ((ba0[ 32 ] << 8) | (ba0[ 33 ] & 0xFF)) & 0x7FFF;
				// ba0 is [ 95 ]
				d0 = ((Road.w_y_coordinate_offsets[ 96 ][ 32 - ba0.length ] << 8) | (Road.w_y_coordinate_offsets[ 96 ][ 33 - ba0.length ] & 0xFF)) & 0x7FFF;
				d0 += w_overall_left_y_shifts[ 52 ];// right
				w_far_section_coordinates[ (sa1i + 7 * 2 + 2 + 4) / 2 ] = (short) d0;
				w_far_section_coordinates[ (sa1i + 7 * 2 + 2 + 10) / 2 ] = (short) d0;
				w_far_section_coordinates[ (sa1i + 4 * 7 * 2 + 2 + 4) / 2 ] = (short) d0;
				w_far_section_coordinates[ (sa1i + 4 * 7 * 2 + 2 + 10) / 2 ] = (short) d0;
			}
			// mdba
			b_draw_bridge_make_far_coords = true;
			if( 47 != b_opponents_road_piece ) {
				return;
			}
			d0 = b_draw_bridge_frame_count;
		}
		// mdbb:
		final int d2 = (d0 & 0x1F) >>> 1;
		int d1 = 0;
		d0 = 0xC6;// base speed
		do {
			d0 += Road.b_draw_bridge_opponents_speed_changes[ d2 ];
			b_opponents_speed_values[ 48 + d1 ] = (byte) d0;
		} while( 3 != ++d1 );
	}

	// get_valid_map_number
	/**
	 * Input: {@link #b_players_map_z}, {@link #b_map_z_shift}
	 * {@link #b_players_map_x}, {@link #b_map_x_shift}
	 *
	 * @return a byte with x and z coordinates. XXXX ZZZZ
	 * < 0 means cs, off-road
	 * > 0 means cc. Lowest 8 bits are d0 byte.
	 * d1 is returned as the 1st byte.
	 * Use instead later:
	 * <pre>
	 * 	bd1 = ((b_players_map_x + b_map_x_shift) & 0x0F)
	 * 		| (((b_players_map_z + b_map_z_shift) << 4) & 0xF0);
	 * </pre>
	 */
	private static int getValidMapNumber() {
		int bd0 = b_players_map_z + b_map_z_shift;
		if( 0 != (bd0 & 0xF0) ) {
			return -bd0;
		}
		bd0 <<= 4;
		int value = bd0;
		bd0 = b_players_map_x + b_map_x_shift;
		if( 0 != (bd0 & 0xF0) ) {
			return -bd0;
		}
		bd0 &= 0x0F;
		bd0 |= value;
		return (bd0 << 8) | (b_road_under_map[ bd0 ] & 0xFF);
	}

	// to_closest_adjacent_map_square
	/**
	 * @param bd0_x players_fine_map_x
	 * @param bd1
	 * @param bd2_z players_fine_map_z
	 * @return bd0. Lowest 8 bits. Others are 0.
	 */
	private static int toClosestAdjacentMapSquare(int bd0_x, int bd1, int bd2_z) {
		// d1 is from get_valid_map_number
		// int bd1 = ((b_players_map_x + b_map_x_shift) & 0x0F)
		// 		| (((b_players_map_z + b_map_z_shift) << 4) & 0xF0);

		b_value_low_factor_high = (byte) bd0_x;
		b_factor_high_value_low = (byte) bd2_z;
		bd0_x &= 0xFF;
		bd2_z &= 0xFF;
		if( bd2_z < bd0_x ) {
			bd0_x += b_factor_high_value_low & 0xFF;
			if( bd0_x >= 256 ) {
				bd0_x = bd1 & 0x0F;
				if( 0x0F == bd0_x ) {
					return 0xFF;
				}
				++bd1;
			} else {
				bd0_x = bd1 & 0xF0;
				if( 0 == bd0_x ) {
					return 0xFF;
				}
				bd1 -= 0x10;
			}
		} else {
			bd0_x += b_factor_high_value_low & 0xFF;
			if( bd0_x >= 256 ) {
				bd0_x = bd1 & 0xF0;
				if( 0xF0 == bd0_x ) {
					return 0xFF;
				}
				bd1 += 0x10;
			} else {
				bd0_x = (bd1 & 0x0F);
				if( 0 == bd0_x ) {
					return 0xFF;
				}
				--bd1;
			}
		}
		return b_road_under_map[ bd1 & 0xFF ] & 0xFF;
	}

	// copy_coords_for_next_section
	private static void copyCoordsForNextSection() {
		int d1_number_of_coords = b_number_of_coords;
		if( b_offset_for_last_two_coords >= 0 ) {
			d1_number_of_coords = b_offset_for_last_two_coords + 4;
		}
		if( b_dont_copy_coords ) {
			b_coord_offset_zero_or_four = 0;// not used
			d1_number_of_coords -= 4;
			return;// TODO verify (x_values[ d1 / 2 ] - 256);// max x
		}
		int d2 = 2;
		do {
			d1_number_of_coords -= 2;
			w_screen_x_coords[ d2 / 2 ] = w_screen_x_coords[ d1_number_of_coords / 2 ];
			w_screen_y_coords[ d2 / 2 ] = w_screen_y_coords[ d1_number_of_coords / 2 ];
			w_coord_y_values_fp5[ d2 / 2 ] = w_coord_y_values_fp5[ d1_number_of_coords / 2 ];

			w_screen_x_coords[ (d2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = w_screen_x_coords[ (d1_number_of_coords + OFFSET_FOR_BOTTOM_X2) / 2 ];
			w_screen_y_coords[ (d2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = w_screen_y_coords[ (d1_number_of_coords + OFFSET_FOR_BOTTOM_X2) / 2 ];
			w_coord_y_values_fp5[ (d2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = w_coord_y_values_fp5[ (d1_number_of_coords + OFFSET_FOR_BOTTOM_X2) / 2 ];

			d2 -= 2;
		} while( d2 >= 0 );
		b_edge_offset_left_x2 = 0;
		b_edge_offset_right_x2 = 2;
		// return;// TODO verify (x_values[ d1 / 2 ] - 256);// max x
	}

	// to_next_road_piece
	/**
	 * @return current road section in bd1 and in {@link #b_current_road_piece}
	 */
	private static int toNextRoadPiece() {
		int next_piece = b_current_road_piece;
		if( ++next_piece >= b_number_of_road_pieces ) {
			next_piece = 0;
		}
		b_current_road_piece = (byte) next_piece;
		return next_piece;
	}

	// to_previous_road_piece
	/**
	 * @return current road section in bd1 and in {@link #b_current_road_piece}
	 */
	private static int toPreviousRoadPiece() {
		int previous_piece = b_current_road_piece;
		if( previous_piece <= 0 ) {
			previous_piece = b_number_of_road_pieces;
		}
		--previous_piece;
		b_current_road_piece = (byte) previous_piece;
		return previous_piece;
	}

	// calculate_wheel_xz_offsets
	private static void calculateWheelXZoffsets() {
		int d4 = ((w_sin_cos_values[ 62 / 2 ] >> 1) - (w_sin_cos_values[ 52 / 2 ] >> 1)) >> 5;// FIXME should be add.w ?
		int d5 = ((w_sin_cos_values[ 56 / 2 ] >> 1) - (w_sin_cos_values[ 66 / 2 ] >> 1)) >> 5;
		int d0 = (w_sin_cos_values[ 68 / 2 ] >> 5);
		int d3 = (w_sin_cos_values[ 70 / 2 ] >> 5);

		// rear_wheel_x_offset = (short)-d0;
		// rear_wheel_z_offset = (short)-d3;
		w_wheel_x_offsets[ REAR_WHEEL ] = (short) -d0;
		w_wheel_z_offsets[ REAR_WHEEL ] = (short) -d3;

		// front_right_wheel_x_offset = front_left_wheel_x_offset = (short)d0;
		// front_right_wheel_z_offset = front_left_wheel_z_offset = (short)d3;
		w_wheel_x_offsets[ RIGHT_FRONT_WHEEL ] = w_wheel_x_offsets[ LEFT_FRONT_WHEEL ] = (short) d0;
		w_wheel_z_offsets[ RIGHT_FRONT_WHEEL ] = w_wheel_z_offsets[ LEFT_FRONT_WHEEL ] = (short) d3;

		// front_left_wheel_x_offset -= d4;
		// front_left_wheel_z_offset -= d5;
		w_wheel_x_offsets[ LEFT_FRONT_WHEEL ] -= d4;
		w_wheel_z_offsets[ LEFT_FRONT_WHEEL ] -= d5;

		// front_right_wheel_x_offset += d4;
		// front_right_wheel_z_offset += d5;
		w_wheel_x_offsets[ RIGHT_FRONT_WHEEL ] += d4;
		w_wheel_z_offsets[ RIGHT_FRONT_WHEEL ] += d5;
	}

	// to_appropriate_road_section
	private static void toAppropriateRoadSection() {
		// Player has crossed the start or end of the road section.
		int d0 = w_surface_z_position >> 8;
		int d3 = b_plus_180_degrees;
		d0 ^= d3;
		if( 0 != (d0 & 0x80) ) {
			// crossed_section_start
			final int d1 = toPreviousRoadPiece();
			fetchNearSectionStuff( d1 );
			if( b_plus_180_degrees >= 0 ) {
				// last_section_surface
				b_surface_start_coord_x2 = (byte) (b_number_of_sections_x2 - 4);
				if( w_surface_z_position < 0 ) {
					return;
				}
			} else {
				// first_section_surface
				b_surface_start_coord_x2 = 0;
				if( w_surface_z_position >= 0 ) {
					return;// bpl same_section_orientation;
				}
			}
		} else {
			// crossed_section_end
			final int d1 = toNextRoadPiece();
			fetchNearSectionStuff( d1 );
			if( b_plus_180_degrees < 0 ) {
				// last_section_surface
				b_surface_start_coord_x2 = (byte) (b_number_of_sections_x2 - 4);
				if( w_surface_z_position < 0 ) {
					return;
				}
			} else {
				// first_section_surface
				b_surface_start_coord_x2 = 0;
				if( w_surface_z_position >= 0 ) {
					return;// bpl same_section_orientation;
				}
			}
		}
		// If the starts or ends of the old and new sections meet each other, rather
		// than the old end meeting the new start (or vice versa).

		// opposite_section_orientation
		w_surface_z_position = (short) ((w_surface_z_position & 0xFF00)
				| ((-((byte) w_surface_z_position)) & 0xFF));
		if( 0 == (w_surface_z_position & 0xFF) ) {
			w_surface_z_position |= 0xFF;
		}
		w_surface_x_position = (short) ((w_surface_x_position & 0xFF00)
				| ((-((byte) w_surface_x_position)) & 0xFF));
		if( 0 == (w_surface_x_position & 0xFF) ) {
			w_surface_x_position |= 0xFF;
		}
		// same_section_orientation
	}

	// calculate_if_car_off_road
	private static void calculateIfCarOffRoad() {
		// Calculate how far the current wheel is off the left or right of the road
		int d0 = w_wheel_road_x_position;
		if( d0 >= 0 ) {
			d0 = Road.ROAD_WIDTH - w_wheel_road_x_position;
		}
		if( d0 < 0 ) {
			d0 = -d0;
		}
		// If the wheel is more than $30 off the road then the car is off the road
		// cior2
		if( d0 <= 0x30 ) {
			// Otherwise use the amount the wheel is off the road to drop the height of
			// the wheel - to make the car fall gradually off the edge.
			// If the new wheel height is less than $1000 then the car is off the road

			d0 &= 0xFF;
			d0 <<= 4;
			int d3 = lw_road_height_fp8 - d0 - 0x100;
			if( d3 >= 0x1000 ) {
				lw_road_height_fp8 = d3;// save reduced wheel height

				// Also store which side the car is falling off
				d3 = w_wheel_road_x_position >> 8;
				d0 = (d3 ^ b_plus_180_degrees) & 0x80;
				if( 0 == d0 ) {
					d0 = 0x40;
				}
				// left.side
				b_which_side_byte = (byte) d0;// should perhaps instead be called 'car_on_edge'
				return;
			}
		}
		// signal_car_is_off_road
		lw_road_height_fp8 = 0x1000;
		d0 = ((b_at_side_byte & 0xFF) >>> 1) | 0x80;// another wheel is off the road
		b_at_side_byte = (byte) d0;
	}

	// calculate_road_wheel_height
	/**
	 * @param d1_wheel_index_x2 (wheel index * 2)
	 */
	private static void calculateRoadWheelHeight(int d1_wheel_index_x2) {
		calculateRoadSurfaceHeight();

		// d1 <<= 1;
		// move.l	#front.left.road.height,a3
		if( b_wheel_off_road ) {
			calculateIfCarOffRoad();
		}
		b_wheel_off_road = false;
		// wheel_not_off_road2
		if( w_pos_players_z_speed < 10 * 256 ) {
			// speed_not_big_enough
			int d0_abs_angle = w_players_x_angle;
			if( d0_abs_angle < 0 ) {
				d0_abs_angle = -d0_abs_angle;
			}
			if( d0_abs_angle <= Trigonometry.ANGLE_7_DEGREES ) {// if |x angle| > $500 degrees
				// Otherwise save the average of the calculated wheel height and the old wheel
				// height.  (This is for when the car is being lowered onto the road.)
				// java: all these values are positive, so there is no reason to convert them into long for the math
				switch( d1_wheel_index_x2 ) {
					case 0:
						lw_front_left_road_height_fp8 = ((lw_front_left_road_height_fp8 + lw_road_height_fp8) >>> 1);
						break;
					case 2:
						lw_front_right_road_height_fp8 = ((lw_front_right_road_height_fp8 + lw_road_height_fp8) >>> 1);
						break;
					case 4:
						lw_rear_road_height_fp8 = ((lw_rear_road_height_fp8 + lw_road_height_fp8) >>> 1);
						break;
					default:
						break;
				}
				return;
			}
		}
		// Save calculated wheel height if speed is greater than $a00 or car is at an
		// x angle of more than $500 degrees forwards or backwards.
		// save_wheel_height
		switch( d1_wheel_index_x2 ) {
			case 0:
				lw_front_left_road_height_fp8 = lw_road_height_fp8;
				break;
			case 2:
				lw_front_right_road_height_fp8 = lw_road_height_fp8;
				break;
			case 4:
				lw_rear_road_height_fp8 = lw_road_height_fp8;
				break;
			default:
				break;
		}
	}

	// calculate_road_wheel_heights
	private static void calculateRoadWheelHeights() {
		b_current_road_piece = b_players_road_piece;
		fetchNearSectionStuff( b_current_road_piece );
		b_at_side_byte = 0;
		int d1_wheel_x2 = 4;// rear-right-left
		// next_wheel
		do {
			if( b_players_road_piece != b_current_road_piece ) {
				b_current_road_piece = b_players_road_piece;
				fetchNearSectionStuff( b_players_road_piece );
			}
			// no_section_change
			b_factor_high_value_low = b_road_width_reduction;

			int d0 = w_players_road_x_position + (w_wheel_x_offsets[ d1_wheel_x2 / 2 ] >> 4);// x offset for current wheel
			if( (d0 < 0) || (d0 >= Road.ROAD_WIDTH) ) {
				b_wheel_off_road = true;
				w_wheel_road_x_position = (short) d0;
				if( d0 >= 0 ) {
					// wheel_off_right
					d0 = 255;
				} else {
					// wheel_off_left
					d0 = 0;
				}
			} else {
				// wheel_not_off_road
				// Reduce wheel_road_x_position from (0 - road width) to (0 - 255) and save it
				// for use by the height calculate routine, which can only take (0 - 255).
				if( d0 < 0 ) {
					d0 = -d0;
				}
				d0 = ((d0 * ((b_factor_high_value_low << 7) & 0x7FFF)) << 1) >> 16;// value is now reduced
				if( d0 > 255 ) {
					d0 = 255;// FIXME BUG - didn 't have the # infront
				}
			}
			// save_surface_x
			w_surface_x_position = (short) (d0 | (w_surface_x_position & 0xFF00));
			if( b_plus_180_degrees < 0 ) {
				d0 ^= 0xFF;// -d0 - 1
			}
			if( 2 * REAR_WHEEL == d1_wheel_x2 ) {
				b_rear_wheel_surface_x_position = (byte) d0;
			}
			// not_rear_wheel

			// Reduce the wheel road z position in a similar way to above.
			d0 = w_wheel_z_offsets[ d1_wheel_x2 / 2 ] >> 3;// z offset for current wheel
			d0 = ((d0 * ((b_road_length_reduction << 7) & 0x7FFF)) << 1) >> 16;// value is now reduced
			d0 += w_normal_distance_into_section;
			w_surface_z_position = (short) d0;

			// this byte is segment number
			// doubling gives coordinates to skip
			b_surface_start_coord_x2 = (byte) ((w_surface_z_position >> 8) << 1);
			if( (b_surface_start_coord_x2 < 0) || (b_surface_start_coord_x2 >= b_number_of_sections_x2_minus2) ) {
				// section_change
				toAppropriateRoadSection();
			}
			// no_section_change2
			// d0 = w_left_y_coord_offset;
			// d0 = ((((d0 >> 8) & 0xFF) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			final byte[] ba4 = w_left_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

			// d0 = w_right_y_coord_offset;
			// d0 = ((((d0 >> 8) & 0xFF) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			final byte[] ba5 = w_right_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

			if( b_plus_180_degrees >= 0 ) {
				// start_with_first_coords
				int d1_surface_coord = b_surface_start_coord_x2;
				w_surface_y_coord1 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
				++d1_surface_coord;

				w_surface_y_coord2 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
				++d1_surface_coord;

				w_surface_y_coord3 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
				++d1_surface_coord;

				w_surface_y_coord4 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
			} else {
				// start_with_last_coords
				int d1_surface_coord = b_number_of_sections_x2 - b_surface_start_coord_x2 - 4;
				w_surface_y_coord4 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
				++d1_surface_coord;

				w_surface_y_coord3 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
				++d1_surface_coord;

				w_surface_y_coord2 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
				++d1_surface_coord;

				w_surface_y_coord1 = (short) fetchSurfaceYcoord( ba4, ba5, d1_surface_coord );
			}
			// got_four_surface_coords
			// d1_wheel_x2 = near_z_coord_1;// java is not required
			calculateRoadWheelHeight( d1_wheel_x2 );
			d1_wheel_x2 -= 2;
		} while( d1_wheel_x2 >= 0 );
	}

	// calculate_actual_wheel_heights
	private static void calculateActualWheelHeights() {
		final int d3 = ((int) Trigonometry.getSin( w_players_x_angle )) << 4;// 16 sin x
		final int d0 = ((int) Trigonometry.getSin( w_players_z_angle )) << 3;// 8 sin z
		lw_rear_actual_height_fp8 = (lw_players_world_y_fp16 - d3) >> 8;

		final int d4 = lw_players_world_y_fp16 + d3;
		lw_front_right_actual_height_fp8 = (d4 - d0) >> 8;
		lw_front_left_actual_height_fp8 = (d4 + d0) >> 8;
	}

	// calculate_xz_speeds
	private static void calculateXZspeeds() {
		// Saves player's actual X and Z speeds by rotating the world speed values.
		int d5 = sinCosMult( w_players_world_x_speed,
				b_sin_cos_offsets[ 2 ] );// add X component
		d5 += sinCosMult( w_players_world_y_speed,
				b_sin_cos_offsets[ 2 + 3 ] );// add Y component
		d5 += sinCosMult( w_players_world_z_speed,
				b_sin_cos_offsets[ 2 + 6 ] );// add Z component
		w_players_z_speed = (short) d5;

		d5 = sinCosMult( w_players_world_x_speed,
				b_sin_cos_offsets[ 0 ] );// add X component
		d5 += sinCosMult( w_players_world_y_speed,
				b_sin_cos_offsets[ 0 + 3 ] );// add Y component
		d5 += sinCosMult( w_players_world_z_speed,
				b_sin_cos_offsets[ 0 + 6 ] );// add Z component
		w_players_x_speed = (short) d5;
	}

	// set_wheel_rotation_speed
	private static void setWheelRotationSpeed() {
		int d0 = w_players_z_speed;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		w_pos_players_z_speed = (short) d0;

		if( !b_touching_road ) {
			// Not touching road, so reduce wheel speed by one quarter
			w_wheel_rotation_speed -= w_wheel_rotation_speed >>> 2;
			return;
		}
		// Touching road
		if( d0 < 0x800 ) {
			// Less than $800, so multiply by 8 and use as wheel speed
			w_wheel_rotation_speed = (char) (d0 << 3);
			return;
		}
		// Otherwise double it, add $3000 and use as wheel speed
		d0 = 0x3000 + (d0 << 1);
		if( d0 >= 0x1_0000 ) {
			d0 = 0xff00;// set to maximum value
		}
		w_wheel_rotation_speed = (char) d0;
	}

	// sin_cos_mult
	/**
	 * @param d0 as word
	 * @param d1 index for {@link #w_sin_cos_values}.
	 * @return d0 as word
	 */
	private static int sinCosMult(int d0, int d1) {
		return ((d0 * w_sin_cos_values[ d1 ]) << 1) >> 16;
	}

	// mult_weight
	/**
	 * @param d1 index for {@link #w_sin_cos_values}.
	 * @return d0 as word
	 */
	private static int multWeight(final int d1) {
		return sinCosMult( CAR_WEIGHT, d1 );
	}

	// mult_minus_weight
	/**
	 * @param d1 index for {@link #w_sin_cos_values}.
	 * @return d0 as word
	 */
	private static int multMinusWeight(final int d1) {
		return sinCosMult( -CAR_WEIGHT, d1 );
	}

	// calculate_gravity_acceleration
	/**
	 * Gravity acts on the Y axis only.
	 * Therefore, only Y components are used.
	 * X acceleration = weight * cos x * sin z
	 * Y acceleration = -weight * cos x * cos z
	 * Z acceleration = -weight * sin x
	 */
	private static void calculateGravityAcceleration() {
		w_gravity_y_acceleration = (short) multMinusWeight( COS_X_COS_Z );
		w_gravity_z_acceleration = (short) multMinusWeight( SIN_X );
		w_gravity_x_acceleration = (short) multWeight( COS_X_SIN_Z );
	}

	// calculate_inclination_sin_cos
	/**
	 * @param d0_sin is effectively the sin of the inclination angle, as word
	 */
	private static void calculateInclinationSinCos(int d0_sin) {
		if( d0_sin < 0 ) {
			d0_sin = -d0_sin;
		}
		int d1 = 0xff;// set maximum inclination
		if( d0_sin < 256 ) {
			d1 = d0_sin;// use inclination if it is not too large
		}
		b_inclination_sin_fp8 = (byte) d1;
		// 128 values in the table
		b_inclination_cos_fp8 = Trigonometry.b_cosine_conversion_table_fp8[ d1 >>> 1 ];
	}

	// calculate_car_collision_acceleration
	/**
	 * average.amount.below.road is the force exerted by the road on the car.
	 * Force is directed through the Y axis of the road surface.
	 * Therefore, only Y components are used.
	 * X acceleration = force * -cosx.sinz
	 * Y acceleration = force * cosx.cosz
	 * Z acceleration = force * sinx
	 */
	private static void calculateCarCollisionAcceleration() {
		w_y_inclination_to_road = 0;
		// Above value is zero because road exists in X and Z planes only.
		// average front height difference
		int d0 = (((lw_front_left_height_difference + lw_front_right_height_difference) >> 1)
				- lw_rear_height_difference) >> 4;// X gradient to road surface

		w_x_inclination_to_road = (short) (d0 ^ 0x8000);

		// Calculate sin and cos of X angle between car and road surface
		calculateInclinationSinCos( d0 );

		b_surface_cosx_fp8 = b_inclination_cos_fp8;
		b_surface_sinx_fp8 = b_inclination_sin_fp8;
		// Z gradient to road surface
		d0 = (lw_front_left_height_difference - lw_front_right_height_difference) >> 3;
		w_z_inclination_to_road = (short) d0;

		// Calculate sin and cos of Z angle between car and road surface
		calculateInclinationSinCos( d0 );

		b_surface_cosx_cosz_fp8 = (byte) (((b_surface_cosx_fp8 & 0xFF) * (b_inclination_cos_fp8 & 0xFF)) >>> 8);

		// surface_sinz
		b_surface_cosx_sinz_fp8 = (byte) (((b_surface_cosx_fp8 & 0xFF) * (b_inclination_sin_fp8 & 0xFF)) >>> 8);

		//*** Calculate car collision X acceleration ***
		int d3 = b_surface_cosx_sinz_fp8 & 0xFF;
		if( w_z_inclination_to_road < 0 ) {
			d3 = -d3;
		}
		w_car_collision_x_acceleration = (short) (((w_average_amount_below_road * (d3 << 7)) << 1) >> 16);

		//*** Calculate car collision Y acceleration ***
		d3 = b_surface_cosx_cosz_fp8 & 0xFF;
		if( w_y_inclination_to_road < 0 ) {
			d3 = -d3;
		}
		w_car_collision_y_acceleration = (short) (((w_average_amount_below_road * (d3 << 7)) << 1) >> 16);

		//*** Calculate car collision Z acceleration ***
		d3 = b_surface_sinx_fp8 & 0xFF;
		if( w_x_inclination_to_road < 0 ) {
			d3 = -d3;
		}
		w_car_collision_z_acceleration = (short) (((w_average_amount_below_road * (d3 << 7)) << 1) >> 16);
	}

	// multiplayer_set_car_position
	private static void multiplayerSetCarPosition() {
		if( (0 != b_machine) && b_drop_start_done ) {
			int d0 = b_1bbe0;
			if( 0 != (d0 & 0x80) ) {
				d0 <<= 1;
				if( b_players_road_piece == b_opponents_road_piece ) {
					if( b_swing_from_left == ((d0 & 0x80) != 0) ) {
						b_swing_from_left = !b_swing_from_left;
					}
				}
			}
		}
	}

	// swing_car
	/**
	 * @param d0 signed adjustment, as byte
	 * @return {@code true} swing magnitude reached, z
	 */
	private static boolean swingCar(int d0) {
		int d4 = 16;
		if( b_swing_from_left ) {
			d0 = -d0;
			d4 = -16;
		}
		d0 = (REDUCTION * (d0 << 8)) >> 8;
		final boolean is_magnitude_reached = ((w_swing_magnitude >> 8) == d4);
		if( !is_magnitude_reached ) {
			w_swing_magnitude += d0;// reduce amount of swing
		}
		// magnitude_reached
		w_players_z_angle = (short) (w_swing_magnitude - (w_players_x_offset_from_road_centre << 5));
		w_overall_difference_below_road = 0;
		return is_magnitude_reached;
	}

	// raise_car_off_ground
	/**
	 * @param d0_height_overhang as byte
	 * @return d0
	 */
	private static int raiseCarOffGround(int d0_height_overhang) {
		int d3 = w_players_smaller_y - w_required_raise_height - (d0_height_overhang << 8);
		d0_height_overhang = (d3 >> 3) - 256;
		if( d0_height_overhang < -512 ) {// 65024 ?
			d0_height_overhang = -512;
		}
		// less_negative
		w_car_collision_y_acceleration -= d0_height_overhang;
		return 2 + ((d3 >>> 8) & 0xFF);
	}

	// lift_car_onto_track
	private static void liftCarOntoTrack() {
		int d1 = b_car_on_chains_countdown & 0xFF;
		if( 0 == d1 ) {
			return;
		}
		if( d1 >= ON_CHAIN_STAGE_SET_FIXED_HEIGHT ) {
			multiplayerSetCarPosition();
			int d0 = 44;// 44 * 256
			if( b_swing_from_left ) {
				d0 = -44;// -44 * 256
			}
			w_swing_magnitude = (short) (d0 << 8);
			--b_car_on_chains_countdown;
			return;
		}
		// lift_car_stage1
		if( ON_CHAIN_STAGE_1 == d1 ) {
			swingCar( 0 );// no adjustment to amount of swing
			final int d0 = raiseCarOffGround( 3 );
			if( 0 == (d0 & 0x80) ) {
				--b_car_on_chains_countdown;
			}
			// car_not_on_chains
			return;
		}
		// lift_car_stage2
		if( ON_CHAIN_STAGE_2 == d1 ) {
			raiseCarOffGround( 4 );
			// reduce amount of swing
			if( swingCar( -1 ) ) {
				int d0 = ON_CHAIN_RACE_MIN_WAIT_KEY + (randomizeLong() & 0x1F);// 175 +16/-15

				String d2 = Text.STR_DROP_START;// int d2 = 44;// drop start
				if( b_drop_start_done ) {
					d2 = Text.STR_PRESS_FIRE;// d2 = 60;// press fire
				}
				if( b_race_mode >= 0 ) {
					d0 = ON_CHAIN_PRACTICE_WAIT_KEY;
				}
				b_car_on_chains_countdown = (byte) d0;
				if( 0 != b_1bb74 ) {
					b_1bb74 = 50;
				}
				d0 = 4;
				signalPromptRequired( d0, d2 );
				return;
			}
			// not_minimum_magnitude
			return;
		}
		// lift_car_stage3
		// no adjustment to amount of swing
		swingCar( 0 );
		raiseCarOffGround( 2 );
		if( b_fourteen_frames_elapsed >= 0 ) {
			--b_car_on_chains_countdown;
			if( 0 == b_car_on_chains_countdown ) {
				++b_car_on_chains_countdown;
			}
		}
		if( b_drop_start_done ) {
			// subsequent_lift
			if( b_boost_flag ) {// if fire not pressed
				return;
			}
		} else {
			if( b_car_on_chains_countdown < 0 ) {
				return;
			}
		}
		// car_off_chains
		b_car_on_chains_countdown = 0;
		b_off_map_status = 0;
		b_prompt_required = false;

		// Car only wants a drop start at the start of the race.
		b_drop_start_done = true;
		// not.off.chains.yet
	}

	// car_to_car_collision
	private static void carToCarCollision() {
		if( 0 != b_cars_collided_delay ) {
			--b_cars_collided_delay;
		}
		if( b_cars_collided ) {
			b_cars_collided = false;
			int d0 = w_opponents_z_speed - w_car_to_car_z_acceleration;
			if( d0 < 0 ) {
				d0 = -d0;
			}
			w_opponents_z_speed = (short) d0;

			d0 = (w_car_to_car_y_acceleration >> 4);
			w_opp_wheel_y_speeds[ OPP_REAR_LEFT_WHEEL ] -= d0;
			w_opp_wheel_y_speeds[ OPP_REAR_RIGHT_WHEEL ] -= d0;
			w_opp_wheel_y_speeds[ OPP_FRONT_WHEEL ] -= d0;

			w_car_collision_x_acceleration += w_car_to_car_x_acceleration;
			w_car_collision_y_acceleration += w_car_to_car_y_acceleration;
			w_car_collision_z_acceleration += w_car_to_car_z_acceleration;

			w_car_to_car_x_acceleration = 0;
			w_car_to_car_y_acceleration = 0;
			w_car_to_car_z_acceleration = 0;

			//*** Play collision sound if necessary ***
			if( 0 == b_cars_collided_delay ) {
				SoundManager.soundEffect( SoundManager.SOUND_HIT_CAR );// hit car
				b_cars_collided_delay = 5;
			}
		}
	}

	// car_collision_detection
	private static void carCollisionDetection() {
		b_grounded_count = 0;
		w_damage_value &= 0x00FF;// TODO why only high byte?
		//*** Front left wheel collision ***
		int d0 = lw_front_left_road_height_fp8 - lw_front_left_actual_height_fp8 - lw_wreck_wheel_height_reduction_fp8;
		lw_front_left_height_difference = d0;
		// limit to minimum or maximum
		if( d0 > 0x1400 ) {
			d0 = 0x1400;
		} else if( d0 < -0x300 ) {
			d0 = -0x300;
		}
		w_new_wheel_differences[ LEFT_FRONT_WHEEL ] = (short) d0;
		int d6 = d0;
		d0 -= w_old_front_left_difference;
		d0 = calculateDifference( d0, d6 );
		if( d0 >= 0 ) {
			final int d4_old_amount_below_road = w_front_left_amount_below_road;
			w_front_left_amount_below_road = (short) d0;
			if( (d0 >= 0x400) && (d4_old_amount_below_road < 0x200) ) {
				++b_grounded_count;// update count of grounded wheels
			}
			// front_left_not_grounded
			d0 = w_front_left_amount_below_road - (b_road_cushion_value << 8);
			if( d0 < 0x700 ) {
				// front_left_not_damaged
				b_damaged_count = 0;
			} else {
				if( d0 > w_damage_value ) {// TODO verify
					w_damage_value = (short) d0;// save damage value if it is larger
				}
				// front_left_damage_is_less
				d0 -= 0x600;
				if( b_fourteen_frames_elapsed >= 0 ) {
					++b_damaged_count;
					if( b_damaged_count < b_damaged_limit ) {
						d0 >>>= 8;
						d0 += d0 >>> 1;
						d0 += (b_front_left_damage & 0xFF);
						if( d0 > 255 ) {
							d0 = 255;
						}
						b_front_left_damage = (byte) d0;
						b_damaged = true;
					}
				}
				// ccd5
				d0 = w_front_left_amount_below_road;
				if( d0 > 0x11FF ) {
					w_front_left_amount_below_road = 0x11FF;
				}
			}
		} else {
			// front_left_above_road
			w_front_left_amount_below_road = 0;

			// front_left_not_damaged
			b_damaged_count = 0;
		}
		// save_front_left_difference
		w_old_front_left_difference = w_new_wheel_differences[ LEFT_FRONT_WHEEL ];

		//*** Front right wheel collision ***

		d0 = lw_front_right_road_height_fp8 - lw_front_right_actual_height_fp8 - lw_wreck_wheel_height_reduction_fp8;
		lw_front_right_height_difference = d0;
		// limit to minimum or maximum
		if( d0 > 0x1400 ) {
			d0 = 0x1400;
		} else if( d0 < -0x300 ) {
			d0 = -0x300;
		}
		w_new_wheel_differences[ RIGHT_FRONT_WHEEL ] = (short) d0;
		d6 = d0;
		d0 -= w_old_front_right_difference;
		d0 = calculateDifference( d0, d6 );
		if( d0 >= 0 ) {
			final int d4_old_amount_below_road = w_front_right_amount_below_road;
			w_front_right_amount_below_road = (short) d0;
			if( (d0 >= 0x400) && (d4_old_amount_below_road < 0x200) ) {
				++b_grounded_count;// update count of grounded wheels
			}
			// front_right_not_grounded
			d0 = w_front_right_amount_below_road - (b_road_cushion_value << 8);
			if( d0 < 0x700 ) {
				// front_right_not_damaged
				b_damaged_count = 0;
			} else {
				if( d0 > w_damage_value ) {
					w_damage_value = (short) d0;// save damage value if it is larger
				}
				// front_right_damage_is_less
				d0 -= 0x600;
				if( b_fourteen_frames_elapsed >= 0 ) {
					++b_damaged_count;
					if( b_damaged_count < b_damaged_limit ) {
						d0 >>>= 8;
						d0 += d0 >>> 1;
						d0 += b_front_right_damage & 0xFF;
						if( d0 > 255 ) {
							d0 = 255;
						}
						b_front_right_damage = (byte) d0;
						b_damaged = true;
					}
				}
				d0 = w_front_right_amount_below_road;
				if( d0 > 0x11FF ) {
					w_front_right_amount_below_road = 0x11FF;
				}
			}
		} else {
			// front_right_above_road
			w_front_right_amount_below_road = 0;

			// front_right_not_damaged
			b_damaged_count = 0;
		}
		// save_front_right_difference
		w_old_front_right_difference = w_new_wheel_differences[ RIGHT_FRONT_WHEEL ];

		//*** Rear wheel collision ***

		d0 = lw_rear_road_height_fp8 - lw_rear_actual_height_fp8 - lw_wreck_wheel_height_reduction_fp8;
		lw_rear_height_difference = d0;
		// limit to minimum or maximum
		if( d0 > 0x1400 ) {
			d0 = 0x1400;
		} else if( d0 < -0x300 ) {
			d0 = -0x300;
		}
		w_new_wheel_differences[ REAR_WHEEL ] = (short) d0;
		d6 = d0;
		d0 -= w_old_rear_difference;
		d0 = calculateDifference( d0, d6 );
		if( d0 >= 0 ) {
			final int d4_old_amount_below_road = w_rear_amount_below_road;
			w_rear_amount_below_road = (short) d0;
			if( (d0 >= 0x400) && (d4_old_amount_below_road < 0x200) ) {
				++b_grounded_count;// update count of grounded wheel
			}
			// rear_not_grounded
			d0 = w_rear_amount_below_road - (b_road_cushion_value << 8);
			if( d0 < 0x700 ) {
				// rear_not_damaged
				b_damaged_count = 0;
			} else {
				if( d0 >= w_damage_value ) {
					w_damage_value = (short) d0;// save damage value if it is larger
				}
				// rear_damage_is_less
				d0 -= 0x600;
				if( b_fourteen_frames_elapsed >= 0 ) {
					++b_damaged_count;
					if( b_damaged_count < b_damaged_limit ) {
						d0 >>>= 8;
						d0 += d0 >>> 1;
						d0 += b_rear_damage & 0xFF;
						if( d0 > 255 ) {
							d0 = 255;
						}
						b_rear_damage = (byte) d0;
						b_damaged = true;
					}
				}
				d0 = w_rear_amount_below_road;
				if( d0 > 0x11ff ) {
					w_rear_amount_below_road = 0x11ff;
				}
			}
		} else {
			// rear_above_road
			w_rear_amount_below_road = 0;
			// rear_not_damaged
			b_damaged_count = 0;
		}
		// save_rear_difference
		w_old_rear_difference = w_new_wheel_differences[ REAR_WHEEL ];

		//***

		d0 = (w_front_left_amount_below_road + w_front_right_amount_below_road) >> 1;
		w_near_x_coord = (short) d0;// average front amount below road
		d0 += w_rear_amount_below_road;
		d0 >>= 1;
		w_average_amount_below_road = (short) d0;

		calculateCarCollisionAcceleration();

		d0 = w_front_left_amount_below_road - w_front_right_amount_below_road;
		int d3 = d0;
		d0 = (d0 << 1) + d0;// difference * 3
		if( d0 < 0 ) {
			d0 = -d0;
		}
		if( d0 > 0x1000 ) {
			d0 = 0x1000;// limit to maximum
		}
		if( d3 < 0 ) {
			d0 = -d0;// correct sign
		}
		w_front_difference_below_road = (short) d0;

		//***
		// average front amount below road
		w_overall_difference_below_road = (short) (w_near_x_coord - w_rear_amount_below_road);

		//***

		b_touching_road = (w_average_amount_below_road != 0);
		if( !b_touching_road ) {
			// Not touching road
			if( 0 == b_car_on_chains_countdown ) {
				d3 = -128;
				d0 = w_players_x_angle;
				if( d0 < 0 ) {
					switch( b_road_id ) {
						case Road.TRACK_SKI_JUMP:
							d3 = -8;
						case Road.TRACK_ROLLER_COASTER:
							// x_angle_less
							d3 -= w_overall_difference_below_road;
							if( (d3 < 0) && (w_players_x_rotation_speed >= -256) ) {
								w_overall_difference_below_road = (short) d3;
							}
					}
				} else {
					// x_angle_plus
					if( d0 >= (Trigonometry.ANGLE_45_DEGREES / 2) ) {// 0x1000 ) {
						d3 = -256;
					}
					// x_angle_less
					d3 -= w_overall_difference_below_road;
					if( (d3 < 0) && (w_players_x_rotation_speed >= -256) ) {
						w_overall_difference_below_road = (short) d3;
					}
				}
			}
		}
		// ccd17
		liftCarOntoTrack();
		w_car_to_road_collision_z_acceleration = w_car_collision_z_acceleration;
		carToCarCollision();

		//*** Play grounded sound if necessary ***
		if( 0 != b_grounded_delay ) {
			--b_grounded_delay;
		}
		if( 0 != b_grounded_count ) {
			d0 = (w_damage_value >>> 8);
			if( d0 < 7 ) {
				d0 = 7;// set volume to minimum of 28
			}
			d0 <<= 2;
			if( d0 > 64 ) {
				d0 = 64;// set volume to maximum of 64
			}
			SoundManager.effects[ SoundManager.SOUND_GROUNDED ].mVolume_fp6 = (short) d0;
			if( 0 == b_grounded_delay ) {
				SoundManager.soundEffect( SoundManager.SOUND_GROUNDED );// grounded
				b_grounded_delay = 5;
			}
		}
	}

	// get_twice_collision_y_acceleration
	/**
	 * @return d0 as word
	 */
	private static int getTwiceCollisionYacceleration() {
		// If on ground, returns value in d0
		if( b_touching_road ) {
			return (w_car_collision_y_acceleration << 1);
		}
		return 0;
	}

	// calculate_x_acceleration
	private static void calculateXAcceleration() {
		final int d4 = w_gravity_x_acceleration + w_car_collision_x_acceleration;
		int d3 = d4 - w_players_x_speed;// speed increase minus current speed
		if( d3 < 0 ) {
			d3 = -d3;
		}
		int d0 = getTwiceCollisionYacceleration();
		if( d3 >= d0 ) {
			if( w_players_x_speed < 0 ) {
				d0 = -d0;
			}
			w_players_x_acceleration = (short) (d4 - d0);
			b_collision_in_air = true;
			return;
		}
		// y_bigger_than_x
		w_players_x_acceleration = (short) (w_car_collision_x_acceleration - w_players_x_speed);
		b_collision_in_air = false;
	}

	// calculate_total_acceleration
	private static void calculateTotalAcceleration() {
		w_players_y_acceleration = (short) (w_gravity_y_acceleration + w_car_collision_y_acceleration);

		int d0 = w_engine_z_acceleration | w_players_z_speed;// high bytes
		if( d0 >= 0 ) {
			if( 0 != (w_engine_z_acceleration & 0xFF) ) {
				w_engine_z_acceleration -= (d0 >> 8);// TODO verify
			}
		}
		int d3 = w_engine_z_acceleration;
		if( d3 < 0 ) {
			d3 = -d3;
		}
		d0 = getTwiceCollisionYacceleration();
		if( d3 >= d0 ) {
			if( w_engine_z_acceleration < 0 ) {
				d0 = -d0;
			}
			w_engine_z_acceleration = (short) d0;
		}
		w_players_z_acceleration = (short) (w_engine_z_acceleration
				+ w_car_collision_z_acceleration
				+ w_gravity_z_acceleration);
		calculateXAcceleration();
	}

	// adjust_steering_acceleration
	/**
	 * @param d4_steering_acceleration steering acceleration in d4 as word
	 */
	private static void adjustSteeringAcceleration(int d4_steering_acceleration) {
		d4_steering_acceleration -= w_players_y_rotation_speed;
		if( !b_touching_road ) {
			// steering.disabled
			d4_steering_acceleration = 0;
		}
		// store_steering_acceleration
		w_players_y_rotation_acceleration = (short) d4_steering_acceleration;
	}

	// align_car_with_road
	/**
	 * @param d4_steering_acceleration steering acceleration as word
	 * @param difference_angle         as word
	 */
	private static void alignCarWithRoad(final int d4_steering_acceleration, final int difference_angle) {

		b_left_right_value = (byte) (difference_angle >> 8);// not needed

		// Following code used to gradually bring the car back in
		// line with the road - this helps steering considerably
		int d0 = -1;// java
		int d2 = w_pos_difference_angle & 0xFF;
		if( 0 != (w_pos_difference_angle >> 8) ) {
			// diff_greater_than_256
			d2 = 0xFF;// set adjustment amount to maximum
			d0 = w_pos_difference_angle - 30 * 256;// 0x1e00
			// java if( d0 >= 0 ) {
			//	bpl diff_large;// just use remainder to adjust y angle
			//}
		}
		if( d0 < 0 ) {
			// diff_less_than_256
			d0 = w_players_z_speed;
			if( d0 < 0 ) {
				d0 = -d0;
			}
			d0 += 0xA00;
			if( d0 > 0x7FFF ) {
				d0 = 0x7F00;// set speed amount to maximum
			}
			// Adjustment of player's Y angle increases as player's speed increases
			d0 = ((d0 * ((d2 << 7) & 0x7FFF)) >>> 15) >>> 7;
			if( 0 == (d0 & 0xFF) ) {
				++d0;// at least do some adjusting
			}
		}
		// diff_large
		if( difference_angle < 0 ) {
			d0 = -d0;
		}
		w_players_y_angle += d0;// adjust player's Y angle now

		// End of car re-alignment section

		// adjust.steering.acceleration
		adjustSteeringAcceleration( d4_steering_acceleration );
	}

	// calculate_steering_acceleration
	/**
	 * @param d0               as byte
	 * @param difference_angle as word
	 */
	private static void calculateSteeringAcceleration(int d0, final int difference_angle) {
		// Steering acceleration increases as player's speed increases
		d0 = ((w_players_z_speed * ((d0 << 7) & 0x7FFF)) << 1) >> 16;// signed
		if( b_left_right_value < 0 ) {
			d0 = -d0;// if steering left, subtract from player's Y angle
		}
		// steering_right
		d0 >>= 3;
		if( w_pos_difference_angle < 30 * 256 ) {
			adjustSteeringAcceleration( d0 );// if difference less than 30
			return;
		}
		alignCarWithRoad( d0, difference_angle );
	}

	// calculate_steering
	private static void calculateSteering() {
		b_current_road_piece = b_players_road_piece;
		fetchNearSectionStuff( b_players_road_piece );

		int d4 = w_section_y_angle - w_players_y_angle;
		int d3 = (b_plus_180_degrees) << 8;
		d4 ^= d3;// difference angle between section and player
		d4 = (int) (short) d4;// java: keep sign of the short angle TODO simplify

		int d2 = 0;// straight
		if( b_curved_section < 0 ) {// if straight piece
			d2 = 2;// curve to right
			if( 0 != ((w_curve_to_left ^ d3) & Trigonometry.ANGLE_180_DEGREES) ) {// if right hand bend
				d2 += 2;// curve to left
			}
		}
		// get_road_steering_change

		// If player is on a curved section then adjust the difference angle
		// move.l	#road.steering.changes,a0
		d4 += Road.w_road_steering_changes[ d2 / 2 ];// adjust difference angle
		int d0 = d4;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		w_pos_difference_angle = (short) d0;
		final int w_difference_angle = d4;// word

		// Save a scaled positive difference angle ranging from 0 to $7fff
		if( d0 >= 0x800 ) {
			d0 = 0x7fff;// set to maximum, 0x800 << 4
		} else {
			d0 <<= 4;
		}
		w_pos_difference_angle_m16 = (short) d0;

		// If on last segment of road section then get data for next section
		d0 = b_number_of_segments - (w_players_distance_into_section >> 8);
		if( d0 < 2 ) {
			final int d1 = toNextRoadPiece();
			fetchNearSectionStuff( d1 );
		}
		// not_last_segment
		b_left_hand_bend = (byte) ((w_curve_to_left >> 8) ^ b_plus_180_degrees);

		if( 0 != b_left_right_value ) {
			// player_is_steering
			// negative if pos.difference.angle is going to increase
			b_value_low_factor_high = (byte) (b_left_right_value ^ (w_difference_angle >> 8));
			if( b_curved_section < 0 ) {
				if( 0 == ((b_left_right_value ^ b_left_hand_bend) & 0x80) ) {
					// steering_into_the_bend
					d0 = b_section_steering_amount + 45;// steering amount +45
				} else {
					// steering_away_from_bend
					b_left_right_value = b_left_hand_bend;
					d0 = b_section_steering_amount - 35;// steering amount -35 FIXME is this a bug? there are sections with 0x20
					// pos_difference_angle_increasing
					calculateSteeringAcceleration( d0, w_difference_angle );
					return;
				}
			} else {
				// straight_piece
				d0 = b_section_steering_amount;
			}
			// plus_current_difference
			if( b_value_low_factor_high >= 0 ) {
				// Add current difference (between player and road) onto steering amount
				d0 += w_pos_difference_angle_m16 >> 8;
			}
			// pos_difference_angle_increasing
			calculateSteeringAcceleration( d0, w_difference_angle );
			return;
		}
		// player_not_steering
		d4 = 0;// zero steering acceleration
		if( b_curved_section < 0 ) {
			b_left_right_value = b_left_hand_bend;
			calculateSteeringAcceleration( b_section_steering_amount, w_difference_angle );
			return;
		}
		// if on a straight piece
		alignCarWithRoad( d4, w_difference_angle );
	}

	// calculate_world_acceleration
	/**
	 * Adds components of player's (i.e. rotated) X, Y and Z accelerations
	 * to give world acceleration values.
	 */
	private static void calculateWorldAcceleration() {
		// add component of rotated X
		int d5 = sinCosMult( w_players_x_acceleration, b_sin_cos_offsets[ 2 + 9 ] );
		// add component of rotated Y
		d5 += sinCosMult( w_players_y_acceleration, b_sin_cos_offsets[ 2 + 12 ] );
		// add component of rotated Z
		d5 += sinCosMult( w_players_z_acceleration, b_sin_cos_offsets[ 2 + 15 ] );
		w_total_world_z_acceleration = (short) d5;

		// add component of rotated X
		d5 = sinCosMult( w_players_x_acceleration, b_sin_cos_offsets[ 1 + 9 ] );
		// add component of rotated Y
		d5 += sinCosMult( w_players_y_acceleration, b_sin_cos_offsets[ 1 + 12 ] );
		// add component of rotated Z
		d5 += sinCosMult( w_players_z_acceleration, b_sin_cos_offsets[ 1 + 15 ] );
		w_total_world_y_acceleration = (short) d5;

		// add component of rotated X
		d5 = sinCosMult( w_players_x_acceleration, b_sin_cos_offsets[ 0 + 9 ] );
		// add component of rotated Y
		d5 += sinCosMult( w_players_y_acceleration, b_sin_cos_offsets[ 0 + 12 ] );
		// add component of rotated Z
		d5 += sinCosMult( w_players_z_acceleration, b_sin_cos_offsets[ 0 + 15 ] );
		w_total_world_x_acceleration = (short) d5;
	}

	// java to avoid goto
	/**
	 * Reduce acceleration values using current speed values.
	 *
	 * @param d0 reduction amount
	 * @param d7 overall reduction factor
	 */
	private static void reduceWorldAccelerationFinal(final int d0, final int d7) {
		w_total_world_x_acceleration -= ((w_players_world_x_speed * d0) >> 16) >> d7;
		w_total_world_y_acceleration -= ((w_players_world_y_speed * d0) >> 16) >> d7;
		w_total_world_z_acceleration -= ((w_players_world_z_speed * d0) >> 16) >> d7;
	}

	// reduce_world_acceleration
	private static void reduceWorldAcceleration() {
		if( b_touching_road ) {
			// Touching road
			int d0 = w_car_to_road_collision_z_acceleration >> 8;
			if( d0 < 0 ) {
				d0 = ~d0;// ^ 0xFF, -(v + 1)
			}
			// if collision.z.acceleration large or if off map
			if( (d0 >= 3) || (b_off_map_status < 0) ) {
				reduceWorldAccelerationFinal( 0x6000, 1 );
				return;
			}
			if( 0 != lw_wreck_wheel_height_reduction_fp8 ) {// if wrecked
				// medium reduction factor
				reduceWorldAccelerationFinal( 0x6000, 3 );
				return;
			}
		}
		// rwa1
		if( 0 == b_car_on_chains_countdown ) {
			// not_on_chains
			int d0 = w_players_x_speed;// reduce accelerations
			if( d0 < 0 ) {// depending on car speed
				d0 = -d0;
			}
			int d3 = w_players_y_speed;
			if( d3 < 0 ) {
				d3 = -d3;
			}
			if( d0 < d3 ) {
				d0 = d3;
			}
			d3 = w_players_z_speed;
			if( d3 < 0 ) {
				d3 = -d3;
			}
			if( d0 < d3 ) {
				d0 = d3;
			}
			// check_slipstream
			if( b_player_close_to_opponent && !b_opponent_behind_player ) {
				d0 -= 20 * 128;// subtract 20*128 to make reduction smaller
				if( d0 < 0 ) {
					d0 = 0;
				}
			}
			// minimum reduction factor
			reduceWorldAccelerationFinal( d0, 5 );
			return;
		}
		// medium reduction factor
		reduceWorldAccelerationFinal( 0x6000, 3 );
	}

	// calculate_xz_rotation_acceleration
	/**
	 * Calculate values using current car rotation speeds and inclination values
	 * between the car and the road, in order to damp the car X and Z angles
	 * and keep the car level with the road, on its X and Z axes.  Also give
	 * effect of acceleration.
	 * <p>
	 * overall.difference.below.road is effectively car X inclination.
	 * <p>
	 * front.difference.below.road is effectively car Z inclination.
	 */
	private static void calculateXZrotationAcceleration() {
		int d0 = w_overall_difference_below_road - (w_players_x_rotation_speed >> 4);
		if( b_touching_road ) {
			// This part lifts the car up at the front during forwards acceleration
			// and, vice versa, dips the front of the car during backwards acceleration.
			d0 += w_players_z_acceleration >> 2;
		}
		// not_touching_road
		w_players_x_rotation_acceleration = (short) d0;

		w_players_z_rotation_acceleration =
				(short) (w_front_difference_below_road - (w_players_z_rotation_speed >> 4));
	}

	// update_players_rotation_speed
	private static void updatePlayersRotationSpeed() {
		w_players_x_rotation_speed += (REDUCTION * w_players_x_rotation_acceleration) >> 8;
		w_players_y_rotation_speed += (REDUCTION * w_players_y_rotation_acceleration) >> 8;
		w_players_z_rotation_speed += (REDUCTION * w_players_z_rotation_acceleration) >> 8;
	}

	// calculate_final_rotation_speed
	private static void calculateFinalRotationSpeed() {
		// Calculate final X and Y rotation speeds by rotating X and Y rotation
		// speeds about the Z axis.
		int d5 = sinCosMult( w_players_x_rotation_speed, b_sin_cos_offsets[ 1 + 18 ] );
		d5 += sinCosMult( w_players_y_rotation_speed, b_sin_cos_offsets[ 1 + 20 ] );
		w_players_final_y_rotation_speed = (short) d5;

		d5 = sinCosMult( w_players_x_rotation_speed, b_sin_cos_offsets[ 0 + 18 ] );
		d5 += sinCosMult( w_players_y_rotation_speed, b_sin_cos_offsets[ 0 + 20 ] );
		w_players_final_x_rotation_speed = (short) d5;

		// Calculate final Z rotation speed by rotating Y rotation speed about
		// the X axis and adding it onto the Z rotation speed.
		w_players_final_z_rotation_speed =
				(short) (w_players_z_rotation_speed + sinCosMult( w_players_final_y_rotation_speed, 4 ));
	}

	// update_players_world_speed
	private static void updatePlayersWorldSpeed() {
		w_players_world_x_speed += (REDUCTION * w_total_world_x_acceleration) >> 8;
		w_players_world_y_speed += (REDUCTION * w_total_world_y_acceleration) >> 8;
		w_players_world_z_speed += (REDUCTION * w_total_world_z_acceleration) >> 8;
	}

	// update_players_position
	private static void updatePlayersPosition() {
		//*** Set player's new position ***

		lw_players_world_x += ((REDUCTION * w_players_world_x_speed) >> 8) << 6;
		lw_players_world_y_fp16 += ((REDUCTION * w_players_world_y_speed) >> 8) << 7;
		lw_players_world_z += ((REDUCTION * w_players_world_z_speed) >> 8) << 6;

		// limit player's height
		if( lw_players_world_y_fp16 > 1000 * 65536 ) {// TODO verify only high word
			lw_players_world_y_fp16 = (1000 * 65536) | (lw_players_world_y_fp16 & 0xFFFF);
		}
		//*** Set player's new angles ***
		w_players_x_angle += (REDUCTION * w_players_final_x_rotation_speed) >> 8;
		w_players_y_angle += (REDUCTION * w_players_final_y_rotation_speed) >> 8;
		w_players_z_angle += (REDUCTION * w_players_final_z_rotation_speed) >> 8;

		//*** Check player's X angle ***
		int d2 = 0;
		if( b_1bb75 && (0xE0 == (b_at_side_byte & 0xFF)) ) {
			d2 = 2;
		}
		int d3 = w_players_x_angle;
		if( d3 >= 0 ) {
			int d0 = Trigonometry.w_maximum_angles[ d2 / 2 ];// positive value from table
			if( d0 < d3 ) {
				// uppa
				w_players_x_angle = (short) d0;// set maximum allowable x angle

				d3 = w_players_x_rotation_speed;
				d0 ^= d3;
				if( d0 >= 0 ) {
					w_players_x_rotation_speed = 0;
				}
			}
			//*** Check player's Z angle ***
			// uppb:
			d3 = w_players_z_angle;
			if( d3 >= 0 ) {
				d0 = Trigonometry.w_maximum_angles[ d2 / 2 ];
				if( d0 < d3 ) {
					// uppd:
					w_players_z_angle = (short) d0;

					d3 = w_players_z_rotation_speed;
					d0 ^= d3;
					if( d0 >= 0 ) {
						w_players_z_rotation_speed = 0;
					}
				}
			} else {
				// uppc:
				d0 = Trigonometry.w_maximum_angles[ (d2 + 4) / 2 ];
				if( d0 >= d3 ) {
					// uppd:
					w_players_z_angle = (short) d0;

					d3 = w_players_z_rotation_speed;
					d0 ^= d3;
					if( d0 >= 0 ) {
						w_players_z_rotation_speed = 0;
					}
				}
			}
		} else {// x angle < 0
			// upp9
			int d0 = Trigonometry.w_maximum_angles[ (d2 + 4) / 2 ];// negative value from table
			if( d0 >= d3 ) {
				// uppa
				w_players_x_angle = (short) d0;// set maximum allowable x angle

				d3 = w_players_x_rotation_speed;
				d0 ^= d3;
				if( d0 >= 0 ) {
					w_players_x_rotation_speed = 0;
				}
			}
			//*** Check player's Z angle ***
			// uppb:
			d3 = w_players_z_angle;
			if( d3 >= 0 ) {
				d0 = Trigonometry.w_maximum_angles[ d2 / 2 ];
				if( d0 < d3 ) {
					// uppd:
					w_players_z_angle = (short) d0;

					d3 = w_players_z_rotation_speed;
					d0 ^= d3;
					if( d0 >= 0 ) {
						w_players_z_rotation_speed = 0;
					}
				}
			} else {// z angle < 0
				// uppc:
				d0 = Trigonometry.w_maximum_angles[ (d2 + 4) / 2 ];
				if( d0 >= d3 ) {
					// uppd:
					w_players_z_angle = (short) d0;

					d3 = w_players_z_rotation_speed;
					d0 ^= d3;
					if( d0 >= 0 ) {
						w_players_z_rotation_speed = 0;
					}
				}
			}
		}
		//***
		// uppe
		int d0 = w_players_x_angle;// high byte
		if( d0 < 0 ) {
			d0 = -d0;
		}
		w_y_shift = (short) -w_players_x_angle;
	}

	// car_movement
	private static void carMovement() {
		makeSinCos();

		calculateWheelXZoffsets();
		calculateRoadWheelHeights();
		calculateActualWheelHeights();

		calculateXZspeeds();
		setWheelRotationSpeed();
		calculateGravityAcceleration();
		carCollisionDetection();
		if( b_1bb72 ) {
			calculateTotalAcceleration();
			calculateSteering();
			calculateWorldAcceleration();
			reduceWorldAcceleration();

			calculateXZrotationAcceleration();
			updatePlayersRotationSpeed();
			calculateFinalRotationSpeed();
		}
		// left_road
		updatePlayersWorldSpeed();
		updatePlayersPosition();
	}

	// update_engine_revs
	private static void updateEngineRevs() {
		int d0;
		if( !b_touching_road ) {
			// Not touching road, so test if joystick is held forwards or backwards
			if( 0 != (b_players_input & (CTRL_PLAYER_ACCELERATE | CTRL_PLAYER_BREAK)) ) {
				d0 = 0x9000;// set required revs.
			} else {
				d0 = 0;
			}
		} else {
			// Touching road
			// uer1
			d0 = w_players_z_speed & 0xFFFF_FFF0;
			if( d0 < 0 ) {
				d0 = -d0;
			}
		}
		// uer2
		d0 += 0x580;
		d0 >>>= 3;
		int d3 = w_engine_revs;
		if( d3 < 192 ) {
			// If engine revs.are low then increase them slowly (e.g.at race start)
			d0 = 2;
		} else {
			// Otherwise calculate revs.change depending on current engine revs.
			// uer3
			d0 -= d3;
			d0 >>= 3;
		}
		// uer4
		w_engine_revs_change = (short) d0;
		// Now adjust revs change
		d0 >>= 8;
		if( d0 < 0 ) {
			// Revs change is negative
			// uer5
			if( !b_touching_road ) {
				// beq uer6;
				// Not touching road, so set revs change to $ffe0 minimum
				// uer6
				if( (0xFF != (d0 & 0xFF)) || (((w_engine_revs_change >> 8) & 0xFF) < 0xE0) ) {
					// uer7
					w_engine_revs_change = (short) 0xFFE0;
				}
			} else {
				// Touching road, so set revs change to $ff00 minimum
				if( 0xFF != (d0 & 0xFF) ) {
					w_engine_revs_change = (short) 0xFF00;
				}
			}
		} else if( 0 != d0 ) {
			// If revs change is $100 or greater, then set to $100
			w_engine_revs_change = 0x100;
		}
		// uer9
		// d0 = randomizeLong() & 0x0F;
		randomizeLong();
		b_engine_fluctuation = 0;// FIXME 0 instead of d0? is this a bug in the Amiga code?
	}

	// car_is_wrecked
	private static void carIsWrecked() {
		if( 0 == b_1bb6c ) {
			if( (0x40 != (b_1bbe2 & 0xFF)) && (0x80 != (b_1bbe2 & 0xFF)) ) {
				b_1bbe2 = b_machine;
			}
			lw_wreck_wheel_height_reduction_fp8 = SCRATCH_HEIGHT_REDUCTION;

			b_1bbdd = (byte) 0x92;

			w_1bc56 = (short) ((w_1bc56 & 0xFF) | (0x8200));

			b_1bb6c = 0x3C;

			signalPromptRequired( 2, Text.STR_WRECK );// wreck
		}
	}

	// test_if_pixel_clear
	/**
	 * java: Test if a pixel is not black
	 *
	 * @param ba0i offset in bytes to byte screen buffer. 16 bit aligned.
	 *             320x200 is (40 bytes x 8 bits) x 200
	 * @param bd4  pixel number, 0-15. Low first, so bits on the screen are:
	 *             0-7 8-15, 0-7 8-15 ...
	 * @return true if the pixel is not black
	 */
	private static boolean testIfPixelClear(final int ba0i, final int bd4) {
		final int y = ba0i / (SCREEN_WIDTH / 8);
		final int x = ((ba0i - y * (SCREEN_WIDTH / 8)) << 3) | (bd4 & 0x0f);
		return 0xFF000000 != screen1.getPixel( SCREEN_AREA_LEFT_X + x, y );
	}

	private static byte b_damage_line_y = 4;

	// damage_line
	/**
	 * Draw the damage line on both screens
	 */
	private static void damageLine() {
		final Canvas g1 = screen1_canvas;
		final Canvas g2 = screen2_canvas;
		g1.save();
		g2.save();
		g1.clipRect( 0, 0, SCREEN_WIDTH, SCREEN_AREA_TOP_Y );
		g2.clipRect( 0, 0, SCREEN_WIDTH, SCREEN_AREA_TOP_Y );
		do {
			int d0 = b_new_damage;
			if( d0 <= b_damage_line_x ) {
				break;
			}
			++b_damage_line_x;
			int d2 = b_damage_line_y;
			if( 0 == (b_damage_line_x & 0x01) ) {
				d0 = randomizeLong();
				if( 0 != (d0 & 0x01) ) {
					if( 0 != (d0 & 0x02) ) {
						if( d2 < 5 ) {
							++d2;
						} else if( 0 == (d0 & 0x04) ) {
							--d2;
						}
					} else {
						// dl4
						if( d2 >= 3 ) {
							--d2;
						} else if( 0 == (d0 & 0x04) ) {
							++d2;
						}
					}
				}
			}
			// dl6
			byte b_damaged_point_color = DAMAGE_POINT_COLOR_1;
			if( d2 > b_damage_line_y ) {
				b_damaged_point_color = DAMAGE_POINT_COLOR_2;
			}
			b_damage_line_y = (byte) d2;

			d0 = b_damage_line_x & 0xFF;
			if( d0 >= MAX_DAMAGE_LINE_X / PF ) {
				--b_damage_line_x;
				// dl8
				carIsWrecked();
				break;
			}
			// dl9
			// move.l	screen_mem,a0
			// add.l	#4,a0
			int ba0i = 4;
			final int d4 = ((b_damage_line_x & 0xFF) + MAX_DAMAGE_LINE_START_X / PF) & 0xFF;
			d0 = d4;
			final int d5 = b_damage_line_y & 0x07;
			d0 >>>= 3;
			d0 &= 0xfe;// x pos
			ba0i += d0;

			d0 = ((d5 << 2) + d5) << 3;// * 40, y pos
			ba0i += d0;

			/* if( testIfPixelClear( ba0i, d4 ) ) {// java: what does this code do?
				// dla
				++b_new_damage;
				if( 0 == b_new_damage ) {
					carIsWrecked();
					return;
				}
				// dlb
				// d2 = 2;
				// dlc
				b_front_left_damage = b_new_damage;
				b_front_right_damage = b_new_damage;
				b_rear_damage = b_new_damage;
				continue;
			} */

			paint.setColor( getRGB( 0 ) );
			plotPixel( g1, ba0i, d4 );
			plotPixel( g2, ba0i, d4 );

			ba0i -= SCREEN_WIDTH / 8 / PF;// 40
			plotPixel( g1, ba0i, d4 );
			plotPixel( g2, ba0i, d4 );

			ba0i -= SCREEN_WIDTH / 8 / PF;// 40
			paint.setColor( getRGB( b_damaged_point_color ) );
			plotPixel( g1, ba0i, d4 );
			plotPixel( g2, ba0i, d4 );
		} while( true );
		g1.restore();
		g2.restore();
	}

	// update_damage
	private static void updateDamage() {
		if( b_damaged ) {
			// average front damage
			// short or int must be used to sum to avoid overflow
			// pay attention: summing must use int16 at least
			b_new_damage = (byte) ((((b_front_left_damage + b_front_right_damage) >> 1) + b_rear_damage) >> 1);// total average damage
			damageLine();
		}
		// no_new_damage
		if( 0 != b_smashed_countdown ) {
			if( SMASHED_COUNTDOWN_MAX != b_smashed_countdown-- ) {
				if( !b_damaged ) {
					return;
				}
			} else {
				// change_smash_to_hole
				copyDamageHole( b_damage_hole_position );
			}
		} else {
			// test_if_smashed
			if( !b_damaged ) {
				return;
			}
			if( w_damage_value >= 0x1400 ) {
				int d2 = b_damage_hole_position;
				if( 0 != d2 ) {
					--d2;
					b_damage_hole_position = (byte) d2;
					copyDamageHoleSmashed( d2 );

					b_smashed_countdown = SMASHED_COUNTDOWN_MAX;
					// move.b	#10, d0
					// move.b	#5, d0 smash
					SoundManager.soundEffect( SoundManager.SOUND_SMASH );// smash
					b_damaged = false;
					return;
				}
			}
		}
		// dam4:
		int d0 = (w_damage_value >> 8) & 0xFF;
		if( d0 < 7 ) {
			d0 = 7;
		}
		d0 <<= 2;
		if( d0 > 64 ) {
			d0 = 64;
		}
		SoundManager.effects[ SoundManager.SOUND_CREAK ].mVolume_fp6 = (short) d0;
		SoundManager.soundEffect( SoundManager.SOUND_CREAK );// creak
		b_damaged = false;
	}

	// R_57ac0
	private static void R_57ac0() {
		// move.b	d0,b_players_input
		b_57c6f = (byte) 0x80;
		if( b_57c5b ) {
			if( b_machine < 0 ) {
				byte d2 = 3;
				// _label1
				do {
					if( testKey( b_control_keys[ d2 + 5 ] ) ) {
						b_57c6f = d2;
					}
				} while( --d2 >= 0 );
				b_57c65 = b_57c6f;
			}
			// _label3
			b_57c66 = b_players_input;
			b_57c64 = (byte) 0xE3;
			if( ! R_579b0() ) {
				if( SLAVE == b_machine ) {
					b_players_input = b_57c60;
					b_57c6f = b_57c5f;
				}
			}
		}
		b_57c64 = 0;
	}

	// get_players_input
	/**
	 * return {@link #b_players_input} in d0
	 */
	private static int getPlayersInput() {
		randomizeLong();
		/*
		if( key_array == 0 ) {
			b_players_input = 0;
			return 0;
		}
		int input = b_players_input;// previous state, in case if several buttons are pressed
		if( key_array == KeyEvent.VK_LEFT ) {
			input |= CTRL_PLAYER_LEFT;
		}
		if( key_array == KeyEvent.VK_RIGHT ) {
			input |= CTRL_PLAYER_RIGHT;
		}
		if( key_array == KeyEvent.VK_UP ) {
			input |= CTRL_PLAYER_ACCELERATE;
		}
		if( key_array == KeyEvent.VK_DOWN ) {
			input |= CTRL_PLAYER_BREAK;
		}
		switch( key_array ) {
			case KeyEvent.VK_SPACE:
			case KeyEvent.VK_CONTROL:
			case KeyEvent.VK_SHIFT:
			case KeyEvent.VK_ENTER:
				input |= CTRL_PLAYER_BOOST;
				if( 0 == (input & (CTRL_PLAYER_ACCELERATE | CTRL_PLAYER_BREAK)) ) {
					input |= CTRL_PLAYER_ACCELERATE;
				}
				break;
		}
		*/
		int input = 0;
		synchronized( key_array ) {
			for( final Integer key : key_array ) {
				switch( key ) {
					case KeyEvent.KEYCODE_DPAD_LEFT:
						input |= CTRL_PLAYER_LEFT;
						break;
					case KeyEvent.KEYCODE_DPAD_RIGHT:
						input |= CTRL_PLAYER_RIGHT;
						break;
					case KeyEvent.KEYCODE_DPAD_UP:
						input |= CTRL_PLAYER_ACCELERATE;
						break;
					case KeyEvent.KEYCODE_DPAD_DOWN:
						input |= CTRL_PLAYER_BREAK;
						break;
					case KeyEvent.KEYCODE_SPACE:
					case KeyEvent.KEYCODE_CTRL_RIGHT:
					case KeyEvent.KEYCODE_SHIFT_RIGHT:
					case KeyEvent.KEYCODE_CTRL_LEFT:
					case KeyEvent.KEYCODE_SHIFT_LEFT:
					case KeyEvent.KEYCODE_ENTER:
						input |= CTRL_PLAYER_BOOST;
						break;
				}
			}// for all keys
			if( (0 != (input & CTRL_PLAYER_BOOST)) && (0 == (input & (CTRL_PLAYER_ACCELERATE | CTRL_PLAYER_BREAK))) ) {
				input |= CTRL_PLAYER_ACCELERATE;
			}
		}// sync
		if( 0 != b_machine ) {
			if( ! testKey( KeyEvent.KEYCODE_ESCAPE ) ) {
				R_57ac0();
			}
		}
		b_players_input = (byte) input;
		return input;
	}

	// boost_print
	/**
	 * Increase {@link #b_boost_reserve} and print the value
	 *
	 * @param d0 a byte. a value to add to {@link #b_boost_reserve}
	 */
	private static void boostPrint(int d0) {
		d0 += b_boost_reserve;
		if( d0 > b_boost_max_units ) {
			d0 = b_boost_max_units;
		}
		b_boost_reserve = (byte) d0;

		b_print_column = 9;
		b_print_row = 22;
		b_print_fine_x = 4;
		b_print_fine_y = 2;
		printDigit1( (d0 / 10) % 10 );
		printDigit1( d0 % 10 );
		b_print_fine_x = 0;
		b_print_fine_y = 0;
	}

	// boost_power
	private static void boostPower() {
		if( !(b_boost_flag | (lw_wreck_wheel_height_reduction_fp8 != 0)) ) {
			if( b_accelerating ||
					(0 != (b_players_input & (CTRL_PLAYER_ACCELERATE | CTRL_PLAYER_BREAK))) ) {
				// update_boost
				if( 0 != b_boost_reserve ) {
					if( b_fourteen_frames_elapsed >= 0 ) {
						--boost_unit;
						if( boost_unit < 0 ) {
							boost_unit = boost_unit_value;
							// d0 = 0x99;// subtract 1 from boost reserve
							boostPrint( -1 );
						}
					}
					// boost_on
					b_boost_activated = true;
					w_engine_z_acceleration <<= 1;// double car acceleration
					return;
				}
			}
		}
		// boost_off
		b_boost_activated = false;
	}

	// car_control
	private static void carControl() {
		getPlayersInput();
		int d0 = 0;
		if( b_touching_road ) {
			// if not touching road
			d0 = b_car_on_chains_countdown;
			if( 0 == d0 ) {
				d0 = b_players_input & (CTRL_PLAYER_RIGHT | CTRL_PLAYER_LEFT);
				if( 0 != d0 ) {
					if( CTRL_PLAYER_LEFT != d0 ) {
						// car.right
						d0 = 15;
					} else {
						// car.left
						d0 = -15;
					}
				}
			}
		}
		// left.right.done
		b_left_right_value = (byte) d0;

		b_boost_flag = ((b_players_input & CTRL_PLAYER_BOOST) ^ CTRL_PLAYER_BOOST) != 0;

		int acceleration = 0;// d1 = d2 = 0
		if( w_players_z_speed < 0x7800 ) {
			// car_going_backwards
			if( 0 == b_car_on_chains_countdown ) {
				if( 0 == lw_wreck_wheel_height_reduction_fp8 ) {
					d0 = b_players_input & (CTRL_PLAYER_ACCELERATE | CTRL_PLAYER_BREAK);
					if( CTRL_PLAYER_ACCELERATE == d0 ) {
						// car_accelerate
						acceleration = engine_power;// d1 low, d2 high, in Amiga engine_power is stored in low-endian
						b_accelerating = true;
					} else if( CTRL_PLAYER_BREAK == d0 ) {
						// car_brake
						acceleration = ENGINE_POWER_TO_BREAK;// 0xff10 d1 low d2 high
						b_accelerating = false;
					} else if( b_accelerating ) {
						// car_accelerate
						acceleration = engine_power;
						// accelerating = true;
					}
				}
			}
		}
		// save_accelerate_value
		w_engine_z_acceleration = (short) acceleration;
		boostPower();
	}

	// initialise_opponent_data
	private static void initialiseOpponentData() {
		calculateOpponentsRoadWheelPositions();
		final int sd0 = (0x68 + (randomizeLong() & 0x7f));
		int d1 = OPP_MAX_WHEELS;
		do {
			w_opp_wheel_actual_heights[ d1 ] = (short) (w_opp_wheel_road_heights[ d1 ] + sd0);
		} while( --d1 >= 0 );
	}

	// player_to_side_of_road
	/**
	 * Shift player in his x direction by 160 * 8192.
	 * <p>
	 * Uses cosx.siny and cosx.cosy, but players.x.angle is 0,
	 * so these values become sin y and cos y.
	 * <p>
	 * If player is to start to the right of the road:
	 * <p>
	 * players.world.x + 160 * 8192 cos y
	 * players.world.z - 160 * 8192 sin y
	 * <p>
	 * If player is to start to the left of the road:
	 * <p>
	 * players.world.x - 160 * 8192 cos y
	 * players.world.z + 160 * 8192 sin y
	 * <p>
	 * This is actually x = 160 * 8192, z = 0 being rotated about the y axis
	 * and then added to the player x and z.
	 */
	private static void playerToSideOfRoad() {
		// z
		int d0 = -w_sin_cos_values[ SIN_Y_COS_X ];// cos x * sin y = sin y
		b_copy_swing_from_left = b_swing_from_left;
		int d3 = Road.ROAD_SWING_CAR_POSITION;
		if( b_copy_swing_from_left ) {
			d3 = -d3;
		}
		lw_players_world_z += (((d0 * (d3 << 7)) << 1) >> 16) << 6;
		// x
		d0 = w_sin_cos_values[ COS_Y_COS_X ];// cos x * cos y = cos y
		d3 = Road.ROAD_SWING_CAR_POSITION;
		if( b_copy_swing_from_left ) {
			d3 = -d3;
		}
		lw_players_world_x += (((d0 * (d3 << 7)) << 1) >> 16) << 6;
	}

	// set_players_restart_position
	/**
	 * @param d1_piece road section, d1 as byte
	 */
	private static void setPlayersRestartPosition(int d1_piece) {
		restart:
		for( ; ; ) {
			b_current_road_piece = (byte) d1_piece;
			b_players_road_piece = (byte) d1_piece;
			// Find section to lower car onto;

			int d0_section_can_be_put = Road.sections_car_can_be_put_on[ b_road_section_angle_and_piece[ d1_piece ] & 0x0F ];
			if( d0_section_can_be_put >= 0 ) {
				int d2 = b_1ca2f;
				if( 0 != d2 ) {
					--d2;
					do {
						d0_section_can_be_put = d1_piece;
						if( d0_section_can_be_put == b_DAT_1c8e8[ d2 ] ) {
							// previous.restart.position;
							d1_piece = toPreviousRoadPiece();
							continue restart;
						}
					} while( --d2 >= 0 );
				}
				break;
			}
			// previous.restart.position;
			d1_piece = toPreviousRoadPiece();
		}
		// sprp2:
		jClearLoopPlayersVariables();

		b_car_on_chains_countdown = (byte) ON_CHAIN_STAGE_START_OFF_GROUND;

		fetchNearSectionStuff( d1_piece );
		b_map_x_shift = (byte) (b_road_piece_xz_positions[ d1_piece ] & 0x0F);
		b_map_z_shift = (byte) ((b_road_piece_xz_positions[ d1_piece ] >>> 4) & 0x0F);

		// Put player into the middle of the map square
		lw_players_world_x = (((b_map_x_shift & 0xFF) << 7) + 128 / 2) << 16;
		lw_players_world_z = (((b_map_z_shift & 0xFF) << 7) + 128 / 2) << 16;

		// Put player just above ground to prevent immediate damage
		lw_players_world_y_fp16 = 0x04_00_0000;

		// Set player's y angle
		int d1_flags = 0;
		if( (Road.PIECE_4 == b_near_section_piece) || (Road.PIECE_10 == b_near_section_piece) ) {// TODO replace numbers
			// add.45.degrees:
			d1_flags = Trigonometry.ANGLE_45_DEGREES;// if piece 4 or 10
		}
		// TODO verify set only high byte. is low byte = 0?
		d1_flags += (w_rough_piece_angle ^ (b_plus_180_degrees << 8));
		w_players_y_angle = (short) ((d1_flags & 0xFF00) | (w_players_y_angle & 0xFF));

		//********
		setRoadPositionValues();
		calculatePlayersRoadPosition();
		setRoadCentreValues();
		carMovement();
		lw_players_world_y_fp16 = 16 * 65536;// 0x0010_0000;// 16

		if( b_drop_start_done ) {
			// If not a drop start then put the car
			// at a fixed height above the road
			lw_players_world_y_fp16 = (lw_rear_road_height_fp8 << 9) + 0x18_0000;
			b_car_on_chains_countdown = (byte) ON_CHAIN_STAGE_SET_FIXED_HEIGHT;
		}
		w_required_raise_height = (short) (lw_rear_road_height_fp8 >>> 2);

		playerToSideOfRoad();

		lw_front_left_road_height_fp8 = 0x1000;
		lw_front_right_road_height_fp8 = 0x1000;
		lw_rear_road_height_fp8 = 0x1000;

		lw_front_left_actual_height_fp8 = 0x1000;
		lw_front_right_actual_height_fp8 = 0x1000;
		lw_rear_actual_height_fp8 = 0x1000;

		w_old_front_left_difference = 0;
		w_old_front_right_difference = 0;
		w_old_rear_difference = 0;

		setRoadPositionValues();
		b_1bbea = (byte) 176;
		b_1bbe9 = 8;
	}

	// calculate_if_winning
	/**
	 * @return {@code true} if winning
	 */
	static boolean calculateIfWinning() {

		if( b_opponents_lap != b_players_lap ) {// if on different laps
			return b_opponents_lap < b_players_lap;
		}

		int p = b_players_road_piece - b_start_finish_section;
		if( p < 0 ) {
			p += b_number_of_road_pieces;
		}
		int o = b_opponents_road_piece - b_start_finish_section;
		if( o < 0 ) {
			o += b_number_of_road_pieces;
		}
		o -= p;
		if( o == 0 ) {
			if( w_difference_between_players == 0 ) {
				return b_machine == MASTER;
			}
			return w_difference_between_players < 0;
		}
		return o < 0;
	}

	private static final int SIN_Y_SIN_X = 0;
	private static final int COS_Y_SIN_X = 1;
	private static final int SIN_Y_COS_X = 2;
	private static final int COS_Y_COS_X = 3;
	private static final int SIN_X = 4;
	private static final int COS_X = 5;
	private static final int SIN_Y_SIN_X_SIN_Z = 6;
	private static final int SIN_Y_SIN_X_COS_Z = 7;
	private static final int COS_Y_SIN_X_SIN_Z = 8;
	private static final int COS_Y_SIN_X_COS_Z = 9;
	private static final int SIN_Y_SIN_Z = 10;
	private static final int SIN_Y_COS_Z = 11;
	private static final int COS_Y_SIN_Z = 12;
	private static final int COS_Y_COS_Z = 13;
	private static final int COS_X_SIN_Z = 14;
	private static final int COS_X_COS_Z = 15;
	private static final int SIN_Z = 16;
	private static final int COS_Z = 17;
	private static final int M_SIN_Z = 18;
	private static final int COS_Y_SIN_Z__M__SIN_Y_SIN_X_COS_Z = 20;
	private static final int M__COS_Y_SIN_X_COS_Z__M__SIN_Y_SIN_Z = 21;
	private static final int P__COS_Y_COS_Z__P__SIN_Y_SIN_X_SIN_Z = 22;
	private static final int COS_Y_SIN_X_SIN_Z__M__SIN_Y_COS_Z = 23;
	private static final int M__COS_X_SIN_Z = 24;

	private static final int SIN_X_SIN_Z_SIN_DY = 26;
	private static final int SIN_X_SIN_Z_COS_DY = 28;
	private static final int COS_Z_COS_DY = 31;
	private static final int COS_Z_SIN_DY = 33;
	private static final int COS_X_SIN_DY = 34;
	private static final int COS_X_COS_DY = 35;

	// sin_cos_offsets
	private static final byte[] b_sin_cos_offsets = {
			P__COS_Y_COS_Z__P__SIN_Y_SIN_X_SIN_Z, COS_Y_SIN_Z__M__SIN_Y_SIN_X_COS_Z, SIN_Y_COS_X,
			M__COS_X_SIN_Z, COS_X_COS_Z, SIN_X,
			COS_Y_SIN_X_SIN_Z__M__SIN_Y_COS_Z, M__COS_Y_SIN_X_COS_Z__M__SIN_Y_SIN_Z, COS_Y_COS_X,

			P__COS_Y_COS_Z__P__SIN_Y_SIN_X_SIN_Z, M__COS_X_SIN_Z, COS_Y_SIN_X_SIN_Z__M__SIN_Y_COS_Z,
			COS_Y_SIN_Z__M__SIN_Y_SIN_X_COS_Z, COS_X_COS_Z, M__COS_Y_SIN_X_COS_Z__M__SIN_Y_SIN_Z,
			SIN_Y_COS_X, SIN_X, COS_Y_COS_X,

			COS_Z, SIN_Z,
			M_SIN_Z, COS_Z
	};

	// make_sin_cos
	private static void makeSinCos() {
		final short[] a5 = w_sin_cos_values;

		a5[ SIN_Y_COS_Z ] = a5[ SIN_Y_SIN_Z ] = a5[ SIN_Y_SIN_X_COS_Z ]
				= a5[ SIN_Y_SIN_X_SIN_Z ] = a5[ SIN_Y_COS_X ] = Trigonometry.getSin( w_players_y_angle );
		a5[ COS_Y_COS_Z ] = a5[ COS_Y_SIN_Z ] = a5[ COS_Y_SIN_X_COS_Z ]
				= a5[ COS_Y_SIN_X_SIN_Z ] = a5[ COS_Y_COS_X ] = Trigonometry.getCos( w_players_y_angle );

		final int d0 = w_players_y_angle - w_section_y_angle;
		a5[ COS_X_SIN_DY ] = a5[ COS_Z_SIN_DY ] = a5[ SIN_X_SIN_Z_SIN_DY ] = Trigonometry.getSin( d0 );
		a5[ COS_X_COS_DY ] = a5[ COS_Z_COS_DY ] = a5[ SIN_X_SIN_Z_COS_DY ] = Trigonometry.getCos( d0 );

		a5[ SIN_X ] = Trigonometry.getSin( w_players_x_angle );

		a5[ COS_X_COS_Z ] = a5[ COS_X_SIN_Z ] = a5[ COS_X ] = Trigonometry.getCos( w_players_x_angle );

		a5[ COS_Z ] = Trigonometry.getCos( w_players_z_angle );

		a5[ SIN_Z ] = Trigonometry.getSin( w_players_z_angle );

		final short sin_x = a5[ SIN_X ];// sin x *
		a5[ SIN_Y_SIN_X_SIN_Z ] = (short) (((a5[ SIN_Y_SIN_X_SIN_Z ] * sin_x) << 1) >> 16);
		a5[ SIN_Y_SIN_X_COS_Z ] = (short) (((a5[ SIN_Y_SIN_X_COS_Z ] * sin_x) << 1) >> 16);
		a5[ COS_Y_SIN_X_SIN_Z ] = (short) (((a5[ COS_Y_SIN_X_SIN_Z ] * sin_x) << 1) >> 16);
		a5[ COS_Y_SIN_X_COS_Z ] = (short) (((a5[ COS_Y_SIN_X_COS_Z ] * sin_x) << 1) >> 16);

		a5[ SIN_X_SIN_Z_SIN_DY ] = (short) (((a5[ SIN_X_SIN_Z_SIN_DY ] * sin_x) << 1) >> 16);
		a5[ SIN_X_SIN_Z_COS_DY ] = (short) (((a5[ SIN_X_SIN_Z_COS_DY ] * sin_x) << 1) >> 16);

		a5[ SIN_Y_SIN_X ] = a5[ SIN_Y_SIN_X_SIN_Z ];
		a5[ COS_Y_SIN_X ] = a5[ COS_Y_SIN_X_SIN_Z ];

		final short cos_x = a5[ COS_X ];
		a5[ SIN_Y_COS_X ] = (short) (((a5[ SIN_Y_COS_X ] * cos_x) << 1) >> 16);
		a5[ COS_Y_COS_X ] = (short) (((a5[ COS_Y_COS_X ] * cos_x) << 1) >> 16);

		a5[ COS_X_SIN_DY ] = (short) (((a5[ COS_X_SIN_DY ] * cos_x) << 1) >> 16);
		a5[ COS_X_COS_DY ] = (short) (((a5[ COS_X_COS_DY ] * cos_x) << 1) >> 16);

		final short sin_z = a5[ SIN_Z ];
		a5[ SIN_Y_SIN_X_SIN_Z ] = (short) (((a5[ SIN_Y_SIN_X_SIN_Z ] * sin_z) << 1) >> 16);
		a5[ COS_Y_SIN_X_SIN_Z ] = (short) (((a5[ COS_Y_SIN_X_SIN_Z ] * sin_z) << 1) >> 16);
		a5[ SIN_Y_SIN_Z ] = (short) (((a5[ SIN_Y_SIN_Z ] * sin_z) << 1) >> 16);
		a5[ COS_Y_SIN_Z ] = (short) (((a5[ COS_Y_SIN_Z ] * sin_z) << 1) >> 16);
		a5[ COS_X_SIN_Z ] = (short) (((a5[ COS_X_SIN_Z ] * sin_z) << 1) >> 16);

		a5[ SIN_X_SIN_Z_SIN_DY ] = (short) (((a5[ SIN_X_SIN_Z_SIN_DY ] * sin_z) << 1) >> 16);
		a5[ SIN_X_SIN_Z_COS_DY ] = (short) (((a5[ SIN_X_SIN_Z_COS_DY ] * sin_z) << 1) >> 16);

		final short cos_z = a5[ COS_Z ];
		a5[ SIN_Y_SIN_X_COS_Z ] = (short) (((a5[ SIN_Y_SIN_X_COS_Z ] * cos_z) << 1) >> 16);
		a5[ COS_Y_SIN_X_COS_Z ] = (short) (((a5[ COS_Y_SIN_X_COS_Z ] * cos_z) << 1) >> 16);
		a5[ SIN_Y_COS_Z ] = (short) (((a5[ SIN_Y_COS_Z ] * cos_z) << 1) >> 16);
		a5[ COS_Y_COS_Z ] = (short) (((a5[ COS_Y_COS_Z ] * cos_z) << 1) >> 16);
		a5[ COS_X_COS_Z ] = (short) (((a5[ COS_X_COS_Z ] * cos_z) << 1) >> 16);

		a5[ COS_Z_COS_DY ] = (short) (((a5[ COS_Z_COS_DY ] * cos_z) << 1) >> 16);
		a5[ COS_Z_SIN_DY ] = (short) (((a5[ COS_Z_SIN_DY ] * cos_z) << 1) >> 16);

		a5[ COS_Y_SIN_Z__M__SIN_Y_SIN_X_COS_Z ] = (short) (a5[ COS_Y_SIN_Z ] - a5[ SIN_Y_SIN_X_COS_Z ]);
		a5[ M__COS_Y_SIN_X_COS_Z__M__SIN_Y_SIN_Z ] = (short) (-a5[ COS_Y_SIN_X_COS_Z ] - a5[ SIN_Y_SIN_Z ]);
		a5[ P__COS_Y_COS_Z__P__SIN_Y_SIN_X_SIN_Z ] = (short) (a5[ COS_Y_COS_Z ] + a5[ SIN_Y_SIN_X_SIN_Z ]);
		a5[ COS_Y_SIN_X_SIN_Z__M__SIN_Y_COS_Z ] = (short) (a5[ COS_Y_SIN_X_SIN_Z ] - a5[ SIN_Y_COS_Z ]);
		a5[ M__COS_X_SIN_Z ] = (short) (-a5[ COS_X_SIN_Z ]);
		a5[ M_SIN_Z ] = (short) (-a5[ SIN_Z ]);
	}

	// calculate_screen_x2
	private static void calculateScreenX2(int sd0_x, final int sd1_off, final int sd3_z) {
		final long r = Trigonometry.calculatePerspectiveCoord( sd0_x, sd3_z );
		w_screen_x_coords[ sd1_off / 2 ] = (short) ((((int) (short) r - w_x_shift) >> 3) * PF);

		w_perspective_z = (short) Trigonometry.calculatePerspectiveValue( r );
	}

	// calculate_screen_x
	/**
	 * @param sd1 x point index to store in the array {@link #w_screen_x_coords}.
	 *            return: {@link #w_screen_x_coords}, {@link #w_perspective_z}
	 */
	private static void calculateScreenX(final int sd1) {
		int sd0_x = w_near_x_coord;
		int sd3 = w_near_z_coord;
		if( b_track_preview ) {
			sd0_x >>= 1;
			sd3 >>= 1;
			w_perspective_z = (short) (sd3 + 49 * 256);
			sd3 >>= 1;
			sd3 += 0x4900;
			w_screen_x_coords[ sd1 / 2 ] =
					(short) ((((int) (short) Trigonometry.calculatePerspectiveCoord( sd0_x, sd3 ) - w_x_shift) >> 3) * PF);
			return;
		}
		// calculate_screen_x2
		calculateScreenX2( sd0_x, sd1, sd3 );
	}

	// calculate_screen_y2
	/**
	 * @param d0 y coordinate
	 * @param d1 offset to {@link #w_screen_y_coords}
	 * @param d3 perspective z as word
	 */
	private static void calculateScreenY2(final int d0, final int d1, final int d3) {
		w_screen_y_coords[ d1 / 2 ] =
				(short) ((((int) (short) Trigonometry.calculatePerspectiveCoord( d0, d3 ) - w_y_shift) >> 3) * PF);
	}

	// calculate_screen_y
	/**
	 * in: {@link #w_perspective_z}
	 *
	 * @param sd1 y point index to store in the array {@link #w_screen_y_coords}.
	 *            return: {@link #w_screen_y_coords}
	 */
	private static void calculateScreenY(final int sd1) {
		int sd0 = (w_y_perspective_shift - w_coord_y_values_fp5[ sd1 / 2 ]) >> 2;
		if( b_track_preview ) {
			sd0 = (sd0 * 19483) >> 15;// 19483 / 32768
		}
		calculateScreenY2( sd0, sd1, w_perspective_z );
	}

	// z_rotate, z_rotate1
	/**
	 * Rotate X and Y around Z axis
	 *
	 * @param coordIndexX2 an index of X and Y values in d1 as word
	 */
	private static void zRotate(int coordIndexX2) {
		coordIndexX2 >>= 1;
		final int sd5_x = w_screen_x_coords[ coordIndexX2 ];
		final int sd4_y = w_screen_y_coords[ coordIndexX2 ];

		int sd0 = ((w_sin_cos_values[ COS_Z ] * sd5_x) << 1) >> 16;
		int sd3 = ((w_sin_cos_values[ SIN_Z ] * sd4_y) << 1) >> 16;
		sd0 = ((sd0 + sd3) >> 2) + SCREEN_AREA_WIDTH / 2;// java: SCREEN_AREA_LEFT_X + 128px
		w_screen_x_coords[ coordIndexX2 ] = (short) sd0;

		sd0 = ((w_sin_cos_values[ COS_Z ] * sd4_y) << 1) >> 16;
		sd3 = ((w_sin_cos_values[ SIN_Z ] * sd5_x) << 1) >> 16;
		sd0 = ((sd0 - sd3) >> 2) + SCREEN_AREA_HEIGHT / 2;// java: SCREEN_AREA_TOP_Y + 64px
		w_screen_y_coords[ coordIndexX2 ] = (short) sd0;
	}

	// z_rotate0
	/**
	 * Rotate {@link #w_coord_y_values_fp5} at specified offset
	 *
	 * @param sd1 an offset in bytes to {@link #w_coord_y_values_fp5}
	 */
	private static void zRotate0(final int sd1) {
		if( w_coord_y_values_fp5[ sd1 / 2 ] >= 0 ) {
			zRotate( sd1 );
		}
	}

	// initialise_data
	private static void initialiseData() {
		jClearLoop();

		// League values
		if( 0 != b_league_offset ) {
			w_1baf8 = SUPER_LEAGUE_w_1baf8;
			engine_power = SUPER_LEAGUE_ENGINE_POWER;
			w_opponents_engine_power = SUPER_LEAGUE_OPPONENT_ENGINE_POWER;
			boost_unit_value = SUPER_LEAGUE_BOOST_UNIT_VALUE;
			b_road_cushion_value = SUPER_LEAGUE_ROAD_CUSHION_VALUE;
		} else {
			w_1baf8 = STANDARD_LEAGUE_w_1baf8;
			engine_power = STANDARD_LEAGUE_ENGINE_POWER;
			w_opponents_engine_power = STANDARD_LEAGUE_OPPONENT_ENGINE_POWER;
			boost_unit_value = STANDARD_LEAGUE_BOOST_UNIT_VALUE;
			b_road_cushion_value = STANDARD_LEAGUE_ROAD_CUSHION_VALUE;
		}
		// end

		w_old_speed_bar = (SPEED_BAR_MAX_LENGTH - 1 * PF);
		b_1bbdd = (byte) 0xBA;

		b_time_results_min[ 2 ] = 9;
		b_time_results_min[ 1 ] = 9;
		b_time_results_min[ 0 ] = 9;

		resetTimeResult( (b_racer_id_3 + 12) & 0xFF );

		if( 0 != b_machine ) {
			resetTimeResult( b_opponents_id + 12 );
		}

		makeSinCos();
		w_y_shift = 1024;
		w_1bc56 = (short) 0xff00;
		SoundManager.soundOff();

		b_lap_that_finishes_race = 4;
		setRandomValues();
		lw_random_long[ 3 ] = 0x3B;

		final byte boost = (0 != b_league_offset)
				? b_super_league_boost
				: b_standard_league_boost;
		// Amiga uses binary decimal format
		b_boost_reserve = boost;
		b_boost_max_units = boost;
		b_1bbb5 = b_damage_hole_position;

		synchronized( key_array ) {
			key_array.clear();// key_array = 0;// Arrays.fill( key_array, (short) 0 );
		}

		Arrays.fill( w_screen_coords, (short) 0 );
		// Arrays.fill( end_edge_space, (short) 0 );

		// setSpriteColours();
		// setCIAs();

		id2();
	}

	/**
	 * returns:
	 * sa1 = {@link #w_coord_y_values_fp5}
	 * ba2 = {@link #b_near_section_flags}
	 */
	private static void id2() {
		int i = 62;
		do {
			w_coord_y_values_fp5[ (i + OFFSET_FOR_BOTTOM_X2) / 2 ] = (short) 0x8000;
			b_near_section_flags[ i ] = (byte) 0x80;
			i -= 2;
		} while( i >= 0 );
	}

	// private static int lw_random_long = 0;
	// private static byte b_random_byte = 0;// dc.b 0,0
	private static final byte[] lw_random_long = { 0, 0, 0, 0, 0 };

	// randomize_long
	/**
	 * @return d0 as i16
	 */
	private static int randomizeLong() {
		/*
		final int d0 = ((lw_random_long & 0xFFFF) >>> 4) ^ (lw_random_long >>> 17);
		lw_random_long = ((lw_random_long << 8) & 0xFFFFFF00) | (b_random_byte & 0xFF);
		b_random_byte = (byte) d0;
		*/
		int d0 = (((lw_random_long[ 2 ] & 0xFF) << 8) + (lw_random_long[ 3 ] & 0xFF)) >>> 4;
		final int d3 = (((lw_random_long[ 0 ] & 0xFF) << 8) + (lw_random_long[ 1 ] & 0xFF)) >>> 1;
		d0 ^= d3;
		lw_random_long[ 0 ] = lw_random_long[ 1 ];
		lw_random_long[ 1 ] = lw_random_long[ 2 ];
		lw_random_long[ 2 ] = lw_random_long[ 3 ];
		lw_random_long[ 3 ] = lw_random_long[ 4 ];
		lw_random_long[ 4 ] = (byte) d0;
		return d0;
	}

	// private static int lw_random_seed = 0x3b3b1e49;//, 0x3b3b3562
	private static final byte[] lw_random_seed = { 0x3b, 0x3b, 0x1e, 0x49 };//, 0x3b3b3562

	// set_random_values
	private static void setRandomValues() {
		// lw_random_long = lw_random_seed;
		// b_random_byte = (byte) (lw_random_seed >>> 16);
		System.arraycopy( lw_random_seed, 0, lw_random_long, 0, 4 );
		lw_random_long[ 4 ] = lw_random_long[ 1 ];
	}

	// save_random_values
	private static void saveRandomValues() {
		// lw_random_seed = (lw_random_long & 0xFF00FFFF) | (((int) b_random_byte << 16) & 0x00FF0000);
		System.arraycopy( lw_random_long, 0, lw_random_seed, 0, 4 );
		lw_random_seed[ 1 ] = lw_random_long[ 4 ];
	}

	private static void mainInitialisation2() {

		b_1ca35 = 0;
		b_57c67 = 0;
		w_57c72 = 0;

		Arrays.fill( b_won_races, (byte) 0 );
		Arrays.fill( b_best_lap_races, (byte) 0 );
		Arrays.fill( b_raced, (byte) 0 );
		Arrays.fill( b_total_points, (byte) 0 );
		// Arrays.fill( b_rating_racer_ids, (byte) 0 );
		for( int i = 0; i < b_rating_racer_ids.length; i++ ) {// length is 12
			b_rating_racer_ids[ i ] = (byte) i;
		}

		Arrays.fill( b_DAT_1ca36, (byte) 10 );
	}

	// main_initialisation
	private static void mainInitialisation() {

		Arrays.fill( w_coord_y_values_fp5, (short) 0 );// ds.w 160
		Arrays.fill( w_screen_x_coords, (short) 0 );// ds.w 160
		Arrays.fill( w_screen_y_coords, (short) 0 );// ds.w 160
		Arrays.fill( w_sin_cos_values, (short) 0 );// ds.w 36
		// ds.w 4

		Arrays.fill( b_road_under_map, (byte) 0 );

		Arrays.fill( w_sparks_x, (short) 0 );
		// ds.w 32
		Arrays.fill( w_sparks_y, (short) 0 );
		Arrays.fill( w_sparks_clouds_dx, (short) 0 );
		Arrays.fill( w_sparks_clouds_dy, (short) 0 );
		// ds.w 32

		Arrays.fill( b_left_y_coordinate_ids, (byte) 0 );
		Arrays.fill( b_right_y_coordinate_ids, (byte) 0 );
		Arrays.fill( b_road_piece_xz_positions, (byte) 0 );
		Arrays.fill( b_road_section_angle_and_piece, (byte) 0 );
		Arrays.fill( w_overall_left_y_shifts, (short) 0 );
		Arrays.fill( w_overall_right_y_shifts, (short) 0 );

		Arrays.fill( w_distances_around_road, (short) 0 );

		Arrays.fill( b_DAT_1c8a8, (byte) 0 );
		Arrays.fill( b_DAT_1c8c8, (byte) 0 );
		Arrays.fill( b_DAT_1c8e8, (byte) 0 );

		Arrays.fill( b_time_results_min, (byte) 0 );
		Arrays.fill( b_time_results_sec, (byte) 0 );
		Arrays.fill( b_time_results_ms, (byte) 0 );

		// dc.w 0

		Arrays.fill( b_opponents_speed_values, (byte) 0 );

		Arrays.fill( b_encoded_players_name, (byte) 0 );

		Arrays.fill( b_DAT_1c9c2, (byte) 0 );

		b_players_division = 0;
		b_damage_hole_position = 0;
		b_league_offset = 0;// dc.b 0,0

		Arrays.fill( b_DAT_1c9d2, (byte) 0 );

		Arrays.fill( b_won_races, (byte) 0 );

		Arrays.fill( b_best_lap_races, (byte) 0 );

		Arrays.fill( b_raced, (byte) 0 );

		Arrays.fill( b_total_points, (byte) 0 );

		Arrays.fill( b_rating_racer_ids, (byte) 0 );

		b_number_of_road_pieces = 0;
		b_players_start_piece = 0;
		b_near_start_line_section = 0;
		b_half_a_lap_section = 0;
		w_total_road_distance = 0;
		b_boost_reserve = 0;
		b_1ca21 = 0;
		b_race_mode = 0;
		b_best_lap_racer_id = 0;
		b_race_winner_id = 0;
		b_1ca25 = 0;
		b_1ca26 = 0;
		b_racer_id_1 = 0;
		b_racer_id_2 = 0;
		b_opponents_id = 0;
		b_standard_league_damage_limit = 0;
		b_super_league_damage_limit = 0;
		b_standard_league_boost = 0;
		b_super_league_boost = 0;
		b_1ca2e = 0;
		b_1ca2f = 0;
		// d.b 0
		b_save = false;
		b_start_finish_section = 0;
		b_road_id = 0;
		b_1ca34 = 0;
		b_1ca35 = 0;
		//----

		initialiseData();
		b_5d724 = 0;

		int id1 = b_DAT_1c9c2.length - 1;
		do {
			b_DAT_1c9c2[ id1 ] = (byte) id1;
			randomizeLong();
		} while( --id1 >= 0 );

		mainInitialisation2();

		b_damage_hole_position = 10;
	}

	// set_wheel_frame_number
	private static void setWheelFrameNumber() {
		int frame = b_wheel_frame_number;
		if( w_players_z_speed < 0 ) {
			if( --frame < 0 ) {// wheels rotating backwards
				frame = SPRITE_WHEEL_SPRITES - 1;
			}
		} else {
			if( ++frame >= SPRITE_WHEEL_SPRITES ) {// wheels rotating forwards
				frame = 0;
			}
		}
		b_wheel_frame_number = (byte) frame;
	}

	// update_wheel_rotation
	private static void updateWheelRotation(final Canvas g) {
		int bd0 = (b_wheel_frame_count & 0xFF) + (w_wheel_rotation_speed >>> 8);
		b_wheel_frame_count = (byte) bd0;
		if( bd0 > 255 ) {
			setWheelFrameNumber();
		}

		// java: always use large sprites
		bd0 = b_wheel_frame_number + 37;// 37, 38, 39
		// if( graphic_info[ SPRITE_RIGHT_WHEELS ].mY < WHEEL_TOP_Y ) {// right wheels graphic.info + 10 < 126
			bd0 += 6;// 43, 44, 45 use larger hardware sprites
		// }
		int sd1 = 0;// right
		setWheelSpritePointers( g, bd0, sd1 );

		bd0 = 5 + 37 - b_wheel_frame_number;// 42, 41, 40
		// if( graphic_info[ SPRITE_LEFT_WHEELS ].mY < WHEEL_TOP_Y ) {// left wheels graphic.info + 3 * 16 + 10 < 126
			bd0 += 6;// 48, 47, 46 use larger hardware sprites
		// }
		sd1 = 1;// left
		setWheelSpritePointers( g, bd0, sd1 );
	}

	// frames_wheels_engine
	private static void framesWheelsEngine(final Canvas g) {
		if( b_frame_count != 0 ) {
			--b_frame_count;
		}
		if( fade_frame_count != 0 ) {
			--fade_frame_count;
		}
		if( (b_5d724 < 0) && !b_no_wheel_update ) {
			updateWheelRotation( g );
		}
		int revs = w_engine_revs + w_engine_revs_change;
		if( revs < 0 ) {
			if( b_turn_engine_off ) {
				SoundManager.soundOff();
				return;
			}
			revs = 0;
		}
		w_engine_revs = (short) revs;

		revs += 378;
		int sd3 = 4_800_000 / revs;
		if( sd3 > 0x3FFE ) {
			sd3 = 0x3FFE;
		}
		sd3 |= (b_engine_fluctuation & 0xFF);
		if( sd3 < 124 ) {// lowest possible period
			sd3 = 124;
		}
		SoundManager.pitchEngineSound( sd3 );
		/* TODO make engine sound pitch
		int sd4 = 6;
		do {
			do {
				if( sd3 < 256 ) {
					move.l	#lw_engine_pitch_table, a0
					lea custom, a1
					asl.w	#3, d4
					move.l( a0, d4.w ), aud0lch( a1 )
					move.w 6 (a0, d4.w),aud0len( a1 )
					engine_period = sd3;
					return;
				}
				sd3 >>>= 1;
			} while( --sd4 >= 0 );
			sd4 = 0;
		} while( true );
		*/
	}

	// R_5ab00
	private static void R_5ab00() {
		if( 0 != b_multi_no_of_players ) {
			w_fp_y = (byte) ((b_multi_no_of_players ^ 0xFF) + NUMBER_OF_RACERS);
			w_fp_y2 = NUMBER_OF_RACERS;
			return;
		}
		w_fp_y = b_TAB_5ab42[ b_division ];
		w_fp_y2 = (short) (w_fp_y + 3);
	}

	private static final byte[] b_TAB_58146 = {// TODO verify using b_TAB_58146
			// for each player
			(byte) 0x00, (byte) 0x02, (byte) 0x04, (byte) 0x0a, (byte) 0x16, (byte) 0x2a, (byte) 0x48, (byte) 0x72,
			(byte) 0x01, (byte) 0x01, (byte) 0x03, (byte) 0x06, (byte) 0x0a, (byte) 0x0f, (byte) 0x15, (byte) 0x1c,

			(byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x01, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x02, (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x00, (byte) 0x02,
			(byte) 0x01, (byte) 0x03, (byte) 0x00, (byte) 0x03, (byte) 0x02, (byte) 0x01, (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04,
			(byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x03, (byte) 0x04, (byte) 0x02, (byte) 0x00, (byte) 0x03, (byte) 0x01, (byte) 0x04, (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x05,
			(byte) 0x01, (byte) 0x02, (byte) 0x00, (byte) 0x05, (byte) 0x03, (byte) 0x04, (byte) 0x00, (byte) 0x02, (byte) 0x03, (byte) 0x05, (byte) 0x01, (byte) 0x04, (byte) 0x00, (byte) 0x03, (byte) 0x04, (byte) 0x02,
			(byte) 0x05, (byte) 0x01, (byte) 0x00, (byte) 0x04, (byte) 0x01, (byte) 0x03, (byte) 0x02, (byte) 0x05, (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x05, (byte) 0x06, (byte) 0x00,
			(byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x05, (byte) 0x06, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x04, (byte) 0x03, (byte) 0x05, (byte) 0x06, (byte) 0x02, (byte) 0x00, (byte) 0x04,
			(byte) 0x01, (byte) 0x05, (byte) 0x03, (byte) 0x06, (byte) 0x02, (byte) 0x04, (byte) 0x00, (byte) 0x05, (byte) 0x01, (byte) 0x03, (byte) 0x06, (byte) 0x04, (byte) 0x00, (byte) 0x03, (byte) 0x01, (byte) 0x06,
			(byte) 0x02, (byte) 0x05, (byte) 0x00, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x05, (byte) 0x06, (byte) 0x07, (byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x03, (byte) 0x04, (byte) 0x06,
			(byte) 0x05, (byte) 0x07, (byte) 0x00, (byte) 0x03, (byte) 0x01, (byte) 0x02, (byte) 0x04, (byte) 0x07, (byte) 0x05, (byte) 0x06, (byte) 0x00, (byte) 0x04, (byte) 0x03, (byte) 0x07, (byte) 0x01, (byte) 0x05,
			(byte) 0x02, (byte) 0x06, (byte) 0x00, (byte) 0x05, (byte) 0x01, (byte) 0x04, (byte) 0x02, (byte) 0x07, (byte) 0x03, (byte) 0x06, (byte) 0x00, (byte) 0x07, (byte) 0x01, (byte) 0x06, (byte) 0x02, (byte) 0x05,
			(byte) 0x03, (byte) 0x04, (byte) 0x00, (byte) 0x06, (byte) 0x01, (byte) 0x07, (byte) 0x02, (byte) 0x04, (byte) 0x03, (byte) 0x05
	};

	// R_580d8
	private static void R_580d8() {
		int ba2i = 0;
		int ba1i = 8;
		int ba0i = 16;

		int d3 = w_57c72;
		int d0 = b_multi_no_of_players;
		int d5 = b_TAB_58146[ ba1i + d0 ];
		int d2 = b_TAB_58146[ ba2i + d0 ];
		// FIXME lea( a0, d2.w ), a0
		final int ba0i_3 = b_TAB_58146[ ba0i + d2 ];
		final int ba0i_2 = b_TAB_58146[ ba0i + d2 + 1 ];
		final int ba0i_1 = b_TAB_58146[ ba0i + d2 + 2 ];
		final int ba0i_0 = b_TAB_58146[ ba0i + d2 + 3 ];
		ba0i = (ba0i_3 << 24) | ((ba0i_2 << 16) & 0xFF0000) | ((ba0i_1 << 8) & 0xFF00) | (ba0i_0 & 0xFF);
		final byte[] ba0 = b_TAB_58146;

		while( d3 >= d5 ) {
			d3 -= d5;
		}
		d3 <<= 1;
		d0 = NUMBER_OF_RACERS - 1 - ba0[ ba0i + d3 ];
		int d1 = NUMBER_OF_RACERS - 1 - ba0[ ba0i + d3 + 1 ];
		if( b_machine != MASTER ) {
			final int temp = d0;
			d0 = d1;
			d1 = temp;
		}
		b_racer_id_1 = (byte) d0;
		b_racer_id_3 = (byte) d0;
		b_racer_id_2 = (byte) d1;
	}

	private static final byte[] b_TAB_5abc8 = { 0, 0, 0, 0, 1, 1 };
	private static final byte[] b_TAB_5abce = { 1, 1, 2, 2, 2, 2 };

	// choose_racers_in_division
	private static void chooseRacersInDivision() {
		R_5ab00();
		int d0, d2;
		if( 0 != b_multi_no_of_players ) {
			if( 0 != b_machine ) {
				R_580d8();
				return;
			}
			d2 = NUMBER_OF_RACERS - 1 - b_1ca35;
			b_racer_id_1 = (byte) d2;
			d0 = b_5eb77;
		} else {
			// R_5ab7c:
			b_racer_id_1 = b_DAT_1c9c2[ b_TAB_5abc8[ b_1ca35 ] + w_fp_y ];
			d0 = b_DAT_1c9c2[ b_TAB_5abce[ b_1ca35 ] + w_fp_y ];
			d2 = NUMBER_OF_RACERS - 1;
		}
		// R_5abba:
		b_racer_id_2 = (byte) d0;
		b_racer_id_3 = (byte) d2;
	}

	// R_5abd4
	private static void R_5abd4() {
		int d1 = 0;
		int d0 = randomizeLong() & 0xFF;
		if( d0 >= 160 ) {
			d1 = 0x40;
		}
		int w_near_x_coord_1_flags = d1;
		int d2 = b_racer_id_1;
		d1 = b_racer_id_2;
		b_raced[ d1 ]++;
		b_raced[ d2 ]++;
		d0 = b_1bbb4;
		if( b_racer_id_3 != d1 ) {
			if( b_racer_id_3 != d2 ) {
				// _label3
				if( ((b_DAT_1c9d2[ d2 ] & 0xFF) > (b_DAT_1c9d2[ d1 ] & 0xFF)) || (0 == (randomizeLong() & 0x01)) ) {
					// _label4
					w_near_x_coord_1_flags ^= 0xC0;
				}
			} else {
				d0 ^= 0xC0;
				// _label2
				w_near_x_coord_1_flags = d0;
			}
		} else {
			// _label2
			w_near_x_coord_1_flags = d0;
		}
		// _label5
		int lw_road_height_fp8_racer_id_1, lw_road_height_fp8_1_racer_id_2;
		if( 0 == (w_near_x_coord_1_flags & 0x80) ) {
			lw_road_height_fp8_1_racer_id_2 = d1;
			if( 0 != (w_near_x_coord_1_flags & 0x40) ) {
				// _label8
				lw_road_height_fp8_racer_id_1 = d2;
			} else {
				// _label6
				lw_road_height_fp8_racer_id_1 = d1;
			}
		} else {
			// _label7
			lw_road_height_fp8_1_racer_id_2 = d2;
			if( 0 == (w_near_x_coord_1_flags & 0x40) ) {
				lw_road_height_fp8_racer_id_1 = d1;
			} else {
				// _label8
				lw_road_height_fp8_racer_id_1 = d2;
			}
		}
		// R_5aca6
		if( (0 == b_machine) || !b_1bb5b ) {
			// _label9
			b_won_races[ lw_road_height_fp8_1_racer_id_2 ]++;
			b_best_lap_races[ lw_road_height_fp8_racer_id_1 ]++;
			if( b_players_division == b_division ) {
				b_1ca26 = (byte) lw_road_height_fp8_racer_id_1;
				b_1ca25 = (byte) lw_road_height_fp8_1_racer_id_2;
			}
		}
		// _labela
	}

	private static final byte[] b_TAB_5ab42 = { 9, 6, 3, 0 };

	// prepare_division
	/**
	 * Prepare division
	 */
	private static void prepareDivision() {
		int d1 = 6;// races in division
		if( 0 != b_multi_no_of_players ) {
			d1 = 1;
			b_5eb77 = 1;
			do {
				int d0 = d1;
				int d2 = b_players_division;
				if( 0 != d2 ) {
					d0 = 1 + b_5eb7e + (randomizeLong() & 1);
					if( d0 >= 3 ) {
						d0 -= 3;
					}
					b_5eb7e = (byte) d0;
				}
				d0 += b_TAB_5ab42[ d2 ];// result is 0...12 ? player id
				d2 = d0;
				b_DAT_59140[ d1 ] = (byte) d0;
				b_opponent_attributes[ d1 ] = opponent_attributes_source[ d2 ];
				System.arraycopy( Text.opponents_names_sources[ d2 ], 0, opponents_names[ d1 ], 0, Text.opponents_names_sources[ d2 ].length );
			} while( --d1 >= 0 );
			d1 = 1 + b_multi_no_of_players;
		}
		b_max_races_in_division = (byte) d1;
		b_current_race_number = 0;
	}

	// compute_race_rating
	private static void computeRaceRating() {
		R_5ab00();

		int d2 = w_fp_y;

		// _label1
		do {
			int d1 = b_DAT_1c9c2[ d2 ];
			b_rating_racer_ids[ d2 ] = (byte) d1;
			int d0 = b_won_races[ d1 ] << 1;// 2 pts
			d0 += b_best_lap_races[ d1 ];// 1 pt
			b_total_points[ d1 ] = (byte) d0;
		} while( ++d2 < w_fp_y2 );
		// _label2
		int lw_road_height_fp8_1;
		do {
			lw_road_height_fp8_1 = 0;
			d2 = w_fp_y;

			// _label3
			do {
				int lw_road_height_fp8 = d2;
				int d1 = b_rating_racer_ids[ d2 ];
				d2 = b_rating_racer_ids[ d2 + 1 ];
				// move.l	#b_DAT_1ca02,a1
				boolean is_exchange = (b_total_points[ d1 ] < b_total_points[ d2 ]);
				if( !is_exchange ) {
					if( b_total_points[ d1 ] == b_total_points[ d2 ] ) {
						is_exchange = (b_won_races[ d1 ] < b_won_races[ d2 ]);
						if( !is_exchange ) {
							if( b_won_races[ d1 ] == b_won_races[ d2 ] ) {
								if( 0 != b_multi_no_of_players ) {
									is_exchange = (d1 < d2);
								} else {
									is_exchange = (0 != (randomizeLong() & 0x01));
								}
							}
						}
					}
				}
				if( is_exchange ) {
					// _label5
					b_rating_racer_ids[ lw_road_height_fp8 + 1 ] = (byte) d1;
					b_rating_racer_ids[ lw_road_height_fp8 ] = (byte) d2;
					++lw_road_height_fp8_1;
				}
				// _label6
				d2 = lw_road_height_fp8 + 2;
				if( d2 >= w_fp_y2 ) {
					break;
				}
				--d2;
			} while( true );
			// _label7
		} while( 0 != lw_road_height_fp8_1 );
	}

	private static final byte[] b_TAB_5aaf4 = {
			120, 110, 100, 90, 80, 70, 60, 50, 40, 30, 20, 10
		};

	// promote
	private static void promote() {
		int d1 = NUMBER_OF_RACERS - 1;
		// _label1
		do {
			b_DAT_1c9d2[ d1 ] = (byte) ((randomizeLong() & 0x3f) + b_TAB_5aaf4[ d1 ]);
		} while( --d1 >= 0 );
		d1 = b_1ca35;// TODO verify using
		int d0 = 0;
		// _label2
		do {
			b_division = (byte) d0;
			if( (0 == b_multi_no_of_players) || (b_players_division == d0) ) {
				// _label3
				chooseRacersInDivision();
				R_5abd4();
				computeRaceRating();
			}
			// _label4
			d0 = b_division;
		} while( ++d0 < MAX_DIVISIONS );
	}

	// private static final short[] DAT_58880 = new short[ 4 ];
	/**
	 * 0:min, 1:min, 2:sec, 3:sec, 4:ms, 5:ms
	 */
	private static final byte[] DAT_58880 = new byte[ 6 ];

	// R.5886c
	private static void R_5886c(int d1, int d2) {
		// 0,2,4 : 1,3,5
		DAT_58880[ d2 ] = b_time_results_min[ d1 ];
		DAT_58880[ d2 + 2 ] = b_time_results_sec[ d1 ];
		DAT_58880[ d2 + 4 ] = b_time_results_ms[ d1 ];
	}

	// R.58858
	private static void R_58858(int d1, int d2) {
		b_time_results_min[ d2 ] = DAT_58880[ d1 ];
		b_time_results_sec[ d2 ] = DAT_58880[ d1 + 2 ];
		b_time_results_ms[ d2 ] = DAT_58880[ d1 + 4 ];
	}

	// R_58812
	private static void R_58812(int d1, int d2) {
		// move.l	#time.results.min,a0
		// move.l	#time.results.sec,a1
		// move.l	#time.results.ms,a2
		// move.l	#DAT.58880,a3

		if( 0 != DAT_58880[ d1 ] ) {
			if( 0 == b_time_results_min[ d2 ] ) {
				R_58858( d1, d2 );
				return;
			}
			R_58858( d1, 3 );
			d1 = 3;
			sortTimeResult( d1, d2 );
			/*
			move.l	#time.results.min, a0
			move.l	#time.results.sec, a1
			move.l	#time.results.ms, a2
			move.l	#DAT .58880, a3
			_label4 rts
			*/
		}
	}

	// R_5879e
	/**
	 * @param d0 as byte
	 */
	private static void R_5879e(boolean d0) {
		if( 0 == b_machine ) {
			return;
		}
		if( d0 ) {
			R_58812( 0, b_racer_id_3 + 12 );
			R_58812( 1, b_opponents_id + 12 );
			return;
		}
		R_5886c( b_racer_id_3 + 12, 0 );
		R_5886c( b_opponents_id + 12, 1 );
	}

	// R_58580
	/**
	 * @return true - z
	 */
	private static boolean R_58580() {
		int b_57c68 = testKey( KeyEvent.KEYCODE_F1 ) ? 0 : 0x80;
		if( 0 != b_machine ) {
			if( b_multi_no_of_players < 2 ) {
				return true;
			}
			if( b_machine < 0 ) {
				b_57c66 = (byte) b_57c68;
			}
			b_57c64 = (byte) 0xf4;
			R_579b0();
			if( b_machine != MASTER ) {
				b_57c68 = b_57c60;
			}
		}
		return b_57c68 == 0;
	}

	private static final byte[] b_TAB_5fb3a = {
			(byte) 0x2a, (byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a, (byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a, (byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a,
			(byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a, (byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a, (byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a, (byte) 0x40,
			(byte) 0x21, (byte) 0x22, (byte) 0x44, (byte) 0x2a, (byte) 0x40, (byte) 0x21, (byte) 0x22, (byte) 0x44
	};

	// java: extracted in place R_58472
	/* private static byte[] R_58472(int d0) {
		d0 -= 4;
		int d3 = d0;
		d0 = (d0 << 2) + d3 + 4;
		int d2 = d0;
		int d1 = 4;
		return b_TAB_5fb3a;// ba2
	} */

	// R_5848a TODO java: extract in place
	private static void R_5848a(final byte[] ba1_src, final byte[] ba2_dst, int d1, int d2) {
		do {
			ba2_dst[ d2 ] = ba1_src[ d1 ];
			--d2;
		} while( --d1 >= 0 );
	}

	private static final byte[] b_TAB_58500 = {
			(byte) 0x0e, (byte) 0x45, (byte) 0x01, (byte) 0x02, (byte) 0x03, (byte) 0x04, (byte) 0x05, (byte) 0x06, (byte) 0x07, (byte) 0x08, (byte) 0x09, (byte) 0x0a, (byte) 0x0b, (byte) 0x0c, (byte) 0x41, (byte) 0x42,
			(byte) 0x10, (byte) 0x11, (byte) 0x12, (byte) 0x13, (byte) 0x14, (byte) 0x15, (byte) 0x16, (byte) 0x17, (byte) 0x18, (byte) 0x19, (byte) 0x1a, (byte) 0x1b, (byte) 0x44, (byte) 0x63, (byte) 0x20, (byte) 0x21,
			(byte) 0x22, (byte) 0x23, (byte) 0x24, (byte) 0x25, (byte) 0x26, (byte) 0x27, (byte) 0x28, (byte) 0x29, (byte) 0x2a, (byte) 0x0d, (byte) 0x60, (byte) 0x00, (byte) 0x31, (byte) 0x32, (byte) 0x33, (byte) 0x34,
			(byte) 0x35, (byte) 0x36, (byte) 0x37, (byte) 0x38, (byte) 0x39, (byte) 0x3a, (byte) 0x61, (byte) 0x1c, (byte) 0x64, (byte) 0x40, (byte) 0x67, (byte) 0x50, (byte) 0x51, (byte) 0x52, (byte) 0x53, (byte) 0x54,
			(byte) 0x55, (byte) 0x56, (byte) 0x57, (byte) 0x58, (byte) 0x59, (byte) 0x2b, (byte) 0x2c, (byte) 0x3b, (byte) 0x4c, (byte) 0x47, (byte) 0x4a, (byte) 0x4f, (byte) 0x48, (byte) 0x4e, (byte) 0x5e, (byte) 0x49,
			(byte) 0x4d, (byte) 0x4b, (byte) 0x68, (byte) 0x69, (byte) 0x62, (byte) 0x6a, (byte) 0x66, (byte) 0x65, (byte) 0x6b, (byte) 0x6c, (byte) 0x6d, (byte) 0x6e, (byte) 0x6f, (byte) 0x70, (byte) 0x71, (byte) 0x72,
			(byte) 0x30, (byte) 0x5f, (byte) 0x46, (byte) 0x5a, (byte) 0x5b, (byte) 0x5c, (byte) 0x5d, (byte) 0x3d, (byte) 0x3e, (byte) 0x3f, (byte) 0x2d, (byte) 0x2e, (byte) 0x2f, (byte) 0x1d, (byte) 0x1e, (byte) 0x1f,
			(byte) 0x0f, (byte) 0x3c, (byte) 0x43, (byte) 0x73, (byte) 0x74, (byte) 0x75, (byte) 0x76, (byte) 0x77, (byte) 0x78, (byte) 0x79, (byte) 0x7a, (byte) 0x7b, (byte) 0x7c, (byte) 0x7d, (byte) 0x7e, (byte) 0x7f
	};

	// R_584de
	private static void R_584de() {
		int d3 = 4;
		do {
			final int d0 = b_DAT_7a41a[ d3 ];
			b_DAT_7a41a[ d3 ] = b_TAB_58500[ d0 ];
		} while( --d3 >= 0 );
	}

	// R_584a0
	private static void R_584a0() {
		int d3 = 4;
		// _label1
		do {
			int d0 = b_control_keys[ d3 ];
			int d2 = 0;

			// _label2
			while( d0 != b_TAB_58500[ d2 ] ) {
				if( 0x80 != ++d2 ) {
					continue;
				}
				d2 = 0;
				break;
			}
			// _label3
			b_DAT_5849a[ d3 ] = (byte) d2;
		} while( --d3 >= 0 );
	}

	private static final byte[] b_DAT_5849a = new byte[ 6 ];

	// R_583e8
	private static void R_583e8() {
		if( 0 == b_machine ) {
			return;
		}
		R_584a0();
		if( b_machine != MASTER ) {
			// _label1
			do {
				do {
					if( testKeyF5() ) {
						return;
					}
				} while( receiveDataBlock( b_DAT_7a41a, 5 - 1 ) );

				R_584de();
				// R_58472( b_opponents_id );
				// java: extracted in place
				int d0 = b_opponents_id;
				d0 -= 4;
				int d3 = d0;
				d0 = (d0 << 2) + d3 + 4;
				int d2 = d0;
				int d1 = 4;
				R_5848a( b_DAT_7a41a, b_TAB_5fb3a, d1, d2 );
				if( b_machine == MASTER ) {
					return;
				}
				// _label2
				do {
					if( testKeyF5() ) {
						return;
					}
				} while( transmitDataBlock( b_DAT_5849a, 5 - 1 ) );

			} while( b_machine == MASTER );
		} else {
			// _label2
			do {
				do {
					if( testKeyF5() ) {
						return;
					}
				} while( transmitDataBlock( b_DAT_5849a, 5 - 1 ) );

				if( b_machine != MASTER ) {
					return;
				}
				do {
					if( testKeyF5() ) {
						return;
					}
				} while( receiveDataBlock( b_DAT_7a41a, 5 - 1 ) );

				R_584de();
				// R_58472( b_opponents_id );
				// java: extracted in place
				int d0 = b_opponents_id;
				d0 -= 4;
				int d3 = d0;
				d0 = (d0 << 2) + d3 + 4;
				int d2 = d0;
				int d1 = 4;
				R_5848a( b_DAT_7a41a, b_TAB_5fb3a, d1, d2 );
			} while( b_machine != MASTER );
		}
	}

	// R_5faf2
	private static void R_5faf2(final boolean flag) {
		// move.b	d0,b_value_low_factor_high
		if( 0 != b_multi_no_of_players ) {
			// final byte[] ba2 = R_58472( b_5eb79 );
			// java: extracted in place
			int d2 = b_racer_id_3 - 4;
			d2 = (d2 << 2) + d2 + 4;
			int d1 = 4;
			final byte[] ba2 = b_TAB_5fb3a;
			final byte[] ba1 = b_control_keys;
			if( flag ) {
				// _label1
				do {
					ba1[ d1 ] = ba2[ d2 ];
					--d2;
				} while( --d1 >= 0 );
				// _label2
				return;
			}
			R_5848a( ba1, ba2, d1, d2 );
			R_583e8();
		}
	}

	// R_5867a
	private static void R_5867a() {
		if( 0 == b_machine ) {
			return;
		}
		if( b_machine != MASTER ) {
			// _label1
			do {
				if( testKeyF5() ) {
					return;
				}
			} while( receiveDataBlock( b_DAT_7a41a, 100 - 1 ) );
			// java version
			System.arraycopy( b_DAT_7a41a, 0, b_time_results_min, 0, b_time_results_min.length );
			System.arraycopy( b_DAT_7a41a, b_time_results_min.length, b_time_results_sec, 0, b_time_results_sec.length );
			System.arraycopy( b_DAT_7a41a, b_time_results_min.length + b_time_results_sec.length, b_time_results_ms, 0, b_time_results_ms.length );

			int ba6i = b_time_results_min.length + b_time_results_sec.length + b_time_results_ms.length;
			b_1bbb4 = (byte) (b_DAT_7a41a[ ba6i++ ] ^ 0xC0);
			b_1bbb1 = b_DAT_7a41a[ ba6i++ ];
			b_best_lap_racer_id = b_DAT_7a41a[ ba6i++ ];
			b_race_winner_id = b_DAT_7a41a[ ba6i++ ];

			Text.b_TEXT_5ec48[ 1 ].mText = new String( b_DAT_7a41a, ba6i + 12,  Text.RACER_NAME_LENGTH, StandardCharsets.UTF_8 );
			return;
		}
		// _label4
		// java version
		System.arraycopy( b_time_results_min, 0, b_DAT_7a41a, 0, b_time_results_min.length );
		System.arraycopy( b_time_results_sec, 0, b_DAT_7a41a, b_time_results_min.length, b_time_results_sec.length );
		System.arraycopy( b_time_results_ms, 0, b_DAT_7a41a, b_time_results_min.length + b_time_results_sec.length, b_time_results_ms.length );

		int ba6i = b_time_results_min.length + b_time_results_sec.length + b_time_results_ms.length;
		b_DAT_7a41a[ ba6i++ ] = b_1bbb4;// TODO verify. why is "^ 0xc0" not used ?
		b_DAT_7a41a[ ba6i++ ] = b_1bbb1;
		b_DAT_7a41a[ ba6i++ ] = b_best_lap_racer_id;
		b_DAT_7a41a[ ba6i++ ] = b_race_winner_id;

		System.arraycopy( Text.b_TEXT_5ec48[ 1 ].mText.getBytes( StandardCharsets.UTF_8 ), 0, b_DAT_7a41a, ba6i + 12, 12 );

		// _label7
		do {
			if( testKeyF5() ) {
				return;
			}
		} while( transmitDataBlock( b_DAT_7a41a, 100 - 1 ) );
	}

	// R_5f25a
	private static void R_5f25a(final boolean flag) {
		// b_value_low_factor_high = (byte) d0;
		int d1 = b_road_id;
		byte[] ba1 = b_TAB_648c2;
		int d0 = ba1[ d1 ];
		int d2 = b_league_offset;
		if( 0 != d2 ) {
			d0 += 8;
		}
		d0 <<= 4;
		d1 = d0;
		d2 = 0;

		if( ! flag ) {
			// _label2
			do {
				Text.TextInfo[] lga2 = Text.b_TEXT_5ec48;
				d0 = lga2[ 0 ].mText.charAt( d2 );
				Text.b_TEXT_7a61a[ d1 ] = (byte) d0;
				// move.l	#b_TEXT_5ec48 + 13, a2
				d0 = lga2[ 1 ].mText.charAt( d2 );
				Text.b_TEXT_7a71a[ d1 ] = (byte) d0;
				++d1;
			} while( 12 != ++d2 );
			ba1 = Text.b_TEXT_7a61a;
			ba1[ d1 ] = b_time_results_min[ 14 ];
			ba1[ d1 + 1 ] = b_time_results_sec[ 14 ];
			ba1[ d1 + 2 ] = b_time_results_ms[ 14 ];

			ba1 = Text.b_TEXT_7a71a;
			ba1[ d1 ] = b_time_results_min[ 15 ];
			ba1[ d1 + 1 ] = b_time_results_sec[ 15 ];
			ba1[ d1 + 2 ] = b_time_results_ms[ 15 ];
			return;
		}
		Text.b_TEXT_5ec48[ 0 ].mText = new String( Text.b_TEXT_7a61a, 0, 12 );
		Text.b_TEXT_5ec48[ 1 ].mText = new String( Text.b_TEXT_7a71a, 0, 12 );
		
		b_time_results_min[ 14 ] = Text.b_TEXT_7a61a[ d1 ];
		b_time_results_sec[ 14 ] = Text.b_TEXT_7a61a[ d1 + 1 ];
		b_time_results_ms[ 14 ] = Text.b_TEXT_7a61a[ d1 + 2 ];
		
		b_time_results_min[ 15 ] = Text.b_TEXT_7a71a[ d1 ];
		b_time_results_sec[ 15 ] = Text.b_TEXT_7a71a[ d1 + 1 ];
		b_time_results_ms[ 15 ] = Text.b_TEXT_7a71a[ d1 + 2 ];
	}

	// R_5930c
	private static void R_5930c() {// show race result picture
		int d0 = RACE_RESULT_WON;
		if( b_1bbb4 < 0 ) {
			d0 = RACE_RESULT_LOST;
			if( 0 != (lw_wreck_wheel_height_reduction_fp8 & 0xFFFF) ) {
				d0 = RACE_RESULT_WRECK;
			}
		}
		R_59352( d0 );
	}

	// R_5eae0
	/**
	 *
	 * @param g Canvas
	 * @param d2 a row to print in d2 as byte
	 */
	private static void R_5eae0(final Canvas g, int d2) {
		final byte b_backup_print_row = (byte) d2;
		int d1 = b_track_name_print_x_offsets[ b_road_id ] - 6;
		if( (0 != b_multi_no_of_players) && (0 != b_league_offset) ) {
			d1 -= 2;
		}
		setPrintColumnRow( d1, d2 );

		setTextColor( 15 );
		R_5a67c( g, 11 );// 147 'Track:  The '
		printTrackName( g, b_road_id );
		if( (0 != b_multi_no_of_players) && (0 != b_league_offset) ) {
			d1 = 33;
			d2 = b_backup_print_row;
			setPrintColumnRow( d1, d2 );
			// R_5a67c( g, 7 );// 99 'RESULT'
			jPrintText( g, Text.league_text[ 7 ].mText );// java
		}
	}

	// java: is not required R_5f028
	/* private static int R_5f028(int d0) {
		// andi.b	#%1111,ccr
		// move.w	d1,-(sp)
		// 		clr.w	d1
		int d1 = d0;
		if( 0 != d1 ) {
			int d3 = 1;
			d0 = 0;
			--d1;
			while( d1 >= 0 ) {
				// bra _label2

				// _label1
				d0 += d3;// abcd.b d3, d0

				// _label2
				// dbra d1, _label1
				--d1;
			}
		}
		// _label3
		// move.w	(sp)+,d1
		return d0;
	} */

	// R_5edd2
	private static void R_5edd2(int d0) {
		if( d0 < 10 ) {
			// move.b d0, -(sp)
			printOneSpace();
			// R_5edf8();
			printDigit( d0 % 10 );
			return;
		}
		// java: is not required d0 = R_5f028( d0 );
		printTwoDigitValue( d0 );
	}

	// R_5edca
	private static void R_5edca(int d0) {
		if( d0 < 10 ) {
			printDigit( d0 );
			return;
		}
		R_5edd2( d0 );
	}

	// R_5edba
	private static void R_5edba(int d0) {
		// move.b	d0,-(sp)
		printTwoSpaces();
		// move.b	(sp)+,d0
		R_5edd2( d0 );
	}

	// display_race_number
	/**
	 * Print: "Race 1 of 2"
	 *
	 * @param g Canvas
	 * @param d0 y position, d0 as byte
	 */
	private static void displayRaceNumber(final Canvas g, int d0) {
		Text.league_text[ 1 ].mY = (byte) d0;// 13 + 2
		setTextColor( 1 );
		printLeagueText( g, 1 );// 13 'RACE  '

		R_5edca( b_current_race_number + 1 );
		printLeagueText( g, 18 );// 244 ' of '
		d0 = b_max_races_in_division;
		if( 0 != b_multi_no_of_players ) {
			d0 <<= 1;
		}
		R_5edca( d0 );
	}

	// R_5914a
	private static void R_5914a(final Canvas g, int d3) {
		jPrintText( g, Text.b_TEXT_5915e[ d3 ] );
	}

	// display_race_opponents_pictures
	private static void displayRaceOpponentsPictures(final Canvas g) {
		chooseRacersInDivision();

		displayPlayerPicture( g, b_racer_id_1, 12 );

		displayPlayerPicture( g, b_racer_id_2, 13 );
	}

	// R_5f074
	private static void R_5f074(final Canvas g) {
		if( 0 != b_1bbb1 ) {
			setTextColor( 1 );
			int d1 = 0xb8;
			b_menu_bar_type = 4;
			b_menu_bar_index = 3;
			if( 0 != (b_1bbb1 & 1) ) {
				d1 = 16;// 227 New track record
				b_menu_bar_index = 16;
				b_menu_bar_type = 1;
				R_5a67c( g, d1 );
				if( 0xC0 == (b_1bbb1 & 0xC0) ) {
					// R_5a67c( g, 9 );// 111 's'
					jPrintCharacter( 's' );// java
				}
			}
			setTextColor( 0 );
			doFillBar( g, 3 );
			if( b_1bbb1 < 0 ) {
				R_5ec7e( g, 2 );// 35 'Race Time:'
				R_5a67c( g, 15 );// 214 'name2'
				printOneSpace();
				printMinSecMs( 15 );
			}
			if( 0 != (b_1bbb1 & 0x40) ) {
				setPrintColumnRow( 5, b_current_print_row + 1 );
				R_5ec7e( g, 3 );// 47 'Best Lap : '
				R_5a67c( g, 14 );// 201 'name1'
				printOneSpace();
				printMinSecMs( 14 );
			}
		}
	}

	private static byte b_58bbb = 0;
	private static byte b_x_offset_for_players_name = 0;

	// print_players_name
	/**
	 * Java: arguments are added.
	 * Amiga prints the text into the source picture area. lw_L_5924c += 1760;
	 *
	 * @param column a column defined in 'font width' units.
	 * @param row a row defined in 'font height' units.
	 */
	private static void printPlayersName(int column, int row) {
		final boolean sp_1 = b_or_char_with_screen;
		b_or_char_with_screen = true;
		// lw_L_5924c += 1760;// +44*40, y offset from the top of the picture
		b_print_fine_x = (byte) (b_x_offset_for_players_name + 3);
		b_font_narrow = true;
		int d1 = b_58bbb;// java << 4; is not required
		// byte[] ba0 = b_opponents_names;
		// lea	1(a0,d1.w),a0
		final char[] ca0 = opponents_names[ d1 ];
		int d0 = 0;
		int d3 = Text.RACER_NAME_LENGTH - 1;
		do {
			if( ' ' != ca0[ d3 + 1 ] ) {
				break;
			}
			++d0;
		} while( --d3 >= 0 );
		// _label2
		d1 = d0 >>> 1;
		int d2 = 0;
		setPrintColumnRow( column + d1, row + d2 );

		setTextColor( 1 );
		// clearPrintFineY();// java, {@link #b_print_fine_y} is set by a caller

		d2 = Text.RACER_NAME_LENGTH;
		d1 = 0;
		// _label3
		do {
			jPrintCharacter( opponents_names[ b_58bbb ][ 1 + d1 ] );
			++d1;
		} while( --d2 > 0 );

		b_print_fine_y = 0;// java
		b_print_fine_x = 0;
		b_font_narrow = false;
		b_or_char_with_screen = sp_1;
	}

	// FIXME R_58dfc
	private static void R_58dfc() {
	}

	// FIXME R_58e20
	private static void R_58e20() {
	}

	/**
	 * race opponent's ids ?
	 */
	private static final byte[] b_DAT_59140 = { 0, 0 };

	// java: is not required, player id to picture id
	// private static final byte[] b_player_id_to_picture_id = { 3, 7, 11, 9, 6, 8, 10, 4, 0, 1, 5, 2 };

	private static final byte[] b_TAB_58bbc = {
			0x0b, 0x0b, 0x0b, 0x0b, 0x05, 0x02, 0x0e, 0x01, 0x04, 0x0f, 0x09, 0x0b
		};

	// java x y coordinates for pictures TODO 0-11 are not used
	private static final short[] lw_TAB_58aa4 = {
			0, 0,// 0
			0, 0,
			0, 0,
			0, 0,
			0, 0,
			0, 0,// 5
			0, 0,
			0, 0,
			0, 0,
			0, 0,
			0, 0,// 10
			0, 0,
			// 12, 13 opponents in a race
			50 * PF, 92 * PF,
			162 * PF, 92 * PF,
			// 14, 15 opponents in race result view
			50 * PF, 138 * PF,
			162 * PF, 138 * PF,
			// 16, 17, 18 rating_in_division
			32 * PF, 87 * PF,
			107 * PF, 87 * PF,
			182 * PF, 87 * PF,
			// 12 opponents, 4 x 3, by columns
			// division 1
			2 * PF, 12 * PF,
			2 * PF, 67 * PF,// 20
			2 * PF, 122 * PF,
			// division 2
			82 * PF, 12 * PF,
			82 * PF, 67 * PF,
			82 * PF, 122 * PF,
			// division 3
			162 * PF, 12 * PF,// 25
			162 * PF, 67 * PF,
			162 * PF, 122 * PF,
			// division 4
			242 * PF, 12 * PF,
			242 * PF, 67 * PF,
			242 * PF, 122 * PF,// 31
	};

	private static final int DRIVER_PICTURE_TEXT_Y_SHIFT = 44 * PF;

	private static final int[] b_player_id_to_picture_id = {
			R.drawable.p0, R.drawable.p1, R.drawable.p2, R.drawable.p3,
			R.drawable.p4, R.drawable.p5, R.drawable.p6, R.drawable.p7,
			R.drawable.p8, R.drawable.p9, R.drawable.p10, R.drawable.p11
	};

	// display_player_picture
	/**
	 * @param g Canvas
	 * @param d0_playerID player id in d0 as byte. 0...11
	 * @param d3 index of the x, y coordinates of a picture in {@link #lw_TAB_58aa4}
	 */
	private static void displayPlayerPicture(final Canvas g, int d0_playerID, int d3) {
		b_58bbb = (byte) d0_playerID;
		// move.w	d6,-(sp)
		// move.l	a6,-(sp)
		if( 0 != b_multi_no_of_players ) {
			if( (0 == b_machine) && (d0_playerID != b_racer_id_3) ) {
				d0_playerID &= 1;
				d0_playerID = b_DAT_59140[ d0_playerID ];
			} else {
				d0_playerID = NUMBER_OF_RACERS - 1;
			}
		}
		// _label3
		if( d0_playerID < NUMBER_OF_RACERS ) {
			if( d3 < (lw_TAB_58aa4.length / 2) ) {
				final int b_58aa2 = d0_playerID;
				// java: recoding is not required
				d0_playerID = b_player_id_to_picture_id[ d0_playerID ];
				if( 0 != b_x_offset_for_players_name ) {
					// java: R_58b2c(); is not required
					// int d3 = b_x_offset_for_players_name & 0x0F;
					final int x = lw_TAB_58aa4[ d3 << 1 ];// 80 = 5 * 16 = width
					final int y = lw_TAB_58aa4[ (d3 << 1) + 1 ];// 55 = height
					g.drawBitmap( jGetImage( d0_playerID ), x, y, paint );
				} else {
					// _label4
					boolean b_58bc8 = false;// TODO using the var
					if( 0 != b_multi_no_of_players ) {
						if( 11 == b_58aa2 ) {
							b_58bc8 = true;
							makeMasks( b_TAB_58bbc[ b_58bbb ] );
							// lw_L_58bca = d6;
							// lw_L_58bce = d7;
							// lw_L_58bd2 = b_TAB_58bd6;
						}
					}
					// _label5
					final int x = lw_TAB_58aa4[ d3 << 1 ];// 80 = 5 * 16 = width
					final int y = lw_TAB_58aa4[ (d3 << 1) + 1 ];// 55 = height
					g.drawBitmap( jGetImage( d0_playerID ), x, y, paint );
				}
				// _labela
				if( (NUMBER_OF_RACERS - 1) == b_58aa2 ) {
					final int x = lw_TAB_58aa4[ d3 << 1 ];// 80 = 5 * 16 = width
					final int y = DRIVER_PICTURE_TEXT_Y_SHIFT + lw_TAB_58aa4[ (d3 << 1) + 1 ];// 55 = height, +44 - offset for text
					b_print_fine_y = (byte) ((y % (Text.TEXT_ROW_HEIGHT * PF)) / PF);
					printPlayersName( x / (Text.COLUMN_SMALL_FONT_WIDTH * PF), y / (Text.TEXT_ROW_HEIGHT * PF)  );
				}
			}
		}
		// _labelb
		// move.l	(sp)+,a6
		// move.w	(sp)+,d6
	}

	// R_58e7a
	private static void R_58e7a(final Canvas g) {
		if( b_1bbaa ) {
			fourPrintFineY();
			displayRaceNumber( g, 10 );
			chooseRacersInDivision();
			R_648d2( g, 17 );

			clearPrintFineY();

			printLeagueText( g, 7 );// 96 'RESULT'

			fourPrintFineY();
			setTextColor( 1 );
			R_5914a( g, 1 );// 5 Winner 2pts     Best Lap 1pt
			displayPlayerPicture( g, b_1ca25, 14 );
			displayPlayerPicture( g, b_1ca26, 15 );
			copyScreenAndWaitForFire( g );
			return;
		}
		clearPrintFineY();
		displayRaceNumber( g, 11 );// 'Race 1 of 2'
		R_5914a( g, 0 );// 'V'
		displayRaceOpponentsPictures( g );
		setTextColor( 15 );

		clearPrintFineY();
		R_5eae0( g, 20 );

		R_5f25a( true );
		R_5f074( g );
		copyScreenAndWaitForFire( g );
	}

	// R_64668
	private static void R_64668(final Canvas g, final boolean flag) {
		b_1bbaa = flag;
		displayDivision( g, false );
		int d0 = b_division << 1;// 2 roads in division
		b_value_low_factor_high = (byte) d0;
		d0 = b_1ca35;
		if( 0 != b_multi_no_of_players ) {
			d0 = b_5eb77 ^ 0x01;
		}
		d0 &= 0x01;
		d0 += b_value_low_factor_high;
		getRoadId( d0 );
		if( 0 == (b_1bbb1 & 0x01) ) {
			R_58e7a( g );
			return;
		}
		R_5eae0( g, 11 );
		R_5f074( g );
		copyScreenAndWaitForFire( g );
	}

	// show_race_result
	private static void showRaceResult(final Canvas g) {
		R_64668( g, true );
	}

	// display_opponents_before_race
	private static void displayOpponentsBeforeRace(final Canvas g) {
		R_64668( g, false );
	}

	// display_result_time_and_points
	private static void displayResultTimeAndPoints(final Canvas g) {
		setTextColor( 0 );
		b_menu_bar_index = 14;
		b_menu_bar_type = 0;
		R_5ec7e( g, 2 );// 35 'Race Time: '
		int d1 = b_race_winner_id;
		if( 0 != d1 ) {
			b_won_races[ d1 ]++;
			printOpponentsName( g, d1 );
			printLeagueText( g, 16 );// 233 ' 2pts'
		}
		setPrintColumnRow( 5, b_current_print_row + 1 );
		R_5ec7e( g, 3 );// 'Best Lap : '
		d1 = b_best_lap_racer_id;
		if( 0 != d1 ) {
			b_best_lap_races[ d1 ]++;
			printOpponentsName( g, d1 );
			printLeagueText( g, 17 );// 239 ' 1pt'
		}
		computeRaceRating();
	}

	// R_5eac8
	private static void R_5eac8() {
		setPrintColumnRow( 5, b_current_print_row );
		++b_menu_bar_index;
	}

	// R_64718
	/**
	 *
	 * @param g Canvas
	 * @return true - nz
	 */
	private static boolean R_64718(final Canvas g) {
		setTextColor( 0 );
		int d1 = b_rating_racer_ids[ b_start_offset_for_rating_in_divisions ];
		b_edge_x1_offset = (byte) d1;
		printOpponentsName( g, d1 );
		printOneSpace();
		d1 = b_edge_x1_offset;
		if( b_1bbcb ) {
			printMinSecMs( d1 );
			printTwoSpaces();
			printMinSecMs( d1 + 12 );
		} else {
			R_5edd2( b_raced[ d1 ] );
			R_5edba( b_won_races[ d1 ] );
			R_5edba( b_best_lap_races[ d1 ] );
			printTwoSpaces();
			R_5edd2( b_total_points[ d1 ] );
		}
		++b_start_offset_for_rating_in_divisions;
		return b_start_offset_for_rating_in_divisions != w_fp_y2;
	}

	private static final byte[] b_TAB_5eb64 = {
			12, 12, 12, 12, 11, 11, 10, 10, 19, 19, 19, 18, 17, 16, 15, 15
		};

	// FIXME R_5e93e
	private static void R_5e93e(final Canvas g) {
		computeRaceRating();
		b_menu_bar_positions[ 14 ] = 16;
		Text.b_TEXT_5ec92[ 7 ].mY = 14;// b_TEXT_5ec92+142 'DRIVER      BEST-LAP RACE-TIME'
		if( b_1bbcb && b_5eb7a ) {
			clearMenu( g );
			setTextColor( 1 );
			setPrintColumnRow( 11, 9 );
			R_5ec7e( g, 0 );// 'TRACK BONUS POINTS'
			int d0 = b_multi_no_of_players;
			if( d0 >= 5 ) {
				fourPrintFineY();
			}
			int d1 = b_multi_no_of_players;
			d0 = b_TAB_5eb64[ d1 ];
			int d2 = d0;
			b_menu_bar_positions[ 14 ] = (byte) (d0 + 2);
			R_5eae0( g, d2 );
			setTextColor( 0 );
			displayResultTimeAndPoints( g );
			d0 = b_multi_no_of_players;
			if( (6 != d0) && (5 != d0) ) {
				fourPrintFineY();
			}
			// _label2
			d1 = b_multi_no_of_players;
			d0 = b_TAB_5eb64[ d1 + 8 ];
			Text.b_TEXT_5ec92[ 7 ].mY = (byte) d0;// 142 'DRIVER      BEST-LAP RACE-TIME'
			d0 += 2;
			d2 = b_multi_no_of_players;
			if( 7 == d2 ) {
				--d0;
			}
			b_menu_bar_positions[ 14 ] = (byte) d0;
			// _label6
			setTextColor( 1 );
			R_5ec7e( g, 0x8c );
		} else {
			// _label4
			displayDivision( g, false );
			setTextColor( 1 );
			if( ! b_1bbcb ) {
				printLeagueText( g, 10 );// 134 'RESULTS TABLE'

				if( b_57c67 >= 3 ) {
					setTextColor( 15 );
					fourPrintFineY();
					R_5ec7e( g, 1 );// 19 "FINAL SEASON"
					clearPrintFineY();
				}
			} else {
				// _label5
				R_5eae0( g, 11 );
				// _label6
				setTextColor( 1 );
				R_5ec7e( g, 7 );// 140 'DRIVER      BEST-LAP RACE-TIME'
			}
		}
		// _label7
		b_menu_bar_index = 14;
		doFillBar( g, b_multi_no_of_players + 2 );

		do {
			R_5eac8();
			++b_current_print_row;
		} while( R_64718( g ) );
	}

	// display_rating_in_division
	private static void displayRatingInDivision(final Canvas g) {
		final int d6 = b_start_offset_for_rating_in_divisions;
		b_x_offset_for_players_name = 14;
		displayPlayerPicture( g, b_rating_racer_ids[ d6 ], 16 );
		b_x_offset_for_players_name = 9;
		displayPlayerPicture( g, b_rating_racer_ids[ d6 + 1 ], 17 );
		b_x_offset_for_players_name = 4;
		displayPlayerPicture( g, b_rating_racer_ids[ d6 + 2 ], 18 );
		b_x_offset_for_players_name = 0;
		fourPrintFineY();
		setTextColor( 1 );
		b_print_fine_x = 4;
		R_5914a( g, 6 );// 73 'First     Second     Third'
		b_print_fine_x = 0;
		setTextColor( 15 );
		int b_5913e = 4;

		// _label1
		do {
			b_edge_x1_offset = b_rating_racer_ids[ b_start_offset_for_rating_in_divisions ];
			setTextColor( 15 );
			setPrintColumnRow( b_5913e, 19 );
			fourPrintFineY();
			R_5914a( g, 2 );// 37 ' Raced  '
			setTextColor( 1 );
			R_5edd2( b_raced[ b_edge_x1_offset ] );

			setTextColor( 15 );
			setPrintColumnRow( b_5913e, 21 );
			clearPrintFineY();
			R_5914a( g, 3 );// 46 ' Wins   '
			setTextColor( 1 );
			R_5edd2( b_won_races[ b_edge_x1_offset ] );

			setTextColor( 15 );
			setPrintColumnRow( b_5913e, 22 );
			fourPrintFineY();
			R_5914a( g, 4 );// 55 ' Laps   '
			setTextColor( 1 );
			R_5edd2( b_best_lap_races[ b_edge_x1_offset ] );

			setTextColor( 15 );
			setPrintColumnRow( b_5913e, 24 );
			clearPrintFineY();
			R_5914a( g, 5 );// 64 ' Points '
			setTextColor( 1 );
			R_5edd2( b_total_points[ b_edge_x1_offset ] );

			b_5913e += 11;
			++b_start_offset_for_rating_in_divisions;
		} while( w_fp_y2 != b_start_offset_for_rating_in_divisions );
		copyScreenAndWaitForFire( g );
	}

	// show_rating_in_division
	private static void showRatingInDivision(final Canvas g) {
		R_5ab00();
		b_start_offset_for_rating_in_divisions = (byte) w_fp_y;
		if( 0 != b_multi_no_of_players ) {
			R_5e93e( g );
			copyScreenAndWaitForFire( g );
			return;
		}
		displayDivision( g, false );
		displayRatingInDivision( g );
	}

	// R_5ef76
	private static void R_5ef76(final Canvas g) {
		b_1bbcb = true;
		showRatingInDivision( g );
		b_1bbcb = false;
	}

	// R_5ef42
	private static void R_5ef42(final Canvas g) {
		if( (0 == b_machine) || (b_multi_no_of_players >= 2) ) {
			b_5eb7a = true;
			R_5ef76( g );
			showRatingInDivision( g );
			b_5eb7a = false;
		}
	}

	// R_64ad2
	private static int R_64ad2(int d0) {
		int d1 = 11;
		// _label1
		do {
			if( b_DAT_1c9c2[ d1 ] == d0 ) {
				break;
			}
		} while( --d1 >= 0 );
		return d1;
	}

	private static boolean b_no_promo_picture = true;

	private static final byte[] b_TAB_64ac6 = {
			(byte) 0x03, (byte) 0x03, (byte) 0x03, (byte) 0x02, (byte) 0x02, (byte) 0x02, (byte) 0x01, (byte) 0x01, (byte) 0x01, (byte) 0x00, (byte) 0x00, (byte) 0x00
	};

	// display_promotion_and_relegation
	private static void displayPromotionAndRelegation(final Canvas g) {
		b_no_promo_picture = true;

		System.arraycopy( b_rating_racer_ids, 0, b_DAT_1c9c2, 0, b_rating_racer_ids.length );

		R_5ab00();

		setPrintColumnRow( 15, 12 );
		printLeagueText( g, 14 );// 215 ' CHANGES'

		b_menu_bar_index = 1;
		boolean j_is_label4 = false;
		boolean j_is_label5 = false;
		if( 0 == w_fp_y ) {
			if( b_racer_id_3 != b_rating_racer_ids[ 0 ] ) {
				j_is_label4 = true;// bne _label4;
			}
			if( 0 != b_league_offset ) {
				fillBar( g );
				printSelectableTextLine( g, 15 );// 206 'EXCELLENT DRIVING - WELL DONE'
				j_is_label5 = true;// jmp _label5;
			}
		}
		if( !j_is_label4 && !j_is_label5 ) {
			// _label2
			fillBar( g );
			printLeagueText( g, 12 );// 183 'Promotion for  '

			int d2 = w_fp_y;
			int d1 = b_rating_racer_ids[ d2 ];
			if( b_racer_id_3 == d1 ) {
				b_no_promo_picture = false;
			}
			printOpponentsName( g, d1 );
			d2 = w_fp_y;
			if( 0 == d2 ) {
				fillBar( g );
				printSelectableTextLine( g, 13 );// 167 'to the SUPER LEAGUE'
				j_is_label5 = true;// jmp _label5;
			}
		}
		if( j_is_label4 && !j_is_label5 ) {
			// _label4
			int d2 = w_fp_y2 - 1;
			if( (NUMBER_OF_RACERS - 1) != d2 ) {
				fillBar( g );
				printLeagueText( g, 13 );// 199 'Relegation for'
				printOpponentsName( g, b_rating_racer_ids[ w_fp_y2 - 1 ] );
			}
		}
		// _label5
		b_division = 2;
		// _label6
		do {
			R_5ab00();
			int d2 = w_fp_y;
			int d1 = b_DAT_1c9c2[ d2 ];
			b_DAT_1c9c2[ d2 ] = b_DAT_1c9c2[ d2 - 1 ];
			b_DAT_1c9c2[ d2 - 1 ] = (byte) d1;
		} while( --b_division >= 0 );

		int d0 = b_racer_id_3;
		if( d0 == b_rating_racer_ids[ 0 ] ) {
			if( 0 != b_league_offset ) {
				// bne _labela;
				// _labela
				b_damage_hole_position = (byte) ((6 << 1) - 2);
				return;
			}
			b_league_offset = (byte) d0;
			int d1 = NUMBER_OF_RACERS - 1;
			do {
				b_DAT_1c9c2[ d1 ] = (byte) d1;
			} while( --d1 >= 0 );
			d0 = 0;
		} else {
			// _label8
			final int d1 = R_64ad2( d0 );
			d0 = b_TAB_64ac6[ d1 ];
		}
		// _label9
		b_players_division = (byte) d0;
		if( 0 != d0 ) {
			return;
		}
		// _labela
		b_damage_hole_position = (byte) ((6 << 1) - 2);
	}

	// show_promotion_relegation_wait_fire
	private static void showPromotionRelegationWaitFire(final Canvas g) {
		displayDivision( g, false );
		displayPromotionAndRelegation( g );
		copyScreenAndWaitForFire( g );
	}

	private static final int RACE_RESULT_WON = 1;
	private static final int RACE_RESULT_LOST = 2;
	private static final int RACE_RESULT_WRECK = 3;
	private static final int RACE_RESULT_PROMOTION = 4;

	// java: picture names
	private static final int[] lw_TAB_56ffc = {
			R.drawable.won,
			R.drawable.lost,
			R.drawable.wreck,
			R.drawable.promotion
		};

	// R_59352
	/**
	 *
	 * @param d0 1 - won, 2 - lost, 3 - wreck, 4 - promotion in d0 as byte
	 */
	private static void R_59352(final int d0) {
		fadeScreenOut();

		screen2_canvas.drawBitmap( jGetImage( lw_TAB_56ffc[ d0 - 1 ] ), 0, 0, null );

		// _label3
		fadeScreenIn();
		waitForFire();
	}

	// R_5933a
	private static void R_5933a() {// show promo picture conditional
		if( !b_no_promo_picture ) {
			R_59352( RACE_RESULT_PROMOTION );
		}
	}

	// set_current_players_xz
	/**
	 *
	 * @param d0_angle angle in d0 as word
	 */
	private static void setCurrentPlayersXZ(int d0_angle) {
		if( 0 == (lw_players_world_x & 0xFFFF) ) {
			lw_players_world_x++;
		}
		final int x_pos = (lw_players_world_x << 1) >>> 16;// /$8000, because each map square = $800000
		b_players_fine_map_x = (byte) x_pos;
		b_players_map_x = (byte) (x_pos >>> 8);

		if( 0 == (lw_players_world_z & 0xFFFF) ) {
			lw_players_world_z++;
		}
		final int z_pos = (lw_players_world_z << 1) >>> 16;// /$8000, because each map square = $800000
		b_players_fine_map_z = (byte) z_pos;
		b_players_map_z = (byte) (z_pos >>> 8);

		if( 0 != (d0_angle & Trigonometry.ANGLE_180_DEGREES) ) {
			if( 0 != (d0_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				lw_current_players_z = (0x800_0000 - lw_players_world_x);
				lw_current_players_x = lw_players_world_z;
				return;
			}
			lw_current_players_x = (0x800_0000 - lw_players_world_x);
			lw_current_players_z = (0x800_0000 - lw_players_world_z);
			return;
		}
		if( 0 != (d0_angle & Trigonometry.ANGLE_90_DEGREES) ) {
			lw_current_players_z = lw_players_world_x;
			lw_current_players_x = 0x800_0000 - lw_players_world_z;
			return;
		}
		lw_current_players_x = lw_players_world_x;
		lw_current_players_z = lw_players_world_z;
	}

	// set_road_position_values
	private static void setRoadPositionValues() {
		// calculate the closest angle, step is 90 deg
		w_rough_player_angle = (short)
				((w_players_y_angle + Trigonometry.ANGLE_45_DEGREES) & Trigonometry.ANGLE_90_DEGREE_STEP_MASK);

		setCurrentPlayersXZ( w_rough_player_angle );
		w_players_smaller_y = (short) (lw_players_world_y_fp16 >>> 11);

		//*** Calculate y perspective shift value, using x angle ***
		int d3_shift = 0x780;
		int d0_y_shift = w_average_amount_below_road;
		if( d0_y_shift >= 0x500 ) {
			d0_y_shift <<= 1;
			d3_shift = 0x280;
		}
		d0_y_shift += d3_shift;
		if( w_players_x_angle < 0 ) {
			d0_y_shift -= w_players_x_angle >> 1;
		}
		w_y_perspective_shift = (short) ((d0_y_shift >> 4) + w_players_smaller_y);

		//*** Calculate fine road offset values ***
		final int x = -((lw_current_players_x >>> 12) & 0x7FF);// 800 00 00 > 8000 > 0 > FFFF
		// b_road_finer_x_offset	dc.b	0,0
		// w_road_near_x_offset	dc.b	0
		// b_road_finest_x_offset	dc.b	0
		//	dc.w	0
		// b_road_finest_x_offset is the low byte of the road_near_x_offset
		w_road_near_x_offset = (short)((w_road_near_x_offset & 0xFF00)
							| (x & 0xFF));
		b_road_finer_x_offset = (byte)(x >>> 8);

		final int z = -((lw_current_players_z >>> 12) & 0x7FF);// 800 00 00 > 8000 > 0 > FFFF
		// b_road_finer_z_offset	dc.b	0,0
		// w_road_near_z_offset	dc.b	0
		// b_road_finest_z_offset	dc.b	0
		//	dc.w	0
		// b_road_finest_z_offset is the low byte of the road_near_z_offset
		w_road_near_z_offset = (short)((w_road_near_z_offset & 0xFF00)
							| (z & 0xFF));
		b_road_finer_z_offset = (byte)(z >>> 8);

		//*** Calculate x shift value, using y angle ***
		w_x_shift = (short)
				(((w_players_y_angle + Trigonometry.ANGLE_45_DEGREES)
						& Trigonometry.ANGLE_90_DEGREE_MASK_EVEN)
						- Trigonometry.ANGLE_45_DEGREES);
	}

	private static final byte[] b_DAT_7a41a = new byte[ 256 ];

	private static final byte[] b_DAT_7a51a = new byte[ 256 ];

	private static final byte[] b_DAT_7a81a = new byte[ 256 ];

	// TODO is_receive_queue_empty
	/**
	 *
	 * @return eq - no new bytes in the queue.
	 */
	private static int isReceiveQueueEmpty() {
		return 0;// b_next_receive_byte - b_receive_byte_queue_end;
	}

	// received_byte_ready
	/**
	 *
	 * @return eq - no new bytes in the queue.
	 */
	private static int receivedByteReady() {
		return isReceiveQueueEmpty();
	}

	// TODO receive_byte_F5_exit make it work
	/**
	 *
	 * @return d0 as byte.
	 */
	private static byte receiveByteF5exit() {
		byte d0 = 0;// & 0xFF
		w_com_receive_byte_total += ((int)d0) & 0xFF;
		w_com_receive_byte_total++;
		w_com_57c44++;
		return d0;
	}

	// TODO add_received_word make it to work with other machine
	/**
	 *
	 * @return d0 as word
	 */
	private static short addReceivedWord() {
		return (short)(((receiveByteF5exit() & 0xFF) << 8) | (receiveByteF5exit() & 0xFF));
	}

	// FIXME get_next_received_byte
	private static byte getNextReceivedByte() {
		return 0;
	}

	// add_received_byte
	private static void addReceivedByte() {
		int d0 = getNextReceivedByte() & 0xFF;
		w_com_receive_byte_total += d0;
		++w_com_receive_byte_total;
		++w_com_57c44;
		// clr.w	d1
		// clr.w	d2
	}

	// FIXME flush_received_byte
	private static void flushReceivedBytes() {
		if( 0 != receivedByteReady() ) {
			addReceivedByte();
		}
	}

	// R.571aa
	/**
	 * Extracted in place
	 *
	 * @param d0 as byte
	 * @param d3 as byte
	 */
	private static void R_571aa(int d0, int d3) {
		/*
		if( (0x40 == d0) || (0x80 == (d0 & 0xFF)) ) {
			if( 0 == d3 ) {
				d3 = d0;
			} else if( d0 != d3 ) {
				d3 = d0 = 0x80;
			}
		}
		*/
	}

	// retry_receive_byte_3_times
	/**
	 * @return < 0 - cs, >= 0 - cc, a received byte 0x000000dd
	 */
	private static int retryReceiveByte3times() {
		return 0;
	}

	// retry_receive_byte_1000_times
	/**
	 * @return < 0 - cs, >= 0 - cc, a received byte 0x000000dd
	 */
	private static int retryReceiveByte1000times() {
		return 0;
	}

	// R.5786c
	/**
	 *
	 * @return true - cs, false - cc
	 */
	private static boolean R_5786c() {
		begin:
		do {
			int d7 = 8;
			// Receive $125634
			do {
				w_com_receive_byte_total = 0;
				int d0 = retryReceiveByte3times();
				if( d0 >= 0 ) {
					if( 0x12 == d0 ) {
						--d7;
						d0 = retryReceiveByte3times();
						if( d0 >= 0 ) {
							if( 0x56 == d0 ) {
								--d7;
								d0 = retryReceiveByte3times();
								if( d0 >= 0 ) {
									if( 0x34 == d0 ) {
										// Receive info bytes
										--d7;
										d0 = retryReceiveByte3times();
										if( d0 >= 0 ) {
											b_57c5d = (byte) d0;
											d0 = retryReceiveByte3times();
											if( d0 >= 0 ) {
												b_57c5c = (byte) d0;
												d0 = retryReceiveByte3times();
												if( d0 >= 0 ) {
													b_57c60 = (byte) d0;
													d0 = retryReceiveByte3times();
													if( d0 >= 0 ) {
														b_57c5e = (byte) d0;
														d0 = retryReceiveByte3times();
														if( d0 >= 0 ) {
															b_57c5f = (byte) d0;
															w_copy_receive_byte_total = w_com_receive_byte_total;
															d0 = receiveByteF5exit();
															if( d0 == w_copy_receive_byte_total ) {
																// check received bytes total
																delay_2ms();
																if( 0 != receivedByteReady() ) {
																	continue begin;// from the start
																}
																return false;
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			} while( --d7 >= 0 );
			delay_1ms();
			delay_1ms();
			if( 0 == receivedByteReady() ) {
				break;
			}
		} while( true );
		return true;
	}

	// R_5773a
	private static void R_5773a() {
		if( R_5786c() ) {
			return;
		}
		int d4 = b_57c5d;
		if( 0 == b_57c5c ) {
			if( 0 != (d4 & 0x01) ) {
				b_second_machine = (byte)(b_machine ^ 0xC0);
			}
			return;
		}
		if( 0 != (b_57c5c & 0x10) ) {
			return;
		}
		if( 0 != (d4 & 0x20) ) {
			b_1bbb4 = (byte)((d4 & 0xC0) ^ 0xC0);
			b_1bb6f = true;
			b_laps_when_race_finished = 5;
			b_1bbe2 = 0;
			b_1bb64 = 0;
			b_second_machine = 0;
			return;
		}
		if( 0 != (d4 & 0x10) ) {
			b_1bb64 = (byte)(b_machine ^ 0xC0);
			return;
		}
		if( 0 != (d4 & 0x08) ) {
			b_1bbe2 = (byte)(b_machine ^ 0xC0);
			b_1bb6f = true;
			return;
		}
	}

	// R_57440
	private static void R_57440() {
		do {
			if( ! b_57c3a ) {
				b_com_machine = b_second_machine;
				b_com_57c51 = b_1bb64;
				b_com_57c52 = b_1bbe2;
				b_com_57c53 = b_1bb6c;
				b_com_car_on_chains_countdown = b_car_on_chains_countdown;
				int d0 = addReceivedWord();
				if( 0x8765 == (d0 & 0xFFFF) ) {
					if( ! b_57c3a ) {
						w_com_opp_rear_left_actual_height = addReceivedWord();
						w_com_57c48 = addReceivedWord();
						w_com_opp_front_actual_height = addReceivedWord();
						w_opp_smallest_difference = 0;
						w_com_opponents_z_speed = addReceivedWord();
						d0 = (addReceivedWord() * 21845) << 1;// signed
						d0 >>>= 16;
						w_com_opponents_road_x_position = (short) d0;
						d0 = receiveByteF5exit();
						int d3 = b_com_machine;
						// in place jsr R_571aa
						if( (0x40 == d0) || (0x80 == (d0 & 0xFF)) ) {
							if( 0 == d3 ) {
								d3 = d0;
							} else if( d0 != d3 ) {
								d3 = d0 = 0x80;
							}
						}
						b_com_machine = (byte) d3;
						d0 = receiveByteF5exit();
						d3 = b_com_57c51;
						// in place jsr R_571aa
						if( (0x40 == d0) || (0x80 == (d0 & 0xFF)) ) {
							if( 0 == d3 ) {
								d3 = d0;
							} else if( d0 != d3 ) {
								d3 = d0 = 0x80;
							}
						}
						b_com_57c51 = (byte) d3;
						d0 = receiveByteF5exit();
						if( 0 != d0 ) {
							if( (d0 < 0) || (0 == b_com_57c53) || (d0 < b_com_57c53) ) {
								b_com_57c53 = (byte) d0;
							}
						}
						d0 = receiveByteF5exit();
						d3 = b_com_57c52;
						// in place jsr R_571aa
						if( (0x40 == d0) || (0x80 == (d0 & 0xFF)) ) {
							if( 0 == d3 ) {
								d3 = d0;
							} else if( d0 != d3 ) {
								d3 = d0 = 0x80;
							}
						}
						b_com_57c52 = (byte) d3;
						d0 = receiveByteF5exit();
						if( !b_drop_start_done ) {
							if( b_machine != MASTER ) {
								if( ((int) b_com_car_on_chains_countdown & 0xFF) < 0xE4 ) {
									b_com_car_on_chains_countdown = (byte) d0;
								}
							}
						}
						b_com_57c55 = receiveByteF5exit();
						b_com_time_results_min = receiveByteF5exit();
						b_com_time_results_sec = receiveByteF5exit();
						b_com_time_results_ms = receiveByteF5exit();
						b_com_57c59 = receiveByteF5exit();
						b_com_loop_counter = receiveByteF5exit();
						w_copy_receive_byte_total = w_com_receive_byte_total;
						d0 = addReceivedWord();
						if( d0 == w_copy_receive_byte_total ) {
							flushReceivedBytes();
							w_opp_wheel_actual_heights[ OPP_REAR_LEFT_WHEEL ] = w_com_opp_rear_left_actual_height;// why left only?
							w_opp_wheel_actual_heights[ OPP_FRONT_WHEEL ] = w_com_opp_front_actual_height;
							w_opponents_z_speed = w_com_opponents_z_speed;
							w_opponents_road_x_position = w_com_opponents_road_x_position;
							b_second_machine = b_com_machine;
							b_1bb64 = b_com_57c51;
							b_1bbe2 = b_com_57c52;
							b_1bb6c = b_com_57c53;
							b_car_on_chains_countdown = b_com_car_on_chains_countdown;
							b_1bbe0 = b_com_57c55;
							b_57c61 = (byte) (b_com_57c55 & 0x0F);
							b_time_results_min[ 1 ] = b_com_time_results_min;
							b_time_results_sec[ 1 ] = b_com_time_results_sec;
							b_time_results_ms[ 1 ] = b_com_time_results_ms;
							b_1bb76 = b_com_57c59;
							d0 = b_com_loop_counter - b_loop_counter;
							if( d0 < 0 ) {
								delay_100ms();
								transmitPlayersData();
								delay_30ms();
								// repeat from the start
								continue;
							}
							if( 0 != d0 ) {
								b_57c6e = true;
							}
							if( b_race_mode < 0 ) {
								R_641b6();
								calculateDistancesBetweenPlayers();
								opponentPlayerInteraction();
								calculateOpponentsRoadWheelPositions();
							}
							return;
						}
					}
				}
			}
			break;
		} while( true );
		if( 0 != receivedByteReady() ) {
			R_5773a();
			flushReceivedBytes();
		}
		b_opponents_road_piece = b_com_opponents_road_piece;
		w_opponents_distance_into_piece = w_com_opponents_distance_into_piece;
		if( 0 != b_1bb76 ) {
			--b_1bb76;
		}
		b_57c3a = true;
		w_opponents_offset = (short)-1;
	}

	// FIXME transmit_byte_when_ready
	private static void transmitByteWhenReady() {

	}

	// add_and_transmit_byte
	/**
	 *
	 * @param d0 a byte to transmit
	 */
	private static void addAndTransmitByte(int d0) {
		d0 &= 0xFF;
		transmit_byte_total += d0;
		++transmit_byte_total;
		transmitByteWhenReady();
	}

	// add_and_transmit_word
	private static void addAndTransmitWord(final int d0) {
		addAndTransmitByte( d0 >>> 8 );// high byte
		addAndTransmitByte( d0 );// low byte
	}

	// transmit_players_data
	private static void transmitPlayersData() {
		b_57c3a = false;
		transmit_byte_total = 0;
		addAndTransmitByte( b_players_road_piece );
		addAndTransmitByte( w_players_distance_into_section >> 8 );
		int d0 = ((w_players_world_y_speed >> 1) + w_average_amount_below_road) >> 3;
		if( d0 < 0 ) {
			d0 = 0;
		}
		final short w_57c70_backup = (short)d0;

		d0 = (lw_front_left_actual_height_fp8 >> 3) + w_57c70_backup;
		if( d0 < 0 ) {
			d0 = 0;
		}

		int d3 = (lw_front_right_actual_height_fp8 >> 3) + w_57c70_backup;
		if( d3 < 0 ) {
			d3 = 0;
		}

		int d4 = (lw_rear_actual_height_fp8 >> 3) + w_57c70_backup;
		if( d4 < 0 ) {
			d4 = 0;
		}

		final int d7 = (((d0 + d3) & 0xFFFF) >>> 7);
		d3 = (d3 - d0) >> 1;
		int d5 = d4 - d3;
		int d6 = d4 + d3;
		addAndTransmitWord( 0x8765 );
		addAndTransmitWord( d5 );
		addAndTransmitWord( d6 );
		addAndTransmitWord( d7 );
		addAndTransmitWord( w_players_z_speed );
		d0 = w_players_road_x_position;
		if( b_plus_180_degrees < 0 ) {
			d0 = Road.ROAD_WIDTH - d0;
		}
		addAndTransmitWord( d0 );
		if( 0x40 != b_second_machine ) {
			if( 0x80 != (b_second_machine & 0xFF) ) {
				d4 = 0;
				if( testKey( KeyEvent.KEYCODE_P ) ) {// KeyEvent.VK_PAUSE Amiga: P
					d4 = b_machine;
				}
				b_second_machine = (byte) d4;
			}
		}
		addAndTransmitByte( b_second_machine );
		if( 0x40 != b_1bb64 ) {
			if( 0x80 != (b_1bb64 & 0xFF) ) {
				d4 = 0;
				if( testKey( KeyEvent.KEYCODE_ESCAPE ) ) {// testKey( AMIGA_KEY_CODE_ESC ) ) {
					d4 = b_machine;
				}
				b_1bb64 = (byte) d4;
			}
		}
		addAndTransmitByte( b_1bb64 );
		addAndTransmitByte( b_1bb6c );
		addAndTransmitByte( b_1bbe2 );
		addAndTransmitByte( b_car_on_chains_countdown );
		d0 = b_damage_hole_position;
		if( 0 != b_car_on_chains_countdown ) {
			d0 |= 0x80;
		}
		if( b_swing_from_left ) {
			d0 |= 0x40;
		}
		addAndTransmitByte( d0 );
		addAndTransmitByte( b_57c6b );
		addAndTransmitByte( b_57c6c );
		addAndTransmitByte( b_57c6d );
		addAndTransmitByte( b_1bbcc );
		addAndTransmitByte( b_loop_counter );
		addAndTransmitWord( transmit_byte_total );
		if( ! b_57c6e ) {
			b_com_opponents_road_piece = b_opponents_road_piece;
			w_com_opponents_distance_into_piece = w_opponents_distance_into_piece;
			// if( ! b_57c6e ) {
				w_com_receive_byte_total = 0;
				b_opponents_road_piece = receiveByteF5exit();
				w_opponents_distance_into_piece = addReceivedWord();
				if( b_race_mode >= 0 ) {
					return;
				}
				if( b_opponents_road_piece >= b_number_of_road_pieces ) {
					calculateSectionPosition( 255 );
					return;
				}
			// }
		}
		b_57c3a = true;
		b_57c6e = false;
	}

	// FIXME receive_data_block
	private static boolean receiveDataBlock(final byte[] ba6, int d7) {
		return false;
	}

	// FIXME transmit_data_block
	private static boolean transmitDataBlock(final byte[] ba6, int d7) {
		return false;
	}
	
	// update_wheel_positions
	private static void updateWheelPositions() {
		int d1 = LEFT_FRONT_WHEEL;
		int d3 = SPRITE_LEFT_WHEELS;// java, left wheel sprites, 48 / 16 = 3, bytes to index
		do {
			int d0 = w_new_wheel_differences[ d1 / 2 ] + 256;
			if( d0 < 0 ) {
				d0 = 0;
			}
			if( d0 > 2047 ) {// FIXME value?
				d0 = 2047;
			}
			d0 >>>= 3;
			d0 ^= 0xFF;// -d-1 within 256
			d0 <<= 1;

			d0 = Trigonometry.w_cos_table[ d0 / 2 ];
			d0 = (d0 >> (16 - 5)) & 0x1F;// cos/2^11
			d0 ^= 0xFF;
			d0 += b_1bbdd;
			d0 &= 0xFF;// java TODO simplify
			d0 *= PF;// java
			int d4 = b_which_side_byte << 1;
			b_which_side_byte = (byte) d4;
			if( (0 != (d4 & 0x100)) || (d0 > 185 * PF) ) {// FIXME values
				d0 = 185 * PF;
			}
			if( d0 < 151 * PF ) {
				d0 = 151 * PF;
			}
			d0 -= 50 * PF;
			// d0 &= 0xFF; java removed
			// java: only 3 basic sprites are used
			// int a4i = 37;// a4i += 37 * 16;
			// if( d0 < (SCREEN_AREA_HEIGHT - 2) ) {// 126
			//	a4i += 6;// a4i += 6 * 16;
			// }
			// final int d5 = 158 - d0;// java - rectClip is used instead in the {@link #drawWorld}
			d4 = SPRITE_WHEEL_SPRITES - 1;
			do {
				graphic_info[ d3 ].mY = d0;// sa2[ (d3 + 10) / 2 ] = d0;
				// graphic_info[ d3 + a4i ].mY = d0;// sa4[ (d3 + 10) / 2 ] = d0;
				// graphic_info[ d3 ].mLines = d5;
				++d3;
			} while( --d4 >= 0 );
			d3 = SPRITE_RIGHT_WHEELS;// right wheels index
		} while( ++d1 <= RIGHT_FRONT_WHEEL );
		adjust_sprites = true;
	}

	// get_players_xz_coord
	private static void getPlayersXZcoord() {
		if( 0 == (w_rough_difference_angle & Trigonometry.ANGLE_180_DEGREES) ) {
			if( 0 == (w_rough_difference_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				w_near_x_coord = (short) (-w_road_near_x_offset);
				w_near_z_coord = (short)(-w_road_near_z_offset);
				return;
			}
			w_near_x_coord = (short)(-w_road_near_z_offset);
			w_near_z_coord = (short)(0x800 + w_road_near_x_offset);
			return;
		}
		if( 0 == (w_rough_difference_angle & Trigonometry.ANGLE_90_DEGREES) ) {
			w_near_x_coord = (short)(0x800 + w_road_near_x_offset);
			w_near_z_coord = (short)(0x800 + w_road_near_z_offset);
			return;
		}
		w_near_x_coord = (short)(0x800 + w_road_near_z_offset);
		w_near_z_coord = (short)(-w_road_near_x_offset);
	}

	// square_it
	/**
	 *
	 * @param value as word in d0
	 * @return value^2 as int32
	 */
	private static int squareIt(int value) {
		value *= value;// signed
		if( value < 0 ) {
			value = -value;
		}
		return value;
	}

	/** Near section piece perspective correction factors */
	private static final byte[] b_perspective_correction_factors = {
			0, (byte) 0xd4, (byte) 0x80, (byte) 0xd4,
			0, 0, (byte) 0xab, (byte) 0xab,
			0x40, 0x40, 0, 0
		};

	// adjust_perspective
	private static void adjustPerspective() {
		int d4 = squareIt( w_near_x_coord ) + squareIt( w_near_z_coord );// view distance
		int d0 = squareIt( w_perspective_z );
		d0 = (d4 - d0) >>> 8;
		d0 = (((d0 * ((b_perspective_correction_factors[ b_near_section_piece ] << 7) & 0x7FFF)) << 1) >> 16) >> 4;
		w_perspective_z += d0;
	}

	// calculate_players_road_position
	private static void calculatePlayersRoadPosition() {
		byte[] ba5;
		do {
			int d1 = b_current_road_piece;
			fetchNearSectionStuff( d1 );
			// int d0 = w_piece_data_offset;
			// d0 = ((((d0 >> 8) & 0xFF) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			ba5 = w_piece_data_offset;// Road.offsetToPiece( d0 );
			int d0 = d1;
			fetchXZposition( d0 );

			w_rough_difference_angle = (short) (w_rough_player_angle - w_rough_piece_angle);

			if( b_curved_section >= 0 ) {
				if( 0 == (b_curved_section & 0x40) ) {
					getPlayersXZcoord();
					// int d3 = ((ba5[ 3 ] & 0xFF) << 8) | (ba5[ 2 ] & 0xFF);
					final int d3 = Road.getSignedWordDataFromPiece( ba5, 2 );
					w_players_road_x_position = (short) (w_near_x_coord - d3);
					w_normal_distance_into_section = w_near_z_coord;
					w_section_y_angle = w_rough_piece_angle;
					return;
				}
				// _label1
				getPlayersXZcoord();

				d0 = w_near_x_coord - w_near_z_coord;
				d0 = ((d0 * ((Trigonometry.SIN_45_FP8 << 7) & 0x7FFF)) << 1) >> 16;
				// int d3 = ((ba5[ 3 ] & 0xFF) << 8) | (ba5[ 2 ] & 0xFF);
				int d3 = Road.getSignedWordDataFromPiece( ba5, 2 );
				w_players_road_x_position = (short) (d0 - d3);

				d0 = w_near_x_coord + w_near_z_coord;
				d0 = ((d0 * ((ba5[ 7 ] << 7) & 0x7FFF)) << 1) >>> 16;// length reduction
				w_normal_distance_into_section = (short) d0;

				// d3 = ((ba5[ 5 ] & 0xFF) << 8) | (ba5[ 4 ] & 0xFF);
				d3 = Road.getSignedWordDataFromPiece( ba5, 4 );
				w_section_y_angle = (short) (w_rough_piece_angle + d3);
				return;
			}
			// _label2
			int d2 = 2;
			d2 = fetchNearXZcoords( d2 );

			d0 = w_near_x_coord;
			int d3 = w_near_z_coord;
			final long r = Trigonometry.calculatePerspectiveCoord( d0, d3 );
			d0 = (int)(short)r;

			w_perspective_z = (short)Trigonometry.calculatePerspectiveValue( r );

			d0 += w_rough_player_angle;
			if( d0 < 0 ) {
				d0 += Trigonometry.ANGLE_180_DEGREES;
			} else {
				d0 -= Trigonometry.ANGLE_180_DEGREES;
			}
			w_1bc1c = (short) d0;

			w_section_y_angle = (short) (d0 + Trigonometry.ANGLE_90_DEGREES - w_curve_to_left);

			int d4 = 1 - (b_curved_section & 0x03);
			// d3 = (((ba5[ 7 ] & 0xFF) << 8) | (ba5[ 6 ] & 0xFF)) << 6;
			d3 = Road.getSignedWordDataFromPiece( ba5, 6 );
			d0 = (short)(w_1bc1c - (d3 << 6) - w_rough_piece_angle);// TODO verify and simplify
			if( d0 < 0 ) {
				d0 = -d0;
			}
			d0 = ((d0 * ((ba5[ 8 ] << 7) & 0x7FFF)) << 1) >>> 16;
			if( d4 < 0 ) {
				d0 <<= ((-d4) & 7);
			} else {
				d0 >>>= (d4 & 7);
			}
			w_normal_distance_into_section = (short) d0;

			d0 >>>= 7;
			d0 += 2;
			if( d0 >= b_number_of_sections_x2 ) {
				d0 = b_near_section_piece;
				if( (d0 == Road.PIECE_1) || (d0 == Road.PIECE_3) ) {
					// .label8:
					final byte road_height_1 = b_current_road_piece;
					if( 0 != b_plus_180_degrees ) {
						d1 = toPreviousRoadPiece();
					} else {
						d1 = toNextRoadPiece();
					}
					d0 = b_road_section_angle_and_piece[ d1 ] & 0x0F;
					if( d0 == 4 ) {
						continue;// bra calculate.players.road.position
					}
					// .labelb:
					b_current_road_piece = road_height_1;
				}
			}
			break;
		} while( true );
		// .labelc:
		adjustPerspective();

		// int d3 = ((ba5[ 10 ] & 0xFF) << 8) | (ba5[ 9 ] & 0xFF);
		int d3 = Road.getSignedWordDataFromPiece( ba5, 9 ) - w_perspective_z;
		if( w_curve_to_left < 0 ) {
			d3 = -d3;
		}
		w_players_road_x_position = (short)d3;
	}

	// set_road_centre_values
	private static void setRoadCentreValues() {
		b_players_position_flags = 0;

		int x_position_on_road = w_players_road_x_position - Road.ROAD_WIDTH / 2;
		if( b_plus_180_degrees < 0 ) {
			x_position_on_road = -x_position_on_road;
		}
		w_players_x_offset_from_road_centre = (short)x_position_on_road;
		if( x_position_on_road < 0 ) {
			x_position_on_road = -x_position_on_road;
		}
		if( x_position_on_road >= Road.ROAD_WIDTH / 2 ) {
			b_players_position_flags = (byte)0x80;
			if( w_players_x_offset_from_road_centre >= 0 ) {
				b_players_position_flags = 2;
			}
		}
		if( x_position_on_road >= Road.ROAD_POSITION_FOR_SPARKS ) {
			if( 0 == (b_off_map_status & 0x80) ) {
				b_off_map_status = (byte) 0x80;
				b_swing_from_left = (0 != (w_players_x_offset_from_road_centre & 0x8000));
				b_opponents_road_piece_m64_low = 0x10;

				initialiseSparksTable();
			}
			return;
		}
		if( 0 == (b_off_map_status & 0x40) ) {
			b_off_map_status = 0;
			b_1bb75 = false;
		}
	}

	// calculate_difference
	/**
	 *
	 * @param d0 as word. old difference
	 * @param d6 as word. new difference
	 * @return d0 as word. amount below road
	 */
	private static int calculateDifference(int d0, final int d6) {
		int d3 = 276;// FIXME numbers
		// if( 0 != d3 ) {
			d0 *= d3;// signed
			d0 >>= 8;// 276 / 256
		// }
		d0 += d6;
		return d0;
	}

	/* calculate_difference_between_opponent_wheels
	// extracted in place
	private static int calculateDifferenceBetweenOpponentWheels(int d1, int d2) {
		int d0 = opp_wheel_actual_heights[ d1 ] - opp_wheel_actual_heights[ d2 ];
		int d4 = d0;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		return d0;
	} */

	// average_two_values
	// extracted in place
	/**
	 *
	 * @param sa4 array of wheels
	 * @param d1 wheel index
	 * @param d2 wheel index
	 * @return d0, average value. is it used?
	 */
	private static int averageTwoValues(final short[] sa4, int d1, int d2) {
		/*
		final int d0 = (sa4[ d1 / 2 ] + sa4[ d2 / 2 ]) >> 1;
		sa4[ d1 / 2 ] = (short)d0;
		sa4[ d2 / 2 ] = (short)d0;
		 */
		final int d0 = (sa4[ d1 ] + sa4[ d2 ]) >> 1;
		sa4[ d1 ] = (short)d0;
		sa4[ d2 ] = (short)d0;
		return d0;
	}

	private static final short[] w_y_speed_adjustments = { 4, 4, -4 };

	// limit_opponent_wheels
	/**
	 * Adjusts opponent wheel heights and y speeds
	 * to limit car's x and z angle
	 * Especially important when in the air on more extreme tracks
	 * (e.g. Roller Coaster)
	 *
	 * @param d1 wheel index as byte
	 * @param d2 wheel index as byte
	 * @return d4 as word
	 */
	private static int limitOpponentWheels(int d1, int d2) {
		// opp_wheel_y_speed
		short[] sa4 = w_opp_wheel_y_speeds;
		int height_adjust_1 = d1;

		// jsr calculate_difference_between_opponent_wheels
		// d4 = result, d0 = abs( result )
		int d0 = w_opp_wheel_actual_heights[ d1 ] - w_opp_wheel_actual_heights[ d2 ];
		int d4 = d0;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		int d3 = w_max_difference - d0;
		if( d3 < 0 ) {
			if( d4 < 0 ) {
				d1 = d2;
			}
			// d1 = index of the highest wheel
			w_opp_wheel_actual_heights[ d1 ] += d3;// drop highest wheel
			if( d2 != OPP_FRONT_WHEEL ) {// 4
				// Following is reached on first call to this subroutine
				averageTwoValues( sa4, OPP_REAR_LEFT_WHEEL, d2 );// average rear wheel y speeds, then return
				return d4;
			}

			// average rear wheel y speeds
			averageTwoValues( sa4, OPP_REAR_LEFT_WHEEL, OPP_REAR_RIGHT_WHEEL );

			// average front and rear wheels y speeds
			// (both rear wheel values are currently the same)
			averageTwoValues( sa4, OPP_FRONT_WHEEL, OPP_REAR_RIGHT_WHEEL );

			// average rear wheels y speeds again
			averageTwoValues( sa4, OPP_REAR_LEFT_WHEEL, OPP_REAR_RIGHT_WHEEL );

			d2 = OPP_FRONT_WHEEL;
			d1 = height_adjust_1;
		}
		if( OPP_FRONT_WHEEL != d2 ) {
			return d4;// finish if first call to this subroutine
		}
		// Following is reached on second call to this subroutine
		if( ! b_opp_touching_road ) {
			// Adjust wheel y speeds when opponent in air,
			// possibly to make the car pitch forwards
			// .continue:
			d0 = sa4[ d1 ] - sa4[ OPP_FRONT_WHEEL ];
			if( d0 < 16 ) {
				d1 = MAX_WHEELS - 1;
				do {
					sa4[ d1 ] += w_y_speed_adjustments[ d1 ];
				} while( --d1 >= 0 );
			}
		}
		return d4;
	}

	// R.641b6
	private static void R_641b6() {
		// FIXME what numbers? is 80 y shift from bottom ?
		// Calculate Y coordinate of opponent's rear left wheel
		w_coord_y_values_fp5[ OPP_REAR_LEFT_WHEEL_Y / 2 ] =
				(short) (80 + w_opp_wheel_actual_heights[ OPP_REAR_LEFT_WHEEL ]);

		// Calculate Y coordinate of opponents rear right wheel
		w_coord_y_values_fp5[ OPP_REAR_RIGHT_WHEEL_Y / 2 ] =
				(short) (80 + w_opp_wheel_actual_heights[ OPP_REAR_RIGHT_WHEEL ]);

		// Calculate Y coordinate of opponents front wheels
		w_coord_y_values_fp5[ OPP_FRONT_LEFT_WHEEL_Y / 2 ] =
				(short) (80 + w_opp_wheel_actual_heights[ OPP_FRONT_WHEEL ]);

		final int d0 = (w_coord_y_values_fp5[ OPP_REAR_RIGHT_WHEEL_Y / 2 ]
				- w_coord_y_values_fp5[ OPP_REAR_LEFT_WHEEL_Y / 2 ]) >> 1;
		final int d3 = w_coord_y_values_fp5[ OPP_FRONT_LEFT_WHEEL_Y / 2 ] + d0;
		w_coord_y_values_fp5[ OPP_FRONT_RIGHT_WHEEL_Y / 2 ] = (short) d3;

		w_coord_y_values_fp5[ OPP_FRONT_LEFT_WHEEL_Y / 2 ] -= d0;
	}

	// update_opponents_actual_wheel_heights
	private static void updateOpponentsActualWheelHeights() {
		// Unlike the player's car, the opponent's car has
		// one front wheel value and two rear wheel values
		w_opp_smallest_difference = (short)0x8000;
		int d0 = 40;
		if( b_curved_section < 0 ) {
			d0 = 124;// increase collision when on a curve
		}
		w_height_adjust = (short)d0;
		int d7 = 0;
		//*** left rear
		d0 = w_opp_wheel_road_heights[ OPP_REAR_LEFT_WHEEL ]
			- w_opp_wheel_actual_heights[ OPP_REAR_LEFT_WHEEL ];
		if( d0 >= w_opp_smallest_difference ) {
			w_opp_smallest_difference = (short) d0;
		}
		d0 += w_height_adjust;
		// if wheel below road
		if( d0 < -96 ) {
			d0 = -96;// set to maximum amount above road
		}
		int d6 = d0;
		d0 -= w_opp_old_rear_left_difference;
		d0 = calculateDifference( d0, d6 );
		if( d0 < 0 ) {
			d0 = 0;
		}
		if( d0 > 1023 ) {
			d0 = 1023;
		}
		d7 |= d0;
		d0 -= w_height_adjust;
		w_opp_new_wheel_differences[ OPP_REAR_LEFT_WHEEL ] = (short) d0;
		w_opp_old_rear_left_difference = (short) d6;
		//*** right rear
		d0 = w_opp_wheel_road_heights[ OPP_REAR_RIGHT_WHEEL ]
				- w_opp_wheel_actual_heights[ OPP_REAR_RIGHT_WHEEL ];
		if( d0 > w_opp_smallest_difference ) {
			w_opp_smallest_difference = (short) d0;
		}
		d0 += w_height_adjust;
		if( d0 < -96 ) {
			d0 = -96;
		}
		d6 = d0;
		d0 -= w_opp_old_rear_right_difference;
		d0 = calculateDifference( d0, d6 );
		if( d0 < 0 ) {
			d0 = 0;
		}
		if( d0 > 1023 ) {
			d0 = 1023;
		}
		d7 |= d0;
		d0 -= w_height_adjust;
		w_opp_new_wheel_differences[ OPP_REAR_RIGHT_WHEEL ] = (short) d0;
		w_opp_old_rear_right_difference = (short) d6;
		//*** front
		d0 = w_opp_wheel_road_heights[ OPP_FRONT_WHEEL ]
				- w_opp_wheel_actual_heights[ OPP_FRONT_WHEEL ];
		if( d0 > w_opp_smallest_difference ) {
			w_opp_smallest_difference = (short) d0;
		}
		d0 += w_height_adjust;
		// wheel above road
		if( d0 < -96 ) {
			d0 = -96;
		}
		d6 = d0;
		d0 -= w_opp_old_front_difference;
		d0 = calculateDifference( d0, d6 );
		if( d0 < 0 ) {
			d0 = 0;
		}
		if( d0 > 1023 ) {
			d0 = 1023;
		}
		d7 |= d0;// touching road
		d0 -= w_height_adjust;
		w_opp_new_wheel_differences[ OPP_FRONT_WHEEL ] = (short) d0;
		w_opp_old_front_difference = (short) d6;
		//***
		d0 = d7 >> 8;
		d0 |= d7;
		b_opp_touching_road = (d0 != 0);
		//***
		int d4 = w_opp_new_wheel_differences[ OPP_REAR_LEFT_WHEEL ]
				+ w_opp_new_wheel_differences[ RIGHT_FRONT_WHEEL ]
				+ w_opp_new_wheel_differences[ OPP_FRONT_WHEEL ];
		// Make accelerations from 6 parts: wheel difference
		// in question and 1 part of the other two wheels
		// (only one central front wheel is considered).
		d7 = w_opp_new_wheel_differences[ OPP_REAR_LEFT_WHEEL ] << 2;
		d0 = (d4 + d7 + w_opp_new_wheel_differences[ OPP_REAR_LEFT_WHEEL ]) >> 3;
		w_opp_rear_left_y_acceleration = (short) d0;

		d7 = w_opp_new_wheel_differences[ OPP_REAR_RIGHT_WHEEL ] << 2;
		d0 = (d4 + d7 + w_opp_new_wheel_differences[ OPP_REAR_RIGHT_WHEEL ]) >> 3;
		w_opp_rear_right_y_acceleration = (short) d0;

		d7 = w_opp_new_wheel_differences[ OPP_FRONT_WHEEL ] << 2;
		d0 = (d4 + d7 + w_opp_new_wheel_differences[ OPP_FRONT_WHEEL ]) >> 3;
		w_opp_front_y_acceleration = (short) d0;
		//***
		if( 0 != (b_opponent_attributes[ b_opponents_id ] & WHEELIE) ) {
			d0 = (w_opp_wheel_y_speeds[ OPP_FRONT_WHEEL ] | w_opp_front_y_acceleration) & 0xFFFC;
			if( 0 == d0 ) {
				if( 0 == (randomizeLong() & 0x0F) ) {
					// Make opponent do a wheelie
					w_opp_wheel_y_speeds[ OPP_FRONT_WHEEL ] = 160;
				}
			}
		}
		//***
		d0 = (w_opp_rear_left_y_acceleration * REDUCTION) >> 8;
		d0 += w_opp_wheel_y_speeds[ OPP_REAR_LEFT_WHEEL ];
		w_opp_wheel_y_speeds[ OPP_REAR_LEFT_WHEEL ] = (short) d0;

		d0 *= REDUCTION;
		d0 >>= 9;
		w_opp_wheel_actual_heights[ OPP_REAR_LEFT_WHEEL ] += d0;
		//***
		d0 = (w_opp_rear_right_y_acceleration * REDUCTION) >> 8;
		d0 += w_opp_wheel_y_speeds[ OPP_REAR_RIGHT_WHEEL ];
		w_opp_wheel_y_speeds[ OPP_REAR_RIGHT_WHEEL ] = (short) d0;

		d0 *= REDUCTION;
		d0 >>= 9;
		w_opp_wheel_actual_heights[ OPP_REAR_RIGHT_WHEEL ] += d0;
		//***
		d0 = (w_opp_front_y_acceleration * REDUCTION) >> 8;
		d0 += w_opp_wheel_y_speeds[ OPP_FRONT_WHEEL ];
		w_opp_wheel_y_speeds[ OPP_FRONT_WHEEL ] = (short) d0;

		d0 *= REDUCTION;
		d0 >>= 9;
		w_opp_wheel_actual_heights[ OPP_FRONT_WHEEL ] += d0;
		//***
		w_max_difference = 296;// param for limitOpponentWheels
		d4 = limitOpponentWheels( OPP_REAR_LEFT_WHEEL, OPP_REAR_RIGHT_WHEEL );

		w_max_difference = 368;
		int d1 = OPP_REAR_LEFT_WHEEL;
		if( d4 < 0 ) {
			// rear right wheel instead (because this is higher than rear left)
			d1 = OPP_REAR_RIGHT_WHEEL;
		}
		limitOpponentWheels( d1, OPP_FRONT_WHEEL );
		//***
		R_641b6();
	}

	// TAB_5be34 signed bytes
	private static final byte[] b_TAB_5be34 = {
			0x20, 0x50, 0x60, 0x70, 0x70, 0x60, 0x50, 0x20,
			(byte)0xe0, (byte)0xb0, (byte)0xa0, (byte)0x90, (byte)0x90, (byte)0xa0, (byte)0xb0, (byte)0xe0
		};
			
	// randomize_opponents_steering
	private static void randomizeOpponentsSteering() {
		if( ! b_opp_touching_road || (0 != b_machine) ) {
			return;
		}
		int d1 = 0;// left side
		b_1bbbe = 0;
		b_1bbbf = 0;
		b_1bbbd = 0;
		int d0 = b_opponents_random_steering_count;
		if( 0 != d0 ) {
			if( b_fourteen_frames_elapsed >= 0 ) {
				--b_opponents_random_steering_count;
			}
			d0 += b_1bbc2;
			d0 &= 0x0F;
			int d2 = d0;
			d0 = b_TAB_5be34[ d2 ];// z shift
			if( d0 < 0 ) {
				d0 = -d0;
				++d1;// right side
			}
			switch( d1 ) {
				case 0:// left side
					b_1bbbe = (byte) d0;
					break;
				case 1:// right side
					b_1bbbf = (byte) d0;
					break;
			}
			d2 += 5;
			d2 &= 0x0F;
			d0 = b_TAB_5be34[ d2 ];
			b_1bbbd = (byte) d0;
		} else {
			int d2 = b_opponents_road_piece;
			if( b_opponents_speed_values[ d2 ] >= 0 ) {
				if( !b_opponent_behind_player ) {
					if( b_curved_section >= 0 ) {// current piece type ?
						d2 = 8;// shift
						if( b_1bb9d < 0 ) {
							if( 0 != (b_1bb9d & 0x40 ) ) {// diagonal piece (45 degrees)?
								d2 = 16;
							}
							b_1bbc2 = (byte) d2;

							d0 = randomizeLong() & 0x1F;
							if( b_opponents_id >= d0 ) {
								b_opponents_random_steering_count = 16;
							}
						}
					}
				}
			}
		}
		d0 = (b_plus_180_degrees & 0xFF) >>> 1;
		d0 ^= b_curved_section;
		b_1bb9d = (byte)d0;
	}

	// get_opponents_engine_acceleration
	private static void getOpponentsEngineAcceleration() {
		int power = w_opponents_engine_power;
		if( 0 != b_opponents_random_steering_count ) {
			power -= 25;
		}
		if( ! b_opp_touching_road ) {
			power = 0;
		}
		w_opponents_engine_z_acceleration = (short) power;
	}

	// adjust_opponents_engine_acceleration
	private static void adjustOpponentsEngineAcceleration() {
		if( ! b_opp_touching_road ) {
			return;
		}
		int d1 = b_opponents_road_piece;
		int d0 = b_opponents_speed_values[ d1 ];
		if( (d0 >= 0) && (d0 > b_opponent_max_speed) ) {
			d0 = b_opponent_max_speed;
		}
		d0 &= 0x7F;
		b_opponents_required_z_speed = (byte) d0;
		d0 = (w_opponents_z_speed >> 8) - b_opponents_required_z_speed;
		if( d0 == 0 ) {
			b_opponents_required_z_speed_reached = true;
			return;
		}
		if( d0 > 0 ) {
			// Speed is greater than required speed
			b_opponents_required_z_speed_reached = true;
			w_opponents_engine_z_acceleration = (short) -w_opponents_engine_z_acceleration;
			if( d0 < 14 ) {
				return;
			}
		}
		// sometimes_double_acceleration
		if( (b_opponents_speed_values[ d1 ] >= 0) && (d0 < 0) && b_opponents_required_z_speed_reached ) {
			if( d0 >= -2 ) {
				return;
			}
			b_opponents_required_z_speed_reached = false;
		}
		// double.acceleration
		w_opponents_engine_z_acceleration <<= 1;
	}

	// byte_multiply
	/**
	 * Extract in place
	 *
	 * @param d0 unsigned byte
	 * @param {@link #factor1}
	 * @return d3 as a word, #product is low byte, d0 is high byte
	 */
	private static void byteMultiply(int d0) {
		/*
		d3 = d0 = (factor1 & 0xFF) * (d0 & 0xFF);
		product = (byte)d0;
		d0 >>>= 8;
		*/
	}

	// update_opponents_z_speed
	private static void updateOpponentsZspeed() {
		int w_acceleration_adjust = 0;
		if( w_opponents_z_speed >= 0 ) {
			int d0 = w_opponents_z_speed >> 7;
			if( b_player_close_to_opponent && b_opponent_behind_player ) {
				// Reduce speed value if opponent close behind player
				d0 -= 20;
				if( d0 < 0 ) {
					d0 = 0;
				}
			}
			int factor1 = d0;
			// A fraction of the square of the speed is subtracted from the acceleration
			// It has a small effect only
			w_acceleration_adjust = ((w_opponents_z_speed >>> 8) * factor1) >> 6;

			if( b_opp_touching_road ) {
				// Reduce the acceleration further when on the road
				d0 = w_opponents_engine_z_acceleration;
				if( d0 >= 0 ) {
					int d3 = w_opponents_z_speed >> 8;
					// subtract fraction of speed from acceleration
					d0 -= d3;
					if( b_curved_section < 0 ) {
						d0 -= d3;// subtract again when on a curve
						d0 -= 35;// then reduce further
					}
					w_opponents_engine_z_acceleration = (short)d0;
				}
			}
		}
		// acceleration.ok
		int d0 = w_opponents_engine_z_acceleration - w_acceleration_adjust;
		if( b_opp_touching_road ) {
			int d3 = (w_opp_wheel_road_heights[ OPP_REAR_LEFT_WHEEL ]
					+ w_opp_wheel_road_heights[ OPP_REAR_RIGHT_WHEEL ]) >>> 1;
			d3 -= w_opp_wheel_road_heights[ OPP_FRONT_WHEEL ];
			// d3 is - 've when opponent pitched backwards, +' ve when pitched forwards
			int d4 = d3;
			if( d4 < 0 ) {
				d4 = -d4;
			}
			if( d4 >= 512 ) {
				d4 = 510;
			}
			d4 >>>= 1;// pitch value /2
			int d5 = d4;
			d5 >>= 2;// pitch value /8
			d5 += d4;// (5 * pitch value) /8
			if( d3 < 0 ) {
				d5 = -d5;
			}
			// Acceleration is reduced when opponent pitched backwards,
			// increased when pitched forwards i.e.effect of gravity
			d0 += d5;
		}
		// + acceleration
		w_opponents_z_speed += ((d0 * REDUCTION) >> 8);
		if( w_opponents_z_speed < 0 ) {
			w_opponents_z_speed = 0;
		}
	}

	// opponent_movement
	private static void opponentMovement() {
		if( !b_drop_start_done || (0 != b_1bb74) ) {
			return;
		}
		fetchNearSectionStuff( b_opponents_road_piece );

		updateOpponentsActualWheelHeights();
		randomizeOpponentsSteering();
		getOpponentsEngineAcceleration();
		adjustOpponentsEngineAcceleration();
		updateOpponentsZspeed();

		int d3 = (b_road_length_reduction << 7) & 0x7FFF;
		int d0 = ((((w_opponents_z_speed * d3) >> 15) * REDUCTION) >> 8) << 3;
		d3 = d0;
		d0 >>= 8;
		d3 += byte_count;
		byte_count = (byte) d3;
		if( d3 >= 256 ) {
			++d0;
		}
		w_opponents_distance_into_piece += d0;
		d0 = (w_opponents_distance_into_piece >> 8);
		if( d0 >= b_number_of_segments ) {
			d0 -= b_number_of_segments;
			w_opponents_distance_into_piece =
					(short)((w_opponents_distance_into_piece & 0xFF) | (d0 << 8));
			int d1 = b_opponents_road_piece;
			if( ++d1 >= b_number_of_road_pieces ) {
				d1 = 0;
			}
			b_opponents_road_piece = (byte) d1;
		}
	}

	// store_segment_xz_coord
	/**
	 * Near X and Z coordinates are stored into segment X and Z coordinates
	 *
	 * @param coordOffsetX2 offset in d1 for {@link #w_segment_x_coords} and {@link #w_segment_z_coords}
	 * @param pairOffsetX4 offset for coordinate pair (x 4) as word in d2
	 * @return next pairOffsetX4 in d2 as word
	 */
	private static int storeSegmentXZcoord(final int coordOffsetX2, int pairOffsetX4) {
		pairOffsetX4 = fetchNearXZcoords( pairOffsetX4 );
		w_segment_x_coords[ coordOffsetX2 / 2 ] = w_near_x_coord;
		w_segment_z_coords[ coordOffsetX2 / 2 ] = w_near_z_coord;
		return pairOffsetX4;
	}

	// fetch_segment_xz_coords
	/**
	 *
	 * @param segmentDataOffset in d0 as byte
	 */
	private static void fetchSegmentXZcoords(int segmentDataOffset) {
		// d0 = w_piece_data_offset;
		// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
		final byte[] ba3 = w_piece_data_offset;// Road.offsetToPiece( d0 );

		b_piece_coords_offset = (byte) (ba3[ 0 ] + 7);

		if( 0 == b_plus_180_degrees ) {
			// each segment is (x,z), (x,z). x and z are words
			segmentDataOffset <<= 3;
			segmentDataOffset += b_piece_coords_offset;
			int coord_pair_x2 = 0;
			do {// 4 coord pairs L(x,z)-R(x,z)===L(x,z)-R(x,z)
				segmentDataOffset = storeSegmentXZcoord( coord_pair_x2, segmentDataOffset );
				coord_pair_x2 += 2;
			} while( coord_pair_x2 < 8 );
			return;
		}
		// each segment is (x,z), (x,z). x and z are words
		segmentDataOffset = (b_number_of_segments - segmentDataOffset - 1) << 3;
		segmentDataOffset += b_piece_coords_offset;
		int coord_pair_x2 = 6;
		do {// 4 coord pairs L(x,z)-R(x,z)===L(x,z)-R(x,z)
			segmentDataOffset = storeSegmentXZcoord( coord_pair_x2, segmentDataOffset );
			coord_pair_x2 -= 2;
		} while( coord_pair_x2 >= 0 );
	}

	// fetch_surface_y_coord
	/**
	 * @param d1 as byte, offset for the coordinate
	 * @return d0 as word
	 */
	private static int fetchSurfaceYcoord(final byte[] ba4, final byte[] ba5, int d1) {
		int d2 = d1;
		int d0;
		if( b_y_coords_stored_as_words < 0 ) {
			// get_word_y_coord:
			d2 &= 0xFE;
			if( 0 == (d1 & 1) ) {
				// final int d3 = (ba4[ d2 + 1 ] & 0xFF);
				// d0 = (ba4[ d2 ] & 0x7F) << 8;
				// d0 |= d3;
				d0 = Road.getCoordinateAsWord( ba4, d2 );
				d0 += w_overall_left_y_shift;
			} else {
				// right_y_coord:
				// final int d3 = (ba5[ d2 + 1 ] & 0xFF);
				// d0 = (ba5[ d2 ] & 0x7F) << 8;
				// d0 |= d3;
				d0 = Road.getCoordinateAsWord( ba5, d2 );
				d0 += w_overall_right_y_shift;
			}
		} else {
			// get_byte_y_coord:
			if( 0 == (d2 & 1) ) {
				// d0 = ba4[ d2 / 2 ];
				// d0 = ((d0 << 1) & 0xE0) | ((d0 & 0x0F) << 8);
				d0 = Road.getCoordinateAsByte( ba4, d2 / 2 );
				d0 += w_overall_left_y_shift;
			} else {
				// right_y_coord2:
				// d0 = ba5[ d2 / 2 ];
				// d0 = ((d0 << 1) & 0xE0) | ((d0 & 0x0F) << 8);
				d0 = Road.getCoordinateAsByte( ba5, d2 / 2 );
				d0 += w_overall_right_y_shift;
			}
		}
		// got_surface_y_coord:
		d0 >>= 5;
		return d0;
	}

	// calculate_segment_coord
	/**
	 * Java: segment is chosen as the argument and not as an offset like Amiga does.
	 *
	 * @param sa0_segment {@link #w_segment_x_coords} or {@link #w_segment_z_coords}.
	 * @param d1 as byte. index of {@link #w_segment_x_coords} in bytes to read
	 * @param d2 as byte. index of {@link #w_segment_x_coords} in bytes to store
	 */
	private static void calculateSegmentCoord(final short[] sa0_segment, int d1, final int d2) {
		int d0 = sa0_segment[ (d1 + 4) / 2 ] - sa0_segment[ (d1) / 2 ];
		int d3 = w_surface_position;
		d0 *= d3;// signed
		d0 >>= 8;
		if( b_next_segment ) {
			// Otherwise use coordinates from the next segment
			d1 += 4;
			// storeSegmentCoord( sa0, 0, d1, d2 );
			d0 += sa0_segment[ (d1) / 2 ];
			sa0_segment[ (d2) / 2 ] = (short) d0;
			// d1 -= 4;
			return;
		}
		// store_segment_coord:
		d0 += sa0_segment[ (d1) / 2 ];
		sa0_segment[ (d2) / 2 ] = (short) d0;
	}

	// calculate_segment_xz
	/**
	 * @param d0 as byte
	 * @param d1 as byte. index of {@link #w_segment_x_coords} in bytes to read
	 * @param d2 as byte. index of {@link #w_segment_x_coords} in bytes to store
	 */
	private static void calculateSegmentXZ(int d0, int d1, int d2) {
		w_surface_position = (short) d0;

		// segment x coordinates
		calculateSegmentCoord( w_segment_x_coords, d1, d2 );

		// segment z coordinates
		calculateSegmentCoord( w_segment_z_coords, d1, d2 );
	}

	// calculate_segment_xz_at_opp_rear
	/**
	 *
	 * @param d1 0 or 2
	 * @param d2 as byte. index of {@link #w_segment_x_coords} in bytes to store
	 */
	private static void calculateSegmentXZatOppRear(int d1, int d2) {
		int d0 = (w_opponents_distance_into_section_minus64 & 0xFF);// opponents.distance.into.section.minus64 + 1;
		// byte[] ba0 = b_1bbbe;// random z shift ?
		// d0 += ba0[ d1 ];
		switch( d1 ) {// TODO what does it happen?
			case 0:
				d0 += b_1bbbe;
				break;
			case 2:
				d0 += b_opponents_random_steering_count;
				break;
		}
		b_next_segment = (d0 >= 256);// flag indicating moved to a next segment
		d0 &= 0xFF;

		calculateSegmentXZ( d0, d1, d2 );
	}

	// calculate_section_position
	/**
	 * @param d0 as word
	 * @return int64 for d0 and d1, d0 as word, d1 as byte.
	 * 0x00 D1 D0h D0l
	 *
	 * opponent road section (piece) as high word
	 * distance as low word. segment is distance >> 8
	 */
	private static int calculateSectionPosition(int d0) {
		b_current_road_piece = b_opponents_road_piece;
		int d1 = b_opponents_road_piece;
		d0 = w_opponents_distance_into_piece - d0;// distance
		if( d0 < 0 ) {// go to previous piece
			d1 = toPreviousRoadPiece();
			fetchNearSectionStuff( d1 );
			// distance. segment is distance / 256
			d0 += ((int) b_number_of_segments << 8);
		}
		w_opponents_distance_into_section_minus255 = (short) d0;
		b_opponents_road_section_m255 = (byte) d1;
		return ((d1 & 0xFF) << 16) | (d0 & 0xFFFF);
	}

	// calculate_road_surface_height
	/**
	 * Calculates the height of the current road patch,
	 * at the specified x and z position.
	 *
	 * {@link #lw_road_height_fp8} as int32.
	 */
	private static void calculateRoadSurfaceHeight() {
		int d3_fraction_fp8 = w_surface_x_position & 0xFF;
		int d5_close_section_y_fp8 = (w_surface_y_coord2 - w_surface_y_coord1) * d3_fraction_fp8;
		d5_close_section_y_fp8 += (w_surface_y_coord1 << 8);

		int d0_far_section_y_fp8 = (w_surface_y_coord4 - w_surface_y_coord3) * d3_fraction_fp8;
		d0_far_section_y_fp8 += (w_surface_y_coord3 << 8);

		d3_fraction_fp8 = w_surface_z_position & 0xFF;
		d0_far_section_y_fp8 -= d5_close_section_y_fp8;
		int d4_abs_far_section_y_fp8 = d0_far_section_y_fp8;
		if( d4_abs_far_section_y_fp8 < 0 ) {
			d4_abs_far_section_y_fp8 = -d4_abs_far_section_y_fp8;
		}
		if( d4_abs_far_section_y_fp8 >= 0x80_00  ) {
			// needs_a_longword:
			d0_far_section_y_fp8 >>= 3;// fp5 now
			if( d0_far_section_y_fp8 < 0 ) {
				d0_far_section_y_fp8 = -d0_far_section_y_fp8;
				d0_far_section_y_fp8 *= d3_fraction_fp8;// unsigned
				d0_far_section_y_fp8 &= 0xFFFFFF00;
				d0_far_section_y_fp8 = -d0_far_section_y_fp8;
			} else {
				d0_far_section_y_fp8 *= d3_fraction_fp8;// unsigned
			}
			d0_far_section_y_fp8 <<= 3;// fp8 again
		} else {
			// fits_into_a_word:
			if( d0_far_section_y_fp8 < 0 ) {
				d0_far_section_y_fp8 = -d0_far_section_y_fp8;
				d0_far_section_y_fp8 *= d3_fraction_fp8;// unsigned
				d0_far_section_y_fp8 &= 0xFFFFFF00;
				d0_far_section_y_fp8 = -d0_far_section_y_fp8;
			} else {
				d0_far_section_y_fp8 *= d3_fraction_fp8;// unsigned, fp 16 now
			}
		}
		d0_far_section_y_fp8 >>= 8;
		d0_far_section_y_fp8 += d5_close_section_y_fp8;
		lw_road_height_fp8 = d0_far_section_y_fp8;
	}

	// calculate_opponents_road_wheel_height
	/**
	 *
	 * @param d1 wheel id, 0 or 2 or 4. d1/2 is a index, as byte
	 */
	private static void calculateOpponentsRoadWheelHeight(final int d1) {
		calculateRoadSurfaceHeight();
		w_opp_wheel_road_heights[ d1 / 2 ] = (short)(lw_road_height_fp8 >> 3);
	}

	// calculate_opp_front_road_height
	/**
	 * Now have:
	 * segment.x.coords	x1,		x2,		x3,		x4
	 * left edge x, left wheel x, right edge x, right wheel x; at car's rear
	 *
	 * segment.z.coords	z1,		z2,		z3,		z4
	 * left edge z, left wheel z, right edge z, right wheel z; at car's rear
	 */
	private static void calculateOppFrontRoadHeight() {
		// segment_x_coords: offset 0, segment_z_coords: offset 32
		// short[] sa0 = segment_x_coords;
		int d0 = w_segment_x_coords[ 14 / 2 ] - w_segment_x_coords[ 10 / 2 ];// rear right wheel x - rear left wheel x
		w_opp_xdiff = (short)d0;
		d0 >>= 1;
		w_opp_xdiff += d0;

		d0 = w_segment_z_coords[ 14 / 2 ] - w_segment_z_coords[ 10 / 2 ];// rear right wheel z - rear left wheel z
		w_opp_zdiff = (short)d0;
		d0 >>= 1;
		w_opp_zdiff += d0;

		// Calculate x, z of front wheels
		d0 = w_segment_x_coords[ 10 / 2 ];// rear left wheel x
		d0 -= w_opp_zdiff;
		w_segment_x_coords[ 16 / 2 ] = (short)d0;// front left wheel x

		d0 = w_segment_z_coords[ 10 / 2 ];// rear left wheel z
		d0 += w_opp_xdiff;
		w_segment_z_coords[ 16 / 2 ] = (short)d0;// front left wheel z

		d0 = w_segment_x_coords[ 14 / 2 ];// rear right wheel x
		d0 -= w_opp_zdiff;
		w_segment_x_coords[ 20 / 2 ] = (short)d0;// front right wheel x

		d0 = w_segment_z_coords[ 14 / 2 ];// rear right wheel z
		d0 += w_opp_xdiff;
		w_segment_z_coords[ 20 / 2 ] = (short)d0;// front right wheel z

		d0 = w_opponents_distance_into_section_minus64 & 0xFF;
		d0 += 128;// add 128 to get z of opponent 's front
		if( d0 >= 256 ) {
			w_surface_y_coord1 = w_surface_y_coord3;
			w_surface_y_coord2 = w_surface_y_coord4;
			w_surface_y_coord3 = w_surface_y_coord5;
			w_surface_y_coord4 = w_surface_y_coord6;
		}
		w_surface_z_position = (short)((w_surface_z_position & 0xFF00)
				| (d0 & 0x00FF));// replace low byte by d0 byte
		// replace low byte of the surface_x_position by low byte of the opponents_road_x_position
		w_surface_x_position = (short)((w_surface_x_position & 0xFF00)
				| (w_opponents_road_x_position & 0x00FF));

		calculateOpponentsRoadWheelHeight( OPP_FRONT_WHEEL * 2 );
	}

	// calculate_opponents_road_wheel_positions
	/**
	 * calculates opponent's road wheel heights and also x,z positions
	 */
	private static void calculateOpponentsRoadWheelPositions() {

		final int d1_d0 = calculateSectionPosition( 64 );
		// FIXME already assigned
		int d0 = (int)(d1_d0 & 0xFFFF);// word
		int d1 = (int)((d1_d0 & 0xFF0000) >>> 16);// byte

		w_opponents_distance_into_section_minus64 = (short) d0;
		b_opponents_road_piece_m64_high = (byte) d1;// FIXME not used?

		fetchNearSectionStuff( d1 );
		fetchXZposition( d1 );
		w_rough_difference_angle = (short)(w_rough_player_angle - w_rough_piece_angle);

		// d0 = w_left_y_coord_offset;
		// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
		byte[] ba4 = w_left_y_coord_offset;//  Road.offsetToNearPieceYdataBytes( d0 );

		// d0 = w_right_y_coord_offset;
		// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
		byte[] ba5 = w_right_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

		// (segment << 1) | L
		d1 = (w_opponents_distance_into_section_minus64 >> 8) << 1;// left, bit 0: 0 - left, 1 - right
		d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
		w_surface_y_coord1 = (short)d0;

		++d1;// right
		d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
		w_surface_y_coord2 = (short)d0;

		++d1;// left
		d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
		w_surface_y_coord3 = (short)d0;

		++d1;// left
		d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
		w_surface_y_coord4 = (short)d0;

		++d1;// right
		if( d1 >= b_number_of_sections_x2 ) {
			// If on last segment of road section
			// then get data for next section,
			// then get last two coordinates

			d1 = toNextRoadPiece();
			fetchNearSectionStuff( d1 );

			// d0 = w_left_y_coord_offset;
			// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			ba4 = w_left_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

			// d0 = w_right_y_coord_offset;
			// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			ba5 = w_right_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

			d1 = 2;
			d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
			w_surface_y_coord5 = (short) d0;

			++d1;
			d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
			w_surface_y_coord6 = (short) d0;

			// Move back to previous section

			++d1;// FIXME changed in the next call
			d1 = toPreviousRoadPiece();
			fetchNearSectionStuff( d1 );

			// d0 = w_left_y_coord_offset;
			// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			// ba4 = w_left_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );

			// d0 = w_right_y_coord_offset;
			// d0 = ((((d0 & 0xFF00) >>> 8) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			// ba5 = w_right_y_coord_offset;// Road.offsetToNearPieceYdataBytes( d0 );
			// FIXME ba4 and ba5 are not used!
		} else {
			// Get last two coordinates
			// _label1:
			d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
			w_surface_y_coord5 = (short) d0;

			++d1;
			d0 = fetchSurfaceYcoord( ba4, ba5, d1 );
			w_surface_y_coord6 = (short) d0;
			++d1;
		}
		// Have now got all 6 coordinates (i.e. of two segments that opponent is on)
		// _label2:
		// get segment number
		// fetch segment's four world x,z coordinates
		fetchSegmentXZcoords( w_opponents_distance_into_section_minus64 >> 8 );

		// Calculate segment left side x,z at opponents.distance.into.section.minus64
		// source offset = 0, store at offset = 8
		calculateSegmentXZatOppRear( 0, 8 );

		// Calculate segment right side x,z at opponents.distance.into.section.minus64
		// source offset = 2, store at offset = 12
		calculateSegmentXZatOppRear( 2, 12 );

		b_next_segment = false;
		d0 = w_opp_wheel_actual_heights[ OPP_REAR_LEFT_WHEEL ] - w_opp_wheel_actual_heights[ OPP_REAR_RIGHT_WHEEL ];
		if( d0 < 0 ) {
			d0 = -d0;
		}
		d0 >>= 4;
		opponents_x_span = Road.opponents_x_spans[ d0 ];//	half the x distance that the opponent's rear wheels span
		// surface_z_position+1 = opponents_distance_into_section_minus64+1;
		w_surface_z_position =
				(short)((w_surface_z_position & 0xFF00) |
				(w_opponents_distance_into_section_minus64 & 0xFF));

		// Calculate opp.rear.left.road.height
		w_surface_x_position = (short)(w_opponents_road_x_position - opponents_x_span);// x position of rear left wheel
		calculateOpponentsRoadWheelHeight( OPP_REAR_LEFT_WHEEL * 2 );// calculate.opponents.road.wheel.height

		// Calculate rear left wheel x,z
		// source offset = 8, store at offset = 10
		calculateSegmentXZ( w_surface_x_position, 8, 10 );// calculate.segment.xz

		// Calculate opp.rear.right.road.height
		w_surface_x_position = (short)(w_opponents_road_x_position + opponents_x_span);// x position of rear right wheel
		calculateOpponentsRoadWheelHeight( OPP_REAR_RIGHT_WHEEL * 2 );

		// Calculate rear right wheel x,z
		// source offset = 8, store at offset = 14
		calculateSegmentXZ( w_surface_x_position, 8, 14 );

		// Calculate opp.front.road.height
		d0 = w_surface_y_coord1 - w_surface_y_coord3;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		if( d0 >= 20 ) {
			b_do_not_draw_opponents_shadow = true;
		}
		calculateOppFrontRoadHeight();

		d0 = w_surface_y_coord1 - w_surface_y_coord3;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		if( d0 >= 20 ) {
			b_do_not_draw_opponents_shadow = true;
		}

		// Finally calculate section position for opponents.distance.into.section.minus255 (used for drawing)
		calculateSectionPosition( 255 );
	}

	// calculate_distances_between_players
	private static void calculateDistancesBetweenPlayers() {
		int d0 = (w_opponents_distance_into_piece - w_players_distance_into_section) >> 3;
		int d1 = b_opponents_road_piece;
		int d2 = b_players_road_piece;
		d0 += w_distances_around_road[ d1 ] - w_distances_around_road[ d2 ];
		int d5 = d0;
		w_difference_between_players = (short) d0;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		int d4 = w_total_road_distance - d0;
		if( d4 >= d0 ) {// compare two road distances
			d4 = d0;// get the smallest distance
			d5 ^= 0xFFFF8000;// opposite sign -diff-1
		}
		w_smallest_distance_between_players = (short) d4;
		d5 ^= 0xFFFF8000;// opposite sign -diff-1
		// d5 >>>= 5;
		b_opponent_behind_player = (d5 < 0);
	}

	// car_to_car_collision_detection
	private static void carToCarCollisionDetection() {
		if( !b_drop_start_done ) {
			return;
		}
		if( ! b_opp_touching_road || (!b_touching_road) ) {
			int d0 = (w_players_smaller_y - w_opp_wheel_actual_heights[ OPP_REAR_LEFT_WHEEL ]);
			int d4 = d0;
			d0 += 40;
			if( d0 < 0 ) {
				d0 = -d0;
			}
			if( d0 >= 192 ) {
				b_collision_counter = 3;
				return;
			}
			if( 0 != b_collision_counter ) {
				--b_collision_counter;
				int d3 = 256 - d0;
				if( d4 < 0 ) {
					d3 = -d3;
				}
				d3 <<= 4;
				w_car_to_car_y_acceleration = (short) d3;
			}
		}
		int d0 = ((int) b_abs_x_difference) & 0xFF;
		if( d0 < 45 ) {
			d0 = w_smallest_distance_between_players & 0xFF;
			if( d0 < 8 ) {
				d0 = 8;
				if( !b_player_to_right ) {
					d0 = -8;
				}
				w_car_to_car_x_acceleration = (short) ((w_car_to_car_x_acceleration & 0xFF)
						| ((d0 << 8) & 0xFF00));
			}
		}
		// .label8:
		if( ! b_1bbeb ) {
			int d3 = 3;
			d0 = 0;
			if( 0 == b_1bb74 ) {
				d0 = w_opponents_z_speed;
			}
			d0 -= w_players_z_speed;
			if( d0 < 0 ) {
				d3 = -3;
			}
			d0 >>= 1;
			d0 += d3;
			w_car_to_car_z_acceleration = (short) d0;
		}
		// _labelb:
		b_cars_collided = true;
		b_1bbeb = true;
		int d3 = 512;
		d0 = w_car_to_car_x_acceleration;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		d3 += d0;
		d0 = w_car_to_car_y_acceleration;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		d3 += d0;
		d0 = w_car_to_car_z_acceleration;
		if( d0 < 0 ) {
			d0 = -d0;
		}
		d3 += d0;
		d3 >>>= 8;
		// _labelf:
		d0 = (b_front_right_damage & 0xFF) + d3;
		if( d0 > 255 ) {
			d0 = 255;
		}
		b_front_right_damage = (byte)d0;

		d0 = (b_front_left_damage & 0xFF) + d3;
		if( d0 > 255 ) {
			d0 = 255;
		}
		b_front_left_damage = (byte)d0;

		b_damaged = true;
	}

	// move_opponent_to_one_side
	/**
	 * Position opponent on left or right
	 */
	private static void moveOpponentToOneSide() {
		if( (((int) b_abs_x_difference) & 0xFF) >= 56 ) {
			return;
		}
		if( b_player_to_right ) {
			b_opponents_suggested_road_x_position = 32;
			return;
		}
		b_opponents_suggested_road_x_position = (byte)(256 - 32);
	}

	// opponent_obstruct_player
	/**
	 * Put opponent at same position as player
	 */
	private static void opponentObstructPlayer() {
		b_opponents_suggested_road_x_position = b_rear_wheel_surface_x_position;
	}

	// opponent_push_player
	/**
	 * Opponent pushing player off track
	 */
	private static void opponentPushPlayer() {
		int d0 = ((int) b_abs_x_difference) & 0xFF;
		if( d0 >= 56 ) {
			return;
		}
		// TODO make the code more clear
		d0 = ((int) b_rear_wheel_surface_x_position) & 0xFF;
		if( b_player_to_right ) {
			if( d0 >= 96 ) {
				b_opponents_suggested_road_x_position = 32;
				return;
			}
			b_opponents_suggested_road_x_position = (byte)(256 - 32);
			return;
		}
		if( d0 >= (256 - 96) ) {
			b_opponents_suggested_road_x_position = 32;
			return;
		}
		b_opponents_suggested_road_x_position = (byte)(256 - 32);
	}

	// java: to avoid goto
	/**
	 *
	 * @param d1 as byte, opponent index
	 */
	private static void opponentPlayerInteractionLabel4(final int d1) {
		if( 0 != (b_opponent_attributes[ d1 ] & DRIVES_NEAR_EDGE) ) {
			if( !b_opponent_behind_player ) {
				if( (w_smallest_distance_between_players & 0xFF) >= 14 ) {
					opponentPlayerInteractionLabelFar( d1 );// bcc	.far
					return;
				}
			}
		}
		moveOpponentToOneSide();
		opponentPlayerInteractionLabelF();
	}

	// java: to avoid goto
	private static void opponentPlayerInteractionLabelF() {
		int d0 = b_1bbbd;
		if( d0 >= 0 ) {
			if( 0 != d0 ) {
				// _label11
				if( d0 < 16 ) {
					return;
				}
				d0 = 9;
			} else {
				d0 = (((int) b_opponents_suggested_road_x_position) & 0xFF)
						- (w_opponents_road_x_position & 0xFF);
				if( 0 == d0 ) {
					return;
				}
				if( d0 >= 0 ) {
					// _label11
					if( d0 < 16 ) {
						return;
					}
					d0 = 9;
				} else {
					if( d0 >= -16 ) {
						return;
					}
					d0 = -9;
				}
			}
		} else {
			// _label10
			if( d0 >= -16 ) {
				return;
			}
			d0 = -9;
		}
		// _label12
		d0 += w_opponents_road_x_position & 0xFF;
		if( b_opp_touching_road && (d0 >= 32) && (d0 <= (256 - 32)) && (0 == b_machine) ) {
			// TODO verify. do we really need word?
			w_opponents_road_x_position = (short) ((w_opponents_road_x_position & 0xFF00)
					| (d0 & 0xFF));
		}
	}

	// java: to avoid goto
	private static void opponentPlayerInteractionLabelC() {
		int road_height_1 = (byte) 2;

		int d1 = b_opponents_road_piece;
		b_current_road_piece = b_opponents_road_piece;

		do {
			int d0 = b_road_section_angle_and_piece[ d1 ] & 0x0F;
			d0 = Road.sections_car_can_be_put_on[ d0 ];
			if( d0 < 0 ) {
				b_opponents_suggested_road_x_position = (byte) 128;
			}
			d1 = toNextRoadPiece();
			--road_height_1;
		} while( 0 != road_height_1 );

		opponentPlayerInteractionLabelF();
	}

	// java: to avoid goto
	/**
	 *
	 * @param d1 as byte, opponent index
	 */
	private static void opponentPlayerInteractionLabelFar(final int d1) {
		// Player and opponent at least $100 from each other (ahead or behind)
		int d2 = 64;
		if( 0 != (b_opponent_attributes[ d1 ] & DRIVES_NEAR_EDGE) ) {
			d2 = 110;
		}
		if( 0 != (d1 & 1) ) {
			d2 = ~d2;// to other side of road
		}
		// _labelb:
		b_opponents_suggested_road_x_position = (byte) d2;

		// _labelc:
		opponentPlayerInteractionLabelC();
	}

	// opponent_player_interaction
	/**
	 * Calculates opponent movement sideways and collision with player
	 * All of opponent.player.interaction (and subroutines) can be bypassed
	 * and replaced by the below.
	 * This disables player to opponent collisions and fixes the opponent
	 * in the middle of the road
	 * <pre>
	 * opponent.player.interaction.bypass
	 *   move.b	#0,player.close.to.opponent
	 *   move.b	#128,opponents.road.x.position+1
	 *   rts
	 * </pre>
	 */
	private static void opponentPlayerInteraction() {
		final int d1 = b_opponents_id;
		boolean d2 = false;
		b_player_close_to_opponent = false;

		int d0 = w_opponents_road_x_position & 0xFF;
		b_opponents_suggested_road_x_position = (byte) d0;
		d0 -= b_rear_wheel_surface_x_position;
		if( d0 < 0 ) {
			d0 = -d0;
			d2 = true;// flag that player is to right of opponent
		}
		b_abs_x_difference = (byte) d0;// abs diff
		b_player_to_right = d2;

		if( 0 == (w_smallest_distance_between_players & 0xFF00) ) {
			// Player and opponent are within $100 of each other (ahead or behind)
			d0 = (w_smallest_distance_between_players & 0xFF);
			if( d0 < 64 ) {
				if( b_opponent_behind_player || ((((int) b_abs_x_difference) & 0xFF) < 50) ) {
					// Either:
					// Opponent less than 64 behind player
					// OR Player less than 64 behind and less than 50
					// to the left or right of opponent
					b_player_close_to_opponent = true;
				}
			}
			if( d0 < 16 ) {
				if( (0 == b_machine) || (0 == (w_opponents_road_x_position & 0xFFFFFF00)) ) {
					d0 = ((int) b_abs_x_difference) & 0xFF;
					if( d0 < 50 ) {
						d0 = w_players_road_x_position >> 8;
						if( (d0 < 1) ||
								(1 == d0 &&
										(w_players_road_x_position & 0xFF) < 0x80) ) {
							carToCarCollisionDetection();
							opponentPlayerInteractionLabel4( d1 );
							return;
						}
					}
				}
			}
			// Not within 16
			// _label3:
			b_collision_counter = 0;// clear collision values
			b_1bbeb = false;

			d0 = w_smallest_distance_between_players & 0xFF;
			if( d0 < 24 ) {
				// _label4:
				opponentPlayerInteractionLabel4( d1 );
				return;
			}
			if( b_opponent_behind_player ) {
				// _label9:
				opponentPushPlayer();
				opponentPlayerInteractionLabelF();
				return;
			}
			if( d0 < 50 ) {
				d0 = b_opponent_attributes[ d1 ] & OBSTRUCTS_PLAYER;
				if( 0 == d0 ) {
					// .label8:
					opponentPushPlayer();
					opponentPlayerInteractionLabelC();
					return;
				}

				opponentObstructPlayer();// put opponent at same position as player
				opponentPlayerInteractionLabelC();
				return;
			}
			// _label7:
			if( d0 < 200 ) {
				d0 = b_opponent_attributes[ d1 ] & PUSH_PLAYER;// opponent pushing player off track
				if( 0 != d0 ) {
					// .label8:
					opponentPushPlayer();
					opponentPlayerInteractionLabelC();
					return;
				}
			}
		}
		// _far
		opponentPlayerInteractionLabelFar( d1 );
	}

	// set_opponent_draw_flag
	private static void setOpponentDrawFlag() {
		b_opponent_draw_flag = 0;
		if( 0 == b_machine ) {
			return;
		}
		int sd0 = w_opponents_road_x_position;
		if( sd0 >= 0 ) {
			sd0 -= 256;
			if( sd0 < 60 ) {
				return;
			}
			b_opponent_draw_flag = 1;
		} else {
			if( sd0 < -60 ) {
				return;
			}
			b_opponent_draw_flag = (byte) 0x80;
		}
		if( w_opponents_offset >= 0 ) {
			w_opponents_offset += 32;
		}
	}

	private static byte b_simple_poly_count = 0;// dc.b 0,0

	// FIXME simple_poly_fill2
	/**
	 * {@link #b_simple_poly_count} defines a triangle count to fill.
	 *
	 * @param id6
	 * @param id7
	 * @param sa0i
	 * @param sa1i
	 * @param sa2i
	 * @param sa3i
	 */
	private static void simplePolyFill2(final int id6, final int id7,
										int sa0i, int sa1i, int sa2i, int sa3i)
	{
		/*
		// move.l	#start.masks,a5
		int d1 = edge_space[ sa0i / 2 ];
		sa0i += 2;
		int d0 = edge_space[ sa3i / 2 ];
		sa3i += 2;
		if( d0 != d1 ) {// y1.0 == y1.3
			return;
		}
		sa0i += 6;
		sa3i += 6;
		fp_y = (short)d1;// y1.3
		--d1;
		if( d1 < 0 ) {
			return;
		}
		// move.l current.scene, a6
		int ba6i = 0;
		d0 = 0;
		d0 = ((d1 << 2) + d1) << 3;
		ba6i += d0;

		// spf5
		do {
			int d4 = edge_space[ sa0i / 2 ];// flag
			sa0i += 2;
			if( d4 < 0 ) {// a point. there is no line
				--simple_poly_count;
				if( simple_poly_count < 0 ) {
					return;
				}
				sa0i = sa1i;
				sa1i = sa2i;
				d0 = edge_space[ sa0i / 2 ];// y1.1
				sa0i += 2;
				if( d0 != fp_y ) {// y1.1 != y1.3
					return;
				}
				sa0i += 6;
				d4 = edge_space[ sa0i / 2 ];// flag
				sa0i += 2;
				if( d4 < 0 ) {// a point, there is no line
					--simple_poly_count;
					if( simple_poly_count < 0 ) {
						return;
					}
					sa0i = sa1i;
					d0 = edge_space[ sa0i / 2 ];// y1.2
					sa0i += 2;
					if( d0 != fp_y ) {// y1.2 != y1.3
						return;
					}
					sa0i += 6;
					d4 = edge_space[ sa0i / 2 ];// flag
					if( d4 < 0 ) {// a point, there is no line
						return;
					}
				}
			}
			// spf6
			int d5 = edge_space[ sa3i / 2 ];// flag
			sa3i += 2;
			if( d5 < 0 ) {// a point, there is no line
				--simple_poly_count;
				if( simple_poly_count < 0 ) {
					return;
				}
				sa3i = sa2i;
				sa2i = sa1i;
				d0 = edge_space[ sa3i / 2 ];
				sa3i += 2;
				if( d0 != fp_y ) {
					return;
				}
				sa3i += 6;
				d5 = edge_space[ sa3i / 2 ];// flag
				sa3i += 2;
				if( d5 < 0 ) {// a point, there is no line
					--simple_poly_count;
					if( simple_poly_count < 0 ) {
						return;
					}

					sa3i = sa2i;
					d0 = edge_space[ sa3i / 2 ];
					sa3i += 2;
					if( d0 != fp_y ) {
						return;
					}
					sa3i += 6;
					d5 = edge_space[ sa3i / 2 ];
					sa3i += 2;
					if( d5 < 0 ) {// flag
						return;
					}
				}
			}
			// spf7
			if( d5 <= d4 ) {// x1 x2
				if( (d5 != d4) && daft_flag ) {
					return;
				}
			} else {
				// spf8
				d1 = (d4 & 0xF0) >>> 3;
				// lea( a6, d1.w ), a4
				int d3 = d4 >>> 4;
				d1 = d5 >>> 4;
				d1 -= d3;
				if( 0 == d1 ) {
					d4 &= 0x0F;
					d4 <<= 2;
					d4 = sa5[ d4 / 2 ];

					d5 &= 0x0F;
					d5 <<= 2;
					d5 = sa5[ (d5 + 64) / 2 ];

					d4 &= d5;
					fillWord( sa4, sa4i, d4 );
				} else {
					// spf9
					--d1;
					d4 &= 0x0F;
					if( 0 == d4 ) {
						beq spfa;
					}

					d4 <<= 2;
					d4 = sa5[ (sa5i + d4) / 2 ];
					fillWord( sa4, sa4i, d4 );
					--d1;
					if( d1 >= 0 ) {
						// spfa
						d2 = d6;
						d3 = d7;
						swap d2
						swap d3

						// spfb
						move.w d2, (a4) +
						move.w d6, 7998 (a4)
						move.w d3, 15998 (a4)
						move.w d7, 23998 (a4)
						dbra d1, spfb
					}
					// spfc
					d5 &= 0x0F;
					if( 0 != d5 ) {
						d5 <<= 2;
						d4 = sa5[ (d5 + 64) / 2 ];
						fillWord( sa4, sa4i, d4 );
					}
				}
			}
			// spfd
			--fp_y;
			ba6i -= 40;// screen y - 1
			cmp.l current.scene, a6
			bge spf5
		} while( false );
		// spfe
		// d1 = d2 = 0;
		/* */
	}

	private static int sortPoints(int index, int pointIndex) {
		final int x1 = w_screen_coords[ (index + 4) / 2 ];
		final int y1 = w_screen_coords[ (index + 0) / 2 ];
		final int x2 = w_screen_coords[ (index + 6) / 2 ];
		final int y2 = w_screen_coords[ (index + 2) / 2 ];
		if( (jXPoints[ pointIndex - 1 ] == x1 && jYPoints[ pointIndex - 1 ] == y1) ) {
			if( pointIndex == 1 ) {
				int temp = jXPoints[ pointIndex - 1 ];
				jXPoints[ pointIndex - 1 ] = jXPoints[ pointIndex ];
				jXPoints[ pointIndex ] = temp;
				temp = jYPoints[ pointIndex - 1 ];
				jYPoints[ pointIndex - 1 ] = jYPoints[ pointIndex ];
				jYPoints[ pointIndex ] = temp;
			}
			++pointIndex;
			jXPoints[ pointIndex ] = x2;
			jYPoints[ pointIndex ] = y2;
			return pointIndex;
		}
		if( (jXPoints[ pointIndex ] == x1 && jYPoints[ pointIndex ] == y1) ) {
			++pointIndex;
			jXPoints[ pointIndex ] = x2;
			jYPoints[ pointIndex ] = y2;
			return pointIndex;
		}
		if( (jXPoints[ pointIndex ] == x2 && jYPoints[ pointIndex ] == y2) ) {
			++pointIndex;
			jXPoints[ pointIndex ] = x1;
			jYPoints[ pointIndex ] = y1;
			return pointIndex;
		}
		if( (jXPoints[ pointIndex - 1 ] == x2 && jYPoints[ pointIndex - 1 ] == y2) ) {
			if( pointIndex == 1 ) {
				int temp = jXPoints[ pointIndex - 1 ];
				jXPoints[ pointIndex - 1 ] = jXPoints[ pointIndex ];
				jXPoints[ pointIndex ] = temp;
				temp = jYPoints[ pointIndex - 1 ];
				jYPoints[ pointIndex - 1 ] = jYPoints[ pointIndex ];
				jYPoints[ pointIndex ] = temp;
			}
			++pointIndex;
			jXPoints[ pointIndex ] = x1;
			jYPoints[ pointIndex ] = y1;
		}
		return pointIndex;
	}

	// simple_poly_fill
	/**
	 * Fill quadrangle, or 2 adjacent triangles
	 *
	 * @param id6
	 * @param id7
	 * @param sa0i
	 * @param sa1i
	 * @param sa2i
	 * @param sa3i
	 */
	private static void simplePolyFill(final Canvas g, final int id6, final int id7,
									   int sa0i, int sa1i, int sa2i, int sa3i)
	{
		//--- Java version TODO do not draw when completely off-screen
		// vector 1
		jXPoints[ 0 ] = w_screen_coords[ (sa0i + 4) / 2 ];
		jYPoints[ 0 ] = w_screen_coords[ (sa0i + 0) / 2 ];
		jXPoints[ 1 ] = w_screen_coords[ (sa0i + 6) / 2 ];
		jYPoints[ 1 ] = w_screen_coords[ (sa0i + 2) / 2 ];
		// vector 2
		int pi = sortPoints( sa1i, 1 );
		// vector 3
		pi = sortPoints( sa2i, pi );
		// vector 4
		sortPoints( sa3i, pi );
		final boolean is_visible =
				(jXPoints[ 0 ] - jXPoints[ 2 ]) * (jYPoints[ 1 ] - jYPoints[ 2 ]) -
						(jYPoints[ 0 ] - jYPoints[ 2 ]) * (jXPoints[ 1 ] - jXPoints[ 2 ]) > 0;
		if( is_visible ) {
			path.moveTo( jXPoints[ 0 ], jYPoints[ 0 ] );
			path.lineTo( jXPoints[ 1 ], jYPoints[ 1 ] );
			path.lineTo( jXPoints[ 2 ], jYPoints[ 2 ] );
			path.lineTo( jXPoints[ 3 ], jYPoints[ 3 ] );
			path.close();
			g.drawPath( path, paint );
			path.reset();
		}
		//--- end java
		/*
		int sd2 = edge_space[ sa2i / 2 ];
		int sd0 = edge_space[ sa0i / 2 ];
		if( sd0 != edge_space[ sa3i / 2 ] ) {
			if( sd2 == edge_space[ sa1i / 2 ] ) {
				// spf1
				int temp = sa0i;
				sa0i = sa2i;
				sa2i = temp;
				temp = sa1i;
				sa1i = sa3i;
				sa3i = temp;
			} else {
				// spf2
				if( sd0 >= edge_space[ sa3i / 2 ] ) {
					final int temp = sa0i;
					sa0i = sa1i;
					sa1i = sa2i;
					sa2i = sa3i;
					sa3i = temp;
				} else {
					final int temp = sa3i;
					sa3i = sa2i;
					sa2i = sa1i;
					sa1i = sa0i;
					sa0i = temp;
				}
			}
		} else {
			if( sd2 == edge_space[ sa1i / 2 ] ) {
				if( sd0 < sd2 ) {
					// spf1
					int temp = sa0i;
					sa0i = sa2i;
					sa2i = temp;
					temp = sa1i;
					sa1i = sa3i;
					sa3i = temp;
				}
			}
		}
		// spf4
		simple_poly_count = 2;
		simplePolyFill2( id6, id7, sa0i, sa1i, sa2i, sa3i );
		*/
	}

	// fill_triangle TODO do not draw when completely off-screen
	private static void fillTriangle(final Canvas g, int sa0i, int sa1i, int sa2i) {
		//--- java version:
		// vector 1
		jXPoints[ 0 ] = w_screen_coords[ (sa0i + 4) / 2 ];
		jYPoints[ 0 ] = w_screen_coords[ (sa0i + 0) / 2 ];
		jXPoints[ 1 ] = w_screen_coords[ (sa0i + 6) / 2 ];
		jYPoints[ 1 ] = w_screen_coords[ (sa0i + 2) / 2 ];
		// vector 2
		int pi = sortPoints( sa1i, 1 );
		// vector 3
		pi = sortPoints( sa2i, pi );
		// g.drawPoints( jXYPoints, 0, 3 * 2, paint );
		path.moveTo( jXPoints[ 0 ], jYPoints[ 0 ] );
		path.lineTo( jXPoints[ 1 ], jYPoints[ 1 ] );
		path.lineTo( jXPoints[ 2 ], jYPoints[ 2 ] );
		path.close();
		g.drawPath( path, paint );
		path.reset();
		//--- end java version

		/*
		final short sd0 = edge_space[ sa0i / 2 ];// y1
		final int sa3i;
		if( sd0 != edge_space[ sa2i / 2 ] ) {
			if( sd0 != edge_space[ sa1i / 2 ] ) {
				sa3i = sa1i;
				sa1i = sa0i;
				sa0i = sa2i;
				sa2i = sa1i;
			} else {
				sa3i = sa0i;
				sa0i = sa1i;
				sa1i = sa2i;
			}
		} else {
			sa3i = sa2i;
			sa2i = sa1i;
		}
		simple_poly_count = 1;
		simplePolyFill2( 0, 0, sa0i, sa1i, sa2i, sa3i );
		*/
	}

	// edge_off_screen
	/**
	 *
	 * @param ia1i offset to {@link #lw_screen_data} in bytes
	 * @param sa0i offset to {@link #w_screen_coords} in bytes
	 * @param sa2i offset to {@link #w_screen_coords} in bytes
	 * @param bd0 flags, low byte
	 * @param sd4_x1
	 * @param sd6_x2
	 * @param sd7_y2
	 */
	private static void edgeOffScreen(
			int ia1i,
			int sa0i, int sa2i,
			int bd0,
			int sd4_x1, int sd5_y1, int sd6_x2, int sd7_y2)
	{
		int sd3 = w_road_section_offset;
		bd0 |= 0x80;
		lw_screen_data[ (ia1i + sd3) / 4 ] |= bd0 << 24;
		if( sd4_x1 < 0 ) {
			sd4_x1 = 0;
		}
		if( sd4_x1 > 256 ) {
			sd4_x1 = 256;
		}
		if( sd6_x2 < 0 ) {
			sd6_x2 = 0;
		}
		if( sd6_x2 > 256 ) {
			sd6_x2 = 256;
		}
		if( 0 != (bd0 & 0x01) ) {
			w_screen_coords[ sa2i / 2 ] = 128;
			w_screen_coords[ (sa2i + 2) / 2 ] = 128;
			w_screen_coords[ (sa2i + 4) / 2 ] = (short) sd4_x1;
			w_screen_coords[ (sa2i + 6) / 2 ] = (short) sd6_x2;
			w_screen_coords[ sa0i / 2 ] = (short) 0x8000;
			sa0i += 2;
			lw_screen_coords_ptr = sa0i;
			// edge_space = sa0;
			return;
		}
		if( 0 != (bd0 & 0x02) ) {
			w_screen_coords[ sa2i / 2 ] = 0;
			// edge_space[ (sa2i + 2) / 2 ] = 0;
			w_screen_coords[ (sa2i + 4) / 2 ] = (short) sd4_x1;
			w_screen_coords[ (sa2i + 6) / 2 ] = (short) sd6_x2;
			w_screen_coords[ sa0i / 2 ] = (short) 0x8000;
			sa0i += 2;
			lw_screen_coords_ptr = sa0i;
			// edge_space = sa0;
			return;
		}
		if( sd5_y1 < sd7_y2 ) {
			final int t = sd5_y1;
			sd5_y1 = sd7_y2;
			sd7_y2 = t;
		}
		if( 0 != (bd0 & 0x04) ) {
			if( sd5_y1 < 0 ) {
				sd5_y1 = 0;
			}
			if( sd5_y1 > 128 ) {
				sd5_y1 = 128;
			}
			w_screen_coords[ sa2i / 2 ] = (short) sd5_y1;
			if( sd7_y2 < 0 ) {
				sd7_y2 = 0;
			}
			if( sd7_y2 > 128 ) {
				sd7_y2 = 128;
			}
			w_screen_coords[ (sa2i + 2) / 2 ] = (short) sd7_y2;
			w_screen_coords[ (sa2i + 4) / 2 ] = (short) sd4_x1;
			w_screen_coords[ (sa2i + 6) / 2 ] = (short) sd6_x2;
			sd5_y1 -= sd7_y2;
			if( sd5_y1 >= 0 ) {
				while( --sd5_y1 >= 0 ) {
					w_screen_coords[ sa0i / 2 ] = 256;
					sa0i += 2;
				}
			}
			w_screen_coords[ sa0i / 2 ] = (short) 0x8000;
			sa0i += 2;
			lw_screen_coords_ptr = (short) sa0i;
			// edge_space = sa0;
			return;
		}
		if( 0 != (bd0 & 0x08) ) {
			if( sd5_y1 < 0 ) {
				sd5_y1 = 0;
			}
			if( sd5_y1 > 128 ) {
				sd5_y1 = 128;
			}
			w_screen_coords[ sa2i / 2 ] = (short) sd5_y1;
			if( sd7_y2 < 0 ) {
				sd7_y2 = 0;
			}
			if( sd7_y2 > 128 ) {
				sd7_y2 = 128;
			}
			w_screen_coords[ (sa2i + 2) / 2 ] = (short) sd7_y2;
			w_screen_coords[ (sa2i + 4) / 2 ] = (short) sd4_x1;
			w_screen_coords[ (sa2i + 6) / 2 ] = (short) sd6_x2;
			sd5_y1 -= sd7_y2;
			if( sd5_y1 >= 0 ) {
				while( --sd5_y1 >= 0 ) {
					w_screen_coords[ sa0i / 2 ] = 0;
					sa0i += 2;
				}
			}
			w_screen_coords[ sa0i / 2 ] = (short) 0x8000;
			sa0i += 2;
			lw_screen_coords_ptr = sa0i;
			// edge_space = sa0;
		}
	}

	private static short w_straight_edge_count = 0;
	private static short w_straight_edge_value = 0;
	// private static boolean y_saved = false;

	// java clme55 to avoid goto
	// d4 = x1, d6 = x2, d5 = y1, d7 = y2
	private static void no_clipping(int sa0i, int sa2i,
			int d4, int d5, int d6, int d7,
			boolean isYsaved)
	{
		// clme55: there is no clipping
		int d2 = d5 - d7;// y1 - y2
		int d1 = d4 - d6;// x1 - x2
		if( d1 < 0 ) {// left to right
			d1 = -d1;// |dx|
			if( d1 >= d2 ) {// |dx| >= y1 - y2
				if( ! isYsaved ) {
					w_screen_coords[ sa2i / 2 ] = (short)d5;// y1
					w_screen_coords[ (sa2i + 2) / 2 ] = (short)d7;// y2
				}
				w_screen_coords[ (sa2i + 4) / 2 ] = (short)d4;// x1
				w_screen_coords[ (sa2i + 6) / 2 ] = (short)d6;// x2
				int d3 = ~(d1 >>> 1);// x = -|dx| / 2 - 1
				while( d6 != d4 ) {// x1 != x2
					++d4;// x1
					d3 += d2;// x = x + (y1-y2)
					if( (d3 < 0) || (d3 >= 65536) ) {
						d3 -= d1;// - (x1-x2)
						--d5;// y1
						w_screen_coords[ sa0i / 2 ] = (short)d4;// x
						sa0i += 2;
					}
				}
				int d0 = w_straight_edge_count;
				if( d0 >= 0 ) {
					// clme59
					do {
						w_screen_coords[ sa0i / 2 ] = w_straight_edge_count;
						sa0i += 2;
					} while( --d0 >= 0 );
				}
				// clme5a
				w_screen_coords[ sa0i / 2 ] = (short)0x8000;
				lw_screen_coords_ptr = (sa0i + 2);
				// clr.w d1
				// clr.w d2
				return;
			}// end // |dx| >= y1 - y2
			// clme5b // |dx| < y1 - y2
			if( ! isYsaved ) {
				w_screen_coords[ (sa2i) / 2 ] = (short)d5;// y1
				w_screen_coords[ (sa2i + 2) / 2 ] = (short)d7;// y2
			}
			w_screen_coords[ (sa2i + 4) / 2 ] = (short)d4;// x1
			w_screen_coords[ (sa2i + 6) / 2 ] = (short)d6;// x2
			int d3 = ~(d2 >>> 1);
			while( d5 != d7 ) {
				--d5;
				w_screen_coords[ sa0i / 2 ] = (short)d4;// x
				sa0i += 2;
				d3 += d1;
				if( (d3 < 0) || (d3 >= 65536) ) {// TODO verify
					d3 -= d2;
					++d4;
				}
			}
			int d0 = w_straight_edge_count;
			if( d0 >= 0 ) {
				do {
					w_screen_coords[ sa0i / 2 ] = w_straight_edge_value;
					sa0i += 2;
				} while( --d0 >= 0 );
			}
			w_screen_coords[ sa0i / 2 ] = (short) 0x8000;
			sa0i += 2;
			lw_screen_coords_ptr = sa0i;
			// clr.w d1
			// clr.w d2
			return;// end |dx| < y1 - y2
		}
		// clme61
		if( d1 >= d2 ) {// dx >= dy
			if( ! isYsaved ) {
				w_screen_coords[ sa2i / 2 ] = (short)d5;
				w_screen_coords[ (sa2i + 2) / 2 ] = (short)d7;
			}
			w_screen_coords[ (sa2i + 4) / 2 ] = (short)d4;
			w_screen_coords[ (sa2i + 6) / 2 ] = (short)d6;
			int d3 = ~(d1 >>> 1);
			// clme63
			while( d4 != d6 ) {
				--d4;
				d3 += d2;
				if( (d3 < 0) || (d3 >= 65536) ) {// TODO verify
					d3 -= d1;
					--d5;
					w_screen_coords[ sa0i / 2 ] = (short) d4;
					sa0i += 2;
				}
			}
			int d0 = w_straight_edge_count;
			if( d0 >= 0 ) {
				// clme65
				do {
					w_screen_coords[ sa0i / 2 ] = w_straight_edge_value;
					sa0i += 2;
				} while( --d0 >= 0 );
			}
			// clme66
			w_screen_coords[ sa0i / 2 ] = (short) 0x8000;
			sa0i += 2;
			lw_screen_coords_ptr = sa0i;
			// clr.w d1
			// clr.w d2
			return;// end dx >= dy
		}
		// dx < dy
		// clme67
		if( ! isYsaved ) {
			w_screen_coords[ sa2i / 2 ] = (short)d5;// y1
			w_screen_coords[ (sa2i + 2) / 2 ] = (short)d7;// y2
		}
		w_screen_coords[ 4 / 2 ] = (short)d4;// x1
		w_screen_coords[ 6 / 2 ] = (short)d6;// x2
		int d3 = ~(d2 >>> 1);
		// clme69
		while( d5 != d7 ) {
			--d5;
			w_screen_coords[ sa0i / 2 ] = (short)d4;// x
			sa0i += 2;
			d3 += d1;
			if( (d3 < 0) || (d3 >= 65536) ) {// TODO verify
				d3 -= d2;
				--d4;
			}
		}
		int d0 = w_straight_edge_count;
		if( d0 >= 0 ) {
			do {
				w_screen_coords[ sa0i / 2 ] = w_straight_edge_value;
				sa0i += 2;
			} while( --d0 >= 0 );
		}
		w_screen_coords[ sa0i / 2 ] = (short)0x8000;
		sa0i += 2;
		lw_screen_coords_ptr = sa0i;
		// d1 = d2 = 0;
		/* */
	}

	// clip_line_make_edge
	/**
	 *
	 * @param d1 index of the 1st point in x_values and y_values
	 * @param d2 index of the 2nd point in x_values and y_values
	 * Returns: x and y coordinates in a1 = {@link #lw_screen_data}
	 */
	private static void clipLineMakeEdge(int d1, int d2) {
		// g.drawLine( x_values[ d1 / 2 ], y_values[ d1 / 2 ], x_values[ d2 / 2 ], y_values[ d2 / 2 ] );
		// jLogErr( String.format( "( %d, %d ) - ( %d, %d )", x_values[ d1 / 2 ], y_values[ d1 / 2 ], x_values[ d2 / 2 ], y_values[ d2 / 2 ] ) );

		//--- short java version without clipping
		if( lw_screen_coords_ptr >= (2 * END_SCREEN_COORDS) ) {
			if( !b_standard_clip_flag ) {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
				return;
			}
		}
		short x1 = (short) (w_screen_x_coords[ d1 / 2 ] + SCREEN_AREA_LEFT_X);
		short x2 = (short) (w_screen_x_coords[ d2 / 2 ] + SCREEN_AREA_LEFT_X);
		short y1 = (short) (w_screen_y_coords[ d1 / 2 ] + SCREEN_AREA_TOP_Y);
		short y2 = (short) (w_screen_y_coords[ d2 / 2 ] + SCREEN_AREA_TOP_Y);
		int flag = 0;
		if( y1 < y2 ) {// y1 < y2
			short temp = y1;
			y1 = y2;
			y2 = temp;
			temp = x1;
			x1 = x2;
			x2 = temp;
			flag = 0x40_00_0000;
		}
		lw_screen_data[ w_road_section_offset / 4 ] = lw_screen_coords_ptr | flag;
		w_screen_coords[ lw_screen_coords_ptr / 2       ] = y1;
		w_screen_coords[ (lw_screen_coords_ptr + 2) / 2 ] = y2;
		w_screen_coords[ (lw_screen_coords_ptr + 4) / 2 ] = x1;
		w_screen_coords[ (lw_screen_coords_ptr + 6) / 2 ] = x2;
		w_screen_coords[ (lw_screen_coords_ptr + 8) / 2 ] = (short) 0x8000;
		lw_screen_coords_ptr = (lw_screen_coords_ptr + 5 * 2);
		//--- end java version
		/*
		// move.l	#lw_section_data,a1
		// move.l	edge_space_ptr,a0
		int ia1i = 0;// offset in bytes move.l	#section.data,a1
		boolean b_y_saved = false;
		w_straight_edge_count = -1;
		int d0 = w_road_section_offset;
		int sa0i = edge_space_ptr;// move.l	edge.space.ptr,a0
		if( sa0i >= END_EDGE_SPACE ) {
			if( !b_standard_clip_flag ) {
				lw_section_data[ d0 / 4 ] = SECTION_DATA_END_MARKER;
				// d1 = d2 = 0;
				return;
			}
		}
		lw_section_data[ d0 / 4 ] = sa0i;
		int sa2i = sa0i;// move.l	a0,a2
		sa0i += 8;// length of the structure: {y1,y2,x1,x2}
		// move.l	#x.values,a4
		// move.l	#y.values,a5
		int d4 = w_screen_x_values[ d1 / 2 ];// x1
		int d6 = w_screen_x_values[ d2 / 2 ];// x2
		int d5 = w_y_values[ d1 / 2 ];// y1
		int d7 = w_y_values[ d2 / 2 ];// y2
		if( d5 < d7 ) {// y1 < y2
			int temp = d5;
			d5 = d7;
			d7 = temp;// exg	d7,d5
			temp = d4;
			d4 = d6;
			d6 = temp;// exg	d6,d4
			lw_section_data[ (ia1i + d0) / 4 ] |= 0x40_00_0000;
		}
		d0 = 0;
		int d3 = d0;
		if( d4 >= 256 ) {// x1 max
			d0 |= 0x04;
		}
		if( d4 < 0 ) {
			d0 |= 0x08;
		}
		if( d6 >= 256 ) {// x1 max
			d3 |= 0x04;
		}
		if( d6 < 0 ) {
			d3 |= 0x08;
		}
		if( d5 >= 128 ) {// y1 max
			d0 |= 0x01;
		}
		if( d5 < 0 ) {
			d0 |= 0x02;
		}
		if( d7 >= 128 ) {// y2 max
			d3 |= 0x01;
		}
		if( d7 < 0 ) {
			d3 |= 0x02;
		}
		// clmea:
		d1 = d0;// flags, X1Y1
		d2 = d3;// flags, X2Y2
		int d0_1 = d0;// high byte d0 swap d0
		d0 = (d1 | d2) & 0x0F;// X1Y1 | X2Y2
		if( 0 != (d0 & 0x0F) ) {// at least one clip
			d0 = d1 & d2 & 0x0F;// X1Y1 & X2Y2
			if( 0 != (d0 & 0x0F) ) {// off the screen
				edgeOffScreen( ia1i, sa0i, sa2i, d0, d4, d5, d6, d7 );
				// d1 = d2 = 0;
				return;
			}
			// clmeb:
			int temp = d0_1;
			d0_1 = d0;
			d0 = temp;// swap
			boolean not_goto_clme1d = true;
			if( 0 != (d1 & 0x02) ) {
				d1 &= 0x7F;
				d0 = d6 - d4;// dx
				if( d0 < 0 ) {
					d1 |= 0x80;
					d0 = -d0;
				}
				// clmec
				d3 = d7 - d5;// dy
				if( d3 < 0 ) {
					d1 ^= 0x80;
					d3 = -d3;
				}
				// clmed
				d5 = -d5;
				if( d3 < d0 ) {// dy < dx
					// clmee
					if( d5 >= d3 ) {
						d5 = d0;
					} else {
						// clmef
						d5 = (((d5 << 16) / d3) * d0) >> 16;
					}
				} else if( d3 != d0 ) {// dy == dx
					d5 = (((d0 << 16) / d3) * d5) >> 16;// y1 = y1 * (dx << 16) / dy
				}
				// clme10
				if( 0 != (d1 & 0x80) ) {
					d5 = -d5;
				}
				d4 += d5;
				d5 = 0;
			} else {
				// clme12
				if( 0 == (d1 & 0x01) ) {
					// beq clme1d;
					not_goto_clme1d = false;
				} else {
					d1 &= 0x7F;
					d0 = d6 - d4;
					if( d0 < 0 ) {
						d1 |= 0x80;
						d0 = -d0;
					}
					d3 = d7 - d5;
					if( d3 < 0 ) {
						d1 ^= 0x80;
						d3 = -d3;
					}
					d5 -= 128;
					if( d3 < d0 ) {
						// clme15
						if( d5 < d3 ) {
							d5 = (((d5 << 16) / d3) * d0) >> 16;
						} else {
							d5 = d0;
						}
					} else if( d3 != d0 ) {
						d5 = (((d0 << 16) / d3) * d5) >> 16;
					}
					// clme17
					if( 0 == (d1 & 0x80) ) {
						d5 = -d5;
					}
					d4 += d5;
					d5 = 128;
				}
			}
			if( not_goto_clme1d ) {
				// clme19
				d1 &= 0xF0;
				if( d4 >= 256 ) {
					d1 |= 0x04;
				}
				if( d4 < 0 ) {
					d1 |= 0x08;
				}
				// clme1b
				temp = d0_1;
				d0_1 = d0;
				d0 = temp;// swap
				d0 = (d1 | d2) & 0x0F;
				if( 0 == d0 ) {
					// beq clme55;// no clipping
					no_clipping( sa0i, sa2i, d4, d5, d6, d7, b_y_saved );
					return;
				}
				d0 = d1 & d2 & 0x0F;
				if( 0 != d0 ) {
					edgeOffScreen( ia1i, sa0i, sa2i, d0, d4, d5, d6, d7 );
					// d1 = d2 = 0;
					return;
				}
				// clme1c
				temp = d0_1;
				d0_1 = d0;
				d0 = temp;// swap
			}
			//--- clme1d
			if( 0 != (d2 & 0x02) ) {
				d1 &= 0x7F;
				d0 = d6 - d4;
				if( d0 < 0 ) {
					d1 |= 0x80;
					d0 = -d0;
				}
				// clme1e
				d3 = d7 - d5;
				if( d3 < 0 ) {
					d1 ^= 0x80;
					d3 = -d3;
				}
				// clme1f
				d7 = -d7;
				if( d3 < d0 ) {
					if( d7 < d3 ) {
						d7 = (((d7 << 16) / d3) * d0) >> 16;
					} else {
						d7 = d0;
					}
				} else if( d3 != d0 ) {
					d7 = (((d0 << 16) / d3) * d7) >> 16;
				}
				// clme22
				if( 0 != (d1 & 0x80) ) {
					d7 = -d7;
				}
				d6 += d7;
				d7 = 0;
				// clme2b
				d2 &= 0xF0;
				if( d6 < 0 ) {
					d2 |= 0x08;
				}
				if( d6 >= 256 ) {
					d2 |= 0x04;
				}
				// clme2d
				temp = d0_1;
				d0_1 = d0;
				d0 = temp;// swap
				d0 = (d1 | d2) & 0xF;
				if( 0 == d0 ) {
					// beq clme55;
					no_clipping( sa0i, sa2i, d4, d5, d6, d7, b_y_saved );
					return;
				}
				d0 = d1 & d2 & 0xF;
				if( 0 != d0 ) {
					edgeOffScreen( ia1i, sa0i, sa2i, d0, d4, d5, d6, d7 );
					// d1 = d2 = 0;
					return;
				}
				// clme2e
				temp = d0_1;
				d0_1 = d0;
				d0 = temp;// swap
			} else {
				// clme24
				if( 0 != (d2 & 0x01) ) {
					d1 &= 0x7F;
					d0 = d6 - d4;
					if( d0 < 0 ) {
						d1 |= 0x80;
						d0 = -d0;
					}
					d3 = d7 - d5;
					if( d3 < 0 ) {
						d1 ^= 0x80;
						d3 = -d3;
					}
					// clme26
					d7 -= 128;
					if( d3 < d0 ) {
						// clme27
						if( d7 < d3 ) {
							// clme28
							d7 = (((d7 << 16) / d3) * d0) >> 16;
						} else {
							d7 = d0;
						}
					} else if( d3 != d0 ) {
						d7 = (((d0 << 16) / d3) * d7) >> 16;
					}
					// clme29
					if( d1 >= 0 ) {
						d7 = -d7;
					}
					d6 += d7;
					d7 = 128;
					// clme2b
					d2 &= 0xF0;
					if( d6 < 0 ) {
						d2 |= 0x08;
					}
					if( d6 >= 256 ) {
						d2 |= 0x04;
					}
					// clme2d
					temp = d0_1;
					d0_1 = d0;
					d0 = temp;// swap
					d0 = (d1 | d2) & 0xF;
					if( 0 == d0 ) {
						// beq clme55;
						no_clipping( sa0i, sa2i, d4, d5, d6, d7, b_y_saved );
						return;
					}
					d0 = d1 & d2 & 0xF;
					if( 0 != d0 ) {
						edgeOffScreen( ia1i, sa0i, sa2i, d0, d4, d5, d6, d7 );
						// d1 = d2 = 0;
						return;
					}
					// clme2e
					temp = d0_1;
					d0_1 = d0;
					d0 = temp;// swap
				}
			}
			// clme2f
			w_screen_coords[ sa2i / 2 ] = (short)d5;// y1
			w_screen_coords[ (sa2i + 2) / 2 ] = (short)d7;// y2
			b_y_saved = true;
			if( 0 != (d1 & 0x08) ) {
				final int sp_1 = d5;
				d1 &= 0x7F;
				d0 = d6 - d4;
				if( d0 < 0 ) {
					d1 |= 0x80;
					d0 = -d0;
				}
				d3 = d7 - d5;
				if( d3 < 0 ) {
					d1 ^= 0x80;
					d3 = -d3;
				}
				d4 = -d4;
				if( d0 >= d3 ) {
					if( d0 != d3 ) {
						d4 = (((d3 << 16) / d0) * d4) >> 16;
					}
				} else {
					// clme32
					if( d4 >= d0 ) {
						d4 = d3;
					} else {
						d4 = (((d4 << 16) / d0) * d3) >> 16;
					}
				}
				// clme34
				if( d1 < 0 ) {
					d4 = -d4;
				}
				d5 += d4;
				d4 = 0;
				d3 = sp_1;
				if( b_standard_clip_flag ) {
					d3 = d5;
					w_screen_coords[ sa2i / 2 ] = (short)d5;// y1
				}
				d3 -= d5;
				if( d3 >= 0 ) {
					// clme37
					while( --d3 >= 0 ) {
						w_screen_coords[ sa0i / 2 ] = 0;// x(i) = 0
						sa0i += 2;
					}
				}
			} else {
				// clme3a
				if( 0 != (d1 & 0x04) ) {
					final int sp_1 = d5;
					d1 &= 0x7F;
					d0 = d6 - d4;
					if( d0 < 0 ) {
						d1 |= 0x80;
						d0 = -d0;
					}
					d3 = d7 - d5;
					if( d3 < 0 ) {
						d1 ^= 0x80;
						d3 = -d3;
					}
					d4 -= 256;
					if( d0 >= d3 ) {
						if( d0 != d3 ) {
							d4 = (((d3 << 16) / d0) * d4) >> 16;
						}
					} else {
						if( d4 >= d0 ) {
							d4 = d3;
						} else {
							d4 = (((d4 << 16) / d0) * d3) >> 16;
						}
					}
					// clme3f
					if( d1 >= 0 ) {
						d4 = -d4;
					}
					d5 += d4;
					d4 = 256;
					d3 = sp_1;
					if( b_standard_clip_flag ) {
						d3 = d5;
						w_screen_coords[ sa2i / 2 ] = (short)d5;
					}
					d3 -= d5;
					if( d3 >= 0 ) {
						// clme42
						while( --d3 >= 0 ) {
							w_screen_coords[ sa0i / 2 ] = 256;
							sa0i += 2;
						}
					}
				}
			}
			// clme44
			if( 0 != (d2 & 0x08) ) {
				final int sp_1 = d7;
				d1 &= 0x7F;
				d0 = d6 - d4;
				if( d0 < 0 ) {
					d1 |= 0x80;
					d0 = -d0;
				}
				d3 = d7 - d5;
				if( d3 < 0 ) {
					d1 ^= 0x80;
					d3 = -d3;
				}
				d6 = -d6;
				if( d0 >= d3 ) {
					if( d0 != d3 ) {
						d6 = (((d3 << 16) / d0) * d6) >> 16;
					}
				} else {
					// clme47
					if( d6 >= d0 ) {
						d6 = d3;
					} else {
						d6 = (((d6 << 16) / d0) * d3) >> 16;
					}
				}
				// clme49
				if( d1 < 0 ) {
					d6 = -d6;
				}
				d7 += d6;
				d6 = 0;
				d3 = d7;
				d3 -= sp_1;
				--d3;
				if( b_standard_clip_flag ) {
					w_screen_coords[ (sa2i + 2) / 2 ] = (short)d7;
				} else {
					// clme4b
					w_straight_edge_count = (short) d3;
					w_straight_edge_value = 0;
				}
			} else {
				// clme4d
				if( 0 != (d2 & 0x04) ) {
					final int sp_1 = d7;
					d1 &= 0x7F;
					d0 = d6 - d4;
					if( d0 < 0 ) {
						d1 |= 0x80;
						d0 = -d0;
					}
					d3 = d7 - d5;
					if( d3 < 0 ) {
						d1 ^= 0x80;
						d3 = -d3;
					}
					d6 -= 256;
					if( d0 < d3 ) {
						// clme50
						if( d6 < d0 ) {
							d6 = (((d6 << 16) / d0) * d3) >> 16;
						} else {
							d6 = d3;
						}
					} else if( d0 != d3 ) {
						d6 = (((d3 << 16) / d0) * d6) >> 16;
					}
					// clme52
					if( d1 >= 0 ) {
						d6 = -d6;
					}
					d7 += d6;
					d6 = 256;
					d3 = d7;
					d3 -= sp_1;
					--d3;
					if( b_standard_clip_flag ) {
						w_screen_coords[ (sa2i + 2) / 2 ] = (short) d7;
					} else {
						w_straight_edge_count = (short) d3;
						w_straight_edge_value = 256;
					}
				}
			}
		}
		// d4 = x1, d6 = x2, d5 = y1, d7 = y2
		no_clipping( sa0i, sa2i, d4, d5, d6, d7, b_y_saved );
		/* */
	}

	// java variables
	private static int text_color = 0;
	private static int background_color = 0;

	/**
	 * Color for text, 0-15
	 */
	private static int word_col = 0;

	// fill_word
	/**
	 * Fill word with defined color.
	 * Color is defined in {@link #word_col}. 0-15.
	 *
	 * @param a4  screen memory address
	 * @param a4i offset for the a4 but in bytes!
	 * @param sd4 mask to apply
	 * @return next address for short a4, (a4+) in bytes
	 */
	private static int fillWord(final short[] a4, int a4i, final short sd4) {
		a4i /= 2;
		final int sw = (word_col - 2) >>> 4;
		final short sd2 = (short) (~sd4);

		switch( sw ) {
			default:// incorrect state
				return 0;
			case 0:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 1:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 2:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 3:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 4:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 5:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 6:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 7:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] &= sd2;
				break;

			case 8:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 9:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 10:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 11:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] &= sd2;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 12:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 13:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] &= sd2;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 14:
				a4[ a4i ] &= sd2;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;

			case 15:
				a4[ a4i ] |= sd4;
				a4[ 8000 / 2 + a4i ] |= sd4;
				a4[ 16000 / 2 + a4i ] |= sd4;
				a4[ 24000 / 2 + a4i ] |= sd4;
				break;
		}
		return a4i + 2;
	}

	// make_masks
	/**
	 * Set color for filled areas
	 *
	 * @param bd0 color index
	 *            output: id6, id7,
	 *            word_col switch state in fill.word 21968
	 *            {@link #fillWord(short[], int, short)}
	 */
	private static void makeMasks(final int bd0) {
		word_col = bd0;
        /*
        byte bd6 = bd0;
        bd6 <<= 4;// 16 states
        bd6 += 2;
        word_col = bd6;

        int id6 = 0;
        int id7 = 0;
        if (0 != (bd0 & 0x01)) {
            id6 = 0xFFFF0000;
        }
        bd0 >>>= 1;
        id6 <<= 16;
        if (0 != (bd0 & 0x01)) {
            id6 ^= 0x0000FFFF;
        }
        bd0 >>>= 1;
        if (0 != (bd0 & 0x01)) {
            id7 ^= 0x0000FFFF;
        }
        bd0 >>>= 1;
        id7 <<= 16;
        if (0 != (bd0 & 0x01)) {
            id7 ^= 0x0000FFFF;
        }
        // bd0 >>>= 1;
        */
	}

	// set_text_color
	/**
	 * Set color for text
	 *
	 * @param bd0 color index
	 *            output: id6, id7,
	 *            word_col switch state in fill.word 21968
	 *            {@link #fillWord(short[], int, short)}
	 */
	private static void setTextColor(final int bd0) {
		text_color = bd0;

		makeMasks( bd0 );
		//move.l d6, text.masks
		//move.l d7, text.masks+4
	}

	// set_background_color
	/**
	 * Set color for background
	 *
	 * @param bd0 color index
	 *            output: id6, id7,
	 *            word_col switch state in fill.word 21968
	 *            {@link #fillWord(short[], int, short)}
	 */
	private static void setBackgroundColor(final int bd0) {
		background_color = bd0;// java

		makeMasks( bd0 );
	}

	// convert_masks
	/**
	 * Input, i32: d6, d7
	 * <p>
	 * Output:
	 * d1 = d6-16lb d6-16lb
	 * d2 = d7-16lb d7-16lb
	 * d6 = d6-16hb d6-16hb
	 * d7 = d7-16hb d7-16hb
	 */
	private static void convertMasks() {
        /*
        int id0 = id6;
        int id1 = (id6 << 16) | (id6 & 0xFFFF);// 16lb 16lb

        id6 &= 0xFFFF0000;
        id6 |= (id0 >> 16) & 0x0000FFFF;// 16hb 16hb

        id0 = id7;
        id2 = id7;
        id2 = (id7 << 16) | (id7 & 0xFFFF);// 16lb 16lb

        id7 &= 0xFFFF0000;
        id7 |= (id0 >> 16) & 0x0000FFFF;// 16hb 16hb
        */
	}

	// initialise_edges
	private static void initialiseEdges() {
		int ia3i = 0;
		lw_screen_data[ ia3i++ ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ ia3i++ ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ ia3i++ ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ ia3i++ ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ ia3i++ ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ ia3i++ ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ ia3i ] = SECTION_DATA_END_MARKER;
		lw_screen_data[ 30 / 4 ] &= 0xFFFF00FF;// offset 30 is in bytes, hsb
		lw_screen_coords_ptr = 0;
		w_road_section_offset = 32;
	}

	// draw_world_initialisation
	private static void drawWorldInitialisation() {
		setRoadPositionValues();

		b_dont_copy_coords = false;
		b_1bb5c = 0;
		b_pits_are_black = false;
		w_opponents_offset = -1;
		b_do_not_draw_opponents_shadow = false;

		initialiseEdges();
	}

	// draw_horizon
	private static void drawHorizon(final Canvas g) {
		w_screen_x_coords[ 6 / 2 ] = 0x500 * PF;
		w_screen_x_coords[ 4 / 2 ] = -0x500 * PF;

		short sd0 = (short) ((-(w_y_shift >> 3)) * PF);
		if( ! b_unused_flag ) {
			sd0 -= 8 * PF;
		}
		w_screen_y_coords[ 4 / 2 ] = sd0;
		w_screen_y_coords[ 6 / 2 ] = sd0;

		zRotate( 4 );
		zRotate( 6 );

		w_road_section_offset = 0;
		b_standard_clip_flag = true;
		clipLineMakeEdge( 4, 6 );
		b_standard_clip_flag = false;
		// makeMasks( g, GROUND_COLOUR );
		// makeMasks( g, SKY_COLOUR );
		//-- java version
		int x1 = SCREEN_AREA_LEFT_X + w_screen_x_coords[ 4 / 2 ];
		int y1 = SCREEN_AREA_TOP_Y + w_screen_y_coords[ 4 / 2 ];
		int x2 = SCREEN_AREA_LEFT_X + w_screen_x_coords[ 6 / 2 ];
		int y2 = SCREEN_AREA_TOP_Y + w_screen_y_coords[ 6 / 2 ];
		paint.setColor( getRGB( SKY_COLOR ) );
		g.drawRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
				SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH, SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT, paint );
		paint.setColor( getRGB( GROUND_COLOR ) );
		boolean is_upside_down = false;
		if( x1 > x2 ) {
			if( (x1 >= SCREEN_AREA_RIGHT_X) && (x2 >= SCREEN_AREA_RIGHT_X) ) {
				return;// only sky
			}
			if( (y1 < SCREEN_AREA_TOP_Y) && (y2 < SCREEN_AREA_TOP_Y) ) {
				return;// only sky
			}
			if( ((x1 < SCREEN_AREA_LEFT_X) && (x2 < SCREEN_AREA_LEFT_X)
				|| ((y1 >= SCREEN_AREA_BOTTOM_Y) && (y2 >= SCREEN_AREA_BOTTOM_Y))) )
			{// only ground
				g.drawRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
						SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH, SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT, paint );
				return;
			}
			is_upside_down = true;
			int t = x1; x1 = x2; x2 = t;
			t = y1; y1 = y2; y2 = t;
		}
		if( ! is_upside_down ) {
			if( (y1 >= SCREEN_AREA_BOTTOM_Y) && (y2 >= SCREEN_AREA_BOTTOM_Y) ) {
				return;// only sky
			}
			if( (y1 < SCREEN_AREA_TOP_Y) && (y2 < SCREEN_AREA_TOP_Y) ) {// only ground
				g.drawRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
						SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH, SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT, paint );
				return;
			}
		}
		// clip line
		if( x1 < SCREEN_AREA_LEFT_X ) {
			y1 -= (x1 - SCREEN_AREA_LEFT_X) * (y2 - y1) / (x2 - x1);
			x1 = SCREEN_AREA_LEFT_X;
		}
		if( x2 >= SCREEN_AREA_RIGHT_X ) {
			y2 -= (x2 - SCREEN_AREA_RIGHT_X) * (y1 - y2) / (x1 - x2);
			x2 = SCREEN_AREA_RIGHT_X;
		}
		if( (y1 == y2) && (x1 == x2) ) {
			return;// visible only 1 corner point
		}
		if( y1 < SCREEN_AREA_TOP_Y ) {
			x1 -= (y1 - SCREEN_AREA_TOP_Y) * (x2 - x1) / (y2 - y1);
			y1 = SCREEN_AREA_TOP_Y;
		}
		if( y1 > SCREEN_AREA_BOTTOM_Y ) {
			x1 -= (y1 - SCREEN_AREA_BOTTOM_Y) * (x2 - x1) / (y2 - y1);
			y1 = SCREEN_AREA_BOTTOM_Y;
		}
		if( y2 > SCREEN_AREA_BOTTOM_Y ) {
			x2 -= ((y2 - SCREEN_AREA_BOTTOM_Y) * (x1 - x2)) / (y1 - y2);
			y2 = SCREEN_AREA_BOTTOM_Y;
		}
		if( y2 < SCREEN_AREA_TOP_Y ) {
			x2 -= (y2 - SCREEN_AREA_TOP_Y) * (x2 - x1) / (y2 - y1);
			y2 = SCREEN_AREA_TOP_Y;
		}
		// draw polygon
		// draw sloping ground section, 4 points maximum
		if( is_upside_down ) {
			g.drawRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
					SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH,
					SCREEN_AREA_TOP_Y + Math.min( y1, y2 ) - SCREEN_AREA_TOP_Y, paint );
		} else {
			final int y = Math.max( y1, y2 );
			g.drawRect( SCREEN_AREA_LEFT_X, y,
					SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH,
					y + SCREEN_AREA_BOTTOM_Y - y, paint );
		}
		if( y1 != y2 ) {
			final int[] xp = jXPoints;// for speed optimization
			final int[] yp = jYPoints;// for speed optimization
			int sides = 0;// 5 sides maximum
			if( is_upside_down ^ (y1 < y2) ) {// ground on left area of the screen
				if( x1 > SCREEN_AREA_LEFT_X ) {
					xp[ sides ] = SCREEN_AREA_LEFT_X;
					yp[ sides++ ] = y1;
				}
				xp[ sides ] = x1;
				yp[ sides++ ] = y1;
				xp[ sides ] = x2;
				yp[ sides++ ] = y2;
				if( x2 > SCREEN_AREA_LEFT_X ) {
					xp[ sides ] = SCREEN_AREA_LEFT_X;
					yp[ sides++ ] = y2;
				}
			} else {// ground on right area of the screen
				if( x1 < SCREEN_AREA_RIGHT_X ) {
					xp[ sides ] = SCREEN_AREA_RIGHT_X;
					yp[ sides++ ] = y1;
				}
				xp[ sides ] = x1;
				yp[ sides++] = y1;
				xp[ sides ] = x2;
				yp[ sides++ ] = y2;
				if( x2 < SCREEN_AREA_RIGHT_X ) {
					xp[ sides ] = SCREEN_AREA_RIGHT_X;
					yp[ sides++ ] = y2;
				}
			}
			//if( sides > 2 )
			// g.drawPoints( xy, 0, sides, paint );
			path.moveTo( xp[ 0 ], yp[ 0 ] );
			for( int i = 1; i < sides; i++ ) {
				path.lineTo( xp[ i ], yp[ i ] );
			}
			path.close();
			g.drawPath( path, paint );
			path.reset();
		}
		// horizon line
		// g.setColor( Color.RED );
		// g.drawLine( x1, y1, x2, y2 );
	}

	// draw.mountains
	private static void drawMountains(final Canvas g) {
		final int mountain_y_offset = (-(w_y_shift >> 3));

		final int bd6 = ((w_players_y_angle >> 8) - 28) & 0xFF;//Amiga: 28 is -39 deg
		final int bd7 = 44;
		for( int i = Scenery.mountain_count; --i >= 0; ) {
			final int bd0 = ((Scenery.mountain_positions[ i ] & 0xFF) - bd6) & 0xFF;
			if( bd0 < bd7 ) {// abs

				int sd0 = ((bd0 - 28) << 8);// Amiga: 28 is -39 deg
				int sd3 = (w_players_y_angle & 0xFE);
				int mountain_x_offset = ((sd0 - sd3) >> 3);// short

				sd0 = Scenery.mountain_numbers[ i ];

				final Scenery.SceneryData a6_sd = Scenery.mountain_table[ sd0 ].mSceneryData;
				final short[] sa2 = Scenery.mountain_table[ sd0 ].mNumbers;
				int a2i = 0;
				int sd6 = a6_sd.mNumCountLen2 - 1;
				final short[] a6_w = a6_sd.mWdata;
				int a6i = 0;
				int sd1 = (sd6 << 1);
				int a4i = 0;
				int a5i = 0;

				do {
					sd0 = a6_w[ a6i++ ];// x
					if( sd0 < 0 ) {
						sd0 = sa2[ a2i++ ];
					}
					sd0 += mountain_x_offset;
					w_screen_x_coords[ a4i++ ] = (short) (sd0 * PF);

					sd0 = a6_w[ a6i++ ];// y
					if( sd0 < 0 ) {
						sd0 = sa2[ a2i++ ];
					}
					w_screen_y_coords[ a5i++ ] = (short) ((-(sd0 - mountain_y_offset)) * PF);
				} while( --sd6 >= 0 );

				do {
					zRotate( sd1 );
					sd1 -= 2;
				} while( sd1 >= 0 );

				w_road_section_offset = 0;// (int16 x, int16 y) count in bytes?
				final byte[] ba6 = a6_sd.mBdata;
				a6i = 0;
				int mountain_total_edges = ba6[ a6i++ ];
				do {
					sd1 = ba6[ a6i++ ];// point index 1
					final int bd2 = ba6[ a6i++ ];// point index 2
					clipLineMakeEdge( sd1, bd2 );
					w_road_section_offset += 4;
				} while( --mountain_total_edges > 0 );

				int mountain_poly_count = ba6[ a6i++ ];

				do {
					sd0 = ba6[ a6i++ ];// color
					makeMasks( sd0 );
					paint.setColor( getRGB( sd0 ) );

					final int mountain_poly_edges = ba6[ a6i++ ];// poly edge number
					/*-- java version
					int[] x_points = jXPoints;
					int[] y_points = jYPoints;
					for( int pi = 0; pi < mountain_poly_edges; pi++ ) {
						final int edge = ba6[ a6i++ ] / 4;// / 2 (x,y) and / 2 is for short indexing
						x_points[ pi ] = x_values[ edge ];
						y_points[ pi ] = y_values[ edge ];
					}
					g.fillPolygon( x_points, y_points, mountain_poly_edges );
                    // end java /* */

					//
					final int[] ia5 = lw_screen_data;// space for writing data?
                    int id4 = 0xffffffff;
                    byte bd2 = ba6[ a6i++ ];// edge 1, index in bytes for short (x,y)
                    int id0 = ia5[ bd2 / 4 ];
                    id4 &= id0;
                    id0 &= 0xffffff;
                    if( SECTION_DATA_NO_DATA != id0 ) {
                        final int sa0i = id0;

                        bd2 = ba6[ a6i++ ];// edge 2, index in bytes for short (x,y)
                        id0 = ia5[ bd2 / 4 ];

                        id4 &= id0;
                        id0 &= 0xffffff;
                        if( SECTION_DATA_NO_DATA != id0 ) {
                            final int sa1i = id0;

                            bd2 = ba6[ a6i++ ];// edge 3, index in bytes for short (x,y)
                            id0 = ia5[ bd2 / 4 ];

                            id4 &= id0;
                            id0 &= 0xffffff;
                            if( SECTION_DATA_NO_DATA != id0 ) {
                                final int sa2i = id0;

                                if( mountain_poly_edges != 3 ) {
                                    bd2 = ba6[ a6i++ ];// edge 4, index in bytes for short (x,y)
                                    id0 = ia5[ bd2 / 4 ];
                                    id4 &= id0;
                                    id0 &= 0xffffff;
                                    if( SECTION_DATA_NO_DATA != id0 ) {
                                        final int sa3i = id0;
                                        id4 &= 0xf000000;
                                        if( 0 == id4 ) {
											simplePolyFill( g, 0, 0, sa0i, sa1i, sa2i, sa3i );
                                        }
                                    }
                                } else {
                                    id4 &= 0xf000000;
                                    if( 0 == id4 ) {
                                        fillTriangle( g, sa0i, sa1i, sa2i );
                                    }
                                }
                            }
                        }
                    }
                    /* */
				} while( --mountain_poly_count > 0 );
				// movem.l	(sp)+,d1/d6-a0
			}
		}
	}

	// fetch_xz_position
	/**
	 *
	 * @param flags_piece road piece in d0. word, high byte is flags
	 */
	private static void fetchXZposition(int flags_piece) {
		flags_piece &= 0xFF;
		final byte[] ba1 = b_road_piece_xz_positions;
		int d3_map_z = ((ba1[ flags_piece ] >>> 4) & 0x0F) - b_players_map_z;
		int d0_map_x = (ba1[ flags_piece ] & 0x0F) - b_players_map_x;

		if( 0 == (w_rough_player_angle & Trigonometry.ANGLE_180_DEGREES) ) {
			if( 0 != (w_rough_player_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				final int temp = d0_map_x;
				d0_map_x = -d3_map_z;
				d3_map_z = temp;
			}
		} else {
			if( 0 == (w_rough_player_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				d0_map_x = -d0_map_x;
				d3_map_z = -d3_map_z;
			} else {
				final int temp = d0_map_x;
				d0_map_x = d3_map_z;
				d3_map_z = -temp;
			}
		}
		// only high byte
		w_road_piece_x_offset = (short)(((d0_map_x << 8) & 0xFF00) | (w_road_piece_x_offset & 0xFF));
		// only high byte
		w_road_piece_z_offset = (short)(((d3_map_z << 8) & 0xFF00) | (w_road_piece_z_offset & 0xFF));
		// only high bytes
		w_road_near_x_offset = (short)((w_road_near_x_offset & 0xFF)
				| ((((d0_map_x << 3) + (b_road_finer_x_offset & 0xFF)) << 8) & 0xFF00));
		w_road_near_z_offset = (short)((w_road_near_z_offset & 0xFF)
				| ((((d3_map_z << 3) + (b_road_finer_z_offset & 0xFF)) << 8) & 0xFF00));
	}

	// check_x_coord_magnitude
	/**
	 *
	 * @param d1 as word
	 * @return {@code true} c is set, {@code false} c is reset
	 */
	private static boolean checkXcoordMagnitude(final int d1) {
		int d0 = w_screen_x_coords[ d1 / 2 ];
		if( d0 < 0 ) {
			d0 = -d0;
		}
		if( d0 >= 0xc00 ) {
			w_coord_y_values_fp5[ d1 / 2 ] = (short) 0x8000;
			// ori.b	#1, ccr
			return true;
		}
		// andi.b	#%11110, ccr
		return false;
	}

	// make_near_bottom_coords
	/**
	 *
	 * @param d1 as byte
	 * @return new value of d1
	 */
	private static int makeNearBottomCoords(int d1) {
		// final short[] sa4 = x_values;
		// not used final short[] sa5 = y_values;
		int d2 = d1 & 2;// left/right
		d2 = ((d2 == 0) ? b_edge_offset_left_x2 : b_edge_offset_right_x2);
		if( d2 < 0 ) {
			if( ! b_unused_flag ) {
				if( 0 == b_players_position_flags ) {
					return d1;
				}
				if( 0 != ((b_players_position_flags ^ d1) & 2) ) {
					return d1;
				}
			}
		} else {
			// mnbc1:
			if( (d2 < 4) && !b_unused_flag ) {
				d2 += 2 * OFFSET_COPY_LAST_NEAR_SECTION_COORD;
			}
			if( 0 == (d1 & 2) ) {// left
				if( w_screen_x_coords[ d2 / 2 ] < w_screen_x_coords[ d1 / 2 ] ) {
					return d1;
				}
			} else {// right
				if( w_screen_x_coords[ d1 / 2 ] < w_screen_x_coords[ d2 / 2 ] ) {
					return d1;
				}
			}
		}
		// mnbc4:
		w_coord_y_values_fp5[ (d1 + OFFSET_FOR_BOTTOM_X2) / 2 ] = Road.GROUND_Y;// bottom visible value
		w_screen_x_coords[ (d1 + OFFSET_FOR_BOTTOM_X2) / 2 ] = w_screen_x_coords[ d1 / 2 ];// bottom x value
		d1 += OFFSET_FOR_BOTTOM_X2;
		calculateScreenY( d1 );// calculate bottom y coordinate
		zRotate0( d1 );// rotate bottom coordinate
		d1 -= OFFSET_FOR_BOTTOM_X2;
		return d1;
	}

	// detail_near_road
	private static void detailNearRoad() {
		int d0 = w_normal_distance_into_section;
		int d4 = b_number_of_segments << 8;
		if( b_plus_180_degrees < 0 ) {
			d0 = d4 - d0;
		}
		w_players_distance_into_section = (short) d0;
		d0 += 64;
		w_players_distance_into_section_plus64 = (short) d0;

		if( d0 >= d4 ) {
			w_players_distance_into_section_plus64 =
					(short)(0x8000 | (w_players_distance_into_section_plus64 & 0xFF));
			d0 = 0;
		}
		d0 = ((((d0 >>> 8) & 0xFF) + 1) << 1);
		b_coord_pair_to_start_at = (byte) d0;

		d0 <<= 1;
		b_offset_for_coord_pair_to_start_at = (byte) d0;

		if( b_plus_180_degrees < 0 ) {
			d0 = ((d4 - w_normal_distance_into_section) >>> 8) & 0xFF;// TODO verify and simplify
		} else {
			d0 = w_normal_distance_into_section >>> 8;
		}
		int d3 = 8 * 4 - d0;
		if( w_players_distance_into_section_plus64 < 0 ) {
			d3 += b_number_of_segments;
		}
		b_near_sections_done = (byte) d3;
		w_near_sections_done2 = (short) (d3 & 0xFF);
	}

	// make_optional_screen_coord
	/**
	 *
	 * @param d1 x/y screen coord offset x2 as unsigned byte
	 * @param d2 coord segment offset x2 as unsigned byte
	 */
	private static void makeOptionalScreenCoord(final int d1, final int d2) {
		w_near_x_coord = w_segment_x_coords[ d2 / 2 ];
		w_near_z_coord = w_segment_z_coords[ d2 / 2 ];// + 32 is segment_z_coords
		calculateScreenX( d1 );
		calculateScreenY( d1 );
		zRotate0( d1 );
	}

	// make_optional_top_coord
	/**
	 *
	 * @param d1_offset_x2 in d1 as unsigned byte. bit 1: 0 - left top y, 1 - right top y
	 */
	private static void makeOptionalTopCoord(int d1_offset_x2) {
		b_edge_x1_offset = (byte) d1_offset_x2;
		int d0_left_right = d1_offset_x2 & 0x02;
		b_start_offset_for_rating_in_divisions = (byte) d0_left_right;// left/right offset
		if( d1_offset_x2 >= OFFSET_FOR_BOTTOM_X2 ) {
			w_coord_y_values_fp5[ (d1_offset_x2 - 4) / 2 ] = Road.GROUND_Y;
		} else {
			w_coord_y_values_fp5[ (d1_offset_x2 - 4) / 2 ] = (d0_left_right == 0)
					? w_previous_left_top_y : w_previous_right_top_y;
		}

		fetchSegmentXZcoords( w_players_distance_into_section_plus64 >>> 8 );

		calculateSegmentXZ( w_players_distance_into_section_plus64 & 0xFF, b_start_offset_for_rating_in_divisions, 8 );

		d1_offset_x2 = b_edge_x1_offset & 0xFF;
		if( d1_offset_x2 < OFFSET_FOR_BOTTOM_X2 ) {
			int y_fp20 = w_coord_y_values_fp5[ d1_offset_x2 / 2 ] - w_coord_y_values_fp5[ (d1_offset_x2 - 4) / 2 ];
			y_fp20 *= ((w_players_distance_into_section_plus64 & 0xFF) << 7) & 0x7FFF;
			w_coord_y_values_fp5[ (d1_offset_x2 - 4) / 2 ] += (short) (y_fp20 >> 15);
		}
		d1_offset_x2 -= 4;
		makeOptionalScreenCoord( d1_offset_x2, 8 );

		int d4_x1 = w_screen_x_coords[ d1_offset_x2 / 2 ];
		int d5_y1 = w_screen_y_coords[ d1_offset_x2 / 2 ];
		if( (d4_x1 >= 0) && (d4_x1 < SCREEN_AREA_WIDTH) && (d5_y1 >= 0) && (d5_y1 < SCREEN_AREA_HEIGHT) ) {
			int dx = d4_x1 - w_screen_x_coords[ (d1_offset_x2 + 4) / 2 ];
			int dy = d5_y1 - w_screen_y_coords[ (d1_offset_x2 + 4) / 2 ];
			if( (0 == dy) && (0 == dx) ) {
				dy = 1;
			}
			do {
				d4_x1 += dx;
				d5_y1 += dy;
			} while( ((d4_x1 >= 0) && (d4_x1 < SCREEN_AREA_WIDTH)) && ((d5_y1 >= 0) && (d5_y1 < SCREEN_AREA_HEIGHT)) );
		}
		w_screen_x_coords[ d1_offset_x2 / 2 ] = (short) d4_x1;
		w_screen_y_coords[ d1_offset_x2 / 2 ] = (short) d5_y1;
	}

	// make_optional_top_edge
	/**
	 * @param d1_offset_x2, d1 as byte
	 * @param d3_section as word, section
	 */
	private static void makeOptionalTopEdge(int d1_offset_x2, int d3_section) {
		w_road_section_offset = (short) (d3_section << 2);

		b_edge_x1_offset = (byte) d1_offset_x2;
		if( ((d1_offset_x2 >= OFFSET_FOR_BOTTOM_X2) && (w_coord_y_values_fp5[ d1_offset_x2 / 2 ] < 0))
			|| (w_screen_x_coords[ d1_offset_x2 / 2 ] < 0) || (w_screen_x_coords[ d1_offset_x2 / 2 ] >= SCREEN_AREA_WIDTH)
			|| (w_screen_y_coords[ d1_offset_x2 / 2 ] < 0) || (w_screen_y_coords[ d1_offset_x2 / 2 ] >= SCREEN_AREA_HEIGHT)
		)
		{
			lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			return;
		}
		makeOptionalTopCoord( d1_offset_x2 );
		d1_offset_x2 = b_edge_x1_offset & 0xFF;
		clipLineMakeEdge( d1_offset_x2, d1_offset_x2 - 4 );
	}

	// make_optional_top_edges
	private static void makeOptionalTopEdges() {
		int offset = b_offset_for_coord_pair_to_start_at & 0xFF;
		if( 0 != offset ) {
			// set data into 8, 9, 10, 11 and make them as current section
			makeOptionalTopEdge( offset, 8 );
			offset = b_offset_for_coord_pair_to_start_at & 0xFF;
			makeOptionalTopEdge( offset + 2, 9 );
			offset = b_offset_for_coord_pair_to_start_at & 0xFF;
			makeOptionalTopEdge( offset + OFFSET_FOR_BOTTOM_X2, 10 );
			offset = b_offset_for_coord_pair_to_start_at & 0xFF;
			makeOptionalTopEdge( offset + OFFSET_FOR_BOTTOM_X2 + 2, 11 );
			w_road_section_offset = 8 * 4;
		}
	}

	// make_near_road_coords
	private static void makeNearRoadCoords() {
		fetchNearSectionStuff( b_current_road_piece );

		fetchXZposition( b_current_road_piece );

		// TODO verify low byte
		final int angle = w_rough_player_angle - w_rough_piece_angle;
		w_rough_difference_angle = (short) ((angle & 0xFF00) | (w_rough_difference_angle & 0xFF));

		makeNearTopYcoords();
		int offset_to_coord_pair = b_offset_for_coord_pair_to_start_at & 0xFF;
		if( 0 != offset_to_coord_pair ) {
			w_previous_left_top_y = w_coord_y_values_fp5[ (offset_to_coord_pair - 4) / 2 ];
			w_previous_right_top_y = w_coord_y_values_fp5[ (offset_to_coord_pair - 4 + 2) / 2 ];
		}
		// no_previous_y:
		if( b_5d724 >= 0 ) {
			// FIXME unused code
			int flags = w_rough_difference_angle >> 8;
			flags ^= b_plus_180_degrees;
			if( b_1bb93
				&& ((b_curved_section < 0) || (0 == (b_curved_section & 0x40))) )
			{
				// mnrc2:
				flags += 0x40;
			} else {
				// mnrc1:
				if( w_curve_to_left < 0 ) {
					// mnrc2:
					flags += 0x40;
				}
			}
			// mnrc3:
		}
		// mnrc4:
		if( 0 == b_plus_180_degrees ) {
			// start.with.first.coords2:
			// d0 = w_piece_data_offset;
			// d0 = ((((d0 >>> 8) & 0xFF) | ((d0 << 8) & 0xFF00)) - 0xb100) & 0xffff;
			final byte[] ba0 = w_piece_data_offset;// Road.offsetToPiece( d0 );
			b_piece_coords_offset = (byte) (ba0[ 0 ] + 7);

			int d1_coord_offset_x2 = b_coord_offset_zero_or_four;

			// mnrc5:
			do {
				if( w_coord_y_values_fp5[ (d1_coord_offset_x2) / 2 ] >= 0 ) {
					w_coord_y_values_fp5[ (d1_coord_offset_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = (short) 0x8000;// if bottom coordinate is not visible
					if( d1_coord_offset_x2 < (b_offset_for_coord_pair_to_start_at & 0xFF) ) {
						w_coord_y_values_fp5[ (d1_coord_offset_x2) / 2 ] = (short) 0x8000;// top visible value
					} else {
						int d2 = (d1_coord_offset_x2 << 1) + b_piece_coords_offset;
						d2 = fetchNearXZcoords( d2 );
						calculateScreenX( d1_coord_offset_x2 );// calculate top x coordinate
						calculateScreenY( d1_coord_offset_x2 );// calculate top y coordinate

						if( 0 == (b_off_map_status & 0x40)
							|| !checkXcoordMagnitude( d1_coord_offset_x2 ) )// if not too large
						{// check.x.coord.magnitude
							d1_coord_offset_x2 = makeNearBottomCoords( d1_coord_offset_x2 );
						}
						// mnrc8:
						if( 0 == (d1_coord_offset_x2 & 2) ) {
							b_edge_offset_left_x2 = (byte)d1_coord_offset_x2;// left
						} else {
							b_edge_offset_right_x2 = (byte)d1_coord_offset_x2;// right
						}
					}
				}
				// mnrc9:
				d1_coord_offset_x2 += 2;
			} while( d1_coord_offset_x2 != b_number_of_coords );
		} else {
			// start_with_last_coords2:
			// d0 = w_piece_data_offset;
			// d0 = ((((d0 >>> 8) & 0xFF) | ((d0 << 8) & 0xFF00)) - 0xB100) & 0xFFFF;
			final byte[] ba0 = w_piece_data_offset;//  Road.offsetToPiece( d0 );
			b_piece_coords_offset = (byte) (ba0[ 0 ] + 7 + ((b_number_of_sections_x2 - 1) << 2));

			int coord_offset_x2 = b_coord_offset_zero_or_four;

			// mnrca:
			do {
				if( w_coord_y_values_fp5[ (coord_offset_x2) / 2 ] >= 0 ) {
					w_coord_y_values_fp5[ (coord_offset_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = (short) 0x8000;// if bottom coordinate is not visible
					if( coord_offset_x2 < (b_offset_for_coord_pair_to_start_at & 0xFF) ) {
						w_coord_y_values_fp5[ (coord_offset_x2) / 2 ] = (short) 0x8000;// top visible value
					} else {
						int d2 = b_piece_coords_offset - (coord_offset_x2 << 1);
						d2 = fetchNearXZcoords( d2 );
						calculateScreenX( coord_offset_x2 );// calculate top x coordinate
						calculateScreenY( coord_offset_x2 );// calculate top y coordinate
						if( 0 == (b_off_map_status & 0x40)
							|| !checkXcoordMagnitude( coord_offset_x2 ) )
						{
							// mnrcc:
							coord_offset_x2 = makeNearBottomCoords( coord_offset_x2 );// if not too large
						}
						// mnrcd:
						if( 0 == (coord_offset_x2 & 2) ) {
							b_edge_offset_left_x2 = (byte) coord_offset_x2;// left
						} else {
							b_edge_offset_right_x2 = (byte) coord_offset_x2;// right
						}
					}
				}
				// mnrce:
				coord_offset_x2 += 2;
			} while( coord_offset_x2 != b_number_of_coords );
		}
		// save_last_x_coords:
		w_screen_x_coords[ OFFSET_COPY_LAST_NEAR_SECTION_COORD ] = w_screen_x_coords[ (b_edge_offset_left_x2 & 0xFF) / 2 ];
		w_screen_x_coords[ OFFSET_COPY_LAST_NEAR_SECTION_COORD + 1 ] = w_screen_x_coords[ (b_edge_offset_right_x2 & 0xFF) / 2 ];

		int coord_offset_x2 = b_coord_offset_zero_or_four;
		// rotate_top_coords:
		do {// TODO inefficient because there is b_offset_for_coord_pair_to_start_at
			if( w_coord_y_values_fp5[ (coord_offset_x2) / 2 ] >= 0 ) {
				zRotate( coord_offset_x2 );
			}
			coord_offset_x2 += 2;
		} while( coord_offset_x2 != b_number_of_coords );
		b_offset_for_last_two_coords = b_edge_offset_left_x2;
	}

	// make_optional_side_coords
	/**
	 * @param d1 offset. Left: 0, 4, ... Right: 2, 6, ...
	 */
	private static void makeOptionalSideCoords(int d1) {
		b_edge_x1_offset = (byte) d1;
		w_coord_y_values_fp5[ (d1 + OFFSET_FOR_BOTTOM_X2) / 2 ] = Road.GROUND_Y;// bottom visible value
		fetchSegmentXZcoords( d1 >>> 2 );

		d1 = b_edge_x1_offset;
		makeOptionalScreenCoord( d1 + OFFSET_FOR_BOTTOM_X2, d1 & 0x02 );
	}

	// mose0
	/**
	 *
	 * @param d1_offset_x2 offset for Y coordinates in d1. {@link #w_coord_y_values_fp5}
	 *           Left: 0, 4, ...
	 *           Right: 2, 6, ...
	 */
	private static void makeOptionalSideEdgesMose0(int d1_offset_x2) {
		d1_offset_x2 -= 2 * 2;// previous left or right
		if( w_coord_y_values_fp5[ d1_offset_x2 / 2 ] < 0 ) {
			// mose3:
			d1_offset_x2 -= 2 * 2;
			if( d1_offset_x2 < 0 ) {
				return;
			}
		} else {
			if( w_coord_y_values_fp5[ (d1_offset_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0 ) {
				// mose5:
				do {
					d1_offset_x2 -= 2 * 2;
					if( d1_offset_x2 < 0 ) {
						return;
					}
				} while( (w_coord_y_values_fp5[ d1_offset_x2 / 2 ] < 0)
						|| (w_coord_y_values_fp5[ (d1_offset_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0) );

				if( d1_offset_x2 < (b_offset_for_coord_pair_to_start_at & 0xFF) ) {
					return;
				}
				// mose6:
				makeOptionalSideCoords( d1_offset_x2 );
				b_near_section_flags[ d1_offset_x2 & 0xFFFD ] |= 0x40;
				d1_offset_x2 -= 4;
			} else {
				final boolean flag_mose3;
				if( (d1_offset_x2 & 0xFFFD) == (b_offset_for_coord_pair_to_start_at & 0xFF) ) {
					// mose2
					flag_mose3 = ( (0 == b_players_position_flags)
							|| (0 == (((w_players_x_offset_from_road_centre >>> 14) ^ d1_offset_x2) & 0x02)) );
				} else {
					flag_mose3 = ( 0 == (w_current_far_section_flag & (0 == (d1_offset_x2 & 0x02) ? 0x8000 : 0x4000)) );
				}

				if( flag_mose3 ) {
					// mose3:
					d1_offset_x2 -= 2 * 2;
					if( d1_offset_x2 < 0 ) {
						return;
					}
				} else {
					// mose6:
					makeOptionalSideCoords( d1_offset_x2 );
					b_near_section_flags[ d1_offset_x2 & 0xFFFD ] |= 0x40;
					d1_offset_x2 -= 2 * 2;
				}
			}
		}// else w_coord_visible_values[ d1 / 2 ] < 0
		// mose4:
		while( d1_offset_x2 >= 0 ) {
			while( (w_coord_y_values_fp5[ d1_offset_x2 / 2 ] < 0)
					|| (w_coord_y_values_fp5[ (d1_offset_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] < 0) ) {
				d1_offset_x2 -= 2 * 2;
				if( d1_offset_x2 < 0 ) {
					return;
				}
			}
			b_near_section_flags[ d1_offset_x2 & 0xFFFD ] |= 0x40;
			// mose5:
			do {
				d1_offset_x2 -= 2 * 2;
				if( d1_offset_x2 < 0 ) {
					return;
				}
			} while( (w_coord_y_values_fp5[ d1_offset_x2 / 2 ] < 0)
					|| (w_coord_y_values_fp5[ (d1_offset_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0) );

			if( d1_offset_x2 < (b_offset_for_coord_pair_to_start_at & 0xFF) ) {
				return;
			}
			// mose6:
			makeOptionalSideCoords( d1_offset_x2 );
			b_near_section_flags[ d1_offset_x2 & 0xFFFD ] |= 0x40;
			d1_offset_x2 -= 2 * 2;
		}
		d1_offset_x2 += 2 * 2;

		if( 8 * 4 != w_road_section_offset ) {
			final int backup = w_road_section_offset;
			int d3_prev = w_road_section_offset - 4 * 4;
			if( 0 != d1_offset_x2 ) {
				d3_prev += 4;
			}
			// mose7:
			w_road_section_offset = (short) d3_prev;
			clipLineMakeEdge( d1_offset_x2 + OFFSET_FOR_BOTTOM_X2, d1_offset_x2 );
			w_road_section_offset = (short) backup;
		}
		// mose8:
	}

	// make_optional_side_edges
	/**
	 * Returns:
	 * sa6 to coord.visible.values
	 */
	private static void makeOptionalSideEdges() {
		w_current_far_section_flag =
				w_far_section_coordinates[ lw_far_section_ptrs[ b_current_road_piece ] / 2 ];

		makeOptionalSideEdgesMose0( b_number_of_coords );// left side

		// mose0
		makeOptionalSideEdgesMose0( b_number_of_coords + 2 );// right side
	}

	private static void setOpponentsOffset1(int d0) {
		d0 -= 32;
		if( w_opp_smallest_difference >= -256 ) {// 0xFF00
			// pit flag
			if( 0 == (lw_screen_data[ (d0 + 30) / 4 ] & (PIT << 8)) ) {// 28 + 2, btst #5,30(a1,d0.w)
				d0 -= 32;
			}
		}
		w_opponents_offset = (short) (d0 & 0xFFE0);
	}

	// set_opponents_offset
	private static void setOpponentsOffset() {
		b_opponents_coord = (byte) 0xFF;
		setOpponentsOffset1( w_road_section_offset );
	}

	// adjust_y_using_x
	private static void adjustYusingX() {
		int offset_1_x2 = (b_edge_x1_offset & 0xFF);
		int i = 2;// x1, x2 to adjust y1, y2
		do {
			offset_1_x2 ^= 2;
			int x = w_screen_x_coords[ offset_1_x2 / 2 ] - (SCREEN_AREA_WIDTH / 2);
			if( x < 0 ) {
				x = -x;
			}
			x -= (SCREEN_WIDTH / 2);
			if( x >= 0 ) {
				x >>>= 3;
				w_screen_y_coords[ offset_1_x2 / 2 ] += x;
			}
		} while( --i > 0 );
	}

	// make_near_road_edges2
	/**
	 * sa6 = {@link #w_coord_y_values_fp5}
	 */
	private static void makeNearRoadEdges2() {
		// For other near sections
		if( w_road_section_offset >= 47 * 8 * 4 ) {
			return;
		}
		int d4 = 0xFF;
		if( b_opponents_road_section_m255 == b_current_road_piece ) {
			d4 = (w_opponents_distance_into_section_minus255 >> 8) << 2;
		}
		b_opponents_coord = (byte) d4;

		if( 0 == b_opponents_coord ) {
			setOpponentsOffset();
		}
		b_edge_offset_left_x2 = 0;
		makeNearRoadEdgesCommon( 4, false, false );// from start
	}

	// mnre6 java: common part
	private static void makeNearRoadEdgesCommon(int d1_offset_1_x2, boolean isMnre9, boolean isMnre13) {
		do {
			if( isMnre9 || isMnre13 || (w_coord_y_values_fp5[ d1_offset_1_x2 / 2 ] >= 0) ) {
				isMnre9 = false;
				// mnre9( offset_x2 );
				int d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
				b_edge_x1_offset = (byte) d1_offset_1_x2;

				b_yellow_road_lines = (byte) (((b_other_road_line_color << 1) ^ d1_offset_1_x2) & 0x04);

				b_pit_and_start_byte = b_near_section_flags[ d1_offset_1_x2 ];

				adjustYusingX();

				if( (12 * 4) != w_road_section_offset ) {
					if( w_coord_y_values_fp5[ (d2_offset_2_x2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ ( d1_offset_1_x2) / 2 ] >= 0 )
					{
						clipLineMakeEdge( d1_offset_1_x2, d2_offset_2_x2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ (w_road_section_offset) / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
					if( w_coord_y_values_fp5[ (d2_offset_2_x2 + 2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ (d1_offset_1_x2 + 2) / 2 ] >= 0 )
					{
						d2_offset_2_x2 += 2;
						d1_offset_1_x2 += 2;
						clipLineMakeEdge( d1_offset_1_x2, d2_offset_2_x2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
					if( w_coord_y_values_fp5[ (d2_offset_2_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ (d1_offset_1_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0 )
					{
						d2_offset_2_x2 += OFFSET_FOR_BOTTOM_X2;
						d1_offset_1_x2 += OFFSET_FOR_BOTTOM_X2;
						clipLineMakeEdge( d1_offset_1_x2, d2_offset_2_x2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
					if( w_coord_y_values_fp5[ (d2_offset_2_x2 + OFFSET_FOR_BOTTOM_X2 + 2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ (d1_offset_1_x2 + OFFSET_FOR_BOTTOM_X2 + 2) / 2 ] >= 0 )
					{
						d2_offset_2_x2 += OFFSET_FOR_BOTTOM_X2 + 2;
						d1_offset_1_x2 += OFFSET_FOR_BOTTOM_X2 + 2;
						clipLineMakeEdge( d1_offset_1_x2, d2_offset_2_x2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
				}
				// mnre12
				if( !isMnre13 && (0 != b_yellow_road_lines)
						&& (b_pit_and_start_byte < 0)
						&& (0 == (b_pit_and_start_byte & 0x40)) )
				{
					// if side edges are not needed
					lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					lw_screen_data[ (w_road_section_offset + 4) / 4 ] = SECTION_DATA_END_MARKER;
					w_road_section_offset += 8;
					// mnre18( offset_1_x2 );
					// return;
				} else {
					isMnre13 = false;
					// mnre13( offset_1_x2 );
					// a6 = coord_visible_values
					if( w_coord_y_values_fp5[ (d1_offset_1_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ (d1_offset_1_x2) / 2 ] >= 0 ) {// if bottom left y and left y
						clipLineMakeEdge( d1_offset_1_x2, d1_offset_1_x2 + OFFSET_FOR_BOTTOM_X2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
					if( w_coord_y_values_fp5[ (d1_offset_1_x2 + 2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ (d1_offset_1_x2 + OFFSET_FOR_BOTTOM_X2 + 2) / 2 ] >= 0 ) {// if right y and bottom right y
						clipLineMakeEdge( d1_offset_1_x2 + OFFSET_FOR_BOTTOM_X2 + 2, d1_offset_1_x2 + 2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
				}
				// mnre18( offset_1_x2 );
				if( b_pit_and_start_byte < 0 ) {
					// if start edge not needed
					lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					w_road_section_offset += 4;
				} else {
					if( w_coord_y_values_fp5[ (d1_offset_1_x2) / 2 ] >= 0
							&& w_coord_y_values_fp5[ (d1_offset_1_x2 + 2) / 2 ] >= 0 )
					{// if left y and right y
						clipLineMakeEdge( d1_offset_1_x2 + 2, d1_offset_1_x2 );
						d1_offset_1_x2 = b_edge_x1_offset & 0xFF;
						d2_offset_2_x2 = b_edge_offset_left_x2 & 0xFF;
					} else {
						lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
					}
					w_road_section_offset += 4;
				}
				// java: moved later to store full int32
				//section_data[ road_section_offset / 4 ] =
				//		(current_road_section << 24)// copy colour flag
				//				| (near_section_byte1 & 0xFF);// not needed

				int color = SIDE_LINES_COLOR;// darkest red
				if( b_near_sections_done >= 38 ) {
					color = SIDE_LINES_COLOR_BRIGHTER;// brighter red
					if( b_near_sections_done >= 42 ) {
						color = SIDE_LINES_COLOR_BRIGHTEST;// brightest red
						if( b_near_sections_done >= 44 ) {
							color |= 0x80;// no left or right lines
						}
					}
				} else {
					if( 0 != b_yellow_road_lines ) {
						color = SIDE_LINES_COLOR_YELLOW;// yellow
					}
				}
				lw_screen_data[ w_road_section_offset / 4 ] =
						(b_current_road_piece << 24)// copy colour flag
								| ((color & 0xFF) << 16)// set road line colour
								| ((b_pit_and_start_byte & 0xFF) << 8)// copy pit/start line flag
								| (b_curved_section & 0xFF);// not needed

				w_road_section_offset += 4;
				b_edge_offset_left_x2 = (byte) d1_offset_1_x2;
				if( w_road_section_offset >= 47 * 8 * 4 ) {
					return;
				}
			}
			// mnre7:
			b_near_section_flags[ d1_offset_1_x2 ] = (byte)0x80;
			if( d1_offset_1_x2 >= (b_opponents_coord & 0xFF) ) {
				setOpponentsOffset();
			}
			d1_offset_1_x2 += 4;
			++b_near_sections_done;
		} while( d1_offset_1_x2 < b_number_of_coords );
	}

	// make_near_road_edges
	/**
	 * sa6 = {@link #w_coord_y_values_fp5}
	 */
	private static void makeNearRoadEdges() {
		// the closest near section
		int d4 = 0xFF;
		if( b_opponents_road_section_m255 == b_current_road_piece ) {
			d4 = (w_opponents_distance_into_section_minus255 >> 8) << 2;
		}
		b_opponents_coord = (byte)d4;

		b_edge_offset_left_x2 = 0;
		w_road_section_offset = 12 * 4;
		final int d1_offset_x2 = b_offset_for_coord_pair_to_start_at & 0xFF;
		if( 0 != d1_offset_x2 ) {
			for( int d0_offset = d1_offset_x2 - 4; d0_offset >= 0; d0_offset -= 4 ) {
				b_near_section_flags[ d0_offset ] = (byte) 0x80;
			}
			b_near_sections_done += (d1_offset_x2 >>> 2) - 1;
		}
		// mnre9( offset_x2 );
		makeNearRoadEdgesCommon( d1_offset_x2, true, false );
	}

	// make_far_road_coords
	private static void makeFarRoadCoords() {
		int sa6i = lw_far_section_ptrs[ b_current_road_piece ];// to far_section_coordinates
		int d1_section_x2 = 4;

		// mfrc1:
		do {// for sections
			final short d0_flags_and_section = w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
			w_far_section_flags[ d1_section_x2 / 2 ] = d0_flags_and_section;
			w_current_far_section_flag = d0_flags_and_section;
			b_current_road_piece = (byte) d0_flags_and_section;

			fetchXZposition( d0_flags_and_section );

			// w_road_near_x_offset	dc.b	0
			// b_road_finest_x_offset	dc.b	0
			int offset = (((b_road_finer_x_offset) << 8) | (w_road_near_x_offset & 0xFF)) >> 1;
			offset += (w_road_piece_x_offset >> 8) << 10;
			w_road_piece_x_offset = (short) offset;

			// w_road_near_z_offset	dc.b	0
			// b_road_finest_z_offset	dc.b	0
			offset = (((b_road_finer_z_offset) << 8) | (w_road_near_z_offset & 0xFF)) >> 1;
			offset += (w_road_piece_z_offset >> 8) << 10;
			w_road_piece_z_offset = (short) offset;
			// mfrc2:
			do {// left/right
				int x, z;
				if( 0 == (w_rough_player_angle & Trigonometry.ANGLE_180_DEGREES) ) {
					if( 0 == (w_rough_player_angle & Trigonometry.ANGLE_90_DEGREES) ) {
						x = w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
						z = w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
					} else {
						z = w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
						x = 0x800 - w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
					}
				} else {
					if( 0 == (w_rough_player_angle & Trigonometry.ANGLE_90_DEGREES) ) {
						x = 0x800 - w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
						z = 0x800 - w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
					} else {
						z = 0x800 - w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
						x = w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
					}
				}
				// mfrc6
				x >>= 1;
				z >>= 1;
				x += w_road_piece_x_offset;
				z += w_road_piece_z_offset;
				calculateScreenX2( x, d1_section_x2, z );// calculate top x coordinate

				int y = w_far_section_coordinates[ sa6i / 2 ]; sa6i += 2;
				w_coord_y_values_fp5[ d1_section_x2 / 2 ] = (short) y;// top visible value

				y = (w_y_perspective_shift - y) >> 3;

				calculateScreenY2( y, d1_section_x2, w_perspective_z );// calculate top y coordinate

				x = w_screen_x_coords[ d1_section_x2 / 2 ];
				boolean is_bottom_visible = false;// java, goto mfrca or mfrcb
				if( 0 == (d1_section_x2 & 2) ) {
					final int old_x = w_old_left_x;
					w_old_left_x = (short) x;
					if( x < old_x ) {
						is_bottom_visible = true;
					} else if( x != old_x ) {
						if( 0 != (w_current_far_section_flag & 0x8000) ) {
							is_bottom_visible = true;
						}
					} else {// x >= old_x
						is_bottom_visible = (0 != b_players_position_flags) || (0 != (w_current_far_section_flag & 0x8000));
					}
				} else {
					final int old_x = w_old_right_x;
					w_old_right_x = (short) x;
					if( x > old_x ) {
						is_bottom_visible = true;
					} else if( x != old_x ) {
						if( 0 != (w_current_far_section_flag & 0x4000) ) {
							is_bottom_visible = true;
						}
					} else {// x <= old_x
						is_bottom_visible = (0 != b_players_position_flags) || (0 != (w_current_far_section_flag & 0x4000));
					}
				}
				if( is_bottom_visible ) {
					// mfrca:
					w_coord_y_values_fp5[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = Road.GROUND_Y;// bottom visible value
					w_screen_x_coords[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = w_screen_x_coords[ (d1_section_x2) / 2 ];// bottom x value

					y = ((w_y_perspective_shift - Road.GROUND_Y) >> 3);
					d1_section_x2 += OFFSET_FOR_BOTTOM_X2;
					calculateScreenY2( y, d1_section_x2, w_perspective_z );// calculate bottom y coordinate
					zRotate( d1_section_x2 );// rotate bottom coordinate
					d1_section_x2 -= OFFSET_FOR_BOTTOM_X2;
				} else {
					// mfrcb:
					w_coord_y_values_fp5[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] = (short) 0x8000;// if bottom coordinate is not visible
				}
				// mfrcc:
				zRotate( d1_section_x2 );// rotate top coordinate

				if( sa6i >= lw_end_far_sections_ptr ) {
					sa6i = 0;
				}
				// mfrcd:
				d1_section_x2 += 2;
			} while( 0 != (d1_section_x2 & 0x02) );// if four coordinates are not done
			if( d1_section_x2 >= b_road_visible_range_table[ b_current_road_piece ] ) {
				break;
			}
		} while( ( (w_screen_x_coords[ (d1_section_x2 - 4) / 2 ] >= 0)
						&& (w_screen_x_coords[ (d1_section_x2 - 4) / 2 ] < SCREEN_AREA_WIDTH) )
					|| ( (w_screen_x_coords[ (d1_section_x2 - 2) / 2 ] >= 0)
						&& (w_screen_x_coords[ (d1_section_x2 - 2) / 2 ] < SCREEN_AREA_WIDTH) )
				||
				( (w_coord_y_values_fp5[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2 - 4) / 2 ] >= 0)
					&& (w_screen_x_coords[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2 - 4) / 2 ] >= 0)
						&& (w_screen_x_coords[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2 - 4) / 2 ] < SCREEN_AREA_WIDTH) )
				||
				( (w_coord_y_values_fp5[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2 - 2) / 2 ] >= 0)
					&& (w_screen_x_coords[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2 - 2) / 2 ] >= 0)
						&& (w_screen_x_coords[ (d1_section_x2 + OFFSET_FOR_BOTTOM_X2 - 2) / 2 ] < SCREEN_AREA_WIDTH) ) );
		// mfrcf:
		b_max_far_coord = (byte) d1_section_x2;
	}

	// make_far_road_edges
	private static void makeFarRoadEdges() {
		b_opponent_reached = false;
		int d2_offset_point_2_x2 = 0;
		int d1_offset_point_1_x2 = 4;
		// mfre1:
		do {
			if( w_road_section_offset >= 47 * 32 ) {
				return;
			}
			b_edge_x1_offset = (byte) d1_offset_point_1_x2;
			b_edge_offset_left_x2 = (byte) d2_offset_point_2_x2;
			if( (w_coord_y_values_fp5[ d2_offset_point_2_x2 / 2 ] >= 0)
				&& (w_coord_y_values_fp5[ d1_offset_point_1_x2 / 2 ] >= 0) )
			{
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfre3:
			w_road_section_offset += 4;
			if( (w_coord_y_values_fp5[ (d2_offset_point_2_x2 + 2) / 2 ] >= 0)
				&& (w_coord_y_values_fp5[ (d1_offset_point_1_x2 + 2) / 2 ] >= 0) )
			{
				d2_offset_point_2_x2 += 2;
				d1_offset_point_1_x2 += 2;
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfre5:
			w_road_section_offset += 4;
			if( w_coord_y_values_fp5[ (d2_offset_point_2_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0
				&& w_coord_y_values_fp5[ (d1_offset_point_1_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0 )
			{
				d2_offset_point_2_x2 += OFFSET_FOR_BOTTOM_X2;
				d1_offset_point_1_x2 += OFFSET_FOR_BOTTOM_X2;
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfre7
			w_road_section_offset += 4;
			if( w_coord_y_values_fp5[ (d2_offset_point_2_x2 + OFFSET_FOR_BOTTOM_X2 + 2) / 2 ] >= 0
				&& w_coord_y_values_fp5[ (d1_offset_point_1_x2 + OFFSET_FOR_BOTTOM_X2 + 2) / 2 ] >= 0 )
			{
				d2_offset_point_2_x2 += OFFSET_FOR_BOTTOM_X2 + 2;
				d1_offset_point_1_x2 += OFFSET_FOR_BOTTOM_X2 + 2;
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfre9
			w_road_section_offset += 4;
			if( w_coord_y_values_fp5[ (d1_offset_point_1_x2 + OFFSET_FOR_BOTTOM_X2) / 2 ] >= 0
				&& w_coord_y_values_fp5[ (d1_offset_point_1_x2) / 2 ] >= 0 )
			{
				d2_offset_point_2_x2 = d1_offset_point_1_x2 + OFFSET_FOR_BOTTOM_X2;
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfreb
			w_road_section_offset += 4;
			if( w_coord_y_values_fp5[ (d1_offset_point_1_x2 + 2) / 2 ] >= 0
				&& w_coord_y_values_fp5[ (d1_offset_point_1_x2 + OFFSET_FOR_BOTTOM_X2 + 2) / 2 ] >= 0 )
			{
				d2_offset_point_2_x2 = d1_offset_point_1_x2 + 2;
				d1_offset_point_1_x2 += OFFSET_FOR_BOTTOM_X2 + 2;
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfred:
			w_road_section_offset += 4;
			if( w_coord_y_values_fp5[ (d1_offset_point_1_x2) / 2 ] >= 0
				&& w_coord_y_values_fp5[ (d1_offset_point_1_x2 + 2) / 2 ] >= 0 )
			{
				d2_offset_point_2_x2 = d1_offset_point_1_x2;
				d1_offset_point_1_x2 += 2;
				clipLineMakeEdge( d1_offset_point_1_x2, d2_offset_point_2_x2 );
				d1_offset_point_1_x2 = b_edge_x1_offset & 0xFF;
				d2_offset_point_2_x2 = b_edge_offset_left_x2 & 0xFF;
			} else {
				lw_screen_data[ w_road_section_offset / 4 ] = SECTION_DATA_END_MARKER;
			}
			// mfref
			w_road_section_offset += 4;
			final int d3 = w_road_section_offset;
			int b_factor_high_value_low = w_far_section_flags[ d1_offset_point_1_x2 / 2 ];
			int d0 = b_factor_high_value_low & 0xFF;
			// set 3 and 1 bytes only, color and pit
			lw_screen_data[ d3 / 4 ] =
					// copy colour flag
					(d0 << 24)
					// copy pit / start line flag
					| (b_factor_high_value_low & 0xFF00)
					// do not change existing values in other bytes
					| (lw_screen_data[ d3 / 4 ] & 0x00FF_00FF);

			if( ! b_opponent_reached && (d0 == (b_opponents_road_section_m255 & 0xFF)) ) {
				d0 = d3 + 4;
				setOpponentsOffset1( d0 );
				b_opponent_reached = true;
			}
			// mfre10
			w_road_section_offset += 4;
			d2_offset_point_2_x2 = d1_offset_point_1_x2;
			d1_offset_point_1_x2 += 2 * 2;// + 2x2
		} while( d1_offset_point_1_x2 < b_max_far_coord );
	}

	// rotate_opponent
	/**
	 * rotate opponent's corner point
	 *
	 * @param sd1 index of x value
	 * @param sd2 index of near coord
	 */
	private static void rotateOpponent(int sd1, final int sd2) {
		w_near_x_coord = w_segment_x_coords[ sd2 / 2 ];
		w_near_z_coord = w_segment_z_coords[ sd2 / 2 ];

		calculateScreenX( sd1 );
		calculateScreenY( sd1 );

		final short sd0 = w_screen_x_coords[ sd1 / 2 ];
		sd1 += 36;
		w_screen_x_coords[ sd1 / 2 ] = sd0;// take copy for shadow x coordinate
		calculateScreenY( sd1 );
		zRotate0( sd1 );
		zRotate0( sd1 - 36 );
	}

	// opponents.wheel
	private static void opponentsWheel() {
		int sd0 = w_1bc88;
		w_screen_x_coords[ 258 / 2 ] = (short) sd0;
		sd0 += w_1bc6c;
		w_screen_x_coords[ 260 / 2 ] = (short) sd0;

		sd0 = w_1bc8a;
		w_screen_y_coords[ 258 / 2 ] = (short) sd0;
		sd0 += w_1bc70;
		w_screen_y_coords[ 260 / 2 ] = (short) sd0;

		sd0 = w_1bc88 - w_1bc66;
		w_screen_x_coords[ 256 / 2 ] = (short) sd0;
		sd0 += w_1bc6c;
		w_screen_x_coords[ 262 / 2 ] = (short) sd0;

		sd0 = w_1bc8a - w_1bc6a;
		w_screen_y_coords[ 256 / 2 ] = (short) sd0;
		sd0 += w_1bc70;
		w_screen_y_coords[ 262 / 2 ] = (short) sd0;

		sd0 = w_car_x_shift;
		int sd3 = w_car_y_shift;

		w_screen_x_coords[ 256 / 2 ] += sd0;
		w_screen_y_coords[ 256 / 2 ] += sd3;

		w_screen_x_coords[ 258 / 2 ] += sd0;
		w_screen_y_coords[ 258 / 2 ] += sd3;

		w_screen_x_coords[ 260 / 2 ] += sd0;
		w_screen_y_coords[ 260 / 2 ] += sd3;

		w_screen_x_coords[ 262 / 2 ] += sd0;
		w_screen_y_coords[ 262 / 2 ] += sd3;

		w_screen_x_coords[ 260 / 2 ]++;
		w_screen_x_coords[ 262 / 2 ]++;
		w_screen_y_coords[ 256 / 2 ]++;
		w_screen_y_coords[ 262 / 2 ]++;

		clipLineMakeEdge( 256, 258 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 258, 260 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 260, 262 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 262, 256 );
		w_road_section_offset += 4;
	}

	// opponents_left_wheel
	private static void opponentsLeftWheel() {
		w_1bc88 = (short) -w_1bc64;
		w_1bc8a = (short) -w_1bc68;
		opponentsWheel();
	}

	// opponents_right_wheel
	private static void opponentsRightWheel() {
		w_1bc88 = (short) (w_1bc64 - w_1bc6c);
		w_1bc8a = (short) (w_1bc68 - w_1bc70);
		opponentsWheel();
	}

	// opponents_sides
	private static void opponentsSides() {
		int sd0 = w_1bc6c;
		w_screen_x_coords[ 276 / 2 ] = (short) sd0;

		sd0 -= w_1bc6e;
		w_screen_x_coords[ 278 / 2 ] = (short) sd0;

		sd0 = w_1bc70;
		w_screen_y_coords[ 276 / 2 ] = (short) sd0;

		sd0 -= w_1bc72;
		w_screen_y_coords[ 278 / 2 ] = (short) sd0;

		sd0 = -w_1bc6c;
		w_screen_x_coords[ 274 / 2 ] = (short) sd0;

		sd0 -= w_1bc6e;
		w_screen_x_coords[ 272 / 2 ] = (short) sd0;

		sd0 = -w_1bc70;
		w_screen_y_coords[ 274 / 2 ] = (short) sd0;

		sd0 -= w_1bc72;
		w_screen_y_coords[ 272 / 2 ] = (short) sd0;

		sd0 = w_car_x_shift;
		int sd3 = w_car_y_shift;

		w_screen_x_coords[ 272 / 2 ] += sd0;
		w_screen_y_coords[ 272 / 2 ] += sd3;

		w_screen_x_coords[ 274 / 2 ] += sd0;
		w_screen_y_coords[ 274 / 2 ] += sd3;

		w_screen_x_coords[ 276 / 2 ] += sd0;
		w_screen_y_coords[ 276 / 2 ] += sd3;

		w_screen_x_coords[ 278 / 2 ] += sd0;
		w_screen_y_coords[ 278 / 2 ] += sd3;

		w_screen_x_coords[ 276 / 2 ]++;
		w_screen_x_coords[ 278 / 2 ]++;
		w_screen_y_coords[ 272 / 2 ]++;
		w_screen_y_coords[ 278 / 2 ]++;

		w_road_section_offset = 47 * 32 + 32;
		clipLineMakeEdge( 272, 274 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 274, 276 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 276, 278 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 278, 272 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 264, 272 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 270, 278 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 266, 274 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 268, 276 );

		w_road_section_offset = 47 * 32 + 96;
		opponentsLeftWheel();
		opponentsRightWheel();
	}

	// opponents_back
	private static void opponentsBack() {
		w_road_section_offset = 47 * 32 + 64;
		opponentsLeftWheel();// 64, 68, 72, 76
		opponentsRightWheel();// 80, 84, 88, 92
		w_road_section_offset = 47 * 32 + 16;

		w_screen_x_coords[ 270 / 2 ] = (short) (w_1bc6c - w_1bc6e);
		w_screen_y_coords[ 270 / 2 ] = (short) (w_1bc70 - w_1bc72);

		w_screen_x_coords[ 264 / 2 ] = (short) (-w_1bc6c - w_1bc6e);
		w_screen_y_coords[ 264 / 2 ] = (short) (-w_1bc70 - w_1bc72);

		w_screen_x_coords[ 268 / 2 ] = (short) (w_1bc66 + w_1bc74 + w_1bc7c);
		w_screen_y_coords[ 268 / 2 ] = (short) (w_1bc6a + w_1bc78 + w_1bc80);

		w_screen_x_coords[ 266 / 2 ] = (short) (w_1bc66 - w_1bc74 - w_1bc7c);
		w_screen_y_coords[ 266 / 2 ] = (short) (w_1bc6a - w_1bc78 - w_1bc80);

		int sd0 = w_car_x_shift;
		int sd3 = w_car_y_shift;

		w_screen_x_coords[ 264 / 2 ] += sd0;
		w_screen_y_coords[ 264 / 2 ] += sd3;

		w_screen_x_coords[ 266 / 2 ] += sd0;
		w_screen_y_coords[ 266 / 2 ] += sd3;

		w_screen_x_coords[ 268 / 2 ] += sd0;
		w_screen_y_coords[ 268 / 2 ] += sd3;

		w_screen_x_coords[ 270 / 2 ] += sd0;
		w_screen_y_coords[ 270 / 2 ] += sd3;

		w_screen_x_coords[ 268 / 2 ]++;
		w_screen_x_coords[ 270 / 2 ]++;
		w_screen_y_coords[ 264 / 2 ]++;
		w_screen_y_coords[ 270 / 2 ]++;

		// 16, 20, 24, 28
		clipLineMakeEdge( 264, 266 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 266, 268 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 268, 270 );
		w_road_section_offset += 4;

		clipLineMakeEdge( 270, 264 );
		w_road_section_offset += 4;
	}

	// opponent_fatten
	private static void opponentFatten() {
		int sd0 = w_1bc64;
		if( sd0 < 0 ) {
			sd0 = -sd0;
		}
		sd0 >>>= 1;
		w_1bc6c = (short) sd0;
		sd0 >>>= 1;
		w_1bc74 = (short) sd0;
		sd0 >>>= 1;
		w_1bc7c = (short) sd0;

		if( w_1bc64 < 0 ) {
			w_1bc6c = (short) -w_1bc6c;
			w_1bc74 = (short) -w_1bc74;
			w_1bc7c = (short) -w_1bc7c;
		}

		sd0 = w_1bc66;
		if( sd0 < 0 ) {
			sd0 = -sd0;
		}
		sd0 >>>= 1;
		w_1bc6e = (short) sd0;
		sd0 >>>= 1;
		w_1bc76 = (short) sd0;
		sd0 >>>= 1;
		w_1bc7e = (short) sd0;

		if( w_1bc66 < 0 ) {
			w_1bc6e = (short) -w_1bc6e;
			w_1bc76 = (short) -w_1bc76;
			w_1bc7e = (short) -w_1bc7e;
		}

		sd0 = w_1bc68;
		if( sd0 < 0 ) {
			sd0 = -sd0;
		}
		sd0 >>>= 1;
		w_1bc70 = (short) sd0;
		sd0 >>>= 1;
		w_1bc78 = (short) sd0;
		sd0 >>>= 1;
		w_1bc80 = (short) sd0;

		if( w_1bc68 < 0 ) {
			w_1bc70 = (short) -w_1bc70;
			w_1bc78 = (short) -w_1bc78;
			w_1bc80 = (short) -w_1bc80;
		}

		sd0 = w_1bc6a;
		if( sd0 < 0 ) {
			sd0 = -sd0;
		}
		sd0 >>>= 1;
		w_1bc72 = (short) sd0;
		sd0 >>>= 1;
		w_1bc7a = (short) sd0;
		sd0 >>>= 1;
		w_1bc82 = (short) sd0;

		if( w_1bc6a < 0 ) {
			w_1bc72 = (short) -w_1bc72;
			w_1bc7a = (short) -w_1bc7a;
			w_1bc82 = (short) -w_1bc82;
		}
	}

	// rotate_opponent2
	/**
	 * make opponent x and y values
	 *
	 * @param sd1 index of x and y values
	 */
	private static void rotateOpponent2(final int sd1) {
		int sd4 = (w_screen_x_coords[ (sd1 + 4) / 2 ] - w_screen_x_coords[ sd1 / 2 ]) >> 1;// right x - left x
		int sd5 = (w_screen_y_coords[ (sd1 + 4) / 2 ] - w_screen_y_coords[ sd1 / 2 ]) >> 1;// right y - left y

		int sd0 = sd4;
		if( sd0 < 0 ) {
			sd0 = -sd0;
		}
		if( sd0 > 255 ) {
			sd0 = 255;
		}
		w_1bc6a = (short) sd0;// abs x diff / 2

		sd0 = sd5;
		if( sd0 < 0 ) {
			sd0 = -sd0;
		}
		if( sd0 > 255 ) {
			sd0 = 255;
		}
		w_1bc68 = (short) sd0;// abs y diff / 2

		w_1bc64 = w_1bc6a;// x diff / 2

		if( sd4 < 0 ) {
			w_1bc64 = (short) -w_1bc64;
		} else {
			w_1bc6a = (short) -w_1bc6a;
		}
		w_1bc66 = w_1bc68;// y diff / 2

		if( sd5 < 0 ) {
			w_1bc66 = (short) -w_1bc66;
			w_1bc68 = (short) -w_1bc68;
		}

		w_1bc66 >>= 1;
		w_1bc6a >>= 1;

		w_car_x_shift = (short) (w_screen_x_coords[ sd1 / 2 ] + sd4);
		w_car_y_shift = (short) (w_screen_y_coords[ sd1 / 2 ] + sd5);

		opponentFatten();
	}

	// opponents_shadow
	private static void opponentsShadow() {

		w_road_section_offset = 47 * 32 + 128;
		clipLineMakeEdge( 280, 284 );

		w_road_section_offset += 4;
		clipLineMakeEdge( 280, 282 );

		w_road_section_offset += 4;
		clipLineMakeEdge( 282, 286 );

		w_road_section_offset += 4;
		clipLineMakeEdge( 284, 286 );

		w_road_section_offset += 4;
	}

	private static final int OPP_REAR_LEFT_WHEEL_Y = 244;
	private static final int OPP_REAR_RIGHT_WHEEL_Y = 248;
	private static final int OPP_FRONT_LEFT_WHEEL_Y = 246;
	private static final int OPP_FRONT_RIGHT_WHEEL_Y = 250;

	private static final int OPP_SHADOw_Y1 = 280;
	private static final int OPP_SHADOw_Y2 = 282;
	private static final int OPP_SHADOw_Y3 = 284;
	private static final int OPP_SHADOw_Y4 = 286;

	// $599e2 make_opponent
	private static void makeOpponent() {
		int id0 = w_coord_y_values_fp5[ OPP_REAR_LEFT_WHEEL_Y / 2 ];
		int id3 = w_opp_wheel_road_heights[ OPP_REAR_LEFT_WHEEL ];
		id3 += 80;
		if( id0 < id3 ) {
			id3 = id0;// use lower of road and wheel height
		}
		w_coord_y_values_fp5[ OPP_SHADOw_Y1 / 2 ] = (short) id3;// shadow y1

		id0 = w_coord_y_values_fp5[ OPP_REAR_RIGHT_WHEEL_Y / 2 ];
		id3 = w_opp_wheel_road_heights[ OPP_REAR_RIGHT_WHEEL ];
		id3 += 80;
		if( id0 < id3 ) {
			id3 = id0;
		}
		w_coord_y_values_fp5[ OPP_SHADOw_Y3 / 2 ] = (short) id3;// shadow y3
		int id4 = (w_opp_wheel_road_heights[ OPP_REAR_RIGHT_WHEEL ]
				- w_opp_wheel_road_heights[ OPP_REAR_LEFT_WHEEL ]) >> 1;
		int id5 = w_opp_wheel_road_heights[ OPP_FRONT_WHEEL ] - id4;
		id0 = w_coord_y_values_fp5[ OPP_FRONT_LEFT_WHEEL_Y / 2 ];
		id3 = id5;// front left road height
		id3 += 80;
		if( id0 < id3 ) {
			id3 = id0;
		}
		w_coord_y_values_fp5[ OPP_SHADOw_Y2 / 2 ] = (short) id3;// shadow y2

		id5 = w_opp_wheel_road_heights[ OPP_FRONT_WHEEL ] + id4;
		id0 = w_coord_y_values_fp5[ OPP_FRONT_RIGHT_WHEEL_Y / 2 ];
		id3 = id5;// front right road height
		id3 += 80;
		if( id0 < id3 ) {
			id3 = id0;
		}
		w_coord_y_values_fp5[ OPP_SHADOw_Y4 / 2 ] = (short) id3;// shadow y4

		// Create screen coordinates of opponent's 4 corners
		rotateOpponent( OPP_REAR_LEFT_WHEEL_Y, 10 );

		rotateOpponent( OPP_REAR_RIGHT_WHEEL_Y, 14 );

		rotateOpponent( OPP_FRONT_LEFT_WHEEL_Y, 16 );

		rotateOpponent( OPP_FRONT_RIGHT_WHEEL_Y, 20 );

		rotateOpponent2( OPP_REAR_LEFT_WHEEL_Y );

		// left wheel: 64, 68, 72, 76
		// right wheel: 80, 84, 88, 92
		// back: 16, 20, 24, 28
		opponentsBack();

		final int sd3 = w_1bc6a - 1;
		w_screen_x_coords[ 280 / 2 ] -= w_1bc66;
		w_screen_x_coords[ 284 / 2 ] -= w_1bc66;
		w_screen_y_coords[ 280 / 2 ] -= sd3;
		w_screen_y_coords[ 284 / 2 ] -= sd3;
		rotateOpponent2( OPP_FRONT_LEFT_WHEEL_Y );

		// sides: 32, 36, 40, 44, 48, 52, 56, 60
		// left wheel: 96, 100, 104, 108
		// right wheel: 112, 116, 120, 124
		opponentsSides();

		w_screen_x_coords[ 282 / 2 ] -= w_1bc66;
		w_screen_x_coords[ 286 / 2 ] -= w_1bc66;
		w_screen_y_coords[ 282 / 2 ] -= w_1bc6a;
		w_screen_y_coords[ 286 / 2 ] -= w_1bc6a;
		opponentsShadow();
	}

	private static final int MIN_DISTANCE_TO_DRAW_OPPONENT = 10;
	private static final int MAX_DISTANCE_TO_DRAW_OPPONENT = 3200;

	// draw_car_subr1
	private static void drawCarSubr1(final Canvas g, int o1, int o2, int o3, int o4, int color) {
		//
		final int id3 = w_road_section_offset;
        int id0 = lw_screen_data[ (id3 + o1) / 4 ];
        int id4 = id0;
        id0 &= 0xffffff;
        if( SECTION_DATA_NO_DATA != id0 ) {
            final int sa0i = id0;
            id0 = lw_screen_data[ (id3 + o2) / 4 ];
            id4 &= id0;
            id0 &= 0xffffff;
            if( SECTION_DATA_NO_DATA != id0 ) {
				final int sa1i = id0;
                id0 = lw_screen_data[ (id3 + o3) / 4 ];
                id4 &= id0;
                id0 &= 0xffffff;
                if( SECTION_DATA_NO_DATA != id0 ) {
					final int sa2i = id0;
                    id0 = lw_screen_data[ (id3 + o4) / 4 ];
                    id4 &= id0;
                    id0 &= 0xffffff;
                    if( SECTION_DATA_NO_DATA != id0 ) {
						final int sa3i = id0;
                        id4 &= 0xf000000;
                        if( 0 == id4 ) {
                            id0 = color;
                            word_col = (id0 << 4) + 2;
							paint.setColor( getRGB( color ) );
                            int id6 = 0;
                            int id7 = 0;
                            if( 0 != (id0 & 0x01) ) {
                                id6 = 0x0000FFFF;
                            }
                            id6 <<= 16;
                            if( 0 != (id0 & 0x02) ) {
                                id6 = ~id6;
                            }
                            if( 0 != (id0 & 0x04) ) {
                                id7 = 0xFFFFFFFF;
                            }
                            id7 <<= 16;
                            if( 0 != (id0 & 0x08) ) {
                                id7 = ~id7;
                            }
                            simplePolyFill( g, id6, id7, sa0i, sa1i, sa2i, sa3i );
                        }
                    }
                }
            }
        }
		/* */
	}

	// draw_opponent
	private static void drawOpponent(final Canvas g) {
		if( b_opponent_behind_player
			|| (w_smallest_distance_between_players < MIN_DISTANCE_TO_DRAW_OPPONENT)
			|| (w_smallest_distance_between_players >= MAX_DISTANCE_TO_DRAW_OPPONENT) )
		{
			return;
		}
		final short backup_road_section_offset = w_road_section_offset;
		w_road_section_offset = 47 * 32;
		makeOpponent();
		b_daft_flag = true;
		w_road_section_offset = 47 * 32;
		// draw shadow only if the car is fully on the track
		if( !b_do_not_draw_opponents_shadow
				&& (w_opponents_road_x_position >= 28)
				&& (w_opponents_road_x_position <= 228) ) {
			w_road_section_offset += 128;
			// shadow
			drawCarSubr1( g, 0, 4, 8, 12, CAR_SHADOW_COLOR );
			w_road_section_offset -= 128;
		}
		// back: 16, 20, 24, 28
		// left wheel: 64, 68, 72, 76
		// right wheel: 80, 84, 88, 92
		// sides: 32, 36, 40, 44, 48, 52, 56, 60
		// left wheel: 96, 100, 104, 108
		// right wheel: 112, 116, 120, 124

		// left_front_wheel
		drawCarSubr1( g, 96, 100, 104, 108, CAR_WHEEL_COLOR );
		// right_front_wheel
		drawCarSubr1( g, 112, 116, 120, 124, CAR_WHEEL_COLOR );
		// left_body_panel
		drawCarSubr1( g, 32, 56, 16, 48, CAR_BODY_PANEL_COLOR );
		// right_body_panel
		drawCarSubr1( g, 24, 60, 40, 52, CAR_BODY_PANEL_COLOR );
		// rear_body_panel
		drawCarSubr1( g, 16, 20, 24, 28, CAR_REAR_BODY_PANEL_COLOR );
		// top_body_panel
		drawCarSubr1( g, 56, 36, 60, 20, CAR_TOP_BODY_PANEL_COLOR );
		// bottom_body_panel
		drawCarSubr1( g, 48, 28, 52, 44, CAR_BOTTOM_BODY_PANEL_COLOR );
		// left_rear_wheel
		drawCarSubr1( g, 64, 68, 72, 76, CAR_WHEEL_COLOR );
		// right_rear_wheel
		drawCarSubr1( g, 80, 84, 88, 92, CAR_WHEEL_COLOR );
		// no_right_rear_wheel
		b_daft_flag = false;
		w_road_section_offset = backup_road_section_offset;
	}

	private static boolean b_clip_flag1 = false;
	private static boolean b_clip_flag2 = false;
	private static byte b_clip_flag3 = 0;
	/** draw surface: 0, near right: 0x02, near left: 0x80 */
	private static byte b_drawing_surface_flags = 0;

	// java: common part of draw_far_right_sides and draw_far_left_sides
	private static void drawFarSides(final Canvas g, final int o1, final int o2, final int o3, final int o4) {
		int d3_section_off = w_road_section_offset;
		int d7_color = SIDES_COLOR_A;
		if( 0 != (lw_screen_data[ (d3_section_off + 28) / 4 ] & 0x01000000) ) {
			d7_color = b_sides_color;
		}
		int d0 = lw_screen_data[ (d3_section_off + o1) / 4 ];
		int d4 = d0;
		d0 &= 0xffffff;
		if( SECTION_DATA_NO_DATA != d0 ) {
			final int sa0i = d0;
			d0 = lw_screen_data[ (d3_section_off + o2) / 4 ];
			d4 &= d0;
			d0 &= 0xffffff;
			if( SECTION_DATA_NO_DATA != d0 ) {
				final int sa1i = d0;
				d0 = lw_screen_data[ (d3_section_off + o3) / 4 ];
				d4 &= d0;
				d0 &= 0xffffff;
				if( SECTION_DATA_NO_DATA != d0 ) {
					final int sa2i = d0;
					d0 = lw_screen_data[ (d3_section_off + o4) / 4 ];
					d4 &= d0;
					d0 &= 0xffffff;
					if( SECTION_DATA_NO_DATA != d0 ) {
						final int sa3i = d0;
						d4 &= 0xf000000;
						if( 0 == d4 ) {
							d0 = d7_color;
							word_col = (d0 << 4) + 2;// set color
							paint.setColor( getRGB( d7_color ) );

							int d6 = d7_color = 0;

							if( 0 != (d0 & 0x01) ) {
								d6 = 0xFFFF0000;
							}
							if( 0 != (d0 & 0x02) ) {
								d6 |= 0xFFFF;
							}
							if( 0 != (d0 & 0x04) ) {
								d7_color = 0xFFFF0000;
							}
							if( 0 != (d0 & 0x08) ) {
								d7_color |= 0xFFFF;
							}
							simplePolyFill( g, d6, d7_color, sa0i, sa1i, sa2i, sa3i );
						}
					}
				}
			}
		}
	}

	// draw_far_right_sides
	private static void drawFarRightSides(final Canvas g) {
		drawFarSides( g, 12, -12, 4, 20 );
	}

	// draw_far_left_sides
	private static void drawFarLeftSides(final Canvas g) {
		drawFarSides( g, 8, 16, 0, -16 );
	}

	// draw_start_line
	private static void drawStartLine(final Canvas g) {
		final int d0 = lw_screen_data[ (w_road_section_offset + 24) / 4 ] & 0xffffff;
		if( SECTION_DATA_NO_DATA != d0 ) {
			paint.setColor( getRGB( START_LINE_COLOR ) );
			plotLine( g, d0 );
		}
	}

	// draw_road_surface
	private static void drawRoadSurface(final Canvas g) {
		b_drawing_surface_flags = 0;
		b_section_flags2 = 0;
		b_draw_road_surface_flag = true;
		int d3_section_off = w_road_section_offset;
		w_copy_road_section_offset = (short) d3_section_off;

		int d0 = lw_screen_data[ (d3_section_off + 24) / 4 ];
		int d4 = d0;
		d0 &= 0xffffff;
		if( SECTION_DATA_NO_DATA != d0 ) {
			int ia1i = SECTION_SIDE_PTRS_OFF;
			int ia2i = SECTION_SIDE_PTRS_OFF;
			b_fill_polygon_color = ROAD_COLOR_A;
			if( 0 != (lw_screen_data[ (d3_section_off + 28) / 4 ] & 0x01_00_0000) ) {// byte 3
				b_fill_polygon_color = ROAD_COLOR_B;
			}
			b_section_flags2 = (byte) (lw_screen_data[ (d3_section_off + 30) / 4 ] >>> 8);// byte 1
			if( 0 != (b_section_flags2 & PIT) ) {
				b_fill_polygon_color = ROAD_PIT_COLOR;
			}

			lw_section_side_ptrs[ ia1i++ ] = d0;
			d0 = lw_screen_data[ (d3_section_off + 24) / 4 ] >>> 24;
			b_start_line_flag = (byte) d0;
			b_left_side_flag = (byte) d0;
			b_right_side_flag = (byte) (d0 ^ 0x40);

			if( 8 * 4 != w_copy_road_section_offset ) {
				// save_right_left_sides
				do {
					d0 = lw_screen_data[ (d3_section_off + 4) / 4 ];
					d4 &= d0;
					d0 &= 0xffffff;
					if( SECTION_DATA_NO_DATA != d0 ) {
						b_right_side_flag = (byte) (lw_screen_data[ (d3_section_off + 4) / 4 ] >>> 24);
						lw_section_side_ptrs[ ia1i++ ] = d0;

						d0 = lw_screen_data[ (d3_section_off) / 4 ];
						d4 &= d0;
						d0 &= 0xffffff;
						if( SECTION_DATA_NO_DATA != d0 ) {
							b_left_side_flag = (byte) (lw_screen_data[ (d3_section_off) / 4 ] >>> 24);
							lw_section_side_ptrs[ --ia2i ] = d0;

							d3_section_off -= 8 * 4;
							if( d3_section_off == w_next_road_section_offset ) {
								break;// here d3 != road_section_offset, beq save.next.start.line;
							}
							if( 8 * 4 != d3_section_off ) {
								continue;// bne save_right_left_sides;
							}
							break;// bra closest_section;
						}
						// no_left_side
						--ia1i;
					}
					break;
				} while( true );
				if( 8 * 4 != d3_section_off ) {// java
					// no_right_side
					if( d3_section_off != w_road_section_offset ) {
						// save_next_start_line
						d0 = lw_screen_data[ (d3_section_off + 24) / 4 ];
						d4 &= d0;
						d0 &= 0xffffff;
						if( SECTION_DATA_NO_DATA != d0 ) {
							b_right_side_flag = (byte) (lw_screen_data[ (d3_section_off + 24) / 4 ] >>> 24);
							lw_section_side_ptrs[ ia1i++ ] = d0;
							// bra now_draw_surface;
							// now_draw_surface
							d4 &= 0xf000000;
							if( 0 == d4 ) {
								fillPolygon( g, ia1i, ia2i );
							}
						}
					}
					// surface_done
					b_draw_road_surface_flag = false;
					if( 0 != (b_section_flags2 & 1) ) {
						drawStartLine( g );
					}
					return;
				}
			}// if( 32 != copy_road_section_offset )
			// closest_section
			d0 = lw_screen_data[ (d3_section_off + 4) / 4 ];
			d4 &= d0;
			d0 &= 0xffffff;
			if( SECTION_DATA_NO_DATA != d0 ) {
				b_right_side_flag = (byte) (lw_screen_data[ (d3_section_off + 4) / 4 ] >>> 24);
				lw_section_side_ptrs[ ia1i++ ] = d0;
			}
			// no_closest_right_side
			int sa5i = lw_section_side_ptrs[ ia1i - 4 / 4 ];
			int d6_x, d7_y;
			if( 0 != (b_right_side_flag & 0x40) ) {
				d7_y = w_screen_coords[ sa5i / 2 ];// y1
				d6_x = w_screen_coords[ (sa5i + 4) / 2 ];// x1
			} else {
				d7_y = w_screen_coords[ (sa5i + 2) / 2 ];// y2
				d6_x = w_screen_coords[ (sa5i + 6) / 2 ];// x2
			}
			final int sp1_x = d6_x;// x1/x2
			int d5_y = d7_y;// y1/y2

			d0 = lw_screen_data[ d3_section_off / 4 ];
			d4 &= d0;
			d0 &= 0xffffff;
			if( SECTION_DATA_NO_DATA != d0 ) {
				b_left_side_flag = (byte) (lw_screen_data[ d3_section_off / 4 ] >>> 24);
				lw_section_side_ptrs[ --ia2i ] = d0;
			}
			// no_closest_left_side
			sa5i = lw_section_side_ptrs[ ia2i ];
			if( 0 != (b_left_side_flag & 0x40) ) {
				d7_y = w_screen_coords[ sa5i / 2 ];// y1
				d6_x = w_screen_coords[ (sa5i + 4) / 2 ];// x1
			} else {
				d7_y = w_screen_coords[ (sa5i + 2) / 2 ];// y2
				d6_x = w_screen_coords[ (sa5i + 6) / 2 ];// x2
			}

			clipClosestSection( ia1i, ia2i, sp1_x, d5_y, d6_x, d7_y );// x1/x2, y1/y2, x1/x2, y1/y2

			if( !b_closest_section_flag ) {
				// now_draw_surface
				d4 &= 0xf000000;
				if( 0 == d4 ) {
					// on android a thin line exists between them
					// fillPolygon( g, ia1i, ia2i );
					// java: the simplest version to fill the closest area
					//jXPoints[ 0 ] = d6_x;
					//jYPoints[ 0 ] = d7_y;
					// right top
					//jXPoints[ 1 ] = sp1_x;
					//jYPoints[ 1 ] = d5_y;
					// right bottom
					//jXPoints[ 2 ] = sp1_x;
					//jYPoints[ 2 ] = SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT;
					// left bottom
					//jXPoints[ 3 ] = d6_x;
					//jYPoints[ 3 ] = SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT;
					//
					//--- java version TODO do not draw when completely off-screen
					paint.setColor( getRGB( b_fill_polygon_color ) );
					int point_count = 1;
					int index = lw_section_side_ptrs[ ia2i ];
					jXPoints[ 0 ] = w_screen_coords[ (index + 4) / 2 ];
					jYPoints[ 0 ] = w_screen_coords[ (index + 0) / 2 ];
					jXPoints[ 1 ] = w_screen_coords[ (index + 6) / 2 ];
					jYPoints[ 1 ] = w_screen_coords[ (index + 2) / 2 ];
					while( ++ia2i < ia1i ) {
						index = lw_section_side_ptrs[ ia2i ];
						point_count = sortPoints( index, point_count );
					}
					// ++point_count;
					path.moveTo( jXPoints[ 0 ], jYPoints[ 0 ] );
					for( int i = 1; i <= point_count; i++ ) {
						path.lineTo( jXPoints[ i ], jYPoints[ i ] );
					}
					//
					// path.moveTo( d6_x, d7_y );
					path.lineTo( sp1_x, d5_y );
					path.lineTo( sp1_x, SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT );
					path.lineTo( d6_x, SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT );
					path.close();
					g.drawPath( path, paint );
					path.reset();
				}
			}
		}
		// surface_done
		b_draw_road_surface_flag = false;
		if( 0 != (b_section_flags2 & 1) ) {
			drawStartLine( g );
		}
	}

	// draw_far_road
	private static void drawFarRoad(final Canvas g) {
		do {
			w_road_section_offset -= 8 * 4;
			if( w_road_section_offset < w_far_road_limit ) {
				return;
			}
			w_next_road_section_offset = (short) (w_road_section_offset - 8 * 4);

			if( 0 == b_at_side_byte ) {
				drawFarLeftSides( g );
				drawFarRightSides( g );
				drawOpponentTest1( g );
				drawOpponentTest2( g );
				drawRoadSurface( g );
				drawOpponentTest3( g );
			} else {
				// left_side
				if( w_players_x_offset_from_road_centre < 0 ) {
					drawOpponentTest1( g );
					drawFarRightSides( g );
					drawRoadSurface( g );
					drawOpponentTest3( g );
					drawFarLeftSides( g );
					drawOpponentTest2( g );
				} else {
					// right_side
					drawOpponentTest2( g );
					drawFarLeftSides( g );
					drawRoadSurface( g );
					drawOpponentTest3( g );
					drawFarRightSides( g );
					drawOpponentTest1( g );
				}
			}
		} while( true );
	}
	
	// drs_sub
	/**
	 * @param d0_x as word
	 * @param d1_y1 as word
	 * @param d2_y2 as word
	 * @return true - z
	 */
	private static boolean drsSub(int ia1i, int ia2i,
							   int d0_x, int d1_y1, int d2_y2)
	{
		if( d1_y1 < d2_y2 ) {
			final int temp = d1_y1; d1_y1 = d2_y2; d2_y2 = temp;
		}
		int sa0i = lw_screen_coords_ptr / 2;
		if( sa0i <= END_SCREEN_COORDS ) {// if( sa0i <= w_screen_coords.length ) {// FIXME why not < ?
			if( (d2_y2 >= 0) && (d2_y2 < (SCREEN_AREA_HEIGHT + 1 * PF)) && (d1_y1 >= 0) && (d1_y1 < (SCREEN_AREA_HEIGHT + 1 * PF)) ) {
				w_screen_coords[ sa0i++ ] = (short)d1_y1;
				w_screen_coords[ sa0i++ ] = (short)d2_y2;
				w_screen_coords[ sa0i++ ] = (short)d0_x;
				w_screen_coords[ sa0i++ ] = (short)d0_x;
				d1_y1 -= d2_y2;
				while( --d1_y1 >= 0 ) {
					w_screen_coords[ sa0i++ ] = (short) d0_x;
				}
				w_screen_coords[ sa0i++ ] = (short)0x8000;
				if( b_clip_flag1 ) {
					lw_section_side_ptrs[ --ia2i ] = lw_screen_coords_ptr;
				} else {
					lw_section_side_ptrs[ ia1i++ ] = lw_screen_coords_ptr;
				}
				lw_screen_coords_ptr = (short) (sa0i << 1);
				// d1_y1 = 0;
				// d2_y2 = d1_y1;
				return true;
			}
		}
		// d1_y1 = 1;
		// d2_y2 = d1_y1;
		return false;
	}
	
	// clip_sub1
	/**
	 * @param ia1i an offset to {@link #lw_section_side_ptrs}
	 * @param ia2i an offset to {@link #lw_section_side_ptrs}
	 * @param d5_y1 y1, as word
	 * @param d6_x2 x2, as word
	 * @param d7_y2 y2, as word
	 * @return d5, < 0 - cs, >= 0 - cc
	 */
	private static int clipSub1(final int ia1i, final int ia2i, int d5_y1, int d6_x2, int d7_y2) {
		if( (256 != d6_x2) || (d5_y1 > d7_y2) ) {
			// cs11:
			int d0 = 256;
			int d1 = d5_y1;
			int d2 = 128;
			if( ! drsSub( ia1i, ia2i, d0, d1, d2 ) ) {
				b_closest_section_flag = true;
				return d5_y1;
			}
			d5_y1 = 128;
			return d5_y1 | 0x80000000;// ori.b	#1,ccr
		}
		int d0 = 256;
		int d1 = d5_y1;
		int d2 = d7_y2;
		if( ! drsSub( ia1i, ia2i, d0, d1, d2 ) ) {
			b_closest_section_flag = true;
			return d5_y1;
		}
		return d5_y1;// andi.b	#%11110,ccr
		/* */
	}

	// clip_sub2
	/**
	 * @param ia1i an offset to {@link #lw_section_side_ptrs}
	 * @param ia2i an offset to {@link #lw_section_side_ptrs}
	 * @param d5 y1, as word
	 * @param d6 x2, as word
	 * @param d7 y2, as word
	 * @return d5, < 0 - cs
	 */
	private static int clipSub2(final int ia1i, final int ia2i, int d5, final int d6, final int d7) {
		if( (0 != d6) || (d5 < d7) ) {
			// cs21
			final int d0 = 0;
			final int d1 = d5;
			final int d2 = 0;
			if( ! drsSub( ia1i, ia2i, d0, d1, d2 ) ) {
				b_closest_section_flag = true;
				return d5;
			}
			d5 = 0;
			return d5 | 0x80000000;
		}
		final int d0 = 0;
		final int d1 = d5;
		final int d2 = d7;
		if( ! drsSub( ia1i, ia2i, d0, d1, d2 ) ) {
			b_closest_section_flag = true;
			return d5;
		}
		return d5;
	}

	// clip_sub3
	/**
	 *
	 * @param d6_x x
	 * @param d7_y y
	 * @return true - cs
	 */
	private static boolean clipSub3(final int d6_x, final int d7_y) {
		if( (SCREEN_AREA_HEIGHT != d7_y) || (w_clip_word_x1 < d6_x) ) {
			w_clip_word_x1 = 0;
			return true;
		}
		return false;
	}

	// clip_sub4
	/**
	 *
	 * @param d6_x x
	 * @param d7_y y
	 * @return true - cs
	 */
	private static boolean clipSub4(final int d6_x, final int d7_y) {
		if( (0 != d7_y) || (w_clip_word_x1 > d6_x) ) {
			w_clip_word_x1 = SCREEN_AREA_WIDTH;
			return true;
		}
		return false;
	}

	// clip_sub5
	/**
	 * @param d1_x x1 or x2 as word
	 * @param d2_y y1 or y2 as word
	 * @return d3, also: d1_x = 0, d2_y = 0 as words
	 */
	private static int clipSub5(int d1_x, int d2_y) {
		int d3 = 0;
		if( 0 == d2_y ) {
			d3 = d1_x;
		} else {
			d3 += SCREEN_AREA_WIDTH;
			if( d1_x == SCREEN_AREA_WIDTH ) {
				d3 += d2_y;
			} else {
				d3 += SCREEN_AREA_HEIGHT;
				if( d2_y == SCREEN_AREA_HEIGHT ) {
					d3 += SCREEN_AREA_WIDTH - d1_x;
				} else {
					d3 += 384 * PF - d2_y;// TODO what is 384?
				}
			}
		}
		// d1_x = 0;
		// d2_y = 0;
		return d3;
	}

	// clip_sub
	/**
	 * Set {@link #b_clip_flag2} if |p1 - p2| > 384
	 *
	 * @param d0_x1 x1, as word
	 * @param d5_y1 y1, as word
	 * @param d6_x2 x2, as word
	 * @param d7_y2 y2, as word
	 * return: {@link #b_clip_flag2}
	 */
	private static void clipSub(int d0_x1, int d5_y1, int d6_x2, int d7_y2) {
		final int d3 = clipSub5( d6_x2, d7_y2 ) - clipSub5( d0_x1, d5_y1 );
		b_clip_flag2 = (d3 < -384 * PF) || (d3 >= 384 * PF);// TODO what is 384?
	}

	// fp0
	private static int fp0(int ia1i, int ia2i) {
		int d2 = bottom_ptrs_offset;
		// move.l	#bottom.ptrs,a4
		// fp1
		do {
			int d0 = bottom_ptrs[ d2 / 10 ].mD0_i16;
			if( d0 >= 0 ) {
				if( ((ia1i <= ia2i) &&
					((ia1i <= bottom_ptrs[ d2 / 10 ].mD2_i32) && (ia2i >= bottom_ptrs[ d2 / 10 ].mD2_i32)))
					||
					((ia1i > ia2i) && ((ia1i < bottom_ptrs[ d2 / 10 ].mD2_i32) || (ia2i >= bottom_ptrs[ d2 / 10 ].mD2_i32))) )
				{
					// fp5
					int d1 = d2;
					// fp6
					for( d2 -= 10; d2 >= 0; d2 -= 10 ) {
						if( d0 >= bottom_ptrs[ d2 / 10 ].mD0_i16 ) {
							continue;
						}
						if( ia1i <= ia2i ) {
							if( ia1i > bottom_ptrs[ d2 / 10 ].mD2_i32 ) {
								continue;
							}
							if( ia2i < bottom_ptrs[ d2 / 10 ].mD2_i32 ) {
								continue;
							}
						} else {
							if( ia1i >= bottom_ptrs[ d2 / 10 ].mD2_i32 ) {
								if( ia2i < bottom_ptrs[ d2 / 10 ].mD2_i32 ) {
									continue;
								}
							}
						}
						// fp8
						d1 = d2;
						d0 = bottom_ptrs[ d1 / 10 ].mD0_i16;
					}
					// fp9
					bottom_ptrs[ d1 / 10 ].mD0_i16 = (short) 0x8000;
					return d0;
				}
			}
			// fp2
			d2 -= 10;
		} while( d2 >= 0 );
		return 0x8000;// d0
	}

	// fill_polygon
	/**
	 *
	 * @param g Canvas
	 * @param ia1i end, offset to {@link #lw_section_side_ptrs}, pointer to int32
	 * @param ia2i start, offset to {@link #lw_section_side_ptrs}, pointer to int32
	 */
	private static void fillPolygon(final Canvas g, int ia1i, int ia2i) {
		//--- java version TODO do not draw when completely off-screen
		paint.setColor( getRGB( b_fill_polygon_color ) );
		int point_count = 1;
		int index = lw_section_side_ptrs[ ia2i ];
		jXPoints[ 0 ] = w_screen_coords[ (index + 4) / 2 ];
		jYPoints[ 0 ] = w_screen_coords[ (index + 0) / 2 ];
		jXPoints[ 1 ] = w_screen_coords[ (index + 6) / 2 ];
		jYPoints[ 1 ] = w_screen_coords[ (index + 2) / 2 ];
		while( ++ia2i < ia1i ) {
			index = lw_section_side_ptrs[ ia2i ];
			point_count = sortPoints( index, point_count );
		}
		// ++point_count;
		path.moveTo( jXPoints[ 0 ], jYPoints[ 0 ] );
		for( int i = 1; i <= point_count; i++ ) {
			path.lineTo( jXPoints[ i ], jYPoints[ i ] );
		}
		g.drawPath( path, paint );
		path.reset();
		//--- end java version
		/*
		fp_level = 0;// how many calls are done to unload stack data
		fp_end = ia1i;
		fp_start = ia2i;
		ia1i -= ia2i;
		fp_diff = ia1i;
		if( 0 != ia1i ) {
			int d7 = 0;
			int d6 = (start_line_flag & 0x40) ^ 0x40;
			final Bottom[] ia4 = bottom_ptrs;
			int ia4i = 0;
			int ia0i = SECTION_SIDE_PTRS_OFF;// section_side_ptrs;
			int sa5i = section_side_ptrs[ ia0i++ ];
			int ia3i = ia0i;
			// bra fpf;
			// fpf
			int d3 = edge_space[ sa5i / 2 ];// y1
			int d4 = edge_space[ (sa5i + 2) / 2 ];// y2
			if( ia0i == fp_end ) {
				ia0i -= fp_diff;
			}
			// fpa
			// fp10
			while( ia3i != SECTION_SIDE_PTRS_OFF ) {
				ia3i = ia0i;
				sa5i = section_side_ptrs[ ia0i++ ];
				int d0 = edge_space[ sa5i / 2 ];// y1 != y2
				if( d0 != edge_space[ (sa5i + 2) / 2 ] ) {
					// fpb
					if( d0 == d4 ) {
						// fpc
						if( 0 != d6 ) {
							bne fpd;
						}
						d6 = 0;
					} else {
						if( d3 != edge_space[ (sa5i + 2) / 2 ] ) {
							bne fpd;
						}
						d6 = 0x40;
					}
				} else {
					if( SECTION_SIDE_PTRS_OFF == ia3i ) {
						if( 0 != (start_line_flag & 0x40) ) {
							// fpc
							if( 0 != d6 ) {
								// fpd
								d6 ^= 0x40;
								if( 0 == d6 ) {

									ia4[ ia4i + d7 / 10 ].mD6_i32 = ia3i;

									int d2 = ia0i - 8 / 4;
									if( d2 < fp_start ) {
										d2 += fp_diff;
									}
									// fpe
									ia4[ ia4i + d7 / 10 ].mD2_i32 = d2;
									ia4[ ia4i + d7 / 10 ].mD0_i16 = (short) d0;

									d7 += 10;// size of the structure
								}
							}
							d6 = 0;
						}
					}
				}
				// fpf
				d3 = edge_space[ sa5i / 2 ];
				d4 = edge_space[ (sa5i + 2) / 2 ];
				if( ia0i == fp_end ) {
					ia0i -= fp_diff;
				}
				// fp10
			}// while( ia3i != SECTION_SIDE_PTRS_OFF );

			d7 -= 10;
			if( d7 >= 0 ) {
				if( 0 != d7 ) {
					bottom_ptrs_offset = (short) d7;
					int d2 = bottom_ptrs_offset;
					int d0 = ia4[ ia4i + d2 / 10 ].mD0_i16;
					int d1 = d2;
					// fp11
					for( d2 -= 10; d2 >= 0; d2 -= 10 ) {
						if( d0 < ia4[ ia4i + d2 / 10 ].mD0_i16 ) {
							d0 = ia4[ ia4i + d2 / 10 ].mD0_i16;
							d1 = d2;
						}
					}
					ia4[ ia4i + d1 / 10 ].mD0_i16 = (short)0x8000;
					ia4[ ia4i + d1 / 10 ].mD6_i32 = ia1i;
					ia4[ ia4i + d1 / 10 ].mD2_i32 = ia2i;
					d0 = fp0( ia1i, ia2i );
					fp_y_flag = (short)d0;
					if( d0 >= 0 ) {
						ia4[ ia4i + d1 / 10 ].mD6_i32 = fp_ptr1;
						ia4[ ia4i + d1 / 10 ].mD2_i32 = fp_ptr2;
					}
				} else {
					fp_y_flag = (short)0x8000;
					ia4[ ia4i ].mD6_i32 = ia1i;
					ia4[ ia4i ].mD2_i32 = ia2i;
				}
				// fp15
				int sa0i = section_side_ptrs[ ia1i ];
				int sa3i = section_side_ptrs[ ia2i ];

				int d0 = fp_colour;
				d6 = (d0 << 4) + 2;
				word_col = d6;

				d6 = d7 = 0;
				lsr.b	#1, d0
				bcc mask1.set
				not.w d6
				mask1.set
				swap d6
				lsr.b	#1, d0
				bcc mask2.set
				not.w d6
				mask2.set
				lsr.b	#1, d0
				bcc mask3.set
				not.w d7
				mask3.set
				swap d7
				lsr.b	#1, d0
				bcc mask4.set
				not.w d7
				mask4.set
				move.l	#start.masks, a5

				move.w( a0 ) +, d1
				move.w( a3 ) +, d0
				cmp.w d1, d0
				bne fp2c

				addq.l	#6, a0
				addq.l	#6, a3
				move.w d1, fp.y
				subq.w	#1, d1
				cmpi.w	#128, d1
				bcc fp2c

				move.l current.scene, a6
				clr.l d0
				move.w d1, d0
				asl.w	#2, d0
				add.w d1, d0
				asl.w	#3, d0
				add.l d0, a6

				fp1a:
				int d0 = fp_y;
				if( d0 == fp_y_flag ) {
					move.w fp.y, -(sp)
					move.l a6, -(sp)
					move.l a3, -(sp)
					move.l a2, -(sp)
					move.l fp.ptr1, -(sp)
					++fp_level;
					move.l fp.ptr2, a2
					move.l( a2 ), a3
					add.l	#8, a3
					jsr fp0
					fp_y_flag = d0;
					if( d0 >= 0 ) {
						move.l 6 (a4, d1.w),fp.ptr1
						move.l 2 (a4, d1.w),fp.ptr2
					}
					// fp1b:
					bra fp1a
				}
				fp1c move.w( a0 ) +, d4
				bpl fp1f

				fp1d addq.l	#4, a1
				cmp.l fp.end, a1
				blt fp1e

				sub.l fp.diff, a1

				fp1e cmp.l a2, a1
				beq fp29

				move.l( a1 ), a0
				move.w( a0 ) +, d4
				cmp.w fp.y, d4
				bne fp2c

				addq.l	#6, a0
				move.w( a0 ) +, d4
				bmi fp1d

				fp1f move.w( a3 ) +, d5
				bpl fp22

				fp20 cmp.l fp.start, a2
				bne fp21

				add.l fp.diff, a2

				fp21 move.l - (a2), a3
				cmp.l a1, a2
				beq fp29

				move.w( a3 ) +, d5
				cmp.w fp.y, d5
				bne fp2c

				addq.l	#6, a3
				move.w( a3 ) +, d5
				bmi fp20

				fp22 cmp.w d4, d5
				bgt fp23
				beq fp28

				tst.b daft.flag
				bpl fp28
				bra fp28

				fp23 move.w d4, d1
				andi.w	#$f0, d1
				lsr.w	#3, d1
				lea( a6, d1.w ), a4
				move.w d4, d3
				move.w d5, d1
				lsr.w	#4, d3
				lsr.w	#4, d1
				sub.w d3, d1
				bne fp24

				andi.w	#$f, d4
				asl.w	#2, d4
				move.w( a5, d4.w ), d4

				andi.w	#$f, d5
				asl.w	#2, d5
				move.w 64 (a5, d5.w),d5
				and.w d5, d4
				jsr fill.word
				bra fp28

				fp24 subq.b	#1, d1
				andi.w	#$f, d4
				beq fp25

				asl.w	#2, d4
				move.w( a5, d4.w ), d4
				jsr fill.word
				subq.w	#1, d1
				bmi fp27

				fp25 move.l d6, d2
				move.l d7, d3
				swap d2
				swap d3

				fp26 move.w d2, (a4) +
				move.w d6, 7998 (a4)
				move.w d3, 15998 (a4)
				move.w d7, 23998 (a4)
				dbra d1, fp26

				fp27 andi.w	#$f, d5
				beq fp28

				asl.w	#2, d5
				move.w 64 (a5, d5.w),d4
				fillWord( a4, a4i, sd4 );

				fp28 subq.w	#1, fp.y
				sub.l	#40, a6
				cmp.l current.scene, a6
				bge fp1a

				// fp29:
				if( 0 != fp_level ) {
					move.l( sp ) +, a1
					move.l( sp ) +, a2
					move.l( sp ) +, a3
					move.l( sp ) +, a6
					move.w( sp ) +, fp.y
					--fp_level;
					move.l( a1 ), a0
					add.l	#8, a0
					jsr fp0
					move.w d0, fp.y.flag
					bmi fp2a

					move.l 6 (a4, d1.w),fp.ptr1
					move.l 2 (a4, d1.w),fp.ptr2
					fp2a:
					bra fp1a
				}
			}
			// fp2b:
			// d1 = 0;
			// d2 = 0;
			return;
		}
		fp2c:
		while( 0 != fp_level ) {
			move.l( sp ) +, a1
			move.l( sp ) +, a2
			move.l( sp ) +, a3
			move.l( sp ) +, a6
			move.w( sp ) +, fp.y
			--fp_level;
			move.l( a1 ), a0
			add.l	#8, a0
		}
		d1 = 0;
		d2 = 0;
		/* */
	}

	private static short w_clip_word_x1 = 0;

	// clip_closest_section
	/**
	 * @param ia1i an offset to {@link #lw_section_side_ptrs}
	 * @param ia2i an offset to {@link #lw_section_side_ptrs}
	 * @param d0_x1 x1, as word
	 * @param d5_y1 y1, as word
	 * @param d6_x2 x2, as word
	 * @param d7_y2 y2, as word
	 */
	private static void clipClosestSection(final int ia1i, final int ia2i, int d0_x1, int d5_y1, int d6_x2, int d7_y2) {
		//
		b_closest_section_flag = false;
		b_clip_flag1 = false;
		/*
		clipSub( d0_x1, d5_y1, d6_x2, d7_y2 );
		if( b_clip_flag2 ) {
			boolean j_is_cc6 = true;
			if( 0 == b_drawing_surface_flags ) {// road surface
				j_is_cc6 = (0 != b_players_position_flags);
			} else {
				if( 0 != b_players_position_flags ) {
					if( 0 == (b_players_position_flags ^ b_drawing_surface_flags) ) {
						int d3 = w_players_x_offset_from_road_centre;
						if( d3 < 0 ) {
							d3 = -d3;
						}
						if( (d3 >= 197) && (d3 < 576) ) {
							if( b_drawing_surface_flags >= 0 ) {
								if( 0 != d7_y2 ) {
									j_is_cc6 = (0 != d6_x2) || (d7_y2 >= 64);
								}
								j_is_cc6 |= (d5_y1 <= d7_y2);
							} else {
								// ccs3:
								if( 0 != d5_y1 ) {
									j_is_cc6 = (256 != d0_x1) || (d5_y1 >= 64);
								}
								j_is_cc6 |= (d7_y2 <= d5_y1);
							}
						}
					}
				}
			}
			// ccs6:
			if( j_is_cc6 && b_clip_flag2 ) {
				int temp = d0_x1;
				d0_x1 = d6_x2;
				d6_x2 = temp;
				temp = d5_y1;
				d5_y1 = d7_y2;
				d7_y2 = temp;
				b_clip_flag1 = true;
				// x1 <> x2, y1 <> y2
			}
		}
		// ccs7:
		// ccs8:
		w_clip_word_x1 = (short)d0_x1;
		b_clip_flag3 = 0;
		boolean is_loop = false;// java
		if( d0_x1 == 256 ) {// x1
			// ccsa:
			d5_y1 = clipSub1( ia1i, ia2i, d5_y1, d6_x2, d7_y2 );
			if( d5_y1 >= 0 ) {
				return;// bcc end.clip2
			}
			d5_y1 &= 0x7FFFFFFF;// java, remove flag

			// ccsb:
			if( ! clipSub3( d6_x2, d7_y2 ) ) {
				return;// bcc end.clip2
			}
			// ccsc:
			d5_y1 = clipSub2( ia1i, ia2i, d5_y1, d6_x2, d7_y2 );
			if( d5_y1 >= 0 ) {
				return;// bcc end.clip2
			}
			d5_y1 &= 0x7FFFFFFF;// java, remove flag
			is_loop = true;
		} else if( d0_x1 == 0 ) {// x1
			// ccsc:
			d5_y1 = clipSub2( ia1i, ia2i, d5_y1, d6_x2, d7_y2 );
			if( d5_y1 >= 0 ) {
				return;// bcc end.clip2
			}
			d5_y1 &= 0x7FFFFFFF;// java, remove flag
			is_loop = true;
		} else if( d5_y1 == 128 ) {// y1
			// ccsb:
			if( ! clipSub3( d6_x2, d7_y2 ) ) {
				return;// bcc end.clip2
			}
			// ccsc:
			d5_y1 = clipSub2( ia1i, ia2i, d5_y1, d6_x2, d7_y2 );
			if( d5_y1 >= 0 ) {
				return;// bcc end.clip2
			}
			d5_y1 &= 0x7FFFFFFF;// java, remove flag
			is_loop = true;
		} else if( d5_y1 == 0 ) {// y1
			is_loop = true;
		}
		if( is_loop ) {
			// ccsd:
			if( ! clipSub4( d6_x2, d7_y2 ) ) {
				return;// bcc end.clip2
			}
			// ccs9:
			while( b_clip_flag3 < 2 ) {
				b_clip_flag3++;
				// ccsa:
				d5_y1 = clipSub1( ia1i, ia2i, d5_y1, d6_x2, d7_y2 );
				if( d5_y1 >= 0 ) {
					return;// bcc end.clip2
				}
				d5_y1 &= 0x7FFFFFFF;// java, remove flag

				// ccsb:
				if( !clipSub3( d6_x2, d7_y2 ) ) {
					return;// bcc end.clip2
				}
				// ccsc:
				d5_y1 = clipSub2( ia1i, ia2i, d5_y1, d6_x2, d7_y2 );
				if( d5_y1 >= 0 ) {
					return;// bcc end.clip2
				}
				d5_y1 &= 0x7FFFFFFF;// java, remove flag
				// ccsd:
				if( !clipSub4( d6_x2, d7_y2 ) ) {
					return;// bcc end.clip2
				}
			}
		}
		// end.clip:
		b_closest_section_flag = true;
		// end.clip2:
		// rts
		/* */
	}

	// java: common part of draw_near_left_sides and draw_near_right_sides
	/**
	 *
	 * @param g Canvas
	 * @param scOff1 offset to {@link #lw_screen_data}
	 * @param scOff2 offset to {@link #lw_screen_data}
	 * @param scOff3 offset to {@link #lw_screen_data}
	 */
	private static void drawNearSides(final Canvas g, final int scOff1, final int scOff2, final int scOff3) {
		int d0, d4;
		int offset_x4 = w_road_section_offset;
		// dnls1:
		do {
			w_copy_road_section_offset = (short) offset_x4;
			d4 = d0 = lw_screen_data[ (offset_x4 + scOff1) / 4 ];
			d0 &= 0xFFFFFF;
			if( SECTION_DATA_NO_DATA != d0 ) {
				break;
			}
			// dnls2:
			do {
				offset_x4 -= 8 * 4;
				if( offset_x4 <= w_next_road_section_offset ) {
					return;
				}
			} while( SECTION_DATA_NO_DATA ==
					(lw_screen_data[ (offset_x4 + scOff1) / 4 ] & 0xFFFFFF) );
		} while( true );

		// dnls3:
		b_fill_polygon_color = SIDES_COLOR_A;
		if( 0 != (lw_screen_data[ (offset_x4 + 28) / 4 ] & 0x01_00_0000) ) {
			b_fill_polygon_color = b_sides_color;
		}
		// dnls4:
		if( b_use_lines_color ) {
			b_fill_polygon_color = b_side_lines_color;
		}

		int ia1i = SECTION_SIDE_PTRS_OFF;
		int ia2i = SECTION_SIDE_PTRS_OFF;
		lw_section_side_ptrs[ ia1i++ ] = d0;
		final byte flag = (byte)(lw_screen_data[ (offset_x4 + scOff1) / 4 ] >>> 24);
		b_start_line_flag = flag;
		b_left_side_flag = flag;
		b_right_side_flag = (byte) (flag ^ 0x40);
		if( (8 * 4) != w_copy_road_section_offset ) {
			// dnls6:
			do {
				d0 = lw_screen_data[ (offset_x4 + scOff2) / 4 ];
				d4 &= d0;
				d0 &= 0xffffff;
				if( SECTION_DATA_NO_DATA != d0 ) {
					b_right_side_flag = (byte) (lw_screen_data[ (offset_x4 + scOff2) / 4 ] >>> 24);
					lw_section_side_ptrs[ ia1i++ ] = d0;

					d0 = lw_screen_data[ (offset_x4 + scOff3) / 4 ];
					d4 &= d0;
					d0 &= 0xffffff;
					if( SECTION_DATA_NO_DATA != d0 ) {
						b_left_side_flag = (byte) (lw_screen_data[ (offset_x4 + 8) / 4 ] >>> 24);
						lw_section_side_ptrs[ --ia2i ] = d0;
						offset_x4 -= 8 * 4;

						if( offset_x4 != w_next_road_section_offset ) {
							if( offset_x4 != 32 ) {
								continue;// bne dnls6;
							}
							break;// bra dnlsa;
						}
					} else {
						// dnls7:
						--ia1i;
						// dnls8:
						if( offset_x4 == w_road_section_offset ) {
							return;
						}
					}
				} else {
					// dnls8:
					if( offset_x4 == w_road_section_offset ) {
						return;
					}
				}
				// dnls9:
				d0 = lw_screen_data[ (offset_x4 + scOff1) / 4 ];
				d4 &= d0;
				d0 &= 0xffffff;
				if( SECTION_DATA_NO_DATA == d0 ) {
					return;
				}
				b_right_side_flag = (byte) (lw_screen_data[ (offset_x4 + scOff1) / 4 ] >>> 24);
				lw_section_side_ptrs[ ia1i++ ] = d0;
				// bra dnlsd;
				// dnlsd:
				if( 0 == (d4 & 0xf000000) ) {
					fillPolygon( g, ia1i, ia2i );
				}
				return;
			} while( true );
		}
		// dnlsa:
		d0 = lw_screen_data[ (offset_x4 + scOff2) / 4 ];
		d4 &= d0;
		d0 &= 0xffffff;
		if( SECTION_DATA_NO_DATA != d0 ) {
			b_right_side_flag = (byte) (lw_screen_data[ (offset_x4 + scOff2) / 4 ] >>> 24);
			lw_section_side_ptrs[ ia1i++ ] = d0;
		}
		// dnlsb:
		int sa5i = lw_section_side_ptrs[ ia1i - 4 / 4 ];
		final short[] sa5 = w_screen_coords;
		// extracted in place: getCoordPair( sa5, sa5i, d0 );
		int x, y;
		if( 0 != (b_right_side_flag & 0x40) ) {
			y = sa5[ sa5i / 2 ];
			x = sa5[ (sa5i + 4) / 2 ];
		} else {
			y = sa5[ (sa5i + 2) / 2 ];
			x = sa5[ (sa5i + 6) / 2 ];
		}
		int x1 = x;
		int y1 = y;

		d0 = lw_screen_data[ (offset_x4 + scOff3) / 4 ];
		d4 &= d0;
		d0 &= 0xffffff;
		if( SECTION_DATA_NO_DATA != d0 ) {
			b_left_side_flag = (byte) (lw_screen_data[ (offset_x4 + scOff3) / 4 ] >>> 24);
			lw_section_side_ptrs[ --ia2i ] = d0;
		}
		// dnlsc:
		sa5i = lw_section_side_ptrs[ ia2i ];
		// getCoordPair();
		if( 0 != (b_left_side_flag & 0x40) ) {
			y = sa5[ sa5i / 2 ];
			x = sa5[ (sa5i + 4) / 2 ];
		} else {
			y = sa5[ (sa5i + 2) / 2 ];
			x = sa5[ (sa5i + 6) / 2 ];
		}

		clipClosestSection( ia1i, ia2i, x1, y1, x, y );

		if( b_closest_section_flag ) {
			return;
		}
		// dnlsd:
		if( 0 == (d4 & 0xf000000) ) {
			fillPolygon( g, ia1i, ia2i );
		}
	}

	// draw.near.left.sides
	private static void drawNearLeftSides(final Canvas g) {
		b_drawing_surface_flags = (byte)0x80;
		drawNearSides( g, 16, 0, 8 );
	}

	// draw.near.right.sides
	private static void drawNearRightSides(final Canvas g) {
		b_drawing_surface_flags = 2;
		drawNearSides( g, 20, 12, 4 );
	}

	// java: common part of draw_left_side_lines and draw_right_side_lines
	/**
	 *
	 * @param g Canvas
	 * @param sc_off1 offset to {@link #lw_screen_data}
	 */
	private static void drawSideLines(final Canvas g, final int sc_off1) {
		int d3 = w_road_section_offset;
		while( d3 != w_next_road_section_offset ) {
			int d0 = (lw_screen_data[ (d3 + 29) / 4 ] & 0x00_FF_0000) >> 16;// byte
			if( (0 == (d0 & 0x80)) && (3 != d0) ) {
				paint.setColor( getRGB( b_side_lines_color ) );
				d0 = lw_screen_data[ (d3 + sc_off1) / 4 ] & 0xffffff;
				if( SECTION_DATA_NO_DATA != d0 ) {
					plotLine( g, d0 );
				}
			}
			d3 -= 32;
		}
	}

	// draw_left_side_lines
	private static void drawLeftSideLines(final Canvas g) {
		drawSideLines( g, 16 );
	}

	// draw_right_side_lines
	private static void drawRightSideLines(final Canvas g) {
		drawSideLines( g, 20 );
	}

	// java: common part of draw_left_road_lines and draw_right_side_lines
	/**
	 *
	 * @param g Canvas
	 * @param sc_off1 offset to {@link #lw_screen_data}
	 */
	private static void drawRoadLines(final Canvas g, final int sc_off1) {
		int d3 = w_road_section_offset;
		while( d3 != w_next_road_section_offset ) {
			int d0 = (lw_screen_data[ (d3 + 29) / 4 ] & 0x00_FF_0000) >> 16;// byte
			if( 0 == (d0 & 0x80) ) {
				paint.setColor( getRGB( d0 ) );
				d0 = lw_screen_data[ (d3 + sc_off1) / 4 ] & 0xffffff;
				if( SECTION_DATA_NO_DATA != d0 ) {
					plotLine( g, d0 );
				}
			}
			d3 -= 32;
		}
	}

	// draw_left_road_lines
	private static void drawLeftRoadLines(final Canvas g) {
		drawRoadLines( g, 0 );
	}

	// draw_right_side_lines
	private static void drawRightRoadLines(final Canvas g) {
		drawRoadLines( g, 4 );
	}

	private static void drawOpponentTestDot1(final Canvas g) {
		if( w_next_road_section_offset <= w_opponents_offset ) {
			drawOpponent( g );// draw.opponent
			w_opponents_offset = -1;
		}
	}

	// draw_opponent_test1
	private static void drawOpponentTest1(final Canvas g) {
		if( b_opponent_draw_flag > 0 ) {
			drawOpponentTestDot1( g );
		}
	}

	// draw_opponent_test2
	private static void drawOpponentTest2(final Canvas g) {
		if( b_opponent_draw_flag < 0 ) {
			drawOpponentTestDot1( g );
		}
	}

	// draw_opponent_test3
	private static void drawOpponentTest3(final Canvas g) {
		if( 0 == b_opponent_draw_flag ) {
			drawOpponentTestDot1( g );
		}
	}

	// draw_near_road
	private static void drawNearRoad(final Canvas g) {
		w_road_section_offset -= 8 * 4;
		if( w_road_section_offset < 16 * 4 ) {
			return;
		}
		do {
			int d3 = w_road_section_offset;
			do {
				d3 -= 8 * 4;
				if( 0 == d3 ) {
					break;
				}
			} while( 0 != (lw_screen_data[ (d3 + 30) / 4 ] & 0x0000_8000) );// (+30) < 0
			w_next_road_section_offset = (short) d3;
			if( d3 < w_dnr_value ) {
				return;
			}
			if( 0 == b_at_side_byte ) {
				drawNearLeftSides( g );
				drawLeftSideLines( g );
				drawNearRightSides( g );
				drawRightSideLines( g );
				drawOpponentTest1( g );
				drawOpponentTest2( g );
				drawRoadSurface( g );
				drawLeftRoadLines( g );
				drawRightRoadLines( g );
				drawOpponentTest3( g );
			} else {
				// dnr_left_side:
				if( w_players_x_offset_from_road_centre < 0 ) {
					drawOpponentTest1( g );
					drawNearRightSides( g );
					drawRightSideLines( g );
					drawRoadSurface( g );
					drawRightRoadLines( g );
					drawOpponentTest3( g );
					drawNearLeftSides( g );
					drawLeftSideLines( g );
					d3 = w_road_section_offset;
					if( (d3 != w_copy_road_section_offset) && (0 == b_car_on_chains_countdown) ) {
						final short sp1 = w_road_section_offset;
						w_road_section_offset = w_copy_road_section_offset;
						drawLeftRoadLines( g );
						w_road_section_offset = sp1;
					} else {
						drawLeftRoadLines( g );
					}
					drawOpponentTest2( g );
				} else {
					// dnr_right_side:
					drawOpponentTest2( g );
					drawNearLeftSides( g );
					drawLeftSideLines( g );
					drawRoadSurface( g );
					drawLeftRoadLines( g );
					drawOpponentTest3( g );
					drawNearRightSides( g );
					drawRightSideLines( g );
					d3 = w_road_section_offset;
					if( (d3 != w_copy_road_section_offset) && (0 == b_car_on_chains_countdown) ) {
						final short sp1 = w_road_section_offset;
						w_road_section_offset = w_copy_road_section_offset;
						drawRightRoadLines( g );
						w_road_section_offset = sp1;
					} else {
						drawRightRoadLines( g );
					}
					drawOpponentTest1( g );
				}
			}
			w_road_section_offset = w_next_road_section_offset;
		} while( w_road_section_offset != 0 );
	}

	// initialise_sparks_table
	private static void initialiseSparksTable() {
		Arrays.fill( w_sparks_y, (short) (SPARK_Y_DEFAULT / PF) );
	}

	// draw_dust_clouds
	private static void drawDustClouds(final Canvas g) {
		// effect = 6;// off road
		int d0 = w_pos_players_z_speed >> 8;
		if( d0 > 16 ) {
			d0 = 16;// set to maximum
		}
		b_ferocity_of_sparks_or_clouds = (byte) d0;

		int d1 = 15;
		if( 0 != b_machine ) {
			d1 = 3;
		}
		d0 = 450 + (randomizeLong() & 0x1c);
		SoundManager.effects[ SoundManager.SOUND_OFF_ROAD ].mPeriod = (short) d0;// off-road period
		sparksOrClouds( g, d1, SoundManager.SOUND_OFF_ROAD );// off-road
	}

	// plot_pixel
	/**
	 * @param g    Canvas
	 * @param ba0i offset in bytes to byte screen buffer. 16 bit aligned.
	 *             320x200 is (40 bytes x 8 bits) x 200
	 * @param bd4  pixel number, 0-15. Low first, so bits on the screen are:
	 *             0-7 8-15, 0-7 8-15 ...
	 */
	private static void plotPixel(final Canvas g, final int ba0i, int bd4) {
        /*
        // High first, so bits on the screen are: 15-8 7-0, 15-8 7-0 ...
        int bd0 = (bd4 & 0x0F) ^ 0x0F;
        if( bd0 >= 8 ) {
            bd0 &= 7;
            ba0[ ba0i + 0] |= 1 << bd0; or &= ~(1 << bd0);
            ba0[ ba0i + 8000 ] |= 1 << bd0; or &= ~(1 << bd0);
            ba0[ ba0i + 16000 ] |= 1 << bd0; or &= ~(1 << bd0);
            ba0[ ba0i + 24000 ] |= 1 << bd0; or &= ~(1 << bd0);
            return;
        }
        ba0[ ba0i + 1 ] |= 1 << bd0; or &= ~(1 << bd0);
        ba0[ ba0i + 8001 ] |= 1 << bd0; or &= ~(1 << bd0);
        ba0[ ba0i + 16001 ] |= 1 << bd0; or &= ~(1 << bd0);
        ba0[ ba0i + 24001 ] |= 1 << bd0; or &= ~(1 << bd0);
        */
		int y = (ba0i / (SCREEN_WIDTH / 8 / PF));
		final int x = (((ba0i - y * (SCREEN_WIDTH / 8 / PF)) << 3) | (bd4 & 0x0f)) * PF;
		y *= PF;
		g.drawRect( x, y, x + 1 * PF, y + 1 * PF, paint );
	}

	// plot_line
	/**
	 * Working area is for x: 0...255, for y: 0...127
	 *
	 * @param g Canvas
	 * @param sa3i offset in bytes to {@link #w_screen_coords}
	 */
	private static void plotLine(final Canvas g, final int sa3i) {
		// paint.setStrokeWidth( LINE_WIDTH );
		g.drawLine( w_screen_coords[ (sa3i + 4) / 2 ], w_screen_coords[ (sa3i) / 2 ],
				w_screen_coords[ (sa3i + 6) / 2 ], w_screen_coords[ (sa3i + 2) / 2 ], paint );
		/*
		// 0:y1, 2:y2, 4:x1, 6:x2,
		if( sa3[ (sa3i + 2) / 2 ] >= 128 ) {// y1 >= 128
			return;
		}
		int sa2i = sa3i;
		sa3i += 8;// next
		int d5 = sa3[ sa2i / 2 ] - 1;// y1 - 1
		int d4 = sa3[ (sa2i + 4) / 2 ];// x2
		if( 0 != d4 ) {
			if( d4 < 256 ) {// x2
				int d6 = sa3[ sa3i ];// y1 next
				sa3i += 2;// to y2 next
				if( d6 < 0 ) {
					d6 = sa3[ (sa2i + 6) / 2 ];// x2
				}
			} else {
				// pl1:
				if( d4 != 256 ) {// x2
					return;
				}
				--d4;// x2 = 255
				++d5;// y1 java: to avoid bra pl3;
				int d6;
				do {
					--d5;// y1-1
					// pl3:
					d6 = sa3[ (sa3i) / 2 ];// y1 next
					sa3i += 2;// y2 next
				} while( d6 == 256 );// y1 next
				if( d6 < 0 ) {
					d6 = sa3[ (sa2i + 6) / 2 ];// x2
					if( d6 == 256 ) {
						return;
					}
				}
			}
		} else {
			++d5;// java to avoid pl5:
			int d6;
			do {
				--d5;
				d6 = sa3[ sa3i / 2 ];
				sa3i += 2;
			} while( 0 == d6 );
			if( d6 < 0 ) {
				d6 = sa3[ (sa2i + 6) / 2 ];
				if( 0 == d6 ) {
					return;
				}
			}
		}
		// pl6:// d4 = x2, d5 = y1, d6 = y1 next
		if( d5 >= 128 ) {// y1
			return;
		}
		if( d4 >= 256 ) {// x2
			return;
		}
		sa3i -= 2;// to y1 next
		int d0 = sa3[ (sa2i + 4) / 2 ] - sa3[ (sa2i + 6) / 2 ];// x1 - x2
		if( d0 >= 0 ) {
			int ba0i = 0;// move.l current.scene, a0
			ba0i += (d4 >>> 3) & 0xFE;// d4 is x, to byte offset
			ba0i += ((d5 << 2) + d5) << 3;// d5 is y, * 40
			int d2 = 0;
			int d6 = sa3[ sa3i / 2 ];// y1 next
			sa3i += 2;// to y2 next
			if( d6 < 0 ) {
				if( d2 < 0 ) {
					return;
				}
				d2 = 0x80;
				sa3i -= 2;// to y1 next
				d6 = sa3[ (sa2i + 6) / 2 ];// x2
			}
			pl7:
			if( d6 == 0 ) {
				d6 = 0xffff;
			}
			pl8:
			do {
				plotPixel( g, ba0i, d4 );
				if( d6 == d4 ) {
					d6 = sa3[ sa3i ];
					sa3i += 2;
					if( d6 < 0 ) {
						if( d2 < 0 ) {
							return;
						}
						d2 = 0x80;
						sa3i -= 2;
						d6 = sa3[ (sa2i + 6) / 2 ];
					}
					pl9:
					if( d6 == 0 ) {
						d6 = 0xffff;
					}
					pla:
					--d5;
					if( d5 < 0 ) {
						return;
					}
					ba0i -= 40;// y - 1
					if( d6 == d4 ) {
						continue;// beq pl8;
					}
				}
				plb:
				d0 = d4 & 0x0F;
				--d4;
				if( 0 == d0 ) {
					if( d4 < 0 ) {
						return;
					}
					ba0i -= 2;
				}
				// plc:
			} while( true );
		}
		ple:
		int ba0i = 0;// move.l current.scene, a0
		ba0i += (d4 >>> 3) & 0xFE;// d4 is x, to byte offset
		ba0i += ((d5 << 2) + d5) << 3;// d5 is y, * 40
		d2 = 0;
		d6 = sa3[ sa3i / 2 ];
		sa3i += 2;
		if( d6 < 0 ) {
			if( d2 < 0 ) {
				return;
			}
			d2 = 0x80;
			sa3i -= 2;
			d6 = sa3[ (sa3i + 6) / 2 ];
		}
		plf:
		if( d6 == 0 ) {
			d6 = 0xffff;
		}
		// pl10
		plotPixel( g, ba0i, d4 );
		if( d6 == d4 ) {
			d6 = sa3[ (sa3i) / 2 ];
			sa3i += 2;
			if( d6 < 0 ) {
				if( d2 < 0 ) {
					bmi pl15;
				}
				d2 = 0x80;
				sa3i -= 2;
				d6 = sa3[ (sa2i + 6) / 2 ];
			}
			if( d6 == 0 ) {
				d6 = 0xffff;
			}
			pl12:
			--d5;
			if( d5 < 0 ) {
				return;
			}
			ba0i -= 40;// y - 1
			if( d6 == d4 ) {
				beq pl10;
			}
		}
		pl13 addq.w	#1, d4
		move.w d4, d0
		andi.w	#$f, d0
		bne pl14

		cmp.w	#256, d4
		bge pl16

		add.l	#2, a0
		pl14 bra pl10
		pl15 rts
		pl16 rts
		 */
	}

	// FIXME draw_spark_sub2_sub
	private static void drawSparkSub2Sub(int sa0i, int sa1i, int sd3) {
        /*
		sa1i >>= 1;// java
        int d0 = (sa1[ sa1i++ ] << 16) | 0xFFFF;
        d0 = getROR32( d0, sd3 );
        int d6 = (d0 & 0xFFFF0000) | (d0 >>> 16);// hd0 hd0
        int sp_d7 = (d0 << 16) | (d0 & 0xFFFF);// ld0 ld0

        d0 = sa1[ sa1i++ ];
        int d7 = d0;
        d0 &= 0xFFFF0000;
        d0 >>>= sd3;
        d7 <<= 16;
        d7 >>>= sd3;

        int d4 = (d0 & 0xFFFF0000) | (d7 >>> 16);
        int sp_d7_2 = (d0 << 16) | (d7 & 0xFFFF);

        d0 = sa1[ sa1i++ ];
        d7 = d0;
        d0 &= 0xFFFF0000;
        d0 >>>= sd3;
        d7 <<= 16;
        d7 >>>= sd3;
        int d5 = (d0 & 0xFFFF0000) | (d7 >>> 16);
        d7 = (d0 << 16) | (d7 & 0xFFFF);
        if( w_6a168 < 16 ) {
            d0 = (sa0[ sa0i / 2 ] << 16) | (sa0[ (8000 + sa0i) / 2 ] & 0xFFFF);
            d0 &= d6;
            d0 |= d4;
            sa0[ (8000 + sa0i) / 2 ] = (short)d0;
            d0 = (d0 << 16) | (d0 >>> 16);
            sa0[ (sa0i) / 2 ] = (short)(d0);
            sa0i += 2;

            d0 = (sa0[ (15998 + sa0i) / 2 ] << 16) | (sa0[ (23998 + sa0i) / 2 ] & 0xFFFF);
            d0 &= d6;
            d0 |= d5;
            sa0[ (23998 + sa0i) / 2 ] = (short)(d0);
            sa0[ (15998 + sa0i) / 2 ] = (short)(d0 >>> 16);
        } else {
            sa0i += 2;
        }

        d4 = sp_d7_2;
        d6 = sp_d7;

        ++w_6a168;
        if( w_6a168 < 16 ) {
            d0 = (d0 & 0xFFFF0000) | (sa0[ (sa0i) / 2 ] & 0xFFFF);
            sa0[ (sa0i) / 2 ] = (short)d0;
            d0 = (d0 << 16) | (d0 >>> 16);
            sa0[ (8000 + sa0i) / 2 ] = (short)(d0);
            d0 &= d6;
            d0 |= d4;
            sa0[ (8000 + sa0i) / 2 ] = (short)(d0);
            sa0[ (16000 + sa0i) / 2 ] = sa0[ (sa0i) / 2 ] = (short)(d0 >>> 16);
            id0 = ((sa0[ 24000 / 2 ] & 0xFFFF) & d6) | d7;
            sa0[ 24000 / 2 ] = (short) d0;
            sa0[ 16000 / 2 ] = (short) (d0 >>> 16);
        }
        /* */
		--w_6a168;
	}

	// TODO java adjust clouds
	private static final Bitmap[] clouds = new Bitmap[ 8 ];
	private static final int CLOUD_OFFSET_X = 0 * PF;
	private static final int CLOUD_OFFSET_Y = -16 * PF;

	private static final short[] w_cloud_x_offsets = {
			32 * PF, 32 * PF, 32 * PF, 40 * PF, 20 * PF, 32 * PF, 32 * PF, 32 * PF
		};

	private static final byte[] b_cloud_types = {
			3, 6, 7, 2, 1, 5, 0, 4, 0, 5, 1, 2, 7, 6, 2, 7
		};

	// draw_cloud
	/**
	 *
	 * @param g Canvas
	 * @param bd1 index x2 of the cloud position
	 *           in {@link #w_sparks_x} and {@link #w_sparks_y}
	 */
	private static void drawCloud(final Canvas g, int bd1) {
		final int bd2_cloud = b_cloud_types[ ((bd1 >>> 1) + b_main_loop_count) & 0x0F ];
		final int sd4_x = SCREEN_AREA_LEFT_X + CLOUD_OFFSET_X + w_sparks_x[ bd1 / 2 ] * PF - w_cloud_x_offsets[ bd2_cloud ];
		final int sd5_y = SCREEN_AREA_TOP_Y + CLOUD_OFFSET_Y + w_sparks_y[ bd1 / 2 ] * PF;
		// java version
		g.drawBitmap( clouds[ bd2_cloud ], sd4_x, sd5_y, paint );
		/*
		int sd0_cloud_id = 29 + (bd2_cloud_x2 >>> 1);
		// draw_spark_sub2
		// drawSparkSub2( (short) bd0, sd4_x, sd5_y );
		sd0_cloud_id = (sd0_cloud_id & 0xFF) << 2;
		int sa1i = lw_graphic_pointers[ sd0_cloud_id / 4 ];
		sd0_cloud_id <<= 2;
		final int[] ia2 = graphic_info;
		int sa2i = ia2[ (sd0_cloud_id + 4) / 4 ];
		// final short[] sa2 = screen;

		// final short[] a0 = screen2;
		int sa0i = 0;// offset to a0 in the screen area
		// ext.l d0, ext.l d5
		sa0i += (sd4_x >>> 3) & 0xFE;
		sa0i += ((sd5_y << 2) + sd5_y) << 3;// y, 5x8 = 40

		short sa6 = sa2[ sa2i ];
		int sd2 = sa2[ (sa2i + 2) / 2 ];
		int sd3 = sd4_x & 0x0F;
		final short w_6a166 = (short) ((sd4_x >> 4) - 2);
		w_6a168 = w_6a166;
		short w_cloud_top_y = (short) (sd5_y - 16);

		do {
			sa2 = sa0;
			sa2i = sa0i;
			int sd1 = sa6;
			if( (w_cloud_top_y >= 0) && (w_cloud_top_y < SCREEN_AREA_HEIGHT) ) {
				do {
					drawSparkSub2Sub( sa0i, sa1i, sd3 );
					++w_6a168;
				} while( --sd1 >= 0 );
			}
			w_6a168 = w_6a166;
			sa0 = ia2[ (sa2i + 40) / 4 ];// address
			++w_cloud_top_y;
		} while( --sd2 >= 0 );
		*/
	}

	private static short w_6a168 = 0;

	// draw_spark
	/**
	 * {@link #w_sparks_x} and {@link #w_sparks_y} as w_sparks_x + 64 in sa4
	 * {@link #b_off_map_status} defines spark or cloud.
	 *
	 * @param g Canvas
	 * @param bd1 x/y offset in d1 as byte
	 * @return false if !=0 or true if ==0
	 */
	private static boolean drawSparkOrCloud(final Canvas g, int bd1) {

		int sd5 = w_sparks_y[ (bd1) / 2 ];

		if( (sd5 >= 0) && (sd5 < SCREEN_AREA_HEIGHT / PF) ) {
			int sd0 = w_sparks_x[ (bd1) / 2 ];// x
			if( (sd0 >= 0) && (sd0 < SCREEN_AREA_WIDTH / PF) ) {
				if( sd5 >= 1 ) {
					if( b_off_map_status < 0 ) {
						drawCloud( g, bd1 );
					} else {
						if( sd0 >= (SCREEN_AREA_WIDTH / PF - 1) ) {// 255-1
							w_sparks_y[ (bd1) / 2 ] = SPARK_Y_OFF_SCREEN / PF;// w_TAB_1c380[ (bd1 + 64) / 2 ] = 210;
							return false;
						}
						sd0 += SCREEN_AREA_LEFT_X / PF;
						sd5 += SCREEN_AREA_TOP_Y / PF;
						int sd4 = sd0;
						// byte[] ba0 = current_scene;
						int ba0i = 0;// offset to ba0
						// d0, d5 ext to int, so now they are ints
						// sd0 < 254, 1 <= sd5 < 128
						ba0i += (sd0 >>> 3) & 0xFE;// x
						ba0i += (((sd5 << 2) + sd5) << 3);// y, sd5 * 40

						// setColor YELLOW
						// fillRect( x - 1, y - 1, 2, 2 )
						// setColor LIGHT YELLOW
						// fillRect( x, y - 1, 2, 2 )
						paint.setColor( getRGB( SPARK_COLOR ) );
						plotPixel( g, ba0i, (byte) sd4 );// sd4 < 0xFE

						sd4++;
						plotPixel( g, ba0i, (byte) sd4 );// x+1, y

						ba0i -= SCREEN_WIDTH / 8 / PF;// 40
						sd4--;
						plotPixel( g, ba0i, (byte) sd4 );// x, y-1

						sd4++;
						paint.setColor( getRGB( SPARK_LIGHT_COLOR ) );
						plotPixel( g, ba0i, (byte) sd4 );// x+1, y-1
					}
					return true;
				}
			}
		}
		w_sparks_y[ (bd1) / 2 ] = SPARK_Y_OFF_SCREEN / PF;// w_TAB_1c380[ (bd1 + 64) / 2 ] = 210;
		return false;
	}

	// sparks_or_clouds
	private static void sparksOrClouds(final Canvas g, int bd1, final int effect) {
		bd1 <<= 1;
		int temp = bd1;

		if( !b_touching_road ) {
			// FIXME Amiga bug: sparks disappear when the car goes off road
			initialiseSparksTable();// if not touching road
			return;
		}
		// Touching road

		SoundManager.soundEffect( effect );

		bd1 = temp;
		do {
			if( drawSparkOrCloud( g, bd1 ) ) {
				w_sparks_clouds_dy[ (bd1) / 2 ] += 2;
				w_sparks_y[ (bd1) / 2 ] += w_sparks_clouds_dy[ (bd1) / 2 ];
				w_sparks_x[ (bd1) / 2 ] += w_sparks_clouds_dx[ (bd1) / 2 ];
			}
			bd1 -= 2;
		} while( bd1 >= 0 );

		bd1 = temp;

		do {
			short sd0 = w_sparks_y[ bd1 / 2 ];// sa4[ (bd1 + 64) / 2 ];
			if( (sd0 < 0) || (sd0 >= SCREEN_AREA_HEIGHT / PF) ) {
				final short sd3 = (short) (randomizeLong() & 0x07);
				sd0 = (short) (b_ferocity_of_sparks_or_clouds >>> 1);
				if( b_off_map_status >= 0 ) {
					sd0 >>>= 1;
				}
				sd0 += sd3;
				sd0 = (short) ~sd0;
				w_sparks_clouds_dy[ bd1 / 2 ] = sd0;// sa5[ (bd1 + 64) / 2 ] = sd0;

				int sd5_x;
				if( b_off_map_status < 0 ) {
					// in place drawSparks2();
					sd5_x = randomizeLong() & 0xFF;
					w_sparks_x[ bd1 / 2 ] = (short) sd5_x;
					w_sparks_y[ bd1 / 2 ] = (short) ((SCREEN_AREA_HEIGHT / PF - 10) + (randomizeLong() & 0x07));
				} else {
					sd5_x = SCREEN_AREA_WIDTH / 4 / PF + (randomizeLong() & 0x7F);
					w_sparks_x[ bd1 / 2 ] = (short) sd5_x;
					w_sparks_y[ bd1 / 2 ] = (short) ((SCREEN_AREA_HEIGHT / PF - 1) + (randomizeLong() | 0xfff8));
				}
				w_sparks_clouds_dx[ bd1 / 2 ] = (short) ((sd5_x - (SCREEN_AREA_WIDTH / 2 / PF)) >> 3);
				drawSparkOrCloud( g, bd1 );
			}
			bd1 -= 2;
		} while( bd1 >= 0 );
	}

	// draw_sparks
	private static void drawSparks(final Canvas g) {
		if( (0 == b_which_side_byte) && (0 == lw_wreck_wheel_height_reduction_fp8) ) {
			return;// if car is not scraping on road
		}
		if( b_off_map_status < 0 ) {
			return;// dust clouds will be drawn instead
		}

		int bd0 = w_pos_players_z_speed >>> 8;
		if( bd0 < 1 ) {
			return;// if speed is not large enough
		}
		if( bd0 > 50 ) {
			bd0 = 50;// set to maximum
		}
		b_ferocity_of_sparks_or_clouds = (byte) bd0;

		int bd2 = randomizeLong() & 0x07;// not used
		bd0 = b_ferocity_of_sparks_or_clouds >>> 1;
		if( bd0 > 0x1F ) {// 31
			bd0 = 0x1F;
		}
		bd0 ^= 0x1F;
		bd0 &= 0xFF;
		bd0 <<= 2;
		bd0 += 170;

		SoundManager.effects[ SoundManager.SOUND_WRECK ].mPeriod = (short) bd0;

		sparksOrClouds( g, 31, SoundManager.SOUND_WRECK );
	}

	// draw_chains
	private static void drawChains(final Canvas g) {
		if( 0 == b_car_on_chains_countdown ) {
			int d0 = b_1bbea;
			if( 96 == d0 ) {
				return;
			}
			d0 -= b_1bbe9;
			b_1bbea = (byte) d0;
			b_1bbe9 += 8;
		}
		// dch1
		int d2 = (b_1bbea & 0xFF);
		// dch2
		do {
			int d3 = (d2 - 48) * PF;// TODO values
			graphic_info[ SPRITE_LEFT_CHAIN_TOP ].mY = d3;// y position of chain tops
			graphic_info[ SPRITE_RIGHT_CHAIN_TOP ].mY = d3;

			d3 += 8 * PF;
			graphic_info[ SPRITE_LEFT_CHAIN_BOTTOM ].mY = d3;// y position of chain bottoms
			graphic_info[ SPRITE_RIGHT_CHAIN_BOTTOM ].mY = d3;
			if( graphic_info[ SPRITE_LEFT_CHAIN_BOTTOM ].mY < 16 * PF ) {
				return;
			}

			copyGraphic( g, SPRITE_LEFT_CHAIN_BOTTOM );
			copyGraphic( g, SPRITE_RIGHT_CHAIN_BOTTOM );

			if( graphic_info[ SPRITE_LEFT_CHAIN_TOP ].mY < 16 * PF ) {
				return;
			}
			copyGraphic( g, SPRITE_LEFT_CHAIN_TOP );
			copyGraphic( g, SPRITE_RIGHT_CHAIN_TOP );

			d2 -= 16;
		} while( true );
	}

	// copy_graphic
	/**
	 * Draw image sprite
	 *
	 * @param g  Canvas
	 * @param d0 index of the image sprite in {@link #graphic_info} in d0 as byte
	 */
	private static void copyGraphic(final Canvas g, final int d0) {
		final GraphicsInfo ga2 = graphic_info[ d0 ];
		if( ga2.mImage != null ) {// TODO remove after testing
			g.drawBitmap( ga2.mImage, ga2.mX, ga2.mY, null );
		}
	}

	// display_race_prompts
	private static void displayRacePrompts() {
		if( b_prompt_required ) {
			if( 0 != b_1bb6c ) {
				if( (b_1bb6c < 3) && (Text.STR_PRESS_FIRE != b_prompt_offset) ) {
					return;
				}
			}
			if( b_1bba4 ) {
				return;
			}
			b_or_char_with_screen = true;
			// b_print_text_on_screen_2 = true;// java: print text on both screens

			setTextColor( b_white_prompts ? 15 : 0 );

			b_copy_prompt_groups = b_prompt_groups;
			final String d2 = b_prompt_offset;
			// final int sp_1 = d1;
			b_print_row = 4;
			b_print_column = 19;
			b_print_fine_x = 3;
			b_print_fine_y = 0;
			if( 2 == b_prompt_groups ) {
				b_print_fine_y = 5;
			}
			// java version
			for( int i = 0; i < d2.length(); i++ ) {
				final char c = d2.charAt( i );
				if( c == '<' ) {
					b_print_fine_x += 4;
				} else if( c == '!' ) {
					b_print_column = 19;// to next row down
					b_print_row = 5;
					b_print_fine_x = 3;
					b_print_fine_y = 2;
				} else {
					jPrintCharacter( c );
					++b_print_fine_x;
				}
			}
			/*
			do {
				// move.l	#race_prompt_text, a2
				int d1 = race_prompt_text[ d2 ];
				if( '!' == d1 ) {
					print_column = 19;// to next row down
					print_row = 5;
					print_fine_x = 3;
					print_fine_y = 2;
				}
				++d2;
				d0 = 3;// print groups of 3 chars.
				prompt_chars = (byte) d0;

				// rp8
				do {
					// move.l	#race_prompt_text, a2
					d0 = race_prompt_text[ d2 ];
					if( '<' == d0 ) {
						print_fine_x += 4;// half a character across
					} else {
						print_character();
						++print_fine_x;
					}
					++d2;
					++d1;
				} while( --prompt_chars != 0 );
			} while( --copy_prompt_groups != 0 );
			*/
			// d1 = sp_1;
			b_or_char_with_screen = false;
			b_print_text_on_screen_2 = false;
			b_print_fine_x = 0;
			b_print_fine_y = 0;
			setTextColor( 0 );
		}
	}
	
	// set_race_prompt
	private static void setRacePrompt(final int d0, final String d2) {
		signalPromptRequired( d0, d2 );
		displayRacePrompts();
	}
	
	// off_map
	private static void offMap() {
		b_off_map_status = (byte) 0xC0;
		if( b_machine != 0 ) {
			transmitPlayersData();
		}
		if( b_race_mode < 0 ) {
			opponentMovement();
			calculateOpponentsRoadWheelPositions();
		}
		w_players_distance_into_section_plus64 = (short)((b_1bb9f << 8) | (w_players_distance_into_section_plus64 & 0xFF));
		w_near_sections_done2 = w_1bc28;
		b_near_sections_done = b_1bb9e;
		// bra no_opponent5
/* FIXME not used ?
        R_1b818();
        drawHorizon();
        drawMountains();
        jmp dwca
*/
	}

	// on_road
	private static void onRoad(int bd0) {
		b_current_road_piece = (byte) bd0;

		calculatePlayersRoadPosition();
		setRoadCentreValues();

		b_players_road_piece = b_current_road_piece;

		if( 0 == (b_off_map_status & 0x40) ) {
			b_1bb9b = b_current_road_piece;
		}

		detailNearRoad();
		b_1bb9f = (byte) (w_players_distance_into_section_plus64 >>> 8);
		w_1bc28 = w_near_sections_done2;
		b_1bb9e = b_near_sections_done;
		if( 0 != b_machine ) {
			transmitPlayersData();
			return;
		}
		if( b_race_mode < 0 ) {
			opponentMovement();
			calculateDistancesBetweenPlayers();
			opponentPlayerInteraction();
			calculateOpponentsRoadWheelPositions();
		}
	}

	// draw_world
	private static void drawWorld(final Canvas g) {
		g.save();
		g.clipRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
				SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH,
				SCREEN_AREA_TOP_Y + SCREEN_AREA_HEIGHT );

		drawWorldInitialisation();

		b_map_x_shift = 0;
		b_map_z_shift = 0;
		int sign_bd1_bd0 = getValidMapNumber();
		if( sign_bd1_bd0 < 0 ) {// use sign
			offMap();
		} else {
			if( 0xFF != (sign_bd1_bd0 & 0xFF) ) {// use d0
				onRoad( sign_bd1_bd0 & 0xFF );
			} else {
				// If map square is empty
				final int bd0 = toClosestAdjacentMapSquare(
						b_players_fine_map_x,
						(sign_bd1_bd0 >> 8) & 0xFF,// use d1
						b_players_fine_map_z );
				if( bd0 != 0xFF ) {
					onRoad( bd0 );
				} else {
					offMap();
				}
			}
		}
// no_opponent5:
		b_edge_offset_left_x2 = (byte) 0x80;
		b_edge_offset_right_x2 = (byte) 0x80;
		b_current_road_piece = b_players_road_piece;
		b_coord_offset_zero_or_four = 0;
		if( 0 != (w_players_distance_into_section_plus64 & 0x8000) ) {
			toNextRoadPiece();
			w_players_distance_into_section_plus64 &= 0x00FF;
		}
		if( 0 == (w_players_distance_into_section_plus64 & 0xFF00) ) {

			final int section = toPreviousRoadPiece();
			if( b_opponents_road_section_m255 == section ) {
				w_opponents_offset = 0;
			}
			toNextRoadPiece();
		}
		makeNearRoadCoords();
		makeOptionalSideEdges();
		makeOptionalTopEdges();
		makeNearRoadEdges();
		b_coord_pair_to_start_at = 0;
		b_offset_for_coord_pair_to_start_at = 0;
		b_coord_offset_zero_or_four = 4;
		copyCoordsForNextSection();
		toNextRoadPiece();

		makeNearRoadCoords();
		makeOptionalSideEdges();
		makeNearRoadEdges2();
		copyCoordsForNextSection();
		b_pits_are_black = true;
		toNextRoadPiece();

		makeNearRoadCoords();
		makeOptionalSideEdges();
		makeNearRoadEdges2();
		copyCoordsForNextSection();
		toNextRoadPiece();

		w_far_road_limit = w_road_section_offset;
		w_old_left_x = w_screen_x_coords[ OFFSET_COPY_LAST_NEAR_SECTION_COORD ];
		w_old_right_x = w_screen_x_coords[ OFFSET_COPY_LAST_NEAR_SECTION_COORD + 1 ];
		makeFarRoadCoords();
		makeFarRoadEdges();

		if( b_race_mode >= 0 ) {
			w_opponents_offset = -1;
		}
		if( 0 != b_machine ) {
			R_57440();
		}
		final short backup_road_section_offset = w_road_section_offset;
		drawHorizon( g );
		drawMountains( g );
		w_road_section_offset = backup_road_section_offset;

		setOpponentDrawFlag();
		drawFarRoad( g );

		w_road_section_offset = w_far_road_limit;
		drawNearRoad( g );

		// dwca:
		if( (0 == b_car_on_chains_countdown) && (b_off_map_status < 0) ) {
			drawDustClouds( g );
		}
		drawSparks( g );

		drawChains( g );

		copyGraphic( g, SPRITE_LEFT_TOP_CORNER );// left top corner
		copyGraphic( g, SPRITE_RIGHT_TOP_CORNER );// right top corner

		updateWheelPositions();
		g.restore();
		g.save();
		g.clipRect( SCREEN_AREA_LEFT_X, SCREEN_AREA_TOP_Y,
				SCREEN_AREA_LEFT_X + SCREEN_AREA_WIDTH,
				SCREEN_AREA_TOP_Y + (158 + 1) * PF - SCREEN_AREA_TOP_Y );
		if( b_5d724 >= 0 ) {// java if added
			copyGraphic( g, SPRITE_RIGHT_WHEELS + b_wheel_frame_number );// right wheel
			copyGraphic( g, (SPRITE_LEFT_WHEELS + SPRITE_WHEEL_SPRITES - 1) - b_wheel_frame_number );// left wheel
			// java
			copyGraphic( g, SPRITE_LEFT_EXHAUST_PANEL );// left exhaust covering wheel
			copyGraphic( g, SPRITE_RIGHT_EXHAUST_PANEL );// right exhaust covering wheel
		}
		copyGraphic( g, SPRITE_ENGINE_BLOCK );// engine block
		// copyGraphic( g, SPRITE_LEFT_EXHAUST_PANEL );// left exhaust covering wheel
		// copyGraphic( g, SPRITE_RIGHT_EXHAUST_PANEL );// right exhaust covering wheel

		// java: draw flames each frame in the {@link #vertbInterrupt}
		if( b_boost_activated ) {

			int bd0 = b_flame_counter;
			if( ++bd0 >= SPRITE_FLAME_SPRITES ) {
				bd0 = 0;
			}
			b_flame_counter = (byte) bd0;

			bd0 += SPRITE_LEFT_FLAMES;// 6, 7, 8
			if( 8 == bd0 ) {
				bd0 = SPRITE_LEFT_FLAME_3;// left flames
			}
			// copyGraphic( g, bd0 );// java: changed to the next code
			graphic_info[ J_SPRITE_CURRENT_LEFT_FLAME ].mImage = graphic_info[ bd0 ].mImage;

			bd0 = b_flame_counter + SPRITE_RIGHT_FLAMES;// 8, 9, 10
			if( 10 == bd0 ) {
				bd0 = SPRITE_RIGHT_FLAME_3;// right flames
			}
			// copyGraphic( g, bd0 );// java: changed to the next code
			graphic_info[ J_SPRITE_CURRENT_RIGHT_FLAME ].mImage = graphic_info[ bd0 ].mImage;
		} else {// java added
			graphic_info[ J_SPRITE_CURRENT_LEFT_FLAME ].mImage = null;
			graphic_info[ J_SPRITE_CURRENT_RIGHT_FLAME ].mImage = null;
		}
		g.restore();
		displayRacePrompts();
	}

	/**
	 *
	 * @param bd2 delay in internal units
	 */
	private static void delay(final int bd2) {
		// TODO verify java to amiga delay time conversion
		jDelay( bd2 * 20 );
	}

	// delay20
	private static void delay20() {
		delay( 20 );
	}

	private static float mAlpha = 0f;

	// fade_screen_in
	/**
	 * Java: the picture must be on {@link #screen2}.
	 * At the end both screens have the same picture.
	 */
	private static void fadeScreenIn() {
		// Amiga just change colors from dark to correct each frame
		final Canvas g2d = screen1_canvas;
		if( mAlpha <= 0.f ) {
			mAlpha = 0f;
			// if( mAlpha < 1f ) {
			paint.setColor( Color.BLACK );
			synchronized( Game.class ) {
				g2d.drawRect( 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, paint );
			}
			do {
				// draw image with alfa
				mAlpha += (1f / 10f);
				mAlpha = Math.min( 1f, mAlpha );
				synchronized( Game.class ) {
					paint.setAlpha( (int)(255f * mAlpha) );
					g2d.drawBitmap( screen2, 0, 0, paint );
				}
				b_frame_count = 5;// 2;
				while( 0 != b_frame_count ) {// min 40 ms to show
					jDelay( 10 );
				}
			} while( mAlpha < 1f );
			paint.setAlpha( 255 );
			while( 0 != fade_frame_count ) {// waiting for 600 ms
				jDelay( 10 );
			}
			// }
			return;
		}
		synchronized( Game.class ) {
			g2d.drawBitmap( screen2, 0, 0, paint );
		}
	}

	// fade_screen_out
	/**
	 * Java: the picture must be on the {@link #screen1}.
	 */
	private static void fadeScreenOut() {
		// Amiga just sets more dark colors each frame
		// if( mAlpha >= 1f ) {
		fade_frame_count = FADE_FRAMES;
		mAlpha = 0f;

		b_which_screen = false;
		final Canvas g2d = screen1_canvas;
		paint.setColor( Color.BLACK );
		do {
			// draw image with alfa
			mAlpha += (1f / 10f);
			mAlpha = Math.min( 1f, mAlpha );
			synchronized( Game.class ) {
				paint.setAlpha( (int)(255f * mAlpha) );
				// paint.setStyle( Paint.Style.FILL );
				// paint.setXfermode( new PorterDuffXfermode( PorterDuff.Mode.SRC_OVER ) );
				g2d.drawRect( 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, paint );
			}
			b_frame_count = 2;
			while( 0 != b_frame_count ) {// min 40 ms to show
				jDelay( 10 );
			}
		} while( mAlpha < 1f );
		paint.setAlpha( 255 );
		while( 0 != fade_frame_count ) {// waiting for 600 ms
			jDelay( 10 );
		}
		mAlpha = 0f;
	}

	// set_current_scene
	private static void setCurrentScene() {
		/* int d3 = screen2;
		d3 += 16 * 40 + 4;
		current_scene = d3;
		visible_screen = screen1; */
		show_new_screen = true;// TODO check in the interrupt
	}

	// R_57b5c
	/**
	 *
	 * @return true if minus, false is positive
	 */
	private static boolean R_57b5c() {
		final int d0 = b_57c6f;
		if( d0 >= 0 ) {
			int d3 = b_map_z + 1;
			if( d0 <= d3 ) {
				b_map_z_shift = b_57c6f;
				return false;
			}
		}
		// _label1
		// d0 = 0x80;
		return true;
	}

	/**
	 * offsets to {@link #track_names} and track ids
	 */
	private static final byte[] b_TAB_648c2 = {
			(byte) 0x00, (byte) 0x02, (byte) 0x01, (byte) 0x03, (byte) 0x06, (byte) 0x07, (byte) 0x04, (byte) 0x05,
			(byte) 0x08, (byte) 0x05, (byte) 0x0c, (byte) 0x05, (byte) 0x05, (byte) 0x08, (byte) 0x0c, (byte) 0x08
		};
	
	// get_main_menu_selection
	/**
	 *
	 * @param g
	 * @param d0 selected menu line index
	 * @param d1
	 * @param d2
	 * @return selected line index in d0 as byte
	 */
	private static int getMainMenuSelection(final Canvas g, int d0, int d1, int d2) {
		--d2;
		int b_map_z = (byte) d2;
		int b_map_x_offset_for_text = (byte) d1;// start offset for menu text in b_TAB_5bcd0
		int b_map_z_shift_selected_line_index = (byte) d0;
		displayDivision( g, false );

		setTextColor( 1 );

		b_1bbca = 0;// fire pressed
		printSelectableTextLine( g, 0 );// 'SELECT'

		setTextColor( 0 );

		gmms1:
		do {
			b_menu_bar_index = 0;

			// gmms2
			do {
				d2 = b_menu_bar_index;
				final int menu_line_index = d2;// menu line index backup
				if( d2 == b_map_z_shift_selected_line_index ) {
					d0 = 0;// confirmed
					if( 0 == b_1bbca ) {
						d0 = 1;// selected
					}
					// gmms3
					b_menu_bar_type = (byte) d0;
				}
				// gmms4
				fillBar( g );

				// "1. "
				// "2. "
				// "3. "
				d0 = menu_line_index + 1;// number is menu index+1
				printDigit( d0 );
				jPrintCharacter( '.' );
				jPrintCharacter( ' ' );

				d2 = menu_line_index + b_map_x_offset_for_text;
				d1 = b_TAB_5bcd0[ d2 ];// menu text index
				printSelectableTextLine( g, d1 );
				if( 24 == b_map_x_offset_for_text ) {
					printDigit( menu_line_index + 1 );
				}
				// gmms5
				if( b_map_z < menu_line_index ) {
					break;
				}
				if( 28 == b_map_x_offset_for_text ) {
					printLeagueText( g, 3 );// 'The '

					d0 = (b_5eb7c << 1) + menu_line_index;
					d2 = d0;
					d1 = b_TAB_648c2[ d2 ];
					printTrackName( g, d1 );
				}
			} while( true );

			// gmms6
			copyScreenPart( g );
			delay( 15 );
			if( 0 != b_1bbca ) {
				clearPrintFineY();
				return b_map_z_shift_selected_line_index;
			}
			// wait_joystick_released
			while( 0 != getPlayersInput() ) {
				jKeyPollDelay();// java
			}
			delay( 2 );

			// gmms8
			do {
				d0 = (getPlayersInput() & CTRL_PLAYER_BOOST);
				b_1bbca = (byte) d0;
				if( 0 != d0 ) {
					continue gmms1;// if fire pressed
				}
				if( 0 == b_machine ) {
					// gmms9
					d2 = b_map_z + 1;
					// gmmsa
					do {
						// move.l	#b_control_keys + 5, a2
						// move.b( a2, d2.w ), d1
						if( testKey( b_control_keys[ 5 + d2 ] ) ) {// FIXME key code
							b_map_z_shift_selected_line_index = (byte) d2;
							continue gmms1;
						}
						// gmmsb
					} while( --d2 >= 0 );
				} else {
					if( ! R_57b5c() ) {
						continue gmms1;
					}
				}
				jKeyPollDelay();// java
				// gmmsc
				d1 = b_map_z_shift_selected_line_index;
				d0 = b_players_input & (CTRL_PLAYER_ACCELERATE | CTRL_PLAYER_BREAK);
			} while( 0 == d0 );// joystick not forward or back
			d0 &= CTRL_PLAYER_ACCELERATE;
			if( 0 == d0 ) {
				// next_menu_bar_down
				if( d1 <= b_map_z ) {
					++d1;
				}
			} else {
				// next_menu_bar_up
				if( --d1 < 0 ) {
					d1 = 0;
				}
			}
			// gmmsf
			b_map_z_shift_selected_line_index = (byte) d1;
		} while( true );
	}

	// underline_text
	/**
	 * Draw a horizontal line
	 *
	 * @param g Canvas
	 * @param color index in d0
	 * @param x1 in d4
	 * @param y in d5
	 * @param x2 in d6
	 */
	private static void underlineText(final Canvas g, final int color, final int x1, final int y, final int x2) {
		paint.setColor( getRGB( color ) );
		g.drawRect( x1, y, x1 + x2 - x1, y + 1 * PF, paint );
	}

	// set_print_column_row
	private static void setPrintColumnRow(final int d1, final int d2) {
		b_print_column = (byte) d1;
		b_print_row = (byte) d2;
	}

	private static byte b_max_name_length = 0;
	private static byte b_joystick_directions = 0;
	private static byte b_input_text_char_position = 0;

	// get_pressed_key
	/**
	 *
	 * @return < 0 means cs, no key code
	 */
	private static int getPressedKey() {
		/*
		if( key_array == 0 ) {
			return -1;
		}
		return key_array;
		*/
		synchronized( key_array ) {
			final Iterator< Integer > it = key_array.iterator();
			if( it.hasNext() ) {
				final Integer key = it.next();
				key_array.clear();
				return key.intValue();
			}
			return -1;
		}
	}

	// wait_for_key
	private static int waitForKey() {
		int key;
		while( (key = getPressedKey()) < 0 ) {
			jKeyPollDelay();// java
		}
		return key;
	}

	//
	private static int releaseAndWaitForKey() {
		while( getPressedKey() >= 0 ) {
			jKeyPollDelay();// java
		}
		return waitForKey();
	}

	// input_text
	/**
	 *
	 * @param d0 index in array to store text
	 * @param d3 text length in d3 as byte
	 */
	private static void inputText(int d0, int d3) {
		// Android:
		sActivity.showKeyboard();

		int b_map_x_shift = (byte) d0;
		int b_max_name_length = (byte) d3;

		b_joystick_directions = 0;
		int d1 = 1;// b_input_text_char_position + b_print_column;
		// bra	next_name_key
		// first_name_key
		background_color = 2;

		next_name_key:
		do {
			int d2 = b_map_x_shift;// java d1 + b_map_x_shift;

			d0 = waitForKey();
			if( b_read_joystick ) {
				encodeJoystick( d0 );
				if( 0 != b_joystick_directions ) {
					if( d1 <= b_max_name_length ) {
						Arrays.fill( opponents_names[ d2 ], d1, b_max_name_length + 1, ' ');
					}
					key_code = 0;// java
					break;
				}
			}

			switch( key_code ) {
				case '\n':// KeyEvent.KEYCODE_ENTER:
					if( d1 <= b_max_name_length ) {
						Arrays.fill( opponents_names[ d2 ], d1, b_max_name_length + 1, ' ');
					}
					break next_name_key;
				case 127:// KeyEvent.KEYCODE_BACK:
					if( d1 > 1 ) {
						--d1;
						opponents_names[ d2 ][ d1 ] = ' ';
						// --b_print_column;
						// jPrintCharacter( ' ' );
						// --b_print_column;
						jPrintCharacter( ASCII_DELETE );
					}
					break;
				case ' ':
					break;
				default:
					if( (d1 <= b_max_name_length) && jIsCharValid( key_code ) ) {
						opponents_names[ d2 ][ d1 ] = (char) key_code;
						jPrintCharacter( (char) key_code );
						// jPrintText( g, String.valueOf( opponents_names[ d2 ] ) );
						++d1;
					}
					break;
			}
			key_code = 0;// java
		} while( true );
		clearPrintFineY();
		// Android:
		sActivity.hideKeyboard();
	}

	// input_name
	private static void inputName() {
		setPrintColumnRow( 14, 16 );

		jPrintCharacter( '>' );

		// _wait
		while( getPressedKey() >= 0 ) {// wait until keys released
			jKeyPollDelay();// java
		}

		b_input_text_char_position = 0;
		inputText( (NUMBER_OF_RACERS - 1 - b_multi_no_of_players), Text.RACER_NAME_LENGTH );// java << 4;
	}

	// get_players_name
	private static void getPlayersName(final Canvas g) {
		clearMenu( g );
		printLeagueText( g, 15 );// 'NAME?'

		b_menu_bar_index = 1;
		fillBar( g );

		underlineText( g, 10, 106 * PF, 133 * PF, 190 * PF );

		copyScreenPart( g );
		// move.l	lw_screen2,-(sp)
		// move.l	lw_screen1,lw_screen2

		// input_again
		do {
			setBackgroundColor( 2 );

			inputName();
			fourPrintFineY();

		} while( ' ' == opponents_names[ (NUMBER_OF_RACERS - 1 - b_multi_no_of_players) ][ 1 ] );// java << 4;

		clearPrintFineY();
		setBackgroundColor( 3 );
		// move.l	(sp)+,lw_screen2
	}

	// FIXME establish_computer_link
	/**
	 *
	 * @return true - cs
	 */
	private static boolean establishComputerLink() {
		return false;
	}

	// main_menu_selection
	/**
	 *
	 * @param g Canvas
	 */
	private static void mainMenuSelection(final Canvas g) {
		main_menu_selection:
		while( true ) {
			b_text_array_selection = (byte) 0x80;

			int d0 = 0;
			b_multi_no_of_players = 0;
			int d2 = 2;
			int d1 = 16;
			d0 = getMainMenuSelection( g, d0, d1, d2 );

			switch( d0 ) {
				case 0:
					// single_player_league_chosen
					getPlayersName( g );
					break main_menu_selection;
				case 1:// multiplayer_chosen
					multiplayer_chosen:
					do {
						getPlayersName( g );
						if( (b_multi_no_of_players < 7) || (0 == b_multi_no_of_players) ) {
							// multiplayer_enter_another:
							do {
								d0 = 0;
								d2 = 1;
								d1 = 20;
								d0 = getMainMenuSelection( g, d0, d1, d2 );
								// 0: enter name, 1: continue
								if( 0 == d0 ) {
									++b_multi_no_of_players;
									continue multiplayer_chosen;
								} else {
									if( 0 == b_multi_no_of_players ) {
										continue;
									} else {
										break main_menu_selection;
									}
								}
							} while( true );
						}
						break main_menu_selection;
					} while( true );
				case 2:
					// computer_link_chosen
					if( establishComputerLink() ) {
						continue main_menu_selection;
					}
					break main_menu_selection;
			}
		}
		// main_menu_selection_done
		// d0 = 0;
		b_text_array_selection = 0;
	}


	/**
	 * offsets to {@link Text#main_game_selection_text}
	 * or {@link Text#b_TEXT_5a69a}
	 * Java: offsets are changed
	 */
	private static final byte[] b_TAB_5bcd0 = {
			// (byte) 0xec, (byte) 0x0a, (byte) 0x14, (byte) 0x2c, (byte) 0x44, (byte) 0x49, (byte) 0x4e, (byte) 0x55, (byte) 0x5c, (byte) 0x6b, (byte) 0x55, (byte) 0x00, (byte) 0x7a, (byte) 0x87, (byte) 0x55, (byte) 0x00,
			// (byte) 0x0a, (byte) 0x1f, (byte) 0x71, (byte) 0x00, (byte) 0x2b, (byte) 0x40, (byte) 0x00, (byte) 0x00, (byte) 0x49, (byte) 0x49, (byte) 0x49, (byte) 0x49, (byte) 0x0a, (byte) 0x0a, (byte) 0x55, (byte) 0x00
			// to {@link #b_TEXT_5a69a}
			16,// Hall of Fame
			1,// Practise
			2,// Start the Racing Season
			3,// Load/Save/Replay
			4,// Load
			5,// Save
			6,// Replay
			7,// Cancel
			8,// LOAD from Tape
			9,// LOAD from Disc
			7,// Cancel
			0,// SELECT
			10,// SAVE to Tape
			11,// SAVE to Disc
			7,// Cancel
			0,// SELECT
			1,// Single Player League
			2,// Multiplayer
			9,// Computer Link
			0,// SELECT
			3,// Enter another driver
			4,// Continue
			0,// SELECT
			0,// SELECT
			5,// Tracks in DIVISION
			5,// Tracks in DIVISION
			5,// Tracks in DIVISION
			5,// Tracks in DIVISION
			1,// Single Player League
			1,// Single Player League
			// to b_TEXT_5a69a
			7,// Cancel
			0// SELECT
		};

	private static boolean b_font_narrow = false;

	// encode_joystick
	/**
	 *
	 * @param d0 as word, high 8 bita are raw key code
	 */
	private static void encodeJoystick(int d0) {
		b_joystick_directions = 0;
		// if( 0 == (d0 & 0xFF) ) {
			// d0 >>>= 8;// joystick code
			switch( d0 ) {
				case KeyEvent.KEYCODE_ESCAPE:
					b_joystick_directions = (byte) 0x80;
					return;
				case KeyEvent.KEYCODE_DPAD_UP:
					b_joystick_directions = (byte) 0x20;
					return;
				case KeyEvent.KEYCODE_DPAD_DOWN:
					b_joystick_directions = (byte) 0x40;
					return;
				case KeyEvent.KEYCODE_DPAD_LEFT:
					b_joystick_directions = (byte) 0x10;
					return;
				case KeyEvent.KEYCODE_DPAD_RIGHT:
					b_joystick_directions = (byte) 0x08;
					return;
			}
		// }
	}

	private static boolean jIsCharValid(final int c) {// java
		return ((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z')) || ((c >= '0') && (c <= '9'));
	}

	// print_character
	/**
	 * Print ASCII character on both screens
	 *
	 * @param c a character for printing
	 */
	private static void jPrintCharacter(char c) {
		if( c == ASCII_DELETE ) {
			--b_print_column;
			final boolean backup = b_or_char_with_screen;
			b_or_char_with_screen = false;
			jPrintCharacter( ' ' );
			b_or_char_with_screen = backup;
			--b_print_column;
			return;
		}
		if( (c >= ' ') && (c < 127) ) {
			c -= ' ';
			int y = b_print_row * Text.TEXT_ROW_HEIGHT * PF + b_print_fine_y * PF;
			if( (0x41 == b_5d724) && ! b_font_narrow ) {
				y -= 8 * PF;
			}
			int x = b_font_narrow ? Text.COLUMN_SMALL_FONT_WIDTH * PF : Text.COLUMN_NORMAL_FONT_WIDTH * PF;
			x *= b_print_column;
			x += b_print_fine_x * PF;
			final byte[] font = b_font_narrow ? Text.font6 : Text.font7;
			final int width = b_font_narrow ? Text.COLUMN_SMALL_FONT_BITS * PF : Text.COLUMN_NORMAL_FONT_BITS * PF;

			final Canvas g1 = screen1_canvas;
			final Canvas g2 = screen2_canvas;

			for( int offset = c << 3, end = offset + 8; offset < end; ++offset, y += PF ) {
				int mask = font[ offset ];
				int x_shift = 0, text_dx = 0, background_dx = 0;
				for( ; x_shift < width; x_shift += PF ) {
					if( (mask & 0x80) != 0 ) {
						text_dx += PF;
						if( background_dx != 0 ) {
							if( !b_or_char_with_screen ) {
								paint.setColor( getRGB( background_color ) );
								if( !b_print_text_on_screen_2 ) {
									g1.drawRect( x + x_shift - background_dx, y,
											x + x_shift - background_dx + background_dx,
											y + 1 * PF, paint );
								}
								g2.drawRect( x + x_shift - background_dx, y,
										x + x_shift - background_dx + background_dx,
										y + 1 * PF, paint );
							}
							background_dx = 0;
						}
					} else {
						background_dx += PF;
						if( text_dx != 0 ) {
							paint.setColor( getRGB( text_color ) );
							if( !b_print_text_on_screen_2 ) {
								g1.drawRect( x + x_shift - text_dx, y,
										x + x_shift - text_dx + text_dx,
										y + 1 * PF, paint );
							}
							g2.drawRect( x + x_shift - text_dx, y,
									x + x_shift - text_dx + text_dx,
									y + 1 * PF, paint );
							text_dx = 0;
						}
					}
					mask <<= 1;
				}
				if( background_dx != 0 ) {
					if( !b_or_char_with_screen ) {
						paint.setColor( getRGB( background_color ) );
						if( !b_print_text_on_screen_2 ) {
							g1.drawRect( x + x_shift - background_dx, y,
									x + x_shift - background_dx + background_dx,
									y + 1 * PF, paint );
						}
						g2.drawRect( x + x_shift - background_dx, y,
								x + x_shift - background_dx + background_dx,
								y + 1 * PF, paint );
					}
				}
				if( text_dx != 0 ) {
					paint.setColor( getRGB( text_color ) );
					if( !b_print_text_on_screen_2 ) {
						g1.drawRect( x + x_shift - text_dx, y,
								x + x_shift - text_dx + text_dx,
								y + 1 * PF, paint );
					}
					g2.drawRect( x + x_shift - text_dx, y,
							x + x_shift - text_dx + text_dx,
							y + 1 * PF, paint );
				}
			}
		}
		++b_print_column;
	}

	/**
	 * Synthetic method.
	 * Print text either at stored position
	 *
	 * @param g Canvas
	 * @param text text to print
	 * @param offset start offset
	 */
	private static void jPrintText(final Canvas g, final String text, int offset) {
		for( int end = text.length(); offset < end; offset++ ) {
			jPrintCharacter( text.charAt( offset ) );
		}
	}

	/**
	 * Synthetic method.
	 * Print text either at stored position
	 *
	 * @param g Canvas
	 * @param text text to print
	 */
	private static void jPrintText(final Canvas g, final String text) {
		paint.setColor( getRGB( text_color ) );
		for( int offset = 0, end = text.length(); offset < end; offset++ ) {
			jPrintCharacter( text.charAt( offset ) );
		}
	}

	/**
	 * Synthetic method.
	 * Print text either at defined position or continue printing
	 *
	 * @param g Canvas
	 * @param text text with position data
	 */
	private static void jPrintText(final Canvas g, final Text.TextInfo text) {
		if( (text.mX >= 0) && (text.mY >= 0) ) {
			b_print_column = text.mX;
			b_print_row = text.mY;
		}
		jPrintText( g, text.mText, 0 );
	}

	// print_league_text
	/**
	 * Print text stored in the {@link Text#league_text}.
	 *
	 * @param g Canvas
	 * @param bd1 array index. Not equal used by Amiga!
	 */
	private static void printLeagueText(final Canvas g, final int bd1) {
		final Text.TextInfo lg = Text.league_text[ bd1 ];
		jPrintText( g, lg );
	}

	private static final int MENU_TOP_Y1 = 65 * PF;
	private static final int MENU_LEFT_X1 = 70 * PF;
	private static final int MENU_LEFT_X2 = 32 * PF;
	private static final int MENU_TOP_Y2 = 70 * PF;
	private static final int MENU_RIGHT_X = 256 * PF;// excluding
	private static final int MENU_BOTTOM_Y = 193 * PF;// excluding

	// clear_menu
	private static void clearMenu(final Canvas g) {
		setTextColor( 1 );
		setBackgroundColor( 3 );

		b_or_char_with_screen = true;

		paint.setColor( getRGB( background_color ) );
		g.drawRect( MENU_LEFT_X1, MENU_TOP_Y1,
				MENU_LEFT_X1 + MENU_RIGHT_X - MENU_LEFT_X1,
				MENU_TOP_Y1 + MENU_TOP_Y2 - MENU_TOP_Y1, paint );
		g.drawRect( MENU_LEFT_X2, MENU_TOP_Y2,
				MENU_LEFT_X2 + MENU_RIGHT_X - MENU_LEFT_X2,
				MENU_TOP_Y2 + MENU_BOTTOM_Y - MENU_TOP_Y2, paint );
	}

	// R_5a67c
	/**
	 * Print text
	 *
	 * @param g Canvas
	 * @param bd1 index of text in {@link Text#main_game_selection_text}
	 */
	private static void R_5a67c(final Canvas g, final int bd1) {
		final Text.TextInfo lg = Text.main_game_selection_text[ bd1 ];
		jPrintText( g, lg );
	}

	// R_5a64e
	/**
	 * Print text
	 *
	 * @param g Canvas
	 * @param bd1 index of text in arrays
	 *        The array is {@link #b_text_array_selection} dependent.
	 *        If it is negative, {@link Text#main_game_selection_text} is used.
	 *        If it is positive, {@link Text#b_TEXT_5a69a} is used.
	 */
	private static void printSelectableTextLine(final Canvas g, final int bd1) {
		if( b_text_array_selection < 0 ) {
			final Text.TextInfo lg = Text.main_game_selection_text[ bd1 ];
			jPrintText( g, lg );
			return;
		}
		final Text.TextInfo lg = Text.b_TEXT_5a69a[ bd1 ];
		jPrintText( g, lg );
	}

	// print_digit
	private static void printDigit(final int digit) {
		jPrintCharacter( (char) ((digit + '0') & 0xFF) );
	}

	// print_digit1
	private static void printDigit1(final int digit) {
		++b_print_fine_x;
		printDigit( digit );
	}

	// display_division
	/**
	 * Prints:
	 * 'DRIVERS CHAMPIONSHIP'
	 * 'DIVISION XX'
	 *
	 * @param g Canvas
	 * @param dontPrintDivision true - do not print 'DIVISION 4'
	 */
	private static void displayDivision(final Canvas g, final boolean dontPrintDivision) {
		b_division = b_players_division;
		clearMenu( g );

		setTextColor( 15 );

		if( dontPrintDivision || (0 != b_text_array_selection) ) {
			return;
		}
		if( 0 == b_multi_no_of_players ) {
			if( 0 == b_league_offset ) {
				Text.league_text[ 0 ].mY = 9;
				printLeagueText( g, 0 );// 'DIVISION '
			} else {
				Text.b_TEXT_5a69a[ 14 ].mY = 9;
				printSelectableTextLine( g, 14 );// 'SUPER DIVISION '
			}
			printDigit( MAX_DIVISIONS - b_division );
			setTextColor( 1 );
			return;
		}
		R_5a67c( g, 12 );// 'DRIVERS CHAMPIONSHIP'
	}

	// copy_screen_part
	/**
	 * Copy menu area to the 2nd screen buffer
	 * @param g1 Canvas
	 */
	private static void copyScreenPart(final Canvas g1) {
		// pseudocode:
		// g2.drawImage( g1.getImage(), MENU_LEFT_X, MENU_TOP_Y, MENU_WIDTH, MENU_HEIGHT );
	}

	// show_menu_screen
	private static void showMenuScreen(final boolean dontPrintDivision) {
		j_color_pallete_offset = COLOR_BASE_MENU;

		fadeScreenOut();

		b_print_text_on_screen_2 = true;
		// 2 off-screen buffers
		// (320/8 x 200) bytes x 4 color indexes = 32000 bytes
		/* screen2 = screen_mem;
		screen1 = screen_mem + 32000; */
		setCurrentScene();

		j_color_pallete_offset = COLOR_BASE_MENU;

		final Canvas g2 = screen2_canvas;
		g2.drawBitmap( jGetImage( R.drawable.menu ), 0, 0, paint );

		b_5d724 = 0x41;
		displayDivision( g2, dontPrintDivision );
		copyScreenPart( g2 );
		b_print_text_on_screen_2 = false;// java

		fadeScreenIn();
	}

	// R_58888
	private static void R_58888() {
		fadeScreenOut();

		final Canvas g2 = screen2_canvas;
		g2.drawBitmap( jGetImage( R.drawable.people ), 0, 0, paint );
		b_print_text_on_screen_2 = true;// java

		int d6 = 0;
		do {
			int d0 = b_DAT_1c9c2[ d6 ];
			int d3 = d6 + 19;
			displayPlayerPicture( g2, d0, d3 );
		} while( ++d6 < NUMBER_OF_RACERS );

		// move.l	(sp)+,lw_screen2
		b_print_text_on_screen_2 = false;// java
		fadeScreenIn();

		waitForFire();
		showMenuScreen( false );
	}
	
	// display_players
	private static void displayPlayers(final Canvas g) {
		if( 0 != b_multi_no_of_players ) {
			showRatingInDivision( g );
			return;
		}
		R_58888();
	}

	// FIXME R_579b0
	// for datalink
	/**
	 *
	 * @return true for cs
	 */
	private static boolean R_579b0() {
		return false;
	}

	private static final byte[] b_TAB_7a01a = new byte[ 256 ];

	// R_609ae
	/**
	 *
	 * @param d0 as byte
	 * @return true - cs
	 */
	private static boolean R_609ae(int d0) {
		loop:
		do {
			b_menu_bar_index = (byte) d0;
			int ba0i = 0;
			int d1 = 4;
			// _label1
			do {
				if( b_menu_bar_index >= 0 ) {
					b_TAB_7a01a[ d1 + 27 ] = lw_random_long[ ba0i++ ];
				} else {
					// _label3
					if( b_1ca34 >= 0 ) {
						// _label2
						return false;
					}
					lw_random_long[ ba0i++ ] = b_TAB_7a01a[ d1 + 27 ];
				}
				// _label4
			} while( --d1 >= 0 );
			if( b_menu_bar_index >= 0 ) {
				d1 = Text.RACER_NAME_LENGTH - 1;
				// _label5
				do {
					b_encoded_players_name[ d1 ] = (byte) (opponents_names[ 11 ][ 1 + d1 ] ^ 0x3B);
				} while( --d1 >= 0 );
			}
			// _label6
			d1 = 26;// TODO simplify the code with addressing

			// _label7
			do {
				if( b_menu_bar_index < 0 ) {
					lw_road_height_fp8 = ((b_TAB_7a01a[ d1 ] & 0xFF) << 8) | (b_TAB_7a01a[ 37 + d1 ] & 0xFF);
				}
				// _label8
				int d2 = 0;
				b_factor_high_value_low = (byte) d2;

				boolean is_store = false;// java
				// _label9
				do {
					++b_factor_high_value_low;
					if( 0 == b_factor_high_value_low ) {
						++d2;
						if( 0 != (d2 & 0x80) ) {
							// bmi _label15;
							// _label15
							lw_random_long[ 3 ] = 59;
							d0 = b_menu_bar_index;
							continue loop;
						}
					}
					// _labela
					d0 = randomizeLong();
					b_thousands = (byte) d0;
					if( b_menu_bar_index >= 0 ) {
						// move.l	#b_encoded_players_name,a1
						// if( (d0 & 0xFF) != (b_encoded_players_name[ d1 ] & 0xFF) ) {
						//	continue;// TODO why are random values used ?
						//}
						switch( d1 ) {
							case 24:
								if( (b_players_division & 0xFF) != (d0 & 0xFF) ) {
									continue;
								}
								break;
							case 25:
								if( (b_damage_hole_position & 0xFF) != (d0 & 0xFF) ) {
									continue;
								}
								break;
							case 26:
								if( (b_league_offset & 0xFF) != (d0 & 0xFF) ) {
									continue;
								}
								break;
							default:
								if( d1 >= b_encoded_players_name.length ) {
									if( (b_DAT_1c9c2[ d1 - b_encoded_players_name.length ] & 0xFF) != (d0 & 0xFF) ) {
										continue;
									}
									break;
								}
								if( (b_encoded_players_name[ d1 ] & 0xFF) != (d0 & 0xFF) ) {
									continue;
								}
								break;
						}
						// move.l	#b_TAB_7a01a,a1
						b_TAB_7a01a[ d1 ] = (byte) d2;
						b_TAB_7a01a[ 37 + d1 ] = b_factor_high_value_low;
						is_store = false;
						break;
						// goto _labelc;
					} else {
						// _labelb
						if( (((lw_road_height_fp8 >> 8) & 0xFF) == (d2 & 0xFF))
								&& ((lw_road_height_fp8 & 0xFF) == (b_factor_high_value_low & 0xFF)) ) {
							is_store = true;
							break;
						}
					}
				} while( true );
				if( is_store ) {
					b_DAT_7a41a[ d1 ] = b_thousands;
				}
				// _labelc
			} while( --d1 >= 0 );
			ba0i = 0;
			d1 = 4;
			int d2 = 9;

			// _labeld
			do {
				d0 = lw_random_long[ ba0i++ ];
				if( b_menu_bar_index >= 0 ) {
					b_TAB_7a01a[ 32 + d1 ] = (byte) d0;
					if( d0 >= 0 ) {
						continue;
					}
				}
				// _labele
				if( b_TAB_7a01a[ 32 + d1 ] != d0 ) {
					// bne _label15;
					// _label15
					lw_random_long[ 3 ] = 59;
					d0 = b_menu_bar_index;
					continue loop;
				}
				// _labelf
			} while( --d1 >= 0 );
			if( b_menu_bar_index < 0 ) {
				// b_encoded_players_name
				//	ds.b	12
				// b_DAT_1c9c2
				//	ds.b	12
				// b_1c9ce	dc.b	0
				// b_damage_hole_position	dc.b	0
				// b_league_offset	dc.b	0,0
				d1 = 26;// TODO simplify the code with addressing
				// _label10
				do {
					if( (0 == b_multi_no_of_players) || (d1 >= 24) ) {
						// _label11
						// java code for b_encoded_players_name[ d1 ] = ba0[ d1 ];
						switch( d1 ) {
							case 24:
								b_players_division = b_DAT_7a41a[ d1 ];
								break;
							case 25:
								b_damage_hole_position = b_DAT_7a41a[ d1 ];
								break;
							case 26:
								b_league_offset = b_DAT_7a41a[ d1 ];
								break;
							default:
								if( d1 >= b_encoded_players_name.length ) {
									b_DAT_1c9c2[ d1 - b_encoded_players_name.length ] = b_DAT_7a41a[ d1 ];
									break;
								}
								b_encoded_players_name[ d1 ] = b_DAT_7a41a[ d1 ];
								break;
						}
					}
					// _label12
				} while( --d1 >= 0 );
				d1 = Text.RACER_NAME_LENGTH - 1;
				// _label13
				do {
					d0 = b_encoded_players_name[ d1 ] ^ 0x3B;
					opponents_names[ 11 ][ 1 + d1 ] = (char) d0;
				} while( --d1 >= 0 );
			}
			// _label14
			b_1ca34 = (byte) 0x80;
			return false;

			// _label15
			// lw_random_long[ 3 ] = 59;
			// d0 = b_menu_bar_index;
		} while( d0 >= 0 );
		return true;
	}

	// FIXME R_5837e
	private static void R_5837e() {

	}

	// R_57de6
	private static void R_57de6(final Canvas g, final Text.TextInfo[] text, int d3) {
		jPrintText( g, text[ d3 ] );
	}

	// R_57dc2
	private static void R_57dc2(final Canvas g, int d3) {
		R_57de6( g, Text.b_LINK_TEXT_57e00, d3 );
	}

	// please_wait
	private static void pleaseWait(final Canvas g) {
		clearMenu( g );

		b_menu_bar_index = 1;
		fillBar( g );

		R_57dc2( g, 3 );// 46 'Please wait'

		clearPrintFineY();
	}

	// print_one_space
	private static void printOneSpace() {
		jPrintCharacter( ' ' );
	}

	// print_two_spaces
	private static void printTwoSpaces() {
		jPrintCharacter( ' ' );
		jPrintCharacter( ' ' );
	}

	// print_three_spaces
	private static void printThreeSpaces() {
		jPrintCharacter( ' ' );
		jPrintCharacter( ' ' );
		jPrintCharacter( ' ' );
	}

	// print_two_digit_value
	/**
	 * Amiga: print bcd number 00-99
	 *
	 * @param d0 binary value 0...99 in d0 as byte
	 */
	private static void printTwoDigitValue(int d0) {
		printDigit( d0 / 10 );
		printDigit( d0 % 10 );
	}

	// print_min_sec_ms
	private static void printMinSecMs(int d1) {
		int d0 = b_time_results_min[ d1 ];
		if( d0 < 9 ) {
			d0 |= b_time_results_sec[ d1 ];
			if( 0 != d0 ) {
				printDigit( b_time_results_min[ d1 ] );
				jPrintCharacter( ':' );
				printTwoDigitValue( b_time_results_sec[ d1 ] );
				jPrintCharacter( '.' );
				printTwoDigitValue( b_time_results_ms[ d1 ] );
				return;
			}
		}
		int d2 = 7;
		do {
			jPrintCharacter('-');
		} while( --d2 != 0 );
	}

	// R_5ec7e
	/**
	 *
	 * @param g Canvas
	 * @param d1 index in {@link Text#b_TEXT_5ec92}. Not the same as in Amiga
	 */
	private static void R_5ec7e(final Canvas g, int d1) {
		jPrintText( g, Text.b_TEXT_5ec92[ d1 ] );
	}

	// display_hall_of_fames
	private static void displayHallOfFames() {
		j_color_pallete_offset = 0;
		b_5d724 = 0x42;
		fadeScreenOut();

		// move.l	lw_screen2,-(sp)
		// move.l	lw_screen1,lw_screen2
		final Canvas g = screen2_canvas;// java: for fade in
		int d1 = 127;
		int d2 = 127;
		int d0 = b_league_offset;
		if( 0 != d0 ) {
			d2 = 0xFF;
		}
		// _label1
		// java: temp arrays for names
		// {@link #b_road_under_map} is used instead of
		// {@link #b_road_section_angle_and_piece} and {@link #w_overall_left_y_shifts}
		// because 128 bytes are required
		do {
			d0 = Text.b_TEXT_7a61a[ d2 ];
			// b_road_section_angle_and_piece[ d1 ] = (byte) d0;
			b_road_under_map[ d1 ] = (byte) d0;
			d0 = Text.b_TEXT_7a71a[ d2 ];
			// move.l	#w_overall_left_y_shifts+28,a1
			// move.b	d0,(a1,d1.w)
			b_road_under_map[ 128 + d1 ] = (byte) d0;
			--d2;
		} while( --d1 >= 0 );
		setTextColor( 15 );
		b_or_char_with_screen = true;
		g.drawBitmap( jGetImage( R.drawable.hall_of_fame ), 0, 0, null );
		b_print_fine_x = 2;
		d1 = 4;// 59 HALL of FAME
		d0 = b_league_offset;
		if( 0 != d0 ) {
			setTextColor( 7 );
			R_5ec7e( g, 5 );// Amiga 75 SUPER LEAGUE
		}
		setTextColor( 8 );
		R_5ec7e( g, 6 );// Amiga 91 TRACK  DRIVER   LAP-TIME    DRIVER  RACE-TIME
		b_menu_bar_index = 7;

		// _label3
		do {
			setPrintColumnRow( 0, ((7 - b_menu_bar_index) << 1) + 9 );

			d1 = b_menu_bar_index << 1;
			b_print_fine_x = 1;
			setTextColor( 1 );// white, Amiga: 15
			jPrintCharacter( b_tracks_short_names[ d1 ] );
			jPrintCharacter( b_tracks_short_names[ 1 + d1 ] );
			printOneSpace();
			b_print_fine_x = 4;
			b_prompt_chars = 0;

			// _label4
			do {
				d1 = ((b_menu_bar_index << 4) | b_prompt_chars) & 0xFF;
				setTextColor( 7 );

				d2 = Text.RACER_NAME_LENGTH;
				// _label5
				do {
					// jPrintCharacter( (char) (b_road_section_angle_and_piece[ d1 ] & 0xFF) );
					jPrintCharacter((char) (b_road_under_map[ d1 ] & 0xFF) );
					++d1;
				} while( --d2 != 0 );

				setTextColor( 1 );// white, Amiga: 15
				printOneSpace();
				--b_print_fine_x;
				// move.l	#b_road_section_angle_and_piece,a1
				b_time_results_min[ 0 ] = b_road_under_map[ d1 ];
				b_time_results_sec[ 0 ] = b_road_under_map[ d1 + 1 ];
				b_time_results_ms[ 0 ] = b_road_under_map[ d1 + 2 ];
				d1 = 0;
				printMinSecMs( d1 );
				b_prompt_chars ^= 0x80;
				if( 0 != (b_prompt_chars & 0x80) ) {
					printTwoSpaces();
					b_print_fine_x = 2;
					continue;
				}
				break;
			} while( true );
			// _label6
			--b_menu_bar_index;
		} while( b_menu_bar_index >= 0 );
		b_print_fine_x = 0;
		fadeScreenIn();
		waitForFire();
		// move.l	(sp)+,lw_screen2
		showMenuScreen( false );
	}

	// R_5fb62
	/**
	 *
	 * @param g Canvas
	 */
	private static int menuSelectDivision(final Canvas g) {
		int d0 = 0x80;
		b_text_array_selection = (byte) d0;
		b_5eb7d &= 0xFE;
		if( 0 == b_5eb7d ) {
			int d2 = 3;
			d0 = b_players_division ^ 3;
			int d1 = 24;
			d0 = getMainMenuSelection( g, d0, d1, d2 );
			d0 ^= 3;
			b_5eb7c = (byte) d0;
		}
		b_text_array_selection = 0x40;
		int d2 = 2;
		d0 = (b_race_mode & 1);
		final int d1 = 28;
		d0 = getMainMenuSelection( g, d0, d1, d2 );
		d2 = 0;
		b_text_array_selection = (byte) d2;
		return d0;
	}

	// R_5bca8
	private static void R_5bca8() {
		// R_5bca8
		b_1ca34 = b_1bbd0;
		System.arraycopy( b_road_under_map, 0, b_TAB_7a01a, 0, 256 );
	}

	// get_road_id
	/**
	 * Get road id
	 * Store a road id in the {@link #b_road_id}
	 *
	 * @param d0 track index
	 */
	private static void getRoadId(int d0) {
		int d1 = d0;
		b_road_id = b_TAB_648c2[ d1 ];
		if( 0 != b_league_offset ) {
			d1 ^= 1;
		}
		b_1ca21 = b_TAB_648c2[ d1 + 8 ];// FIXME is no used
	}

	// R_625a8
	private static void R_625a8(final Canvas g, final int d1) {
		jPrintText( g, Text.b_TAB_625ce[ d1 ] );
	}

	// R_1b82a
	/**
	 *
	 * @param g Canvas
	 * @param d1 index of {@link Text#b_TAB_625ce}
	 */
	private static void R_1b82a(final Canvas g, int d1) {
		final boolean backup_1 = b_or_char_with_screen;
		final byte backup_2 = b_print_column;
		final byte backup_3 = b_print_row;

		// print error message on the screen2 only
		b_print_text_on_screen_2 = true;// java
		// move.l	lw_screen2,-(sp)
		// move.l	lw_screen1,lw_screen2
		// move.w	d1,-(sp)
		b_or_char_with_screen = false;
		setTextColor( 1 );
		setBackgroundColor( 3 );

		// java: 2 texts together, print on 2nd screen
		R_625a8( g, 10 );// new line
		R_625a8( g, 11 );// 208 Disc error: retry or escape
		b_print_text_on_screen_2 = false;// java

		R_625a8( g, d1 );
		// move.l	(sp)+,lw_screen2
		b_print_row = backup_3;
		b_print_column = backup_2;
		b_or_char_with_screen = backup_1;
	}

	/**
	 * 10 : current load file index
	 * 11 : current save file index
	 * 26, 27, 28, 29 : '/','R','B','5'
	 * i * 16 + 15 : 0, marker
	 */
	private static final byte[] b_TEXT_7a21a = {
			'D', 'O', 'S', (byte) 128,'D', 'O', 'S', (byte) 129,'D', 'O', 'S', (byte) 130,'D', 'O', 'S', (byte) 131,
			'D', 'O', 'S', (byte) 132,'D', 'O', 'S', (byte) 133,'D', 'O', 'S', (byte) 134,'D', 'O', 'S', (byte) 135,
			'D', 'O', 'S', (byte) 136,'D', 'O', 'S', (byte) 137,'D', 'O', 'S', (byte) 138,'D', 'O', 'S', (byte) 139,
			'D', 'O', 'S', (byte) 140,'D', 'O', 'S', (byte) 141,'D', 'O', 'S', (byte) 142,'D', 'O', 'S', (byte) 143,
			'D', 'O', 'S', (byte) 144,'D', 'O', 'S', (byte) 145,'D', 'O', 'S', (byte) 146,'D', 'O', 'S', (byte) 147,
			'D', 'O', 'S', (byte) 148,'D', 'O', 'S', (byte) 149,'D', 'O', 'S', (byte) 150,'D', 'O', 'S', (byte) 151,
			'D', 'O', 'S', (byte) 152,'D', 'O', 'S', (byte) 153,'D', 'O', 'S', (byte) 154,'D', 'O', 'S', (byte) 155,
			'D', 'O', 'S', (byte) 156,'D', 'O', 'S', (byte) 157,'D', 'O', 'S', (byte) 158,'D', 'O', 'S', (byte) 159,
			'D', 'O', 'S', (byte) 160,'D', 'O', 'S', (byte) 161,'D', 'O', 'S', (byte) 162,'D', 'O', 'S', (byte) 163,
			'D', 'O', 'S', (byte) 164,'D', 'O', 'S', (byte) 165,'D', 'O', 'S', (byte) 166,'D', 'O', 'S', (byte) 167,
			'D', 'O', 'S', (byte) 168,'D', 'O', 'S', (byte) 169,'D', 'O', 'S', (byte) 170,'D', 'O', 'S', (byte) 171,
			'D', 'O', 'S', (byte) 172,'D', 'O', 'S', (byte) 173,'D', 'O', 'S', (byte) 174,'D', 'O', 'S', (byte) 175,
			'D', 'O', 'S', (byte) 176,'D', 'O', 'S', (byte) 177,'D', 'O', 'S', (byte) 178,'D', 'O', 'S', (byte) 179,
			'D', 'O', 'S', (byte) 180,'D', 'O', 'S', (byte) 181,'D', 'O', 'S', (byte) 182,'D', 'O', 'S', (byte) 183,
			'D', 'O', 'S', (byte) 184,'D', 'O', 'S', (byte) 185,'D', 'O', 'S', (byte) 186,'D', 'O', 'S', (byte) 187,
			'D', 'O', 'S', (byte) 188,'D', 'O', 'S', (byte) 189,'D', 'O', 'S', (byte) 190,'D', 'O', 'S', (byte) 191,
			'D', 'O', 'S', (byte) 192,'D', 'O', 'S', (byte) 193,'D', 'O', 'S', (byte) 194,'D', 'O', 'S', (byte) 195,
			'D', 'O', 'S', (byte) 196,'D', 'O', 'S', (byte) 197,'D', 'O', 'S', (byte) 198,'D', 'O', 'S', (byte) 199,
			'D', 'O', 'S', (byte) 200,'D', 'O', 'S', (byte) 201,'D', 'O', 'S', (byte) 202,'D', 'O', 'S', (byte) 203,
			'D', 'O', 'S', (byte) 204,'D', 'O', 'S', (byte) 205,'D', 'O', 'S', (byte) 206,'D', 'O', 'S', (byte) 207,
			'D', 'O', 'S', (byte) 208,'D', 'O', 'S', (byte) 209,'D', 'O', 'S', (byte) 210,'D', 'O', 'S', (byte) 211,
			'D', 'O', 'S', (byte) 212,'D', 'O', 'S', (byte) 213,'D', 'O', 'S', (byte) 214,'D', 'O', 'S', (byte) 215,
			'D', 'O', 'S', (byte) 216,'D', 'O', 'S', (byte) 217,'D', 'O', 'S', (byte) 218,'D', 'O', 'S', (byte) 219,
			'D', 'O', 'S', (byte) 220,'D', 'O', 'S', (byte) 221,'D', 'O', 'S', (byte) 222,'D', 'O', 'S', (byte) 223,
			'D', 'O', 'S', (byte) 224,'D', 'O', 'S', (byte) 225,'D', 'O', 'S', (byte) 226,'D', 'O', 'S', (byte) 227,
			'D', 'O', 'S', (byte) 228,'D', 'O', 'S', (byte) 229,'D', 'O', 'S', (byte) 230,'D', 'O', 'S', (byte) 231,
			'D', 'O', 'S', (byte) 232,'D', 'O', 'S', (byte) 233,'D', 'O', 'S', (byte) 234,'D', 'O', 'S', (byte) 235,
			'D', 'O', 'S', (byte) 236,'D', 'O', 'S', (byte) 237,'D', 'O', 'S', (byte) 238,'D', 'O', 'S', (byte) 239,
			'D', 'O', 'S', (byte) 240,'D', 'O', 'S', (byte) 241,'D', 'O', 'S', (byte) 242,'D', 'O', 'S', (byte) 243,
			'D', 'O', 'S', (byte) 244,'D', 'O', 'S', (byte) 245,'D', 'O', 'S', (byte) 246,'D', 'O', 'S', (byte) 247,
			'D', 'O', 'S', (byte) 248,'D', 'O', 'S', (byte) 249,'D', 'O', 'S', (byte) 250,'D', 'O', 'S', (byte) 251,
			'D', 'O', 'S', (byte) 252,'D', 'O', 'S', (byte) 253,'D', 'O', 'S', (byte) 254,'D', 'O', 'S', (byte) 255
	};

	// FIXME R_62e86
	private static void R_62e86(final int d0, final int d1, final int d2, final int d3) {
	}

	// FIXME R_5ceb4
	private static void R_5ceb4() {
	}

	// R_62e78
	private static void R_62e78(final int d0, final int d1, final int d2, final int d3) {
		R_62e86( d0, d1, d2, d3 );
		R_5ceb4();
	}

	// set_finish_editing
	private static int setFinishEditing() {
		b_finish_file_name_editing = true;
		return -1;
	}

	// publish_drive_error_status
	/**
	 *
	 * @param g Canvas
	 * @param d0 0 OK
	 *    28 'Disc write protected',
	 *    34 'Drive not ready',
	 *    'Disc error: retry or escape'
	 * @return status
	 */
	private static int publishDriveErrorStatus(final Canvas g, int d0) {
		b_finish_file_name_editing = false;
		if( 0 == d0 ) {
			return d0;
		}
		// _label1
		int d1 = 2;// 44 'Drive not ready'
		if( 34 != d0 ) {
			d1 = 3;// 60 'Disc write protected'
			if( 28 != d0 ) {
				d1 = 11;// 212 'Disc error: retry or escape'
			}
		}
		// _label2
		R_1b82a( g, d1 );
		waitForKey();
		d0 >>>= 8;
		if( 69 == d0 ) {// 'E'
			return setFinishEditing();
		}
		setBackgroundColor( 3 );

		// java: 2 lines together
		R_1b82a( g, 8 );// 145 '                            '
		R_1b82a( g, 9 );

		delay( 50 );
		d0 = 1;
		return d0;
	}

	// R_62b80
	/**
	 *
	 * @param g Canvas
	 * @param ba0
	 * @param d0 as word
	 * @param d3 as word
	 */
	private static int R_62b80(final Canvas g, final byte[] ba0, int d0, int d3) {
		if( d3 < 800 ) {
			int d1 = d0;
			d0 = 0;
			int d2 = 1;
			// final byte[] a1 = 0x400;
			R_62e78( d0, d1, d2, d3 );
			d1 = 0;
			d2 = 0;

			return publishDriveErrorStatus( g, d0 );
		}
		d0 = (byte) 0x80;
		return publishDriveErrorStatus( g, d0 );
	}

	private static final String DATA_FILE_NAME_PREFIX = "StuntCarRacerData_";

	// save_data_on_disk
	/**
	 *
	 * @param g
	 * @param ba0 an array with data to save
	 * @param d0
	 * @return 0 ok, < 0 ?, > 0 wait disk ready ?
	 */
	private static int saveDataOnDisk(final Canvas g, final byte[] ba0, final int d0) {
		/* final int d3 = 1;
		return R_62b80( g, ba0, d0, d3 );
		*/
		// java variant to save into a file
		try( final FileOutputStream fos = new FileOutputStream( DATA_FILE_NAME_PREFIX + d0 ) ) {
			fos.write( ba0, 0, ba0.length );
		} catch( final Exception e ) {
			return 0;// TODO show error message
		}
		return 0;
	}

	// load_data_from_disk
	/**
	 *
	 * @param g Canvas
	 * @param ba0 an array to read data
	 * @param d0 ?
	 * @return 0 ok, < 0 no disk ?, > 0 wait data ?
	 */
	private static int loadDataFromDisk(final Canvas g, final byte[] ba0, final int d0) {
		/*
		final int d3 = 0;
		return R_62b80( g, ba0, d0, d3 );
		*/
		// java variant to read a file
		try( final FileInputStream fis = new FileInputStream( DATA_FILE_NAME_PREFIX + d0 ) ) {
			int offset = 0, read;
			while( (offset < ba0.length) && (read = fis.read( ba0, offset, ba0.length - offset )) >= 0 ) {
				offset += read;
			}
			// final int read = fis.read( ba0, 0, ba0.length );
			// if( read == ba0.length ) {
			//	return 0;
			// }
		} catch( final Exception ignored ) {
		}
		// return -1;
		return 0;
	}

	private static byte b_load_info_stage = 0;
	
	// R_62c28
	private static void R_62c28(final Canvas g) {
		R_62c28:
		do {
			b_load_info_stage = 1;

			// _label1
			do {
				b_file_name_empty = false;
				b_finish_file_name_editing = false;
				int d0 = 22;
				d0 = loadDataFromDisk( g, b_TEXT_7a21a, d0 );
				if( d0 < 0 ) {
					return;
				}
				if( 0 != d0 ) {
					continue R_62c28;
				}
				// _label2
				d0 = b_TEXT_7a21a[ 10 ];
				final int d3 = b_TEXT_7a21a[ 11 ];
				// '/','R','B','5'
				if( '/' == b_TEXT_7a21a[ 26 ]
					&& 'R' == b_TEXT_7a21a[ 27 ]
					&& 'B' == b_TEXT_7a21a[ 28 ]
					&& '5' == b_TEXT_7a21a[ 29 ] )
				{
					// _label6
					b_current_load_file_index = (byte) d0;
					b_current_save_file_index = (byte) d3;
					return;
				}
				b_file_name_empty = true;
				if( ! b_save ) {
					// load
					// d0 = 0;
					// d3 = 0;
					b_current_load_file_index = 0;
					b_current_save_file_index = 0;
					return;
				}
				// save
				// java: 2 lines together
				R_1b82a( g, 12 );// 240 'Warning: this disc has not'
				R_1b82a( g, 13 );// 'been used for game saving'

				--b_load_info_stage;
				if( b_load_info_stage < 0 ) {
					// d0 = 0;
					// d3 = 0;
					b_current_load_file_index = 0;
					b_current_save_file_index = 0;
					return;
				}
				d0 = releaseAndWaitForKey();
				// if( 0 == (d0 & 0xFF) ) {
					// d0 >>>= 8;
					if( KeyEvent.KEYCODE_ESCAPE == d0 ) {
						b_finish_file_name_editing = true;
						// d0 = 0;
						// d3 = 0;
						b_current_load_file_index = 0;
						b_current_save_file_index = 0;
						return;
					}
				// }
				// _label3
				clearMenu( g );
				b_print_fine_y = 1;
				int d1 = 0;// 0 LOAD game position
				if( b_save ) {
					// save
					d1 = 1;// 22 SAVE game position"
				}
				R_625a8( g, d1 );
				b_print_fine_y = 0;
			} while( true );
		} while( true );
	}

	// print_all_data_name_read
	private static void printAllDataNameRead() {
		b_or_char_with_screen = false;
		setBackgroundColor( 3 );
		b_file_name_color = 15;
		b_file_name_empty_color = 11;
		b_current_file_index = 0;

		// _loop
		do {
			printDataNameRead();
		} while( ++b_current_file_index != MAX_FILES );

		b_or_char_with_screen = true;
	}

	// print_file_data_name
	private static void printFileDataName() {
		b_input_text_char_position = 0;
		int d4 = 6;
		int d5 = b_current_file_index;
		if( d5 >= MAX_FILES ) {
			d5 = 0;
		}
		// _label1
		while( d5 >= FILES_IN_COLUMN ) {
			d5 -= 10;
			d4 += 10;
		}
		// _label2
		d5 += 11;
		b_print_column = (byte) d4;
		b_print_row = (byte) d5;
		int d0 = b_current_file_index << 4;
		final byte[] ba0 = b_TEXT_7a21a;
		// lea	(a0,d0.w),a0;// TODO java: what is the address used ?
		final int ba0i = d0;
		int d3 = 0;

		// _label3
		do {
			if( b_file_name_exists ) {
				d0 = opponents_names[ 12 ][ 1 + d3 ];
			} else {
				// _label4
				if( (0 == ba0[ ba0i + 15 ]) && (! b_file_name_empty) ) {
					setTextColor( b_file_name_color );
					d0 = ba0[ ba0i + d3 ];
					opponents_names[ 12 ][ 1 + d3 ] = (char) d0;
					if( d0 > ' ' ) {
						++b_input_text_char_position;
					} else {
						// _label5
						opponents_names[ 12 ][ 1 + d3 ] = ' ';
						setTextColor( b_file_name_empty_color );
						d0 = '_';
					}
				} else {
					// _label5
					opponents_names[ 12 ][ 1 + d3 ] = ' ';
					setTextColor( b_file_name_empty_color );
					d0 = '_';
				}
			}
			// _label6
			jPrintCharacter( (char) d0 );
		} while( ++d3 != 8 );
	}

	// print_data_name_read
	private static void printDataNameRead() {
		b_file_name_exists = false;
		printFileDataName();
	}

	// print_entered_file_data_name
	private static void printEnteredFileDataName() {
		b_file_name_exists = true;
		printFileDataName();
	}

	// FIXME R_62b22 read joystick
	private static void R_62b22() {

	}

	// R_5f6f2
	/**
	 *
	 * @param g Canvas
	 * @param d1 index in {@link Text#b_TEXT_5f636_11}
	 */
	private static void R_5f6f2(final Canvas g, int d1) {
		jPrintText( g, Text.b_TEXT_5f636_11[ d1 ] );
	}

	private static final byte[] b_TEXT_5f586 = {
			'D','I','R',' ','H','A','L','L','M','P','$',0
		};

	// R_5f4b6
	/**
	 * @param g Canvas
	 * @return true - cs
	 */
	private static boolean R_5f4b6(final Canvas g) {
		int d2 = 0;
		int d1 = 3;
		boolean is_set_code = true;// java to avoid goto
		_label1:
		do {
			int d0 = opponents_names[ 12 ][ 1 + d1 ];
			if( b_TEXT_5f586[ 4 + d1 ] != d0 ) {
				// bne _label2;
				// _label2
				d1 = 1;
				// _label3
				do {
					d0 = opponents_names[ 12 ][ 1 + d1 ];
					if( b_TEXT_5f586[ 8 + d1 ] != d0 ) {
						// bne _label4;
						is_set_code = false;
						break _label1;
					}
					--d1;
				} while ( d1 >= 0 );

				d2 = 1;
				// bra _label4;
				is_set_code = false;
				break;
			}
		} while( --d1 >= 0 );
		if( is_set_code ) {
			d2 = 0x40;
		}
		// _label4
		b_1bb6b = (byte) d2;
		if( ! b_save ) {
			// load
			// beq _label5;
			return false;
		}
		// save
		if( 0 != d2 ) {
			if( 1 != d2 ) {
				// bne _label5;
				return false;
			}
			if( 0 != b_multi_no_of_players ) {
				// _label5
				// andi.b	#%11110,ccr
				return false;
			}
		} else {
			// _label6
			if( 0 == b_multi_no_of_players ) {
				// beq _label5;
				return false;
			}
		}
		// _label7
		fourPrintFineY();

		setPrintColumnRow( 6, 22 );

		R_5f6f2( g, 6 );// 87 'File name is not suitable'
		releaseAndWaitForKey();

		// delete the previous text
		d1 = Text.b_TEXT_5f636_11[ 6 ].mText.length();// 25;
		// _label8
		do {
			jPrintCharacter( ASCII_DELETE );
		} while( --d1 != 0 );

		clearPrintFineY();
		// ori.b	#1,ccr
		return true;
	}

	// print_data_name
	private static void printDataName() {
		setBackgroundColor( 3 );
		b_print_column = b_backup_print_column;
		--b_print_column;
		jPrintCharacter( ' ' );
		b_file_name_color = 15;
		b_file_name_empty_color = 11;
		printDataNameRead();
		b_print_column -= 9;
	}

	// FIXME R_5f796
	/**
	 *
	 * @return true - cs
	 */
	private static boolean R_5f796() {
		/*
		int d2 = 0;
		move.l	w_road_data_offset,a0
		byte[] ba1 = b_DAT_7a81a;
		if( b_menu_bar_index >= 0 ) {
			int d0 = d2;
			d2 = 239;
			ba0[ d2 ] = d0;
			d2 = 31;
			ba0[ d2 ] = d0;
			d2 = 255;
			ba0[ d2 ] = d0;
			d2 = d0;
		}
		// _label3
		do {
			int d1 = 0;

			// _label4
			int d0 = ba0[ d2 ];
			lw_road_height_fp8 = d0;// byte only
			b_thousands = (byte) d1;
			d1 = b_joystick_reads;
			d0 = ba1[ d1 ];
			++b_joystick_reads;
			d1 = b_thousands;
			++d1;
			if( b_menu_bar_index < 0 ) {
				if( d1 == lw_road_height_fp8 ) {
					beq _label6;
				} else {
					bne _label4;
				}
			}
			// _label5
			d0 = lw_road_height_fp8;// byte only
			if( 0 != d0 ) {
				bne _label4;
			}
			d0 = d1;

			// _label6
			move.l w_road_data_offset, a0
			ba0[ d2 ] = (byte) d0;
			d0 = b_joystick_reads;
			d0 += ba1[ d1 ];
			b_joystick_reads = d0;
			if( 14 == d2 ) {
				++d2;
			}
			// _label7
			++d2;
		} while( d2 < 256 );// d2 != 0;
		if( b_menu_bar_index < 0 ) {
			d2 = 0xef;
			int d0 = ba0[ d2 ];
			d2 = 0xFF;
			d0 |= ba0[ d2 ];
			d2 = 31;
			d0 |= ba0[ d2 ];
			if( 0 != d0 ) {
				d0 = 0x81;
				b_1bb94 = (byte) d0;
				return true;
			}
		}
		// _label8
		*/
		return false;
	}

	private static short w_5f720 = 0x683b;

	// R_5f708
	/**
	 *
	 * @return d0 as byte
	 */
	private static int R_5f708() {
		int d0 = (w_5f720 << 2) + w_5f720;
		w_5f720 = (short) d0;
		d0 >>>= 2;
		return d0;
	}

	// FIXME R_5f730
	/**
	 *
	 * @return true - cs
	 */
	private static boolean R_5f730(int d0) {
		/*
		b_menu_bar_index = (byte) d0;
		move.l	#b_DAT_7a41a,w_road_data_offset
		w_5f720 = 0x683b;
		int d1 = 0;
		// _label1
		do {
			d0 = R_5f708();
			b_DAT_7a81a[ d1 ] = (byte) d0;
		} while( ++d1 < 256 );// d1 != 0

		int d2 = 15;
		if( b_menu_bar_index >= 0 ) {
			d0 = b_joystick_reads;
			move.l w_road_data_offset, a0
			move.b d0, (a0, d2.w)
			return R_5f796();
		}
		// _label2
		move.l	w_road_data_offset,a0
		move.b	(a0,d2.w),d0
		b_joystick_reads = d0;
		*/
		return R_5f796();
	}

	// R_5f722
	/**
	 *
	 * @return true - cs
	 */
	private static boolean R_5f722() {
		return R_5f730( 0 );
	}

	// R_5f72c
	/**
	 *
	 * @return true - cs
	 */
	private static boolean R_5f72c() {
		return R_5f730( 0x80 );
	}

	// save_or_read_data
	private static void saveOrReadData() {
		if( b_save ) {
			// save
			int d1 = 127;// TODO clear the code
			// _label1
			do {
				// java:
				final int temp = d1 / Text.OPPONENTS_NAME_FULL_LENGTH;
				b_DAT_7a41a[ 32 + d1 ] = (byte) opponents_names[ 4 + temp ] [ d1 % Text.OPPONENTS_NAME_FULL_LENGTH ];
				// end java

				// if( d1 < 5 * 12 ) {
					// b_DAT_7a41a[ 160 + d1 ] = b_won_races[ d1 ];
				// }
				// java version:
				if( (d1 >= 4 * 12) && (d1 < 5 * 12) ) {
					b_DAT_7a41a[ 160 + d1 ] = b_rating_racer_ids[ d1 ];
				}
				if( (d1 >= 3 * 12) && (d1 < 4 * 12) ) {
					b_DAT_7a41a[ 160 + d1 ] = b_total_points[ d1 ];
				}
				if( (d1 >= 2 * 12) && (d1 < 3 * 12) ) {
					b_DAT_7a41a[ 160 + d1 ] = b_raced[ d1 ];
				}
				if( (d1 >= 1 * 12) && (d1 < 2 * 12) ) {
					b_DAT_7a41a[ 160 + d1 ] = b_best_lap_races[ d1 ];
				}
				if( (d1 >= 0 * 12) && (d1 < 1 * 12) ) {
					b_DAT_7a41a[ 160 + d1 ] = b_won_races[ d1 ];
				}
				// end java

				// _label2
				if( d1 < 12 ) {
					b_DAT_7a41a[ 224 + d1 ] = b_DAT_1ca36[ d1 ];
				}
				// _label3
			} while( --d1 >= 0 );
			b_DAT_7a41a[ 220 ] = b_multi_no_of_players;
			R_5f722();
			return;
		}
		// load
		// _label4
		if( R_5f72c() ) {
			return;
		}
		int d1 = 127;
		// _label5
		do {
			int d0 = b_DAT_7a41a[ 32 + d1 ];

			// java:
			final int temp = d1 / Text.OPPONENTS_NAME_FULL_LENGTH;
			opponents_names[ 4 + temp ] [ d1 % Text.OPPONENTS_NAME_FULL_LENGTH ] = (char) d0;
			// end java

			// if( d1 < 5 * 12 ) {
			//	b_won_races[ d1 ] = b_DAT_7a41a[ 160 + d1 ];
			// }
			// java version:
			if( (d1 >= 4 * 12) && (d1 < 5 * 12) ) {
				b_rating_racer_ids[ d1 ] = b_DAT_7a41a[ 160 + d1 ];
			}
			if( (d1 >= 3 * 12) && (d1 < 4 * 12) ) {
				b_total_points[ d1 ] = b_DAT_7a41a[ 160 + d1 ];
			}
			if( (d1 >= 2 * 12) && (d1 < 3 * 12) ) {
				b_raced[ d1 ] = b_DAT_7a41a[ 160 + d1 ];
			}
			if( (d1 >= 1 * 12) && (d1 < 2 * 12) ) {
				b_best_lap_races[ d1 ] = b_DAT_7a41a[ 160 + d1 ];
			}
			if( (d1 >= 0 * 12) && (d1 < 1 * 12) ) {
				b_won_races[ d1 ] = b_DAT_7a41a[ 160 + d1 ];
			}
			// end java

			// _label6
			if( d1 < 12 ) {
				b_DAT_1ca36[ d1 ] = b_DAT_7a41a[ 224 + d1 ];
			}
			// _label7
		} while( --d1 >= 0 );
		b_multi_no_of_players = b_DAT_7a41a[ 220 ];

		// _label8
	}

	// R_5f04a
	private static void R_5f04a() {
		if( b_save ) {
			// save
			System.arraycopy( b_TAB_7a01a, 0, b_DAT_7a41a, 0, 192 );
			return;
		}
		// load
		System.arraycopy( b_DAT_7a41a, 0, b_TAB_7a01a, 0, 192 );
	}

	// R_5f87c
	/**
	 *
	 * @return trtue - cs
	 */
	private static boolean R_5f87c() {
		/*
		R_5f722();
		w_road_data_offset += 0x100;
		*/
		return R_5f796();
	}

	// R_5f892
	/**
	 *
	 * @return trtute - cs
	 */
	private static boolean R_5f892() {
		/*
		R_5f72c();
		w_road_data_offset += 0x100;
		*/
		return R_5f796();
	}

	// R_5f374
	private static void R_5f374(boolean d0_flag) {
		if( d0_flag ^ b_save ) {
			return;
		}
		if( b_1bb94 < 0 ) {
			return;
		}
		final int d0 = b_1bb6b;
		if( d0 < 0 ) {
			return;
		}
		if( 0 == d0 ) {
			R_5f04a();
			return;
		}
		if( 0x40 != d0 ) {
			saveOrReadData();
			return;
		}
		// _label2
		if( b_save ) {
			// save
			System.arraycopy( Text.b_TEXT_7a61a, 0, b_DAT_7a41a, 0, 256 );
			System.arraycopy( Text.b_TEXT_7a71a, 0, b_DAT_7a51a, 0, 256 );
			R_5f87c();
			return;
		}
		// load
		if( R_5f892() ) {
			return;
		}
		byte[] ba0 = b_DAT_7a41a;
		byte[] ba1 = Text.b_TEXT_7a61a;
		int d1 = 0;
		// _label6
		do {
			if( ba0[ d1 + 12 ] < ba1[ d1 + 12 ] ) {
				// bcs _label7;
				System.arraycopy( ba0, d1, ba1, d1, 16 );
			} else {
				if( ba0[ d1 + 12 ] != ba1[ d1 + 12 ] ) {
					// bne _label9;
				} else {
					if( ba0[ d1 + 13 ] < ba1[ d1 + 13 ] ) {
						// bcs _label7;
						System.arraycopy( ba0, d1, ba1, d1, 16 );
					} else {
						if( ba0[ d1 + 13 ] != ba1[ d1 + 13 ] ) {
							// bne _label9;
						} else {
							if( ba0[ d1 + 14 ] >= ba1[ d1 + 14 ] ) {
								// bcc _label9;
							} else {
								System.arraycopy( ba0, d1, ba1, d1, 16 );
							}
						}
					}
				}
			}
			d1 += 16;
			// _labela
		} while( d1 < 256 );// while( 0 != d1 )

		d1 = 0;

		ba0 = b_DAT_7a51a;
		ba1 = Text.b_TEXT_7a71a;

		// _labelb
		do {
			if( ba0[ 12 + d1 ] < ba1[ 12 + d1 ] ) {
				// bcs _labelc;
				System.arraycopy( ba0, d1, ba1, d1, 16 );
			} else {
				if( ba0[ 12 + d1 ] != ba1[ 12 + d1 ] ) {
					// bne _labele;
				} else {
					if( ba0[ 13 + d1 ] < ba1[ 13 + d1 ] ) {
						// bcs _labelc;
						System.arraycopy( ba0, d1, ba1, d1, 16 );
					} else {
						if( ba0[ 13 + d1 ] != ba1[ 13 + d1 ] ) {
							// bne _labele;
						} else {
							if( ba0[ 14 + d1 ] >= ba1[ 14 + d1 ] ) {
								// bcc _labele;
							} else {
								System.arraycopy( ba0, d1, ba1, d1, 16 );
							}
						}
					}
				}
			}
			d1 += 16;
			// _labelf
		} while( d1 < 256 );// while( 0 != d1 );
	}

	// R_62ade
	private static void R_62ade(final int d0) {
		b_current_file_index = (byte) d0;
		if( b_save ) {
			// save
			b_current_save_file_index = (byte) d0;
			return;
		}
		// load
		b_current_load_file_index = (byte) d0;
	}

	private static final int FILES_IN_COLUMN = 10;
	private static final int MAX_FILES = 3 * FILES_IN_COLUMN;

	// disk_view_go_next
	private static void diskViewGoNext() {
		int d0 = b_current_file_index;
		if( ++d0 >= MAX_FILES ) {
			d0 = 0;
		}
		R_62ade( d0 );
	}

	// disk_view_go_previous
	private static void diskViewGoPrevious() {
		int d0 = b_current_file_index;
		if( --d0 < 0 ) {
			d0 = MAX_FILES - 1;
		}
		R_62ade( d0 );
	}

	// disk_view_go_right
	private static void diskViewGoRight() {
		final int d0 = b_current_file_index + FILES_IN_COLUMN;
		if( d0 < MAX_FILES ) {
			R_62ade( d0 );
		}
	}

	// disk_view_go_left
	private static void diskViewGoLeft() {
		final int d0 = b_current_file_index - FILES_IN_COLUMN;
		if( d0 >= 0 ) {
			R_62ade( d0 );
		}
	}

	// R_62bbe
	private static void R_62bbe(final Canvas g) {
		int d0 = (w_current_file_index_plus_23 - 23) << 4;
		// System.arraycopy( opponents_names[ 12 ], 1, b_TEXT_7a21a, d0, 8 );
		for( int i = 0; i < 8; i++ ) {// TODO java: using chars
			b_TEXT_7a21a[ d0 + i ] = (byte) opponents_names[ 12 ][ 1 + i ];
		}
		b_TEXT_7a21a[ d0 + 15 ] = 0;// marker

		// _label2
		do {
			d0 = 22;
			// move.l	#$47826653, b_TEXT_7a21a + 26 '/', 'R', 'B', '5'
			b_TEXT_7a21a[ 26 ] = '/';
			b_TEXT_7a21a[ 27 ] = 'R';
			b_TEXT_7a21a[ 28 ] = 'B';
			b_TEXT_7a21a[ 29 ] = '5';
			b_TEXT_7a21a[ 10 ] = b_current_load_file_index;
			b_TEXT_7a21a[ 11 ] = b_current_save_file_index;
			d0 = saveDataOnDisk( g, b_TEXT_7a21a, d0 );
			if( 0 == d0 ) {
				break;
			}
		} while( d0 >= 0 );
	}

	private static byte b_current_file_index = 0;

	/** read joystick while text entered */
	private static boolean b_read_joystick = false;
	private static byte b_current_load_file_index = 0;
	private static byte b_current_save_file_index = 0;
	private static byte b_file_name_color = 15;
	private static byte b_file_name_empty_color = 11;
	private static byte b_backup_print_column = 0;
	private static boolean b_file_name_empty = false;
	private static boolean b_file_name_exists = false;

	private static short w_current_file_index_plus_23 = (short) 0x8000;

	// R_627ca
	private static void R_627ca(final Canvas g) {
		R_62c28( g );
		if( b_finish_file_name_editing ) {
			return;
		}
		// _label1
		printAllDataNameRead();
		b_or_char_with_screen = false;

		_label2:
		do {
			b_file_name_color = 15;
			b_file_name_empty_color = 11;
			int d0 = b_current_load_file_index;
			if( b_save ) {
				// save
				d0 = b_current_save_file_index;
			}
			b_current_file_index = (byte) d0;
			b_file_name_color = 0;
			b_file_name_empty_color = 0;
			setBackgroundColor( 2 );
			printDataNameRead();
			b_print_column -= 9;
			setTextColor( 0 );
			jPrintCharacter( '>' );
			R_62b22();// wait until an arrow key pressed?
			b_backup_print_column = b_print_column;
			if( ! b_save ) {
				// load
				_label4: {
					do {
						do {
							d0 = waitForKey();
							encodeJoystick( d0 );
							if( 0 != b_joystick_directions ) {// joystick is used
								// bne _label6;
								break _label4;
							}
						} while( KeyEvent.KEYCODE_ENTER != d0 );
					} while( ' ' == opponents_names[ 12 ][ 1 ] );
					// bra _labelb;
					// _labelb
					if( R_5f4b6( g ) ) {
						continue _label2;
					}
					break _label2;
				}
			} else {
				// save
				// _label5
				do {
					setTextColor( 0 );
					setBackgroundColor( 2 );
					inputText( 12, 8 );// inputText( 192, 8 );

					R_1b82a( g, 8 );// 145 '                            '
					if( 0 == b_joystick_directions ) {
						if( ' ' != opponents_names[ 12 ][ 1 ] ) {
							// _labelb
							if( R_5f4b6( g ) ) {
								continue _label2;
							}
							break _label2;
						}
						R_1b82a( g, 7 );// 127 'Type in file name'
						b_input_text_char_position = 0;
						continue;
					}
					break;
				} while( true );
			}
			// _label6
			if( 0 != (b_joystick_directions & 0x80) ) {// escape
				setFinishEditing();
				return;
			}
			printDataName();
			if( 0 != (b_joystick_directions & 0x40) ) {// down
				// _label9
				diskViewGoNext();
				// _labela
				continue;
			}
			if( 0 != (b_joystick_directions & 0x20) ) {// up
				// _label8
				diskViewGoPrevious();
				continue;
			}
			if( 0 != (b_joystick_directions & 0x08) ) {// right
				// _label7
				diskViewGoRight();
				continue;
			}
			diskViewGoLeft();
		} while( true );
		setBackgroundColor( 2 );
		setTextColor( 1 );
		printEnteredFileDataName();
		R_5f374( true );
		w_current_file_index_plus_23 = (short) ((b_current_file_index & 0xFF) + 23);

		// _labelc
		do {
			final int d0 = w_current_file_index_plus_23;
			final byte[] ba0 = b_DAT_7a41a;
			if( ! b_save ) {
				// load
				// _labelf
				final int ret = loadDataFromDisk( g, ba0, d0 );
				if( 0 == ret ) {
					// _label10
					b_current_load_file_index = b_current_file_index;
					R_62bbe( g );
					R_5f374( false );
					return;
				}
				if( ret < 0 ) {
					return;
				}
				continue;
			}
			// save
			final int ret = saveDataOnDisk( g, ba0, d0 );
			if( 0 == ret ) {
				// _labeld
				int d3 = b_current_file_index;
				if( ++d3 >= MAX_FILES ) {
					d3 = 0;
				}
				// _labele
				b_current_save_file_index = (byte) d3;
				R_62bbe( g );
				return;
			}
			if( ret < 0 ) {
				return;
			}
		} while( true );
	}

	// menu_load_or_save
	private static void menuLoadOrSave(final Canvas g) {
		clearMenu( g );
		copyScreenPart( g );

		R_1b82a( g, 14 );// 298 'Insert formatted game save disc'
		// java added because the previous text does not have the end marker
		R_1b82a( g, 15 );// 'into drive 0.'
		R_1b82a( g, 16 );// 'Press any key to continue'

		releaseAndWaitForKey();
		clearMenu( g );
		setTextColor( 15 );
		int d1 = 0;// 0 LOAD game position
		if( b_save ) {
			// save
			d1 = 1;// 22 SAVE game position
		}
		R_625a8( g, d1 );
		copyScreenPart( g );
		// move.l	lw_screen2,-(sp)
		// move.l	lw_screen1,lw_screen2

		b_read_joystick = true;
		R_627ca( g );
		b_read_joystick = false;

		setBackgroundColor( 3 );
		// move.l	(sp)+,lw_screen2
	}

	// FIXME R_5823c
	private static void R_5823c() {
	}

	// menu_game
	private static void menuGame(final Canvas g) {
		/*
		// R_5ba68
		if( 0 == d0 ) {
			R_5fbc6();
			R_5baea();
			return;
		}
		// R_5ba78
		R_5fb62();
		if( d0 >= 2 ) {
			bcc	R_5baea;
		}
		b_race_mode = d0;
		b_5eb7d = 1;
		d0 = (b_5eb7c << 1) + b_race_mode;
		R_5bcf0();
		delay20();
		return;
		*/
		// R_5baea
		do {
			b_joystick_directions = 0;
			if( 0 != b_5eb7d ) {
				// bne R_5ba78;
				int d0 = menuSelectDivision( g );
				if( d0 >= 2 ) {
					continue;
				}
				b_race_mode = (byte) d0;
				b_5eb7d = 1;
				d0 = (b_5eb7c << 1) + b_race_mode;
				getRoadId( d0 );
				delay20();
				return;
			}

			int d0 = 1;
			if( b_race_mode < 0 ) {
				d0 = 2;
			}
			// _label1
			// 0 hall of fame, 1 practise, 2 start the racing season, 3 load/save/replay
			d0 = getMainMenuSelection( g, d0, 0, 3 );
			if( 2 == d0 ) {// racing
				// beq R_5baae;
				// R_5baae
				R_609ae( 0 );
				b_race_mode = (byte) 0x80;
				return;
			}
			if( d0 < 2 ) {
				// blt R_5ba68;
				// R_5ba68
				if( 0 == d0 ) {// hall of fame
					displayHallOfFames();
					continue;
				}
				// practice, choose a division and a road
				// R_5ba78
				d0 = menuSelectDivision( g );
				if( d0 >= 2 ) {// cancel
					continue;// bcc	R_5baea;
				}
				b_race_mode = (byte) d0;
				b_5eb7d = 1;
				d0 = (b_5eb7c << 1) + d0;// b_race_mode;
				getRoadId( d0 );
				delay20();
				return;
			}

			delay20();
			if( SLAVE == b_machine ) {
				// R_582f4();
				// java: extracted in place because goto is used
				b_1bb94 = 0;
				pleaseWait( g );
				b_57c64 = (byte) 0xB2;
				R_579b0();
				if( !testKeyF5() ) {
					b_1bb6b = b_57c5f;
					if( b_57c60 >= 0 ) {
						// _label3
						if( 32 == b_57c60 ) {
							// beq R_5bae4;
							// R_5bae4
							delay20();
							continue;
						}
						if( 0 != (b_57c60 & 0x10) ) {
							if( 0 == (b_57c60 & 0x08) ) {
								if( 0 == (b_57c60 & 0x01) ) {
									b_save = false;// load
									R_5837e();
								}
							}
						}
						// _label4
						// R_5bc90
						if( 0x40 != b_1bb6b ) {
							// R_5bc9c
							displayPlayers( g );
						}
						continue;
					}
					mainInitialisation2();
					if( 0xc0 != (b_57c60 & 0xFF) ) {
						// _label2
						// bra R_5bbb2;
						// R_5bbb2
						if( (0 != b_multi_no_of_players) || R_609ae( 0x80 ) ) {
							// _label2
							mainInitialisation();
							showMenuScreen( false );
						}
						// R_5bc9c
						displayPlayers( g );
						continue;
					}
				}
				// _label1
				// bra	R_5bb8c;
				// R_5bb8c
				mainGameSelection();
				// R_5bb92
				if( b_machine == MASTER ) {
					b_57c66 = (byte) 0x80;
					b_57c64 = (byte) 0xB2;
					R_579b0();
				}
				// R_5bbb2
				if( (0 != b_multi_no_of_players) || R_609ae( 0x80 ) ) {
					// _label2
					mainInitialisation();
					showMenuScreen( false );
				}
				// R_5bc9c
				displayPlayers( g );
				continue;
			}
			// 0 load, 1 save, 2 replay, 3 cancel
			d0 = getMainMenuSelection( g, 3, 4, 3 );
			if( d0 >= 2 ) {
				// 2 replay, 3 cancel
				if( 2 != d0 ) {
					// cancel
					// bne R_5bac4;
					// R_5bac4
					if( b_machine == MASTER ) {
						b_57c66 = 0x20;
						b_57c64 = (byte) 0xB2;
						R_579b0();
					}
					// R_5bae4
					delay20();
					continue;
				}

				mainInitialisation2();
				if( testKey( KeyEvent.KEYCODE_F1 ) ) {// F1
					if( b_machine == MASTER ) {
						b_57c66 = (byte) 0xC0;
						b_57c64 = (byte) 0xB2;
						R_579b0();
					}
					// R_5bb8c
					mainGameSelection();
				}
				// R_5bb92
				if( b_machine == MASTER ) {
					b_57c66 = (byte) 0x80;
					b_57c64 = (byte) 0xB2;
					R_579b0();
				}
				// R_5bbb2
				if( (0 != b_multi_no_of_players) || R_609ae( 0x80 ) ) {
					// _label2
					mainInitialisation();
					showMenuScreen( false );
				}
			} else {
				// 0 load, 1 save
				// R_5bbda
				b_save = (d0 == 1);
				d0 <<= 2;
				d0 += 8;
				// d1 = d0;

				// _label3
				System.arraycopy( b_TAB_7a01a, 0, b_road_under_map, 0, 256 );

				b_1bbd0 = b_1ca34;
				delay20();
				R_609ae( 0 );
				menuLoadOrSave( g );
				if( 0 == b_1bb6b ) {
					if( !b_finish_file_name_editing && !b_save ) {
						if( R_609ae( 0x80 ) ) {
							b_1bb94 = (byte) 0x81;
							// _label4
							R_5bca8();
						}
					} else {
						// _label4
						R_5bca8();
					}
				}
				// _label5
				if( 0x81 == (b_1bb94 & 0xFF) ) {
					R_1b82a( g, 6 );// 104 Incorrect data found
					waitForKey();
				}
				// _label6
				if( MASTER == b_machine ) {
					R_5823c();
				}
				// R_5bc90
				if( 0x40 == b_1bb6b ) {
					continue;// beq R_5bca2;
				}
			}
			// R_5bc9c
			displayPlayers( g );

			// R_5bca2
		} while( true );// R_5baea;
	}

	private static byte b_menu_bar_type = 2;// 	dc.b	2,0

	// java: is not required, pointers to bars
	/* private static final int[] lw_TAB_64c1a = {
			0x4070, 0x4f70, 0x5e70, 0x6d70, 0x3170
	}; */

	// clear_print_fine_y
	private static void clearPrintFineY() {
		b_print_fine_y = 0;
	}

	// four_print_fine_y
	private static void fourPrintFineY() {
		b_print_fine_y = 4;
	}

	// menu_bar_positions
	/**
	 * Rows for menu bars
	 */
	private static final byte[] b_menu_bar_positions = {
			13, 16, 19, 22,
			16, 19,
			16,
			15, 20, 23,
			10, 14, 18, 22,
			14,
			11, 17,
			12
	};

	// FIXME do_fill_bar
	private static void doFillBar(final Canvas g, int bd0) {
		int b_menu_bar_height = (byte) (bd0 - 2);

		b_print_column = 5;
		b_print_row = b_current_print_row = b_menu_bar_positions[ b_menu_bar_index ];// row
		// String bar;
		int color;
		switch( b_menu_bar_type ) {// index to lw_TAB_64c1a
			case 0:
				color = 0xFF995533;
				// bar = "bar_dark_orange";
				break;
			case 1:
				color = 0xFFff9900;
				// bar = "bar_light_orange";
				break;
			case 2:
				color = 0xFFbbbbbb;
				// bar = "bar_gray";
				break;
			case 4:
				color = 0xFF559900;
				// bar = "bar_wide_green";
				break;
			default:
				// bar = null;
				color = Color.BLACK;
				break;
		}
		// final BufferedImage img = jGetImage( bar );
		final int x = (b_print_column * Text.COLUMN_NORMAL_FONT_WIDTH - 3) * PF;
		int y = ((b_print_row + b_menu_bar_height) * Text.TEXT_ROW_HEIGHT - 9) * PF;
		final int top_y = y - ((Text.TEXT_ROW_HEIGHT * b_menu_bar_height) * PF);
		// 1: width: 222, height: 17
		// top
		paint.setColor( Color.WHITE );
		g.drawRect( x + 1 * PF, top_y, x + 1 * PF + 221 * PF, top_y + 1 * PF, paint );
		// bottom
		paint.setColor( Color.BLACK );
		g.drawRect( x + 1 * PF, y + 16 * PF, x + 1 * PF + 221 * PF, y + 16 * PF + 1 * PF, paint );
		// left
		paint.setColor( 0xFFBBBBBB );
		g.drawRect( x + 0 * PF, top_y, x + 0 * PF + 1 * PF, top_y + y - top_y + 16 * PF, paint );
		// right
		g.drawRect( x + 222 * PF, top_y, x + 222 * PF + 1 * PF, top_y + y - top_y + 16 * PF, paint );
		// fill
		paint.setColor( color );
		g.drawRect( x + 1 * PF, top_y + 1 * PF, x + 1 * PF + 221 * PF, top_y + 1 * PF + y - top_y + (17 - 2) * PF, paint );
		for( ; b_menu_bar_height >= 0; --b_menu_bar_height ) {
			/*
			g.drawImage( img,
					b_print_column * Text.COLUMN_NORMAL_FONT_WIDTH - 3,
					(b_print_row + b_menu_bar_height) * Text.TEXT_ROW_HEIGHT - 9,
					img.getWidth(), img.getHeight(), null );
			*/
			// nails, left/right
			paint.setColor( 0xFFBBBBBB );
			g.drawRect( x + 1 * PF, y + 7 * PF, x + 1 * PF + 1 * PF, y + 7 * PF + 1 * PF, paint );
			g.drawRect( x + 220 * PF, y + 7 * PF, x + 220 * PF + 1 * PF, y + 7 * PF + 1 * PF, paint );
			paint.setColor( Color.WHITE );
			g.drawRect( x + 2 * PF, y + 7 * PF, x + 2 * PF + 1 * PF, y + 7 * PF + 1 * PF, paint );
			g.drawRect( x + 221 * PF, y + 7 * PF, x + 221 * PF + 1 * PF, y + 7 * PF + 1 * PF, paint );
			paint.setColor( Color.BLACK );
			g.drawRect( x + 1 * PF, y + 8 * PF, x + 1 * PF + 1 * PF, y + 8 * PF + 1 * PF, paint );
			g.drawRect( x + 220 * PF, y + 8 * PF, x + 220 * PF + 1 * PF, y + 8 * PF + 1 * PF, paint );
			paint.setColor( 0xFF555555 );
			g.drawRect( x + 2 * PF, y + 8 * PF, x + 2 * PF + 1 * PF, y + 8 * PF + 1 * PF, paint );
			g.drawRect( x + 221 * PF, y + 8 * PF, x + 221 * PF + 1 * PF, y + 8 * PF + 1 * PF, paint );
			//
			y -= Text.TEXT_ROW_HEIGHT;
		}
/*
		int sd4 = b_64c18 << 2;// positive value
		short[] sa0 = title_crunched;
		int sa0i = -32 * 40;// -32y
		sa0i += lw_TAB_64c1a[ sd4 / 4 ];
		short[] sa3 = screen2;
		int sa3i = -8 * 40;// -8y
		sa3i += (((y & 0xff) * 40) << 3) - 40 + 4;
		int sa2i = sa0i + 160;
		int sa4i = sa3i + 40;
		sd4 = 17 - 1;
		if( 3 == b_64c18 ) {
			sd4 = 25 - 1;
		}
		do {
			do {
				int sd3 = 14 - 1;
				do {
					// copy 14 * 16 px = 224 px line from title img to screen1 position
					move.w( a0 ) +, (a3) +
					move.w( a0 ) +, 7998 (a3)
					move.w( a0 ) +, 15998 (a3)
					move.w( a0 ) +, 23998 (a3)
				} while( --sd3 >= 0 );
				sa0i += 48;
				sa3i += 12;
			} while( --sd4 >= 0 );
			if( --b_menu_bar_height < 0 ) {
				break;
			}
			sa0i = sa2i;
			sa4i += 8 * 40;// + 8y
			sa3i = sa4i;
			sd4 = 16 - 1;
		} while( true );
*/
		b_menu_bar_type = 2;

		setTextColor( 0 );
		b_menu_bar_index++;
		fourPrintFineY();
	}

	// fill_bar
	private static void fillBar(final Canvas g) {
		doFillBar( g, 2 );
	}

	// R_64b06
	private static void R_64b06(final Canvas g) {
		b_menu_bar_type = 3;
		fillBar( g );
	}

	private static final byte[] b_track_name_print_x_offsets = { 15, 13, 16, 16, 16, 15, 16, 13 };

	// print_name
	/**
	 *
	 * @param g Canvas
	 * @param a0 text table
	 * @param bd1 index for the text table
	 * @param bd2 how many chars to print
	 */
	private static void printName(final Canvas g, final char[][] a0, int bd1, int bd2) {
		jPrintText( g, String.valueOf( a0[ bd1 ], 0, bd2 ) );
	}

	// print_track_name
	/**
	 *
	 * @param g Canvas
	 * @param bd1 index for {@link #track_names}
	 */
	private static void printTrackName(final Canvas g, int bd1) {
		printName( g, track_names, bd1, TRACK_NAME_LENGTH );
	}

	// print_opponents_name
	/**
	 *
	 * @param g Canvas
	 * @param bd1 index for {@link #opponents_names}
	 */
	private static void printOpponentsName(final Canvas g, int bd1) {
		printName( g, opponents_names, bd1, Text.OPPONENTS_NAME_STORE_LENGTH - 1 );
	}

	// R_648d2
	/**
	 * Prints "xxx V xxx"
	 *
	 * @param d0 as byte
	 */
	private static void R_648d2(final Canvas g, int d0) {
		b_menu_bar_index = (byte) d0;
		chooseRacersInDivision();
		fillBar( g );

		printOpponentsName( g, b_racer_id_1 );
		printLeagueText( g, 4 );// 40 ' V '
		printOpponentsName( g, b_racer_id_2 );

		setTextColor( 1 );
		clearPrintFineY();
	}

	// copy_screen_and_wait_for_fire
	private static void copyScreenAndWaitForFire(final Canvas g) {
		copyScreenPart( g );
		waitForFire();

		clearPrintFineY();
	}

	// display_race_result is not used
	/**
	 * Print race result
	 *
	 * @param g
	 */
	private static void displayRaceResult(final Canvas g) {
		R_648d2( g, 7 );

		printLeagueText( g, 7 );// 'RESULT'

		fillBar( g );

		printLeagueText( g, 8 );// 'Race Winner: '

		printOpponentsName( g, b_1ca25 );

		printLeagueText( g, 16 );// ' 2pts'

		fillBar( g );

		printLeagueText( g, 9 );// 'Fastest Lap: '

		printOpponentsName( g, b_1ca26 );

		printLeagueText( g, 17 );// ' 1pt'

		copyScreenAndWaitForFire( g );
	}

	// print_road_title
	/**
	 * Print road title
	 *
	 * @param g Canvas
	 */
	private static void printRoadTitle(final Canvas g) {
		fourPrintFineY();

		Text.league_text[ 6 ].mX = b_track_name_print_x_offsets[ b_road_id ];// 'The '
		printLeagueText( g, 6 );// 'The '

		printTrackName( g, b_road_id );
		clearPrintFineY();
	}

	// make_car_screens
	/**
	 * Draw car view on both screens
	 */
	private static void makeCarScreens(final Canvas g) {
		final Bitmap img = jGetImage( R.drawable.car );
		// screen1.getGraphics().drawImage( img, 0, 0, null );
		// screen2.getGraphics().drawImage( img, 0, 0, null );
		g.drawBitmap( img, 0, 0, paint );
	}

	// initialise_damage_bar
	private static void initialiseDamageBar() {
		if( 0 != b_multi_no_of_players ) {
			b_damage_hole_position = b_DAT_1ca36[ b_racer_id_3 ];
		}
		initialiseDamageBar2();
	}

	// copy_damage_graphic
	/**
	 * @param d1 damage graphics index to {@link #graphic_info} as byte
	 * @param d2 hole position, as byte
	 */
	private static void copyDamageGraphic(int d1, int d2) {
		// screen1.getGraphics().setClip( null );
		// screen2.getGraphics().setClip( null );
		// move.l	#graphic.info+8,a0// x position in words
		int d0 = ((d2 << 1) + d2 + 6);// d2*3+6
		if( 0 != (d0 & 0x01) ) {
			++d1;
		}
		d0 = (d0 >>> 1) & 0xFF;// ((d2 * 3 + 6) / 2)
		// int d3 = d1 << 4;
		graphic_info[ d1 ].mX = (8 + (d0 << 4)) * PF;// java +8

		// move.l	screen2,-(sp)
		// move.l	screen_mem,screen2
		copyGraphic( screen1_canvas, d1 );// screen 1

		// move.l	#32000,d0
		// add.l	d0,screen2
		copyGraphic( screen2_canvas, d1 );// screen 2

		// move.l	(sp)+,screen2
		// move.w	(sp)+,d1
	}

	// copy_damage_hole
	/**
	 *
	 * @param d2 hole position, as byte
	 */
	private static void copyDamageHole(final int d2) {
		copyDamageGraphic( SPRITE_HOLE_1, d2 );
	}

	// copy_damage_hole_smashed
	/**
	 *
	 * @param d2 hole position, as byte
	 */
	private static void copyDamageHoleSmashed(final int d2) {
		copyDamageGraphic( SPRITE_SMASH_1, d2 );
	}

	// TODO verify copy_damage_clear
	/**
	 * @param d2 as byte
	 */
	private static void copyDamageClear(final int d2) {
		copyDamageGraphic( SPRITE_DAMAGE_BAR_CLEAR_1, d2 );
	}

	// initialise_damage_bar2
	private static void initialiseDamageBar2() {
		int x = 9;
		do {
			if( x < b_damage_hole_position ) {
				copyDamageClear( x );
			} else {
				copyDamageHole( x );
			}
		} while( --x >= 0 );
	}

	// print_lap_boost_text
	/**
	 * Print "L  B"
	 *
	 * @param g Canvas
	 */
	private static void printLapBoostText(final Canvas g) {
		// print "L B ";
		b_print_fine_y = 2;
		b_print_fine_x = 2;
		b_print_column = 5;
		b_print_row = 22;
		jPrintCharacter( 'L' );

		b_print_fine_x += 2;
		b_print_column = 8;
		b_print_row = 22;
		jPrintCharacter( 'B' );

		b_print_fine_y = 0;
		b_print_fine_x = 0;
	}

	// copy_watch_or_flag
	/**
	 * Draw watch of flag in both screen buffers
	 *
	 * @param d0 index of the image sprite in graphic_info in d0 as byte
	 */
	private static void copyWatchOrFlag(final int d0) {
		if( !b_print_text_on_screen_2 ) {// java: have to draw on screen 2 onl for fade in
			// to screen 1
			copyGraphic( screen1_canvas, d0 );
		}
		// to screen 2
		copyGraphic( screen2_canvas, d0 );
	}

	// copy_chequered_flag_dull
	/**
	 *
	 * @param d2 as byte. not 7 - flag is dull
	 */
	private static void copyChequeredFlagDull(final int d2) {
		int d0 = SPRITE_CHEQUERED_FLAG_BRIGHT;// chequered flag bright
		if( 7 != d2 ) {
			d0 = SPRITE_CHEQUERED_FLAG_DULL;// chequered flag dull
		}
		copyWatchOrFlag( d0 );
	}

	// copy_chequered_flag
	/**
	 *
	 * @return d2 as byte. 7 - winning. 11 - otherwise
	 */
	private static int copyChequeredFlag() {
		byte d2 = 11;
		if( calculateIfWinning() ) {// FIXME Amiga bug: shows the bright flag in practice
			d2 = 7;// if winning
		}
		copyChequeredFlagDull( d2 );
		return d2;
	}

	// copy_stop_watch
	/**
	 * Draw watch image on both screens
	 *
	 * @param d2 as byte. 7 - winning. 11 otherwise
	 */
	private static void copyStopWatch(final int d2) {
		int d0 = SPRITE_STOP_WATCH_BRIGHT;// stop watch bright
		if( d2 != 7 ) {
			d0 = SPRITE_STOP_WATCH_DULL;// stop watch dull
		}
		copyWatchOrFlag( d0 );
	}

	// copy_stop_watch0
	private static void copyStopWatch0() {
		int d2 = 11;
		if( 0 == b_1bbb2 ) {
			d2 = 7;
		}
		copyStopWatch( d2 );
	}

	private static final int SPEED_BAR_X = 96 * PF;
	private static final int SPEED_BAR_Y = 176 * PF;
	private static final int SPEED_BAR_HEIGHT = 2 * PF;
	private static final int SPEED_BAR_MAX_LENGTH = 128 * PF;

	// display_speed_bar
	/**
	 * Dra speed bar in both screens
	 */
	private static void displaySpeedBar() {
		int width = w_players_z_speed - 0x1100;
		if( width < 0 ) {
			width = 0;
		}
		width *= 0xB700;
		width >>>= 23;// reduce (183 / 256)
		width *= PF;
		if( width >= SPEED_BAR_MAX_LENGTH ) {// max. length on screen
			width -= SPEED_BAR_MAX_LENGTH;
		}
		w_new_speed_bar = (short) width;
		width -= w_old_speed_bar;
		if( width != 0 ) {
			int line_start = SPEED_BAR_X;
			int color;
			if( width < 0 ) {
				width = -width;
				line_start += w_new_speed_bar;
				color = SPEED_BAR_BACKGROUND_COLOR;
			} else {
				line_start += w_old_speed_bar;
				color = SPEED_BAR_COLOR;
			}
			Canvas g = screen1_canvas;
			paint.setColor( getRGB( color ) );
			g.drawRect( line_start, SPEED_BAR_Y, line_start + width, SPEED_BAR_Y + SPEED_BAR_HEIGHT, paint );
			g = screen2_canvas;
			paint.setColor( getRGB( color ) );
			g.drawRect( line_start, SPEED_BAR_Y, line_start + width, SPEED_BAR_Y + SPEED_BAR_HEIGHT, paint );
		}
		w_old_speed_bar = w_new_speed_bar;
	}

	// new_lap_sub
	/**
	 *
	 * @param d1_id playerID. 0 - player, 1 - opponent, d1 as word
	 */
	private static void newLapSub(int d1_id) {
		int d3 = 0;
		switch( d1_id ) {
			case 0:
				d3 = w_players_distance_into_section;
				break;
			case 1:
				d3 = w_opponents_distance_into_piece;
				break;
		}
		if( 0 == (d3 >> 8) ) {
			int d0 = (d3 & 0xFF) ^ 0xFF;

			d0 = ((d0 & 0xFF) * 13) >>> 8;
			if( d0 >= 10 ) {
				d0 += 6;
			}

			addToLapTime1( d0, d1_id );
		}
	}

	// sort_time_result
	/**
	 * If time at d2 is lower than time at d1,
	 * replace time ar d2 by time at d1
	 *
	 * @param d1 as byte, 0 - player, 1 - opponent
	 * @param d2 as byte, 0...16
	 * @return true - cs, false - cc
	 */
	private static boolean sortTimeResult(int d1, int d2) {;
		int d0 = b_time_results_min[ d1 ];// time, s, high byte
		if( d0 >= b_time_results_min[ d2 ] ) {
			if( d0 != b_time_results_min[ d2 ] ) {
				return true;
			}
			d0 = b_time_results_sec[ d1 ];// time, s, low byte
			if( d0 >= b_time_results_sec[ d2 ] ) {
				if( d0 != b_time_results_sec[ d2 ] ) {
					return true;
				}
				d0 = b_time_results_ms[ d1 ];// time, ms
				if( d0 >= b_time_results_ms[ d2 ] ) {
					return true;
				}
			}
		}
		b_time_results_ms[ d2 ] = b_time_results_ms[ d1 ];
		b_time_results_sec[ d2 ] = b_time_results_sec[ d1 ];
		b_time_results_min[ d2 ] = b_time_results_min[ d1 ];
		return false;
	}

	// add_time
	/**
	 * Time at index 2 += Time at index 1
	 *
	 * @param d1 index 1
	 * @param d2 index 2
	 */
	private static void addTime(final int d1, final int d2) {
		// xx xx. low byte first
		// Amiga uses BCD format.
		// java: binary format
		int carry = 0;
		int d0 = (b_time_results_ms[ d1 ] + b_time_results_ms[ d2 ]);
		if( d0 >= 100 ) {
			d0 = 0;
			carry = 1;
		}
		b_time_results_ms[ d2 ] = (byte)d0;

		d0 = b_time_results_sec[ d1 ] + b_time_results_sec[ d2 ] + carry;
		carry = 0;
		if( d0 >= 60 ) {
			d0 = 0;
			carry = 1;
		}
		b_time_results_sec[ d2 ] = (byte)d0;

		d0 = b_time_results_min[ d1 ] + b_time_results_min[ d2 ] + carry;
		b_time_results_min[ d2 ] = (byte)d0;
	}

	// R_5f21e
	/**
	 * Copy opponent name into a text template
	 *
	 * in: {@link #b_racer_id_3} as byte, player/opponent id for {@link #opponents_names}.
	 * @param d2 a text template id for {@link Text#main_game_selection_text}.
	 */
	private static void R_5f21e(final int d2) {
		final int d1 = b_racer_id_3;// java index << 4;
		// start space is omitted
		Text.main_game_selection_text[ d2 ].mText =
				String.valueOf( opponents_names[ d1 ], 1, Text.RACER_NAME_LENGTH );
	}

	// process_time
	/**
	 *
	 * @param d1 as byte. 0 - player, 1 - opponent
	 */
	private static void processTime(int d1) {
		if( ! sortTimeResult( d1, 12 ) ) {
			b_best_lap_racer_id = b_racer_id_3;
		}
		if( ! sortTimeResult( d1, 14 ) ) {
			R_5f21e( 14 );// java 201 to 14, copy player's name
			b_1bbb1 |= 0x41;
		}
		int d2 = b_racer_id_3;
		sortTimeResult( d1, d2 );

		d2 += 12;
		addTime( d1, d2 );

		if( 4 == b_players_lap ) {
			d1 = d2;
			if( !sortTimeResult( d1, 13 ) ) {
				b_race_winner_id = b_racer_id_3;
			}
			if( !sortTimeResult( d1, 15 ) ) {
				R_5f21e( 15 );// java 214 to 15
				b_1bbb1 |= 0x81;
			}
		}
	}

	// new_lap_sub1
	/**
	 *
	 * @param d1 as byte. 0 - player, - 1 opponent
	 */
	private static void newLapSub1(final int d1) {
		if( (0 == b_machine) || (1 != d1) || (1 == b_opponents_lap) ) {
			return;
		}

		final byte b_backup_players_lap = b_players_lap;
		final byte b_backup_racer_id_3 = b_racer_id_3;

		b_players_lap = b_opponents_lap;
		b_racer_id_3 = b_opponents_id;
		processTime( d1 );

		b_players_lap = b_backup_players_lap;
		b_racer_id_3 = b_backup_racer_id_3;
	}

	// show_lap_time
	private static void showLapTime() {
		printLapTime( b_1bbcc != 0, 2, 0 );
	}

	// start_of_new_lap
	/**
	 *
	 * @param d1 as byte, 0 - player, 1 - opponent
	 */
	private static void startOfNewLap(int d1) {
		if( (0 == d1) && (0 != (b_off_map_status & 0x40)) ) {
			return;
		}
		// sonl1:
		int d0 = 0, d2 = 0;
		switch( d1 ) {
			case 0:
				d0 = b_players_road_piece;
				d2 = b_1bb1e;
				break;
			case 1:
				d0 = b_opponents_road_piece;
				d2 = b_1bb1f;
				break;
		}
		if( d2 < 0 ) {
			if( d0 == b_half_a_lap_section ) {
				switch( d1 ) {
					case 0:
						b_1bb1e = 0;
						break;
					case 1:
						b_1bb1f = 0;
						break;
				}
			}
			return;
		}
		// sonl3:
		if( d0 != b_start_finish_section ) {
			return;
		}
		switch( d1 ) {
			case 0:
				b_1bb1e = (byte)0x80;
				break;
			case 1:
				b_1bb1f = (byte)0x80;
				break;
		}
		switch( d1 ) {
			case 0:
				++b_players_lap;
				d0 = b_players_lap;
				break;
			case 1:
				++b_opponents_lap;
				d0 = b_opponents_lap;
				break;
		}
		if( 1 != d0 ) {
			if( (0 == b_machine) || (1 != d1) || (0 == b_1bb76) ) {
				// sonl4:
				newLapSub( d1 );
			}
		}
		// sonl5:
		newLapSub1( d1 );
		if( 0 == d1 ) {
			d0 = b_players_lap;
			if( 1 != d0 ) {
				processTime( d1 );
				d0 = 27;
				b_1bbcc = (byte) d0;
				showLapTime();
				copyLapTime();
			}
			// sonl6:
			// edge_x1_offset = (byte) d1;

			b_print_column = 6;
			b_print_row = 22;
			b_print_fine_x = 2;
			b_print_fine_y = 2;
			printDigit1( b_players_lap );
			b_print_fine_x = 0;
			b_print_fine_y = 0;

			// d1 = edge_x1_offset;
		}
		// sonl7:
		switch( d1 ) {
			case 0:
				d0 = b_players_lap;
				break;
			case 1:
				d0 = b_opponents_lap;
				break;
		}
		if( 1 != d0 ) {
			d2 = 2;
			if( ! sortTimeResult( d1, d2 ) ) {
				// move.b d1, d0, lsr.b	#1, d0, roxr.b	#2, d0
				b_1bbb2 = (byte) (d1 << 6);// d1 == 0 ? 0 : 0x40
				if( 0 != d0 ) {
					b_1bbcc = 0;
					showBestLapTime();
				}
			}
		}
		// sonl8:
		resetTimeResult( d1 );
	}

	/** Fine Y coordinates for print, 0..3 - row, 4...7 - fine y */
	private static final byte[] b_y_coords_for_lap_time = {
			22, 23, 22, 23,
			2, 4, 2, 4
		};

	// print_lap_time
	/**
	 *
	 * @param flag flag. true - print ms, d0 as byte
	 * @param d1 y coord id, as byte
	 * @param timeID time id, d2 as byte. 0 - current player, 2 - best
	 */
	private static void printLapTime(final boolean flag, int d1, int timeID) {
		// b_1bbe8 = (byte) d2;
		// b_1bb89 = (byte) d0;
		b_print_fine_y = b_y_coords_for_lap_time[ d1 + 4 ];
		b_or_char_with_screen = true;

		b_print_column = 34;
		b_print_row = b_y_coords_for_lap_time[ d1 ];
		b_print_fine_x = 5;
		jPrintCharacter( ':' );

		b_print_column += 2;
		b_print_fine_y -= 2;
		b_print_fine_x = 2;
		jPrintCharacter( '.' );

		b_or_char_with_screen = false;
		b_print_fine_y += 2;
		b_print_column -= 5;
		b_print_fine_x = 6;

		int d0 = b_time_results_min[ timeID ] % 10;// d0 = ba2[ d2 ] & 0x0F;
		printDigit( d0 );

		++b_print_column;
		b_print_fine_x = 3;
		// d2 = b_1bbe8;
		d0 = (b_time_results_sec[ timeID ] / 10) % 10;// d0 = ba2[ d2 ] >> 4;
		printDigit( d0 );

		// d2 = b_1bbe8;
		d0 = b_time_results_sec[ timeID ] % 10;// d0 = ba2[ d2 ] & 0x0F;
		printDigit( d0 );

		b_print_fine_x += 4;
		// d2 = b_1bbe8;
		d0 = (b_time_results_ms[ timeID ] / 10) % 10;// d0 = ba2[ d2 ] >>> 4;
		if( !flag  ) {
			d0 = 0xF0;
		}
		printDigit( d0 );

		// d2 = b_1bbe8;
		d0 = b_time_results_ms[ timeID ] % 10;
		if( !flag) {
			d0 = 0xF0;
		}
		printDigit( d0 );

		b_print_fine_x = 0;
		b_print_fine_y = 0;
	}

	// show_best_lap_time
	private static void showBestLapTime() {
		copyStopWatch0();
		printLapTime( true, 3, 2 );
	}

	// add_to_lap_time1
	/**
	 *
	 * @param d0 time in ms to add, as byte
	 * @param d1 player id, 0 - player, 1 - opponent, d1 as byte
	 */
	private static void addToLapTime1(int d0, final int d1) {
		int d3 = b_time_results_ms[ d1 ];
		d0 += d3;
		if( d0 >= 100 ) {
			b_time_results_ms[ d1 ] = (byte) (d0 - 100);
			d0 = b_time_results_sec[ d1 ];
			++d0;
			b_time_results_sec[ d1 ] = (byte) d0;
			if( d0 >= 60 ) {
				b_time_results_sec[ d1 ] = 0;
				d0 = b_time_results_min[ d1 ];
				if( ++d0 < 10 ) {
					b_time_results_min[ d1 ] = (byte) d0;
				}
			}
			// atlt1:
			if( 0 == d1 ) {
				if( 0 == b_1bbcc ) {
					if( 0 != b_players_lap ) {
						showLapTime();
					}
				}
			}
			return;
		}
		b_time_results_ms[ d1 ] = (byte)d0;
	}

	// add_to_lap_time
	/**
	 *
	 * @param d1 as byte. 0 - player, 1 - opponent
	 */
	private static void addToLapTime(final int d1) {
		addToLapTime1( 19, d1 );// 19 ms per a game loop ?
	}

	// copy_lap_time
	private static void copyLapTime() {
		b_57c6b = b_time_results_min[ 0 ];
		b_57c6c = b_time_results_sec[ 0 ];
		b_57c6d = b_time_results_ms[ 0 ];
	}

	// display_lap_time_sub
	private static void displayLapTimeSub() {
		int d1 = 1;// opponent or player
		do {
			if( 0 == b_laps_when_race_finished ) {
				int d0 = -1;
				switch( d1 ) {
					case 0:
						d0 = b_players_lap;
						break;
					case 1:
						d0 = b_opponents_lap;
						break;
				}
				if( d0 == b_lap_that_finishes_race ) {
					b_laps_when_race_finished = (byte) d0;
					if( 0 == b_1bb6c ) {
						b_1bb6c = 44;
					}
					final int d2 = copyChequeredFlag();
					String s;
					if( 11 != d2 ) {
						s = Text.STR_LAPS_OVER;// d2 = 84;// laps over
						if( b_race_mode < 0 ) {
							s = Text.STR_RACE_WON;// d2 = 8;// race won
						}
					} else {
						// dlts3:
						b_1bbb4 = (byte) 0x80;
						s = Text.STR_RACE_LOST;// d2 = 28;// race lost
					}
					// no_opponent3
					signalPromptRequired( 4, s );// d2 );
				}
			}
			b_1bbb4 = (byte)((b_1bbb4 & 0xBF) | b_1bbb2);
		} while( --d1 >= 0 );
	}

	// dlt6 java: part of displayLapTime
	private static void displayLapTimeLabel6() {
		if( 0 != b_1bb6c ) {
			if( b_1bb6c >= 0 ) {
				b_white_prompts = ((b_1bb6c & 0x04) != 0);// ((d0 >>> 2) & 1) != 0;
				if( (0 == b_car_on_chains_countdown) && (!b_touching_road) ) {
					// Not touching road
					if( b_1bb6c < 6 ) {
						return;
					}
				}
				// dlt7:
				if( b_fourteen_frames_elapsed < 0 ) {
					return;
				}
				if( --b_1bb6c != 0 ) {
					return;
				}
			}
			// dlt8:
			b_1bb6f = true;
			b_1bb6c = (byte) 0x80;
		}
	}

	// display_lap_time
	private static void displayLapTime() {
		++b_loop_counter;
		int d2 = 0;
		int d0 = 0xEE;
		// if( 0 != d0 ) {
			d0 += (b_1bbcf & 0xFF);
			b_1bbcf = (byte) d0;
			if( d0 <= 255 ) {
				--d2;
			}
		// }
		// dlt1:
		b_fourteen_frames_elapsed = (byte) d2;
		d0 = b_1bbcc;
		if( 0 != d0 ) {
			--b_1bbcc;
			if( 0 == b_1bbcc ) {
				showBestLapTime();
			}
		}
		if( b_race_mode < 0 ) {
			final int d1 = 1;// opponent
			if( 0 != b_machine ) {
				if( ! b_57c3a ) {
					if( 0 == b_1bb76 ) {
						// dlt3:
						addToLapTime( d1 );
					}
					startOfNewLap( d1 );
				}
			} else {
				// dlt3:
				addToLapTime( d1 );
				// dlt4:
				startOfNewLap( d1 );
			}
		}
		// no_opponent2
		// for player only
		addToLapTime( 0 );
		copyLapTime();
		startOfNewLap( 0 );
		displayLapTimeSub();
		if( b_1bb75 ) {
			displayLapTimeLabel6();
		}
		// dlt6:
		displayLapTimeLabel6();
	}

	// display_opponents_distance
	/**
	 *
	 * @param g Canvas
	 */
	private static void displayOpponentsDistance(final Canvas g) {
		if( 0 == (b_main_loop_count & 0x03) ) {// do every fourth frame
			int d0 = w_smallest_distance_between_players & 0xFFFF;// TODO verify
			int d3 = d0 >>> 2;
			d0 += d3;
			d0 >>>= 2;// (5/4 distance) / 4

			b_print_fine_x = 1;
			b_print_fine_y = 4;
			b_print_column = 6;
			b_print_row = 23;

			// print distance: ' xxxx', '-xxxx'
			printDigit1( b_opponent_behind_player ? '-' - '0' : ' ' - '0' );
			d3 = d0 / 1000;
			printDigit1( d3 );
			d0 -= d3 * 1000;
			d3 = d0 / 100;
			printDigit1( d3 );
			d0 -= d3 * 100;
			d3 = d0 / 10;
			printDigit1( d3 );
			printDigit1( d0 - d3 * 10 );

			b_print_fine_x = 0;
			b_print_fine_y = 0;

			if( 0 == b_laps_when_race_finished ) {
				copyChequeredFlag();
			}
		}
	}

	// copy_message_panel
	private static void copyMessagePanel(final Canvas g) {
		copyGraphic( g, SPRITE_MESSAGE_PANEL );
	}

	// R_57964
	private static void R_57964() {
		if( 0 == b_machine ) {
			return;
		}
		int d0 = 0;
		if( 0 != b_laps_when_race_finished ) {
			d0 = b_1bbb4 | 0x20;
		}
		if( 0 != b_1bbb4 ) {
			d0 |= 0x10;
		}
		int d3 = b_1bbe2;
		if( d3 == b_machine ) {
			d0 |= 0x08;
		}
		b_57c63 = (byte) d0;
	}

	// copy_preview_background
	/**
	 * Copy 136 lines from 8 to 136 (5440 bytes) from screen_mem to screen1
	 *
	 * @param g Canvas
	 */
	private static void copyPreviewBackground(final Canvas g) {
		// move.l	lw_screen_mem,a0
		// move.l	lw_screen1,a3
		// add.l	#8*40,a0
		// add.l	#8*40,a3
		// move.l	a0,a2
		// add.l	#136*40,a2
		// g.drawBitmap( jGetImage( R.drawable.preview ),
		//		0, 8 * PF, SCREEN_WIDTH, (136 + 8) * PF,
		//		0, 8 * PF, SCREEN_WIDTH, (136 + 8) * PF, paint );
		g.drawBitmap( jGetImage( R.drawable.preview ),
				new Rect( 0, 8 * PF, SCREEN_WIDTH, 8 * PF + (136 + 8) * PF ),
				new Rect( 0, 8 * PF, SCREEN_WIDTH, 8 * PF + (136 + 8) * PF ),
				paint );
	}

	// draw_road_sections_for_preview
	/**
	 *
	 * @param g Canvas
	 * @param x in d1 as byte. -8...+8
	 * @param z in d2 as byte. 0...16
	 */
	private static void drawRoadSectionsForPreview(final Canvas g, int x, int z) {
		// store to restore
		b_map_x = (byte)x;
		b_map_z = (byte)z;
		//
		if( 0 == (w_rough_player_angle & Trigonometry.ANGLE_180_DEGREES) ) {
			if( 0 != (w_rough_player_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				final int temp = x;
				x = z;
				z = -temp;
			}
		} else {
			if( 0 != (w_rough_player_angle & Trigonometry.ANGLE_90_DEGREES) ) {
				final int temp = x;
				x = -z;
				z = temp;
			} else {
				x = -x;
				z = -z;
			}
		}
		b_map_x_shift = (byte)x;
		b_map_z_shift = (byte)z;
		w_near_sections_done2 = 0;
		b_near_sections_done = 0;

		final int d0 = getValidMapNumber();// 1st byte is d1
		if( d0 >= 0 ) {
			if( 0xFF != (d0 & 0xFF) ) {
				b_current_road_piece = (byte) d0;
				b_coord_offset_zero_or_four = 0;
				b_edge_offset_left_x2 = (byte) 0x80;
				b_edge_offset_right_x2 = (byte) 0x80;
				makeNearRoadCoords();
				b_at_side_byte = (byte) 0xE0;
				w_players_x_offset_from_road_centre = (short) 0x8000;

				if( (w_screen_y_coords[ 0 ] > w_screen_y_coords[ 2 / 2 ])
					|| (w_screen_y_coords[ 16 / 2 ] > w_screen_y_coords[ 18 / 2 ])
					|| (w_screen_y_coords[ b_coords_of_last_section / 2 ] > w_screen_y_coords[ (b_coords_of_last_section + 2) / 2 ]) )
				{
					w_players_x_offset_from_road_centre &= 0x00FF;
				}

				b_use_lines_color = ((0 == (b_curved_section & 0xC0))
						&& (0 == (w_rough_difference_angle & Trigonometry.ANGLE_90_DEGREES)));

				b_near_section_flags[ 0 ] = 0;
				b_near_section_flags[ b_coords_of_last_section ] = 0;
				w_road_section_offset = w_dnr_value;

				b_pit_and_start_byte = 0;
				b_edge_offset_left_x2 = 0;
				b_edge_x1_offset = 0;
				w_road_section_offset += 16;

				lw_screen_coords_ptr = 0;
				// mnre13( 0 );
				makeNearRoadEdgesCommon( 0, false, true );
				drawNearRoad( g );
			}
		}
		// restore
		// bd1 = map_x;
		// bd2 = map_z;
	}

	// draw_road_for_preview
	/**
	 * sa6 = {@link #w_coord_y_values_fp5}
	 *
	 * @param g Canvas
	 */
	private static void drawRoadForPreview(final Canvas g) {
		// scan the map from the right bottom by columns from bottom to top.
		// z: vertical, x: horizontal
		int d2 = 16;// x why 16 and not 15?
		do {
			int d1 = 8;// z
			b_1bb93 = false;

			do {// 8...1
				drawRoadSectionsForPreview( g, d1, d2 );
				delay_2ms();// java for Amiga effect
			} while( --d1 > 0 );

			d1 = -8;// z 0xF8, -8...0
			b_1bb93 = true;

			do {// 248...256 or
				drawRoadSectionsForPreview( g, d1, d2 );
				delay_2ms();// java for Amiga effect
			} while( ++d1 <= 0 );
		} while( --d2 >= 0 );
	}

	private static final class CameraPreviewPosition {
		private final byte mX;
		private final byte mZ;
		private final byte mAngle;

		private CameraPreviewPosition(final byte x, final byte z, final byte angle) {
			mX = x;
			mZ = z;
			mAngle = angle;
		}
	}

	/**
	 * camera_preview_position_data
	 *
	 * Camera position data for preview
	 * 4 columns: positions
	 * 3 rows: x, z, y angle
	 */
	static CameraPreviewPosition[] camera_preview_position_data = {
			/*
			// positions: 0, 1, 2, 3
			0x04, 0x00, 0x04, 0x08,// x
			0x00, 0x04,	0x08, 0x04,// z
			0x00, 0x40, (byte)0x80, (byte)0xc0// y angle
			*/
			new CameraPreviewPosition( (byte)0x04, (byte)0x00, (byte)0x00 ),
			new CameraPreviewPosition( (byte)0x00, (byte)0x04, (byte)(Trigonometry.ANGLE_90_DEGREES >> 8) ),
			new CameraPreviewPosition( (byte)0x04, (byte)0x08, (byte)(Trigonometry.ANGLE_180_DEGREES >> 8) ),
			new CameraPreviewPosition( (byte)0x08, (byte)0x04, (byte)(Trigonometry.ANGLE_270_DEGREES >> 8) ),
		};

	// draw_road_preview
	/**
	 * draw road preview
	 */
	private static void drawRoadPreview(final Canvas g1) {
		w_dnr_value = 96;
		b_unused_flag = true;
		final int d1 = camera_preview_position_id & 3;

		lw_players_world_x = camera_preview_position_data[ d1 ].mX << 24;
		lw_players_world_z = camera_preview_position_data[ d1 ].mZ << 24;
		w_players_y_angle = (short)(camera_preview_position_data[ d1 ].mAngle << 8);

		lw_players_world_y_fp16 = 0x03_F0_00_00;

		b_which_screen2 = false;

		w_y_shift = 1792;
		drawWorldInitialisation();

		copyPreviewBackground( screen2_canvas );
		fadeScreenIn();

		b_track_preview = true;
		drawRoadForPreview( g1 );
		b_track_preview = false;
		w_dnr_value = 0;
		b_unused_flag = false;
	}
/*
	b_control_keys
* $44=RETURN
* $22=D
* $21=S
* $40=SPACE
* $2a=HASH
	dc.b	$2a,$40,$21,$22,$44,1,2,3,4,0
*/
	private static final byte[] b_control_keys = {
		KeyEvent.KEYCODE_DPAD_DOWN, KeyEvent.KEYCODE_SPACE,
		KeyEvent.KEYCODE_DPAD_LEFT, KeyEvent.KEYCODE_DPAD_RIGHT,
		KeyEvent.KEYCODE_ENTER,
		KeyEvent.KEYCODE_1, KeyEvent.KEYCODE_2, KeyEvent.KEYCODE_3,
		KeyEvent.KEYCODE_4, KeyEvent.KEYCODE_0
};

	// test_key
	/**
	 * Check key code
	 *
	 * @param bd1 key code
	 * @return true - z
	 */
	private static boolean testKey(int bd1) {
		// return key_array[ bd1 ] - AMIGA_KEY_PRESSED;
		// return key_array == bd1;
		synchronized( key_array ) {
			return key_array.contains( bd1 );
		}
	}

	// test_key_F5
	/**
	 * If F5 pressed
	 *
	 * @return true - z
	 */
	private static boolean testKeyF5() {
		return testKey( KeyEvent.KEYCODE_F5 );// F5
	}

	// data_link_test_key
	/**
	 *
	 * @param d0 value
	 * @param d1 code
	 * @return {@code true} - z
	 */
	static boolean dataLinkTestKey(final int d0, final int d1) {
		if( 0 == b_machine ) {
			return testKey( d1 );
		}
		if( 0 == d0 ) {
			// d0 = 0xFF;
			return false;
		}
		// d0 = 0;
		return true;
	}

	/**
	 * 10654 R.5d980 read input for other views
	 *
	 * @return false = cc, true = cs
	 */
	private static boolean readInputInRoadPreview() {
		// b_5d9da = 0x1F;
		do {
			synchronized( key_array ) {
				key_array.clear();// java to avoid multiple rotations. Amiga draws a road several seconds
			}
			jKeyPollDelay();

			getPlayersInput();
			// if( 0 != (players_input & 0x1F) ) {
				if( 0 != (b_players_input & CTRL_PLAYER_RIGHT) ) {
					++camera_preview_position_id;
					return true;
				}
				if( 0 != (b_players_input & CTRL_PLAYER_LEFT) ) {
					--camera_preview_position_id;
					return true;
				}
			// }
		} while( 0 == (b_players_input & CTRL_PLAYER_BOOST) );
		return false;
	}

	// start_engine_sound
	private static void startEngineSound() {
		SoundManager.soundEffect( SoundManager.SOUND_TICK_OVER );// tick over
		updateEngineRevs();
	}

	// wait_for_fire
	private static void waitForFire() {
		do {
			getPlayersInput();
		} while( 0 != (b_players_input & CTRL_PLAYER_BOOST) );

		delay( 5 );

		do {
			getPlayersInput();
		} while( 0 == (b_players_input & CTRL_PLAYER_BOOST) );
	}

	private static final byte[] b_TAB_5fadc = {
			(byte) 0x00, (byte) 0x00, (byte) 0x00, (byte) 0x01,
			(byte) 0x04, (byte) 0x03, (byte) 0x02,
			// (byte) 0xb4, (byte) 0xa4, (byte) 0x94, (byte) 0x84, (byte) 0x74
		};
			
	// redefine_keys
	private static void redefineKeys(final Canvas g) {
		if( testKey( KeyEvent.KEYCODE_F1 ) ) {
			// move.l lw_screen2, -(sp)
			// move.l lw_screen1, lw_screen2

			_label1:
			do {
				copyMessagePanel( g );
				setRacePrompt( 4, Text.STR_DEFINE_KEYS );// 100

				b_TAB_5fadc[ 1 ] = 1;

				// _label2:
				do {
					delay( 40 );

					int d1 = 4;
					// _label3:
					do {
						copyMessagePanel( g );
						b_TAB_5fadc[ 0 ] = (byte) d1;
						setRacePrompt( 4, Text.b_TAB_5fadc_7[ d1 ] );

						_label4:
						do {
							d1 = KeyEvent.KEYCODE_DPAD_RIGHT;// 103;// Right Amiga

							// _label5:
							do {
								lw_road_height_fp8 = d1;
								if( testKey( d1 ) ) {
									int d0 = lw_road_height_fp8;
									d1 = b_TAB_5fadc[ 0 ];
									final int d2 = b_TAB_5fadc[ 2 + d1 ];
									final byte[] ba2 = b_control_keys;
									if( (0 == b_TAB_5fadc[ 1 ]) && (ba2[ d2 ] != d0) ) {
										copyMessagePanel( g );
										setRacePrompt( d0, Text.STR_FAULT_FOUND );// 212
										delay( 40 );
										continue _label1;// jmp _label1;
									}
									if( (0 != b_TAB_5fadc[ 1 ]) ) {
										// _label6
										ba2[ d2 ] = (byte) d0;
									}
									// _label7
									SoundManager.soundEffect( SoundManager.SOUND_CHIME );

									// _label8
									do {
										jKeyPollDelay();
										d1 = lw_road_height_fp8;
									} while( testKey( d1 ) );
									delay( 3 );
									break _label4;
								} else {
									// _label9
									d1 = lw_road_height_fp8;
									if( --d1 != 0 ) {
										continue;// bne _label5;
									}
									continue _label4;// bra _label4;
								}
							} while( true );
						} while( true );// _label4
						// _labela
						d1 = b_TAB_5fadc[ 0 ];
					} while( --d1 >= 0 );// bpl _label3;
					copyMessagePanel( g );
					b_TAB_5fadc[ 1 ]--;
					if( b_TAB_5fadc[ 1 ] < 0 ) {
						break _label1;
					}
					setRacePrompt( 4, Text.STR_VERIFY_KEYS );// 196
				} while( true );// jmp _label2;
			} while( true );
			// _labelb
			setRacePrompt( 2, Text.STR_PAUSED );// 76
			// move.l( sp ) +, lw_screen2
		}
		// _labelc
	}

	// R_577fa
	private static void R_577fa() {
		transmit_byte_total = 0;

		// Transmit $125634
		addAndTransmitWord( 0x1256 );
		addAndTransmitByte( 0x34 );
		
		// Transmit info bytes
		addAndTransmitByte( b_57c63 );
		addAndTransmitByte( b_57c62 );
		addAndTransmitByte( b_57c66 );
		addAndTransmitByte( b_57c64 );
		addAndTransmitByte( b_57c65 );
		addAndTransmitByte( transmit_byte_total );
	
		delay_10ms();
		delay_5ms();
	}

	// unpause_request
	private static void unpauseRequest(final Canvas g) {
		if( 0 != b_machine ) {

			b_57c63 = 1;
			b_57c64 = (byte) 0xF1;
			if( ! R_579b0() ) {
				// ur1
				do {
					redefineKeys( g );
					if( !testKey( KeyEvent.KEYCODE_0 ) ) {
						if( testKeyF5() ) {
							break;
						}

						R_577fa();
						if( R_5786c() ) {
							continue;
						}

						if( 0xf2 != (b_57c5e & 0xFF) ) {
							continue;
						}
						// ur2
						b_57c64 = (byte) 0xF2;
						R_579b0();
						break;
					}
				} while( true );
			}
		} else {
			// ur3
			do {
				redefineKeys( g );// redefine keys
			} while( !testKey( KeyEvent.KEYCODE_0 ) );
		}
		// ur4
		b_57c63 = 0;
		b_second_machine = 0;
	}

	// pause_request
	/**
	 * Display messages on a visible screen
	 */
	private static void pauseRequest() {
		if( ! b_1bba4 ) {
			if( ! dataLinkTestKey( b_second_machine, KeyEvent.KEYCODE_P ) ) {// KeyEvent.VK_PAUSE Amiga: P
				return;
			}
		}
		if( ! b_1bba4 ) {
			b_1bba4 = true;
			return;
		}
		b_1bba4 = false;
		b_57c5b = true;
		b_no_wheel_update = true;
		SoundManager.soundOff();

		w_engine_revs_change = 0;

		final boolean b_prompt_required_backup = b_prompt_required;
		final String b_prompt_offset_backup = b_prompt_offset;
		final byte b_prompt_groups_backup = b_prompt_groups;

		// move.l	lw_screen2,-(sp)
		// move.l	lw_screen1,lw_screen2		draw into visible screen
		final Canvas g;
		synchronized( Game.class ) {
			g = b_which_screen ? screen2_canvas : screen1_canvas;
		}
		copyMessagePanel( g );

		signalPromptRequired( 2, Text.STR_PAUSED );// paused
		displayRacePrompts();
		// move.l	(sp)+,lw_screen2

		unpauseRequest( g );

		b_prompt_groups = b_prompt_groups_backup;
		b_prompt_offset = b_prompt_offset_backup;
		b_prompt_required = b_prompt_required_backup;

		b_57c5b = false;
		b_no_wheel_update = false;

		startEngineSound();
	}

	// FIXME word_print
	private static void wordPrint() {

	}

	// update_screens
	private static void updateScreens() {
		while( b_frame_count != 0 ) {
			jDelay( 5 );
		}
		b_frame_count = MIN_FRAMES;

		synchronized( Game.class ) {
			b_which_screen = !b_which_screen;
			// screen_mem = b_which_screen
			//		? screen1
			//		: screen2;
			screen_mem_canvas = b_which_screen
					? screen1_canvas
					: screen2_canvas;
		}
/*
		which_screen3 ^= 1;
		Image d0, d3;
		d3 = d0 = screen_mem;
		which_screen = !which_screen;
		if( which_screen ) {
			which_screen2 = true;
			d0 = screen1;
		} else {
			which_screen2 = false;
			d3 = screen1;
		}
		screen1 = d0;
		screen2 = d3;
		setCurrentScene();
 */
	}

	// set_and_preview_road, set course data, draw course
	static void setAndPreviewRoad(final Canvas g) {
		initialiseData();
		Arrays.fill( lw_screen_data, 0 );// java, Amiga bug?

		j_color_pallete_offset = COLOR_BASE_PREVIEW;

		fadeScreenOut();

		b_5d724 = 0x40;

		setTextColor( 15 );
		b_or_char_with_screen = true;

		// g.drawImage( TheApp.getImage( IMG_PREVIEW ), 0, 0, null );
		// g.setColor( TRACK_PREVIEW_BACKGROUND_COLOR );
		// g.fillRect( 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT );
		final Canvas g2 = screen2_canvas;
		g2.drawBitmap( jGetImage( R.drawable.preview ), 0, 0, null );

		if( b_league_offset == 0 ) {
			b_side_lines_color = SIDE_LINES_COLOR;
			b_sides_color = SIDES_COLOR_B;
		} else {
			b_side_lines_color = SUPER_SIDE_LINES_COLOR;
			b_sides_color = SUPER_SIDES_COLOR_B;
		}

		b_print_text_on_screen_2 = true;// java
		printRoadTitle( g2 );// print road title
		b_print_text_on_screen_2 = false;// java

		setTextColor( 3 );

		id2();

		setRoadData1( b_road_id );
		makeRoadUnderMap();
		moveDrawBridge();
		Scenery.initialiseMountains();
		setRoadData2();

		drawRoadPreview( g );// draw road preview

		printLeagueText( g, 5 );// 'Broken by QUARTEX ...'

		while( readInputInRoadPreview() ) {// read input for other views
			drawRoadPreview( g );// draw road preview
		}

		fadeScreenOut();

		b_5d724 = 0;
	}

	// race_and_practise
	private static void raceAndPractise() {

		if( 0 != b_machine ) {
			delay_100ms();
			delay_100ms();
			flushReceivedBytes();
			delay_60ms();
		}

		b_57c5b = false;
		b_57c63 = 0;
		initialiseData();

		// b_which_screen = false;
		// java: have to draw on screen 2 only for fade in
		b_print_text_on_screen_2 = true;
		Canvas g = screen2_canvas;
		makeCarScreens( g );

		j_color_pallete_offset = COLOR_BASE_CARS;

		setBackgroundColor( 14 );
		setTextColor( 0 );

		initialiseDamageBar();

		if( 0 != b_league_offset ) {// super league colours
			b_side_lines_color = SUPER_SIDE_LINES_COLOR;
			b_sides_color = SUPER_SIDES_COLOR_B;
		} else {
			b_side_lines_color = SIDE_LINES_COLOR;
			b_sides_color = SIDES_COLOR_B;
		}

		printLapBoostText( g );

		copyChequeredFlagDull( 11 );
		copyStopWatch( 11 );

		b_opponents_road_piece = b_players_start_piece;
		w_opponents_distance_into_piece = Road.OPPONENTS_START_DISTANCE_INTO_PIECE;
		w_opponents_road_x_position = Road.OPPONENTS_START_ROAD_X_POOSITION;
		initialiseOpponentData();

		byte piece = b_players_start_piece;
		if( b_machine == SLAVE ) {
			b_swing_from_left = true;
		}

		rap3:
		while( true ) {
			setPlayersRestartPosition( piece );
			updateWheelPositions();
			initialiseSparksTable();

			drawWorld( g );

			displayLapTime();
			--b_main_loop_count;

			// java: have to draw on screen 2 only for fade in
			// updateScreens();
			// g = screen_mem.getGraphics();

			carMovement();

			drawWorld( g );

			displayLapTime();
			--b_main_loop_count;
			displaySpeedBar();

			// java: have to draw on screen 2 only for fade in
			b_print_text_on_screen_2 = false;
			// updateScreens();

			b_1bb72 = true;
			// b_5d724 = (byte) 0x80;// update wheels, java: moved after fade in to avoid bright wheels

			delay( 3 );
			fadeScreenIn();

			// java init switching screen
			updateScreens();
			g = screen_mem_canvas;

			b_5d724 = (byte) 0x80;// allow updating wheels

			startEngineSound();

			race_loop:
			while( true ) {
				--b_main_loop_count;
				carControl();

				wordPrint();

				carMovement();
				updateEngineRevs();

				drawWorld( g );
				moveDrawBridge();
				displaySpeedBar();
				updateDamage();
				displayLapTime();
				displayOpponentsDistance( g );

				if( (b_1bb6f & b_which_screen) && (b_1bbcc == 0) ) {

					if( (b_machine == 0) || (b_1bb76 == 0) ) {
						if( (b_car_on_chains_countdown != 0) || b_touching_road ) {
							// Touching road
							if( b_laps_when_race_finished == 0 ) {
								if( (b_machine == 0) || (b_1bbb4 == b_1bbe2) ) {
									copyStopWatch( 11 );
									b_1bbb4 = (byte) 0xC0;
								}
							}

							copyMessagePanel( g );
							setRacePrompt( 4, Text.STR_PRESS_FIRE );// 60 press fire

							w_engine_revs_change = -8;
							w_wheel_rotation_speed = 0;
							if( SMASHED_COUNTDOWN_MAX == b_smashed_countdown ) {
								if( b_damage_hole_position != 0 ) {
									copyDamageHole( b_damage_hole_position );
								}
							}

							updateScreens();

							b_turn_engine_off = true;
							b_57c5b = true;
							R_57964();
							SoundManager.soundOff();
							waitForFire();
							rap11();
							return;
						}
					}
				} // if
				// rapa:
				updateScreens();
				g = screen_mem_canvas;

				pauseRequest();
				if( (0 == b_car_on_chains_countdown) && (b_off_map_status < 0) && b_touching_road )
				{// Touching road
					if( w_players_smaller_y < 0x200 ) {
						b_1bb75 = (b_off_map_status < 0);
					}
					if( --b_opponents_road_piece_m64_low < 0 ) {// the value is set in set.road.centre.values
						++b_opponents_road_piece_m64_low;
						if( b_1bb6c == 0 ) {
							SoundManager.soundOff();
							piece = b_1bb9b;
							continue rap3;// while true, lower car back onto
							// track
						}
					}
				}
				// rapd:
				if( b_car_on_chains_countdown == 0 ) {
					if( b_off_map_status < 0 ) {
						continue race_loop;
					}
					if( !b_touching_road ) {// can't quit if the car is not touching road
						continue race_loop;
					}
				}
				// Touching road
				// rape:
				if( dataLinkTestKey( b_1bb64, KeyEvent.KEYCODE_ESCAPE ) ) {// (byte)0x45 ) ) {// ESCAPE
					break;
				}
			} // race_loop
			break;
		} // rap3

		if( b_laps_when_race_finished == 0 ) {
			if( b_machine != 0 ) {
				b_1bbb4 = 0;
				if( b_machine != b_1bb64 ) {
					R_57964();
				} else {
					b_1bbb4 = (byte) 0xC0;
				}
			} else {
				b_1bbb4 = (byte) 0xC0;
			}
		} else {
			R_57964();
		}
		rap11();
	}

	// rap11:
	private static void rap11() {
		b_5d724 = 0;// java: here to stop drawing wheel on the active screen
		j_color_pallete_offset = COLOR_BASE_MENU;
		fadeScreenOut();

		// b_5d724 = 0;// java: moved earlie
		SoundManager.soundOff();
		if( b_race_mode >= 0 ) {
			b_damage_hole_position = b_1bbb5;
		} else {
			if( b_multi_no_of_players != 0 ) {
				b_DAT_1ca36[ b_racer_id_3 ] = b_damage_hole_position;
				if( b_machine != 0 ) {
					b_DAT_1ca36[ b_opponents_id ] = b_57c61;
					if( b_opponents_lap != 4 ) {
						resetTimeResult( 12 );
					}
				}
				if( b_players_lap != 4 ) {
					resetTimeResult( b_racer_id_3 + 12 );
				}
				b_damage_hole_position = b_1bbb5;
			}
		}
		saveRandomValues();
		b_57c5b = true;
		// java turn off off-screen buffer switching
		synchronized( Game.class ) {
			// screen_mem = screen1;
			screen_mem_canvas = screen1_canvas;
			b_which_screen = false;
		}
	}

	// Entry point
	// main_game_selection
	private static void mainGameSelection() {

		for( int i = 0; i < Text.opponents_names_sources.length; ++i ) {
			System.arraycopy( Text.opponents_names_sources[ i ], 0, opponents_names[ i ], 0, Text.opponents_names_sources[ i ].length );
		}

		b_machine = 0;

		b_57c62 = 0x40;
		if( !b_game_started ) {
			b_game_started = true;
		}
		mainInitialisation();

		Canvas g = screen_mem_canvas;

		// java
		g.drawBitmap( jGetImage( R.drawable.splash ), 0, 0, null );
		jDelay( 3000 );

		showMenuScreen( true );
		mainMenuSelection( g );// single player / multiplayer / computer link
		displayPlayers( g );// display players in divisions

		saveRandomValues();

		b_57c64 = (byte) 0xF3;
		R_579b0();// for data link

		mgs3:
		while( true ) {
			g = screen_mem_canvas;
			b_1bb5b = false;

			b_57c62 = 0x10;
			menuGame( g );// menu selection

			if( testKey( KeyEvent.KEYCODE_HELP ) ) {// if( 0 != testKey( AMIGA_KEY_CODE_HELP ) ) {
				return;// end the game
			}

			// not_to_DOS
			if( b_race_mode >= 0 ) {
				b_57c62 = 18;
				setAndPreviewRoad( g );

				b_57c62 = 0;
				raceAndPractise();

				showMenuScreen( false );
				continue mgs3;
			}

			// mgs5: If race selected
			prepareDivision();
			if( (b_max_races_in_division == 2) && (0 != b_machine) ) {
				b_max_races_in_division = 1;
			}
			// mgs6:
			do {
				int id1 = 23;
				do {
					resetTimeResult( id1 );
					if( id1 < 16 ) {
						b_time_results_min[ id1 ] = 9;
					}
				} while( --id1 >= 0 );

				b_best_lap_racer_id = 0;
				b_race_winner_id = 0;
				// mgs9:
				do {
					b_division = b_players_division;
					chooseRacersInDivision();
					if( b_1bb5b || ((b_racer_id_1 != b_racer_id_3) && (b_racer_id_2 != b_racer_id_3)) ) {
						promote();
					} else {
						b_opponents_id = (b_racer_id_1 != b_racer_id_3)
								? b_racer_id_1
								: b_racer_id_2;

						b_1bbb1 = (byte) 0xC0;
						b_57c62 = 17;
						displayOpponentsBeforeRace( g );
						if( R_58580() ) {
							b_1bb5b = true;
							b_1bbb4 = (byte) 0xC0;
							continue;
						}

						R_5879e( false );

						b_57c62 = 18;
						setAndPreviewRoad( g );

						R_5faf2( true );

						b_57c62 = 0;
						raceAndPractise();

						R_5879e( true );
						R_5867a();

						R_5faf2( false );

						R_5f25a( false );
						promote();
						R_5930c();

						showMenuScreen( false );
						if( 0 != b_1bbb1 ) {
							b_57c62 = 1;
							displayOpponentsBeforeRace( g );
							b_1bbb1 = 0;
						}
						b_57c62 = 2;
						showRaceResult( g );
						b_57c62 = 3;
						showRatingInDivision( g );
						if( 0 != b_multi_no_of_players ) {
							R_5ef76( g );
						}
					}
					// mgsd:
					++b_current_race_number;
					++w_57c72;
					if( ++b_1ca35 >= b_max_races_in_division ) {
						break;
					}
				} while( true );
				b_1ca35 = 0;
				if( 0 == b_multi_no_of_players ) {
					break;
				}
				R_5ef42( g );
				b_1bb5b = false;
				if( --b_5eb77 < 0 ) {
					if( ++b_57c67 >= 4 ) {
						mainInitialisation2();
					}
					continue mgs3;
				}
			} while( true );
			showPromotionRelegationWaitFire( g );
			R_5933a();
			displayPlayers( g );
			mainInitialisation2();
		} // while true
	}

	private volatile boolean mIsRunning = false;

	/** required to show and hide on-screen keyboard */
	private static MainActivity sActivity = null;
	private static Resources sResources = null;

	final void setKeyboardAccessInterface(final MainActivity activity) {
		sActivity = activity;
	}

	/**
	 * Constructor
	 */
	public Game(final Resources resources) {
		sResources = resources;
		// {@link java.awt.image.BufferedImage} is used
		// because test pixel color is used in {@link #damageLine()}
		screen1 = Bitmap.createBitmap( SCREEN_WIDTH, SCREEN_HEIGHT, Bitmap.Config.ARGB_8888 );
		screen2 = Bitmap.createBitmap( SCREEN_WIDTH, SCREEN_HEIGHT, Bitmap.Config.ARGB_8888 );
		// screen_mem = screen1;
		screen1_canvas = new Canvas( screen1 );
		screen2_canvas = new Canvas( screen2 );
		screen_mem_canvas = screen1_canvas;

		try {
			Bitmap img = jGetImage( R.drawable.clouds );
			for( int i = 0; i < clouds.length; i++ ) {
				clouds[ i ] = Bitmap.createBitmap( img, 0, i * 39 * PF, 65 * PF, 39 * PF );
			}
			img.recycle();
			// wheels
			img = jGetImage( R.drawable.wheels );
			for( int i = 0; i < SPRITE_WHEEL_SPRITES; i++ ) {
				graphic_info[ SPRITE_LEFT_WHEELS + i ].mImage =
						Bitmap.createBitmap( img, i * 24 * PF, 0, 24 * PF, 55 * PF );
				graphic_info[ SPRITE_RIGHT_WHEELS + i ].mImage =
						Bitmap.createBitmap( img, (144 - (i + 1) * 24) * PF, 0, 24 * PF, 55 * PF );
			}
			img.recycle();
			// flames
			img = jGetImage( R.drawable.flames );
			for( int i = 0; i < SPRITE_FLAME_SPRITES; i++ ) {
				int gi = SPRITE_LEFT_FLAMES + i;
				if( i == 2 ) {
					gi = SPRITE_LEFT_FLAME_3;
				}
				graphic_info[ gi ].mImage = Bitmap.createBitmap( img, 0, i * 27 * PF, 48 * PF, 27 * PF );
			}
			for( int i = 0; i < SPRITE_FLAME_SPRITES; i++ ) {
				int gi = SPRITE_RIGHT_FLAMES + i;
				if( i == 2 ) {
					gi = SPRITE_RIGHT_FLAME_3;
				}
				graphic_info[ gi ].mImage = Bitmap.createBitmap( img, 48 * PF, i * 27 * PF, 48 * PF, 27 * PF );
			}
			img.recycle();
			// flags
			img = jGetImage( R.drawable.flags );
			graphic_info[ SPRITE_CHEQUERED_FLAG_DULL ].mImage = Bitmap.createBitmap( img, 0, 0, 12 * PF, 8 * PF );
			graphic_info[ SPRITE_CHEQUERED_FLAG_BRIGHT ].mImage = Bitmap.createBitmap( img, 12 * PF, 0, 12 * PF, 8 * PF );
			img.recycle();
			// watch
			img = jGetImage( R.drawable.watch );
			graphic_info[ SPRITE_STOP_WATCH_DULL ].mImage = Bitmap.createBitmap( img, 0, 0, 12 * PF, 7 * PF );
			graphic_info[ SPRITE_STOP_WATCH_BRIGHT ].mImage = Bitmap.createBitmap( img, 12 * PF, 0, 12 * PF, 7 * PF );
			img.recycle();
			//
			graphic_info[ SPRITE_ENGINE_BLOCK ].mImage = jGetImage( R.drawable.engine );
			//
			graphic_info[ SPRITE_LEFT_EXHAUST_PANEL ].mImage = jGetImage( R.drawable.left_exhaust_panel );
			graphic_info[ SPRITE_RIGHT_EXHAUST_PANEL ].mImage = jGetImage( R.drawable.right_exhaust_panel );
			//
			graphic_info[ SPRITE_LEFT_TOP_CORNER ].mImage = jGetImage( R.drawable.left_top_corner );
			graphic_info[ SPRITE_RIGHT_TOP_CORNER ].mImage = jGetImage( R.drawable.right_top_corner );
			//
			img = jGetImage(R.drawable.chain_top );
			graphic_info[ SPRITE_LEFT_CHAIN_TOP ].mImage = img;
			graphic_info[ SPRITE_RIGHT_CHAIN_TOP ].mImage = img;
			//
			img = jGetImage(R.drawable.chain_bottom );
			graphic_info[ SPRITE_LEFT_CHAIN_BOTTOM ].mImage = img;
			graphic_info[ SPRITE_RIGHT_CHAIN_BOTTOM ].mImage = img;
			//
			img = jGetImage( R.drawable.hole );
			graphic_info[ SPRITE_HOLE_1 ].mImage = img;
			graphic_info[ SPRITE_HOLE_2 ].mImage = img;// TODO find the 2nd image for hole
			//
			img = jGetImage( R.drawable.smash );
			graphic_info[ SPRITE_SMASH_1 ].mImage = img;
			graphic_info[ SPRITE_SMASH_2 ].mImage = img;// TODO find the 2nd image for smash
			//
			graphic_info[ SPRITE_MESSAGE_PANEL ].mImage = jGetImage( R.drawable.message );
		} catch( final Exception e ) {
			e.printStackTrace();
		}
		SoundManager.init( resources );
	}

	private final void release() {
		synchronized( Game.class ) {
			screen1.recycle();
			screen2.recycle();
			screen1 = null;
			screen2 = null;
			// screen_mem = null;
		}
	}

	public final Bitmap getScene() {
		synchronized( Game.class ) {
			return b_which_screen
					? screen2
					: screen1;
		}
	}

	// Key listener
	public final void setKey(final int key) {
		// key_array = key;
		synchronized( key_array ) {
			key_array.add( key );
		}
		// Thread.currentThread().interrupt();
	}

	public final void resetKey(final int key) {
		// key_array = key;
		synchronized( key_array ) {
			key_array.remove( key );
		}
		// Thread.currentThread().interrupt();
	}

	public final void setKeyTyped(final int key) {
		key_code = key;
		Thread.currentThread().interrupt();
	}

	// Java
	@Override
	public final void run() {
		mIsRunning = true;
		/* temp initialization
		Game.mainInitialisation();
		Game.id2();
		b_road_id = 5;
		Game.setRoadData1( b_road_id );
		Game.makeRoadUnderMap();
		//Game.moveDrawBridge();
		Scenery.initialiseMountains();
		Game.setRoadData2();
		setAndPreviewRoad( screen_mem.getGraphics() );

		b_race_mode = true;
		b_57c62 = 0;
		Game.raceAndPractise();
		*/
		mainGameSelection();

		jLogErrLn("ended");
	}

	final void pause() {
		SoundManager.pause();
	}

	final void resume() {
		SoundManager.restart();
	}
}
